 /* devIObug.c */
/*****************************************************************
 *
 *      Original Author :            Jim Kowalkowski
 *      Modified By :                Greg Nawrocki
 *      Date:                        10-21-92
 *
 *****************************************************************
 *                         COPYRIGHT NOTIFICATION
 *****************************************************************

 * THE FOLLOWING IS A NOTICE OF COPYRIGHT, AVAILABILITY OF THE CODE,
 * AND DISCLAIMER WHICH MUST BE INCLUDED IN THE PROLOGUE OF THE CODE
 * AND IN ALL SOURCE LISTINGS OF THE CODE.

 * (C)  COPYRIGHT 1993 UNIVERSITY OF CHICAGO

 * Argonne National Laboratory (ANL), with facilities in the States of
 * Illinois and Idaho, is owned by the United States Government, and
 * operated by the University of Chicago under provision of a contract
 * with the Department of Energy.

 * Portions of this material resulted from work developed under a U.S.
 * Government contract and are subject to the following license:  For
 * a period of five years from March 30, 1993, the Government is
 * granted for itself and others acting on its behalf a paid-up,
 * nonexclusive, irrevocable worldwide license in this computer
 * software to reproduce, prepare derivative works, and perform
 * publicly and display publicly.  With the approval of DOE, this
 * period may be renewed for two additional five year periods.
 * Following the expiration of this period or periods, the Government
 * is granted for itself and others acting on its behalf, a paid-up,
 * nonexclusive, irrevocable worldwide license in this computer
 * software to reproduce, prepare derivative works, distribute copies
 * to the public, perform publicly and display publicly, and to permit
 * others to do so.

 *****************************************************************
 *                               DISCLAIMER
 *****************************************************************
 * NEITHER THE UNITED STATES GOVERNMENT NOR ANY AGENCY THEREOF, NOR
 * THE UNIVERSITY OF CHICAGO, NOR ANY OF THEIR EMPLOYEES OR OFFICERS,
 * MAKES ANY WARRANTY, EXPRESS OR IMPLIED, OR ASSUMES ANY LEGAL
 * LIABILITY OR RESPONSIBILITY FOR THE ACCURACY, COMPLETENESS, OR
 * USEFULNESS OF ANY INFORMATION, APPARATUS, PRODUCT, OR PROCESS
 * DISCLOSED, OR REPRESENTS THAT ITS USE WOULD NOT INFRINGE PRIVATELY
 * OWNED RIGHTS.

 *****************************************************************
 * LICENSING INQUIRIES MAY BE DIRECTED TO THE INDUSTRIAL TECHNOLOGY
 * DEVELOPMENT CENTER AT ARGONNE NATIONAL LABORATORY (708-252-2000).
 *****************************************************************

 * Modification Log:
 * -----------------
 * .01  10-21-92        jbk     initialized
 * .02  04-22-93	nda	eliminated record processing at init time
 * .03  04-22-93	nda	check rxMsg.cmd byte after message completion
 * .04  07-06-93        gjn	rewrote a fair amount of the code so that
 *                              1) The BUG is only scanned as often as the
 *                                 lowest scan rate record.
 *                              2) There are different levels of code debugging.
 * .05  09-10-93	gjn	added proper buffer structures for the proper
 *				operation of multiple nodes.
 *      ...
 */

#include	<vxWorks.h>
#include	<stdlib.h>
#include	<stdio.h>
#include	<string.h>
#include	<vme.h>
#include	<math.h>
#include	<tickLib.h>
#include	<alarm.h>
#include        <callback.h>
#include	<dbDefs.h>
#include	<dbAccess.h>
#include	<dbCommon.h>
#include	<fast_lock.h>
#include        <recSup.h>
#include	<devSup.h>
#include	<drvSup.h>
#include	<dbScan.h>
#include	<special.h>
#include	<module_types.h>
#include	<eventRecord.h>
#include	<drvBitBusInterface.h>
#include	<aiRecord.h>
#include	<aoRecord.h>
#include	<biRecord.h>
#include	<boRecord.h>
#include	<mbboRecord.h>
#include	<mbbiRecord.h>
#include	<menuScan.h>
/* commands */
#define READBACK        0x02
#define BOUT		0x12
#define AOUT_0	        0x22
#define AOUT_1	        0x42
/* types */
#define MBBI		0x01
#define MBBO		0x02
#define BO		0x04
#define BI		0x08
#define AO		0x10
#define AI		0x20
/* maximums */
#define MAXLINKS	4
#define MAXNODES	250
#define SHORT_FIRSTBYTE	0x00ff
#define LONG_FIRSTBYTE	0x000000ff
/* Create the dsets for devIObug */
static long init_all();
static long init_ai();
static long init_ao();
static long init_bo();
static long init_bi();
static long init_mbbi();
static long init_mbbo();
static long iobug_rdwr();
typedef struct {
    long number;
    DEVSUPFUN report;
    DEVSUPFUN init;
    DEVSUPFUN init_record;
    DEVSUPFUN get_ioint_info;
    DEVSUPFUN read_write;
    DEVSUPFUN special_linconv;
}   IODSET;
IODSET devAiIObug = {6, NULL, init_all, init_ai, NULL, iobug_rdwr, NULL};
IODSET devAoIObug = {6, NULL, init_all, init_ao, NULL, iobug_rdwr, NULL};
IODSET devBiIObug = {5, NULL, init_all, init_bi, NULL, iobug_rdwr};
IODSET devBoIObug = {5, NULL, init_all, init_bo, NULL, iobug_rdwr};
IODSET devMbbiIObug = {5, NULL, init_all, init_mbbi, NULL, iobug_rdwr};
IODSET devMbboIObug = {5, NULL, init_all, init_mbbo, NULL, iobug_rdwr};
/* xxDSET devXxxx={ 5,report,init,init_rec,get_ioint_info,read_write}; */
static unsigned char ao_cmd[] = {AOUT_0, AOUT_1};
static initFlag = 0;
int devIObugDebug = 0;
/***** devIObugDebug information *****/
/** devIObugDebug >= 0 --- initialization information **/
/** devIObugDebug >= 5 --- errors **/
/** devIObugDebug >= 10 -- messages sent **/
/** devIObugDebug >= 15 -- messages received **/
/** devIObugDebug >= 20 -- record filled from buffer **/
/** devIObugDebug >= 25 -- scan rate timing **/
/** devIObugDebug >= 30 -- buffer / readback values **/
/** devIObugDebug == 1  -- buffer values **/
/** devIObugDebug == 2  -- readback values **/
/** devIObugDebug >= 35 -- calls to process **/
extern struct drvBitBusEt drvBitBus;
#define OFF		0
#define ON		1

struct dprivate {
    struct dpvtBitBusHead bitbus;
    unsigned int link;
    unsigned int node;
    unsigned int port;
    unsigned int signal;
    char type;
    unsigned char cmd;
    struct dbCommon *precord;	/* at end for callback to get it */
};
struct bugnode {
    unsigned char readBuf[11];
    unsigned long lastScan;
    unsigned char nodeAlarm;
};
static struct bugnode *bn[MAXLINKS][MAXNODES];

/* forward references			*/
static int get_data(struct dbCommon * io);
static void send_cntl_trans();
static struct bugnode *get_bn(unsigned int link, unsigned int node);

static struct bugnode *get_bn(unsigned int link, unsigned int node)
{
    if(link>=MAXLINKS || node>=MAXNODES) {
        printf("devBUG: get_bn illegal link %u node %u\n",link,node);
        return(0);
    }
    if(bn[link][node]==0) {
        printf("devBUG: get_bn null link %u node %u\n",link,node);
    }
    return(bn[link][node]);
}

static int io_callback(struct dprivate * pcallback)
{
    struct dbCommon *precord;
    struct rset *prset;
    precord = pcallback->precord;
    prset = (struct rset *) (precord->rset);
    dbScanLock(precord);
    (*prset->process) (precord);
    dbScanUnlock(precord);
    return (0);
}
static struct dprivate *set_bitbus(int prio, unsigned char cmd, struct bitbusio * pbitbusio)
{
    struct dprivate *my_dpvt;
    my_dpvt = (struct dprivate *)calloc(1,sizeof(struct dprivate));
    my_dpvt->bitbus.finishProc = io_callback;
    my_dpvt->bitbus.psyncSem = (SEM_ID *) NULL;
    my_dpvt->bitbus.priority = prio;
    my_dpvt->bitbus.link = pbitbusio->link;
    my_dpvt->bitbus.rxMaxLen = BB_MAX_MSG_LENGTH;
    my_dpvt->bitbus.ageLimit = 120;
    my_dpvt->bitbus.txMsg.length = 9;
    my_dpvt->bitbus.txMsg.route = 0x40;
    my_dpvt->bitbus.txMsg.node = pbitbusio->node;
    my_dpvt->bitbus.txMsg.tasks = 1;
    my_dpvt->bitbus.txMsg.cmd = cmd;
    my_dpvt->bitbus.txMsg.data =
        (unsigned char *) calloc(BB_MAX_DAT_LEN, sizeof(char));
    my_dpvt->bitbus.rxMsg.data = 
        (unsigned char *) calloc(BB_MAX_DAT_LEN, sizeof(char));
    my_dpvt->bitbus.txMsg.data[0] = '\0';
    my_dpvt->bitbus.txMsg.data[1] = '\0';
    if (!bn[pbitbusio->link][pbitbusio->node]) {
        bn[pbitbusio->link][pbitbusio->node]
            = (struct bugnode *)(calloc(1,sizeof(struct bugnode)));
	bn[pbitbusio->link][pbitbusio->node]->lastScan = 1000;
    }
    return (my_dpvt);
}

static long init_all()
{
    int i,j;
    if (initFlag == 1) return (0);
    for(i=0; i<MAXLINKS; i++) {
        for(j=0; j<MAXNODES; j++) bn[i][j] = 0;
    }
    initFlag = 1;
    return (0);
}

static long init_ai(struct aiRecord * ai)
{
    struct bitbusio *pbitbusio = (struct bitbusio *) (&ai->inp.value);
    struct dprivate *my_dpvt;
    if (devIObugDebug > 0)
	printf("devIObug.c: Init ai invoked\n");
    /* type must be an BITBUS_IO */
    if (ai->inp.type != BITBUS_IO) {
	recGblRecordError(S_dev_badBus, (void *) ai,
			  "devIObug (init_record) Illegal IN Bus Type");
	return (S_dev_badBus);
    }
    if (pbitbusio->signal >= 4) {
	recGblRecordError(S_dev_badBus, (void *) ai,
			  "devIObug (init_record) Illegal IN signal number");
	return (S_dev_badBus);
    }
    my_dpvt = set_bitbus(ai->prio, READBACK, pbitbusio);
    my_dpvt->precord = (struct dbCommon *) ai;
    my_dpvt->link = pbitbusio->link;
    my_dpvt->node = pbitbusio->node;
    my_dpvt->port = pbitbusio->port;
    my_dpvt->signal = pbitbusio->signal;
    my_dpvt->type = AI;
    my_dpvt->cmd = READBACK;
    ai->dpvt = (void *) my_dpvt;
    /* set linear conversion slope & offset */
    ai->eslo = (ai->eguf - ai->egul) / 4095.0;
    return (0);
}

static long init_ao(struct aoRecord * ao)
{
    struct bitbusio *pbitbusio = (struct bitbusio *) (&ao->out.value);
    struct dprivate *my_dpvt;
    if (devIObugDebug > 0)
	printf("devIObug.c: Init ao invoked\n");
    /* out must be an BITBUS_IO */
    if (ao->out.type != BITBUS_IO) {
	recGblRecordError(S_dev_badBus, (void *) ao,
			  "devIObug (init_record) Illegal OUT Bus Type");
	return (S_dev_badBus);
    }
    if (pbitbusio->signal >= sizeof(ao_cmd)) {
	recGblRecordError(S_dev_badBus, (void *) ao,
			  "devIObug (init_record) Illegal OUT signal number");
	return (S_dev_badBus);
    }
    my_dpvt = set_bitbus(ao->prio, ao_cmd[pbitbusio->signal], pbitbusio);
    my_dpvt->precord = (struct dbCommon *) ao;
    my_dpvt->link = pbitbusio->link;
    my_dpvt->node = pbitbusio->node;
    my_dpvt->port = pbitbusio->port;
    my_dpvt->signal = pbitbusio->signal;
    my_dpvt->type = AO;
    my_dpvt->cmd = ao_cmd[pbitbusio->signal];
    ao->dpvt = (void *) my_dpvt;
    return (0);
}

static long init_bi(struct biRecord * bi)
{
    struct bitbusio *pbitbusio = (struct bitbusio *) (&bi->inp.value);
    struct dprivate *my_dpvt;
    if (devIObugDebug > 0)
	printf("devIObug.c: Init bi invoked\n");
    /* out must be an BITBUS_IO */
    if (bi->inp.type != BITBUS_IO) {
	recGblRecordError(S_dev_badBus, (void *) bi,
			  "devIObug (init_record) Illegal IN Bus Type");
	return (S_dev_badBus);
    }
    switch (pbitbusio->port) {
    default:
	recGblRecordError(S_dev_badBus, (void *) bi,
			  "devIObug (init_record) Illegal IN port number");
	return (S_dev_badBus);
    case 0:
	if (pbitbusio->signal >= 16) {
	    recGblRecordError(S_dev_badBus, (void *) bi,
			   "devIObug (init_record) Illegal IN signal number");
	    return (S_dev_badBus);
	}
	break;
    case 1:
	if (pbitbusio->signal >= 8) {
	    recGblRecordError(S_dev_badBus, (void *) bi,
			   "devIObug (init_record) Illegal IN signal number");
	    return (S_dev_badBus);
	}
	break;
    }
    bi->mask = 1 << (pbitbusio->signal);
    my_dpvt = set_bitbus(bi->prio, READBACK, pbitbusio);
    my_dpvt->precord = (struct dbCommon *) bi;
    my_dpvt->link = pbitbusio->link;
    my_dpvt->node = pbitbusio->node;
    my_dpvt->port = pbitbusio->port;
    my_dpvt->signal = pbitbusio->signal;
    my_dpvt->type = BI;
    my_dpvt->cmd = READBACK;
    bi->dpvt = (void *) my_dpvt;
    return (0);
}

static long init_bo(struct boRecord * bo)
{
    struct bitbusio *pbitbusio = (struct bitbusio *) (&bo->out.value);
    struct dprivate *my_dpvt;
    if (devIObugDebug > 0)
	printf("devIObug.c: Init bo invoked\n");
    /* out must be an BITBUS_IO */
    if (bo->out.type != BITBUS_IO) {
	recGblRecordError(S_dev_badBus, (void *) bo,
			  "devIObug (init_record) Illegal IN Bus Type");
	return (S_dev_badBus);
    }
    if (pbitbusio->signal >= 8) {
	recGblRecordError(S_dev_badBus, (void *) bo,
			  "devIObug (init_record) Illegal IN signal number");
	return (S_dev_badBus);
    }
    bo->mask = 1 << (pbitbusio->signal);
    my_dpvt = set_bitbus(bo->prio, BOUT, pbitbusio);
    my_dpvt->precord = (struct dbCommon *) bo;
    my_dpvt->link = pbitbusio->link;
    my_dpvt->node = pbitbusio->node;
    my_dpvt->port = pbitbusio->port;
    my_dpvt->signal = pbitbusio->signal;
    my_dpvt->type = BO;
    my_dpvt->cmd = BOUT;
    bo->dpvt = (void *) my_dpvt;
    return (0);
}

static long init_mbbi(struct mbbiRecord * mbbi)
{
    struct bitbusio *pbitbusio = (struct bitbusio *) (&mbbi->inp.value);
    struct dprivate *my_dpvt;
    if (devIObugDebug > 0)
	printf("devIObug.c: Init mbbi invoked\n");
    /* out must be an BITBUS_IO */
    if (mbbi->inp.type != BITBUS_IO) {
	recGblRecordError(S_dev_badBus, (void *) mbbi,
			  "devIObug (init_record) Illegal IN Bus Type");
	return (S_dev_badBus);
    }
    switch (pbitbusio->port) {
    default:
	recGblRecordError(S_dev_badBus, (void *) mbbi,
			  "devIObug (init_record) Illegal IN port number");
	return (S_dev_badBus);
    case 0:
	if (pbitbusio->signal >= 16) {
	    recGblRecordError(S_dev_badBus, (void *) mbbi,
			   "devIObug (init_record) Illegal IN signal number");
	    return (S_dev_badBus);
	}
	break;
    case 1:
	if (pbitbusio->signal >= 8) {
	    recGblRecordError(S_dev_badBus, (void *) mbbi,
			   "devIObug (init_record) Illegal IN signal number");
	    return (S_dev_badBus);
	}
	break;
    }
    mbbi->shft = pbitbusio->signal;
    mbbi->mask <<= mbbi->shft;
    my_dpvt = set_bitbus(mbbi->prio, READBACK, pbitbusio);
    my_dpvt->precord = (struct dbCommon *) mbbi;
    my_dpvt->link = pbitbusio->link;
    my_dpvt->node = pbitbusio->node;
    my_dpvt->port = pbitbusio->port;
    my_dpvt->signal = pbitbusio->signal;
    my_dpvt->type = MBBI;
    my_dpvt->cmd = READBACK;
    mbbi->dpvt = (void *) my_dpvt;
    return (0);
}

static long init_mbbo(struct mbboRecord * mbbo)
{
    struct bitbusio *pbitbusio = (struct bitbusio *) (&mbbo->out.value);
    struct dprivate *my_dpvt;
    if (devIObugDebug > 0)
	printf("devIObug.c: Init mbbo invoked\n");
    /* out must be an BITBUS_IO */
    if (mbbo->out.type != BITBUS_IO) {
	recGblRecordError(S_dev_badBus, (void *) mbbo,
			  "devIObug (init_record) Illegal OUT Bus Type");
	return (S_dev_badBus);
    }
    if (pbitbusio->signal >= 8) {
	recGblRecordError(S_dev_badBus, (void *) mbbo,
			  "devIObug (init_record) Illegal OUT signal number");
	return (S_dev_badBus);
    }
    mbbo->shft = pbitbusio->signal;
    mbbo->mask <<= mbbo->shft;
    my_dpvt = set_bitbus(mbbo->prio, BOUT, pbitbusio);
    my_dpvt->precord = (struct dbCommon *) mbbo;
    my_dpvt->link = pbitbusio->link;
    my_dpvt->node = pbitbusio->node;
    my_dpvt->port = pbitbusio->port;
    my_dpvt->signal = pbitbusio->signal;
    my_dpvt->type = MBBO;
    my_dpvt->cmd = BOUT;
    mbbo->dpvt = (void *) my_dpvt;
    return (0);
}

static long iobug_rdwr(struct dbCommon * io)
{
    struct dprivate *my_dpvt = (struct dprivate *) io->dpvt;
    struct dpvtBitBusHead *pbitbus;
    unsigned long currentTick, currentScan;
    unsigned int counter;
    struct bugnode *bn;
    bn = get_bn(my_dpvt->link,my_dpvt->node);
    if(!bn) return (S_dev_NoInit);
    if (devIObugDebug >= 35)
	printf("devIObug.c: iobug_rdwr called\n");
    if (!io->dpvt)
	return (S_dev_NoInit);
    pbitbus = &(my_dpvt->bitbus);
    if (io->pact == TRUE) {
	/* a transaction to bitbus has completed */
	switch (pbitbus->status) {
	case BB_OK:
	    if (!pbitbus->rxMsg.cmd) {
		if ((devIObugDebug >= 15) || (devIObugDebug == 2))
		    printf("devIObug.c: ->%s<- Filling Buffer for link %d, node %d\n",
                    io->name, my_dpvt->link, my_dpvt->node);
		bn->nodeAlarm = NO_ALARM;
		bn->readBuf[0] = pbitbus->rxMsg.data[0];
		bn->readBuf[1] = pbitbus->rxMsg.data[1];
		bn->readBuf[2] = pbitbus->rxMsg.data[2];
		bn->readBuf[3] = pbitbus->rxMsg.data[3];
		bn->readBuf[4] = pbitbus->rxMsg.data[4];
		bn->readBuf[5] = pbitbus->rxMsg.data[5];
		bn->readBuf[6] = pbitbus->rxMsg.data[6];
		bn->readBuf[7] = pbitbus->rxMsg.data[7];
		bn->readBuf[8] = pbitbus->rxMsg.data[8];
		bn->readBuf[9] = pbitbus->rxMsg.data[9];
		bn->readBuf[10] = pbitbus->rxMsg.data[10];
		if ((devIObugDebug >= 30) || (devIObugDebug == 2)) {
		    for (counter = 0; counter <= 10; counter++) {
			printf("devIObug.c: BITBUS message byte %d is 0x%2.2X\n",
                            counter, bn->readBuf[counter]);
		    }
		}
		if (devIObugDebug >= 10)
		    printf("devIObug.c: ->%s<- Reading from Buffer\n", io->name);
		if(get_data(io)) return(S_dev_NoInit);
	    } else {
		switch (my_dpvt->type) {
		case AI:
		case BI:
		case MBBI:
		    bn->nodeAlarm = INVALID_ALARM;
		    recGblSetSevr(io, READ_ALARM, INVALID_ALARM);
		    break;
		default:
		    recGblSetSevr(io, WRITE_ALARM, INVALID_ALARM);
		    break;
		}
	    }
	    return (0);
	default:
	    recGblSetSevr(io, WRITE_ALARM, MAJOR_ALARM);
	    recGblRecordError(S_dev_badBus, (void *) io,
		      "devIObug (init_record) Initial BITBUS message failed");
	    if (devIObugDebug >= 5)
		printf("devIObug.c: ****** Unknown BITBUS Error **********\n");
	    return (S_dev_badBus);
	}
    } else {
        char type = my_dpvt->type;
	/* data needs to be sent to bitbus */
	if(((type == AI) || (type == BI) || (type == MBBI))
	&&((io->scan >= menuScan10_second) && (io->scan <= menuScan_1_second))){
	    switch (io->scan) {
	    case menuScan10_second:
		currentScan = 10 * vxTicksPerSecond;
		break;
	    case menuScan5_second:
		currentScan = 5 * vxTicksPerSecond;
		break;
	    case menuScan2_second:
		currentScan = 2 * vxTicksPerSecond;
		break;
	    case menuScan1_second:
		currentScan = 1 * vxTicksPerSecond;
		break;
	    case menuScan_5_second:
		currentScan = .5 * vxTicksPerSecond;
		break;
	    case menuScan_2_second:
		currentScan = .2 * vxTicksPerSecond;
		break;
	    case menuScan_1_second:
		currentScan = .1 * vxTicksPerSecond;
		break;
	    default:
		currentScan = 10 * vxTicksPerSecond;
		break;
	    }
	    /*** The following is a timing sequence using vxTicks that only ***/
	    /***  allows a read transaction to be sent to a BUG once every  ***/
	    /*** scan time of the record most frequently scanned.  A read   ***/
	    /***    transaction fills a buffer which the records less       ***/
	    /***     frequently scanned will obtain their values from.      ***/
	    currentTick = tickGet();
	    if ((currentTick - bn->lastScan) >= currentScan) {
		if (devIObugDebug >= 25) {
		    printf("devIObug.c: Scan timer timed out:\n");
		    printf("            current tick time: %lu (ticks)\n",
                        currentTick);
		    printf("            current record scan rate: %lu (ticks)\n"
                        , currentScan);
		    printf("            lastScan time: %lu (ticks)\n",
                        bn->lastScan);
		    printf("            there are %u Ticks/Second\n",
                        vxTicksPerSecond);
		    printf("            resetting lastScan to current tick time\n");
		}
		bn->lastScan = currentTick;
		if (devIObugDebug >= 10)
		    printf("devIObug.c: Sending BITBUS transaction\n");
		send_cntl_trans(io);
		io->pact = TRUE;
	    } else {
		if (devIObugDebug >= 25) {
		    printf("devIObug.c: Scan timer did not time out:\n");
		    printf("            current tick time: %lu (ticks)\n"
                        , currentTick);
		    printf("            current record scan rate: %lu (ticks)\n"
                        , currentScan);
		    printf("            lastScan time: %lu\n" , bn->lastScan);
		}
		if (devIObugDebug >= 10)
		    printf("devIObug.c: ->%s<- Reading from Buffer\n", io->name);
		if(get_data(io)) return(S_dev_NoInit);
		return (0);
	    }
	    /********************* End of timing sequence *********************/
	} else {
	    if (devIObugDebug >= 10)
		printf("devIObug.c: Sending transaction\n");
	    send_cntl_trans(io);
	    io->pact = TRUE;
	}
    }
    /* queue the command */
    if ((*drvBitBus.qReq) (pbitbus, BB_Q_LOW) < 0) {
	recGblSetSevr(io, WRITE_ALARM, MAJOR_ALARM);
	recGblRecordError(S_dev_badBus, (void *) io,
		      "devIObug (init_record) Initial BITBUS message failed");
	if (devIObugDebug >= 10)
	    printf("devIObug.c: Initial BITBUS message failed\n");
	return (S_dev_badBus);
    }
    return (0);
}

static int get_data(struct dbCommon * io)
{
    struct dprivate *my_dpvt = (struct dprivate *) io->dpvt;
    struct dpvtBitBusHead *pbitbus;
    struct biRecord *bi;
    struct aiRecord *ai;
    struct mbbiRecord *mbbi;
    long value;
    unsigned long uvalue;
    unsigned int counter;
    struct bugnode *bn;
    bn = get_bn(my_dpvt->link,my_dpvt->node);
    if(!bn) return (S_dev_NoInit);
    value = 0;
    uvalue = 0;
    pbitbus = &(my_dpvt->bitbus);
    if (devIObugDebug >= 35)
	printf("devIObug.c: get_data called\n");
    if ((devIObugDebug >= 30) || (devIObugDebug == 1)) {
	printf("devIObug.c: ->%s<- Reading from buffer for link %d, node %d\n",
           io->name, my_dpvt->link, my_dpvt->node);
	for (counter = 0; counter <= 10; counter++) {
	    printf("devIObug.c: Byte %d of buffer: 0x%2.2X\n",
                counter, bn->readBuf[counter]);
	}
    }
    if (bn->nodeAlarm)
	recGblSetSevr(io, READ_ALARM, bn->nodeAlarm);
    else {
	switch (my_dpvt->type) {
	case BI:
	    bi = (struct biRecord *) io;
	    switch (my_dpvt->port) {
	    case 0:
		value = ((bn->readBuf[2]) << 8);
		value |= (bn->readBuf[1]);
		break;
	    case 1:
		value = 0;
		value |= (bn->readBuf[0]);
		break;
	    default:
		break;
	    }
	    bi->rval = value & bi->mask;
	    break;
	case MBBI:
	    mbbi = (struct mbbiRecord *) io;
	    switch (my_dpvt->port) {
	    case 0:
		uvalue = ((bn->readBuf[2]) << 8);
		uvalue |= (bn->readBuf[1]);
		break;
	    case 1:
		uvalue = 0;
		uvalue |= (bn->readBuf[0]);
		break;
	    default:
		break;
	    }
	    mbbi->rval = uvalue & mbbi->mask;
	    break;
	case AI:
	    ai = (struct aiRecord *) io;
	    switch (my_dpvt->signal) {
	    case 0:
		ai->rval = (bn->readBuf[4]) << 8 | (bn->readBuf[3]);
		break;
	    case 1:
		ai->rval = (bn->readBuf[6]) << 8 | (bn->readBuf[5]);
		break;
	    case 2:
		ai->rval = (bn->readBuf[8]) << 8 | (bn->readBuf[7]);
		break;
	    case 3:
		ai->rval = (bn->readBuf[10]) << 8 | (bn->readBuf[9]);
		break;
	    default:
		break;
	    }
	    break;
	default:
	    break;
	}
    }
    return(0);
}

static void send_cntl_trans(struct dbCommon * io)
{
    struct dprivate *my_dpvt = (struct dprivate *) io->dpvt;
    struct dpvtBitBusHead *pbitbus;
    struct boRecord *bo;
    struct aoRecord *ao;
    struct mbboRecord *mbbo;
    pbitbus = &(my_dpvt->bitbus);
    if (devIObugDebug >= 35)
	printf("devIObug.c: send_cntl_trans called\n");
    pbitbus->finishProc = io_callback;
    pbitbus->psyncSem = (SEM_ID *) NULL;
    pbitbus->priority = io->prio;
    pbitbus->txMsg.cmd = my_dpvt->cmd;
    pbitbus->txMsg.data[0] = (unsigned char) NULL;/* the stuff to turn on */
    pbitbus->txMsg.data[1] = (unsigned char) NULL;/* the stuff to turn off */
    switch (my_dpvt->type) {
    case BO:
	bo = (struct boRecord *) io;
	pbitbus->txMsg.data[0] = (unsigned char) (bo->rval & LONG_FIRSTBYTE);
	pbitbus->txMsg.data[1] = (unsigned char) (bo->mask & LONG_FIRSTBYTE);
	break;
    case MBBO:
	mbbo = (struct mbboRecord *) io;
	pbitbus->txMsg.data[0] =
	    (unsigned char) (mbbo->rval & LONG_FIRSTBYTE);
	pbitbus->txMsg.data[1] =
	    (unsigned char) (mbbo->mask & LONG_FIRSTBYTE);
	break;
    case AO:
	ao = (struct aoRecord *) io;
	pbitbus->txMsg.data[0] = (unsigned char) ao->rval & LONG_FIRSTBYTE;
	pbitbus->txMsg.data[1] =
	    (unsigned char) (ao->rval >> 8 & LONG_FIRSTBYTE);
	/* rval is long, data is char, break into 2 char short */
	break;
    default:
	break;
    }
    my_dpvt->precord = (struct dbCommon *) io;
    if (devIObugDebug >= 10) {
	printf("devIObug.c: BITBUS message command send: 0x%2.2X\n",
            pbitbus->txMsg.cmd);
	if (pbitbus->txMsg.cmd & 0xF0) {
	    printf("devIObug.c: BITBUS message byte 1 sent: 0x%2.2X\n",
                (unsigned char) pbitbus->txMsg.data[0]);
	    printf("devIObug.c: BITBUS message byte 2 sent: 0x%2.2X\n",
                (unsigned char) pbitbus->txMsg.data[1]);
	}
    }
    return;
}
