#include <vxWorks.h>
#include <semLib.h>
#include "ellLib.h"
#include "fast_lock.h"
#include "link.h"
#include "tsDefs.h"

#ifndef INCmotorUEIPH
#define INCmotorUEIPH
typedef enum {
	motorUEIP_No,
	motorUEIP_Yes
}motorUEIP;
#endif /*INCmotorUEIPH*/

#ifndef INCmotorTORQH
#define INCmotorTORQH
typedef enum {
	motorTORQ_Disable,
	motorTORQ_Enable
}motorTORQ;
#endif /*INCmotorTORQH*/

#ifndef INCmotorSPMGH
#define INCmotorSPMGH
typedef enum {
	motorSPMG_Stop,
	motorSPMG_Pause,
	motorSPMG_Move,
	motorSPMG_Go
}motorSPMG;
#endif /*INCmotorSPMGH*/

#ifndef INCmotorSETH
#define INCmotorSETH
typedef enum {
	motorSET_Use,
	motorSET_Set
}motorSET;
#endif /*INCmotorSETH*/

#ifndef INCmotorMODEH
#define INCmotorMODEH
typedef enum {
	motorMODE_Position,
	motorMODE_Velocity
}motorMODE;
#endif /*INCmotorMODEH*/

#ifndef INCmotorFOFFH
#define INCmotorFOFFH
typedef enum {
	motorFOFF_Variable,
	motorFOFF_Frozen
}motorFOFF;
#endif /*INCmotorFOFFH*/

#ifndef INCmotorDIRH
#define INCmotorDIRH
typedef enum {
	motorDIR_Pos,
	motorDIR_Neg
}motorDIR;
#endif /*INCmotorDIRH*/
#ifndef INCmotorH
#define INCmotorH
typedef struct motorRecord {
	char		name[29]; /*Record Name*/
	char		desc[29]; /*Descriptor*/
	char		asg[29]; /*Access Security Group*/
	unsigned short	scan;	/*Scan Mechanism*/
	unsigned short	pini;	/*Process at iocInit*/
	short		phas;	/*Scan Phase*/
	short		evnt;	/*Event Number*/
	short		tse;	/*Time Stamp Event*/
	DBLINK		tsel;	/*Time Stamp Link*/
	unsigned short	dtyp;	/*Device Type*/
	short		disv;	/*Disable Value*/
	short		disa;	/*Disable*/
	DBLINK		sdis;	/*Scanning Disable*/
	FAST_LOCK	mlok;	/*Monitor fastlock*/
	ELLLIST		mlis;	/*Monitor List*/
	unsigned char	disp;	/*Disable putField*/
	unsigned char	proc;	/*Force Processing*/
	unsigned short	stat;	/*Alarm Status*/
	unsigned short	sevr;	/*Alarm Severity*/
	unsigned short	nsta;	/*New Alarm Status*/
	unsigned short	nsev;	/*New Alarm Severity*/
	unsigned short	acks;	/*Alarm Ack Severity*/
	unsigned short	ackt;	/*Alarm Ack Transient*/
	unsigned short	diss;	/*Disable Alarm Sevrty*/
	unsigned char	lcnt;	/*Lock Count*/
	unsigned char	pact;	/*Record active*/
	unsigned char	putf;	/*dbPutField process*/
	unsigned char	rpro;	/*Reprocess */
	void		*asp;	/*Access Security Pvt*/
	struct putNotify *ppn;	/*addr of PUTNOTIFY*/
	struct pnWaitNode *ppnn;	/*next record PUTNOTIFY*/
	struct scan_element *spvt;	/*Scan Private*/
	struct rset	*rset;	/*Address of RSET*/
	struct dset	*dset;	/*DSET address*/
	void		*dpvt;	/*Device Private*/
	struct dbRecordType *rdes;	/*Address of dbRecordType*/
	struct lockRecord *lset;	/*Lock Set*/
	unsigned short	prio;	/*Scheduling Priority*/
	unsigned char	tpro;	/*Trace Processing*/
	char bkpt;	/*Break Point*/
	unsigned char	udf;	/*Undefined*/
	TS_STAMP	time;	/*Time*/
	DBLINK		flnk;	/*Forward Process Link*/
	float		vers;	/*Code Version*/
	double		off;	/*User Offset (EGU)*/
	unsigned short	foff;	/*Offset-Freeze Switch*/
	short		fof;	/*Freeze Offset*/
	short		vof;	/*Variable Offset*/
	unsigned short	dir;	/*User Direction*/
	unsigned short	set;	/*Set/Use Switch*/
	short		sset;	/*Set SET Mode*/
	short		suse;	/*Set USE Mode*/
	float		velo;	/*Velocity (EGU/s)*/
	float		vbas;	/*Base Velocity (EGU/s)*/
	float		vmax;	/*Max. Velocity (EGU/s)*/
	float		s;	/*Speed (revolutions/sec)*/
	float		sbas;	/*Base Speed (RPS)*/
	float		smax;	/*Max. Speed (RPS)*/
	float		accl;	/*Seconds to Velocity*/
	float		bdst;	/*BL Distance (EGU)*/
	float		bvel;	/*BL Velocity (EGU/s)*/
	float		sbak;	/*BL Speed (RPS)*/
	float		bacc;	/*BL Seconds to Velocity*/
	float		frac;	/*Move Fraction*/
	DBLINK		out;	/*Output Specification*/
	short		card;	/*Card Number*/
	DBLINK		rdbl;	/*Readback Location*/
	DBLINK		dol;	/*Desired Output Loc*/
	unsigned short	omsl;	/*Output Mode Select*/
	DBLINK		rlnk;	/*Readback OutLink*/
	long		srev;	/*Steps per Revolution*/
	float		urev;	/*EGU's per Revolution*/
	float		mres;	/*Motor Step Size (EGU)*/
	float		eres;	/*Encoder Step Size (EGU)*/
	float		rres;	/*Readback Step Size (EGU*/
	double		res;	/*Step Size (EGU)*/
	unsigned short	ueip;	/*Use Encoder If Present*/
	unsigned short	urip;	/*Use RDBL Link If Presen*/
	short		prec;	/*Display Precision*/
	char		egu[16]; /*Engineering Units*/
	float		hlm;	/*User High Limit*/
	float		llm;	/*User Low Limit*/
	float		dhlm;	/*Dial High Limit*/
	float		dllm;	/*Dial Low Limit*/
	float		hopr;	/*High Operating Range*/
	float		lopr;	/*Low Operating Range*/
	short		hls;	/*User High Limit Switch*/
	short		lls;	/*User Low Limit Switch*/
	short		rhls;	/*Raw High Limit Switch*/
	short		rlls;	/*Raw Low Limit Switch*/
	float		hihi;	/*Hihi Alarm Limit (EGU)*/
	float		lolo;	/*Lolo Alarm Limit (EGU)*/
	float		high;	/*High Alarm Limit (EGU)*/
	float		low;	/*Low Alarm Limit (EGU)*/
	unsigned short	hhsv;	/*Hihi Severity*/
	unsigned short	llsv;	/*Lolo Severity*/
	unsigned short	hsv;	/*High Severity*/
	unsigned short	lsv;	/*Low Severity*/
	unsigned short	hlsv;	/*HW Limit Violation Svr*/
	float		rdbd;	/*Retry Deadband (EGU)*/
	short		rcnt;	/*Retry count*/
	short		rtry;	/*Max retry count*/
	short		miss;	/*Ran out of retries*/
	unsigned short	spmg;	/*Stop/Pause/Move/Go*/
	unsigned short	lspg;	/*Last SPMG*/
	short		stop;	/*Stop*/
	short		homf;	/*Home Forward*/
	short		homr;	/*Home Reverse*/
	short		jogf;	/*Jog motor Forward*/
	short		jogr;	/*Jog motor Reverse*/
	short		twf;	/*Tweak motor Forward*/
	short		twr;	/*Tweak motor Reverse*/
	float		twv;	/*Tweak Step Size (EGU)*/
	double		val;	/*User Desired Value (EGU*/
	double		lval;	/*Last User Des Val (EGU)*/
	double		dval;	/*Dial Desired Value (EGU*/
	double		ldvl;	/*Last Dial Des Val (EGU)*/
	long		rval;	/*Raw Desired Value (step*/
	long		lrvl;	/*Last Raw Des Val (steps*/
	double		rlv;	/*Relative Value (EGU)*/
	double		lrlv;	/*Last Rel Value (EGU)*/
	double		rbv;	/*User Readback Value*/
	double		drbv;	/*Dial Readback Value*/
	double		diff;	/*Difference dval-drbv*/
	long		rdif;	/*Difference rval-rrbv*/
	long		pdif;	/*Previous RDIF*/
	long		rrbv;	/*Raw Readback Value*/
	long		rmp;	/*Raw Motor Position*/
	long		rep;	/*Raw Encoder Position*/
	long		rvel;	/*Raw Velocity*/
	short		dmov;	/*Done moving to value*/
	short		movn;	/*Motor is moving*/
	unsigned long	msta;	/*Motor Status*/
	short		lvio;	/*Limit violation*/
	short		tdir;	/*Direction of Travel*/
	short		athm;	/*At HOME*/
	short		pp;	/*Post process command*/
	unsigned short	mip;	/*Motion In Progress*/
	unsigned long	mmap;	/*Monitor Mask*/
	unsigned long	nmap;	/*Monitor Mask (more)*/
	float		dly;	/*Readback settle time (s)*/
	void		*cbak;	/*Callback structure*/
	float		pcof;	/*Proportional Gain*/
	float		icof;	/*Integral Gain*/
	float		dcof;	/*Derivative Gain*/
	unsigned short	cnen;	/*Enable control*/
	char		init[40]; /*Startup commands*/
	char		prem[40]; /*Pre-move commands*/
	char		post[40]; /*Post-move commands*/
	DBLINK		stoo;	/*STOP OutLink*/
	DBLINK		dinp;	/*DMOV Input Link*/
	DBLINK		rinp;	/*RMP Input Link*/
	float		jvel;	/*Jog Velocity (EGU/s)*/
	float		jar;	/*Jog Accel. (EGU/s^2)*/
} motorRecord;
#define motorRecordNAME	0
#define motorRecordDESC	1
#define motorRecordASG	2
#define motorRecordSCAN	3
#define motorRecordPINI	4
#define motorRecordPHAS	5
#define motorRecordEVNT	6
#define motorRecordTSE	7
#define motorRecordTSEL	8
#define motorRecordDTYP	9
#define motorRecordDISV	10
#define motorRecordDISA	11
#define motorRecordSDIS	12
#define motorRecordMLOK	13
#define motorRecordMLIS	14
#define motorRecordDISP	15
#define motorRecordPROC	16
#define motorRecordSTAT	17
#define motorRecordSEVR	18
#define motorRecordNSTA	19
#define motorRecordNSEV	20
#define motorRecordACKS	21
#define motorRecordACKT	22
#define motorRecordDISS	23
#define motorRecordLCNT	24
#define motorRecordPACT	25
#define motorRecordPUTF	26
#define motorRecordRPRO	27
#define motorRecordASP	28
#define motorRecordPPN	29
#define motorRecordPPNN	30
#define motorRecordSPVT	31
#define motorRecordRSET	32
#define motorRecordDSET	33
#define motorRecordDPVT	34
#define motorRecordRDES	35
#define motorRecordLSET	36
#define motorRecordPRIO	37
#define motorRecordTPRO	38
#define motorRecordBKPT	39
#define motorRecordUDF	40
#define motorRecordTIME	41
#define motorRecordFLNK	42
#define motorRecordVERS	43
#define motorRecordOFF	44
#define motorRecordFOFF	45
#define motorRecordFOF	46
#define motorRecordVOF	47
#define motorRecordDIR	48
#define motorRecordSET	49
#define motorRecordSSET	50
#define motorRecordSUSE	51
#define motorRecordVELO	52
#define motorRecordVBAS	53
#define motorRecordVMAX	54
#define motorRecordS	55
#define motorRecordSBAS	56
#define motorRecordSMAX	57
#define motorRecordACCL	58
#define motorRecordBDST	59
#define motorRecordBVEL	60
#define motorRecordSBAK	61
#define motorRecordBACC	62
#define motorRecordFRAC	63
#define motorRecordOUT	64
#define motorRecordCARD	65
#define motorRecordRDBL	66
#define motorRecordDOL	67
#define motorRecordOMSL	68
#define motorRecordRLNK	69
#define motorRecordSREV	70
#define motorRecordUREV	71
#define motorRecordMRES	72
#define motorRecordERES	73
#define motorRecordRRES	74
#define motorRecordRES	75
#define motorRecordUEIP	76
#define motorRecordURIP	77
#define motorRecordPREC	78
#define motorRecordEGU	79
#define motorRecordHLM	80
#define motorRecordLLM	81
#define motorRecordDHLM	82
#define motorRecordDLLM	83
#define motorRecordHOPR	84
#define motorRecordLOPR	85
#define motorRecordHLS	86
#define motorRecordLLS	87
#define motorRecordRHLS	88
#define motorRecordRLLS	89
#define motorRecordHIHI	90
#define motorRecordLOLO	91
#define motorRecordHIGH	92
#define motorRecordLOW	93
#define motorRecordHHSV	94
#define motorRecordLLSV	95
#define motorRecordHSV	96
#define motorRecordLSV	97
#define motorRecordHLSV	98
#define motorRecordRDBD	99
#define motorRecordRCNT	100
#define motorRecordRTRY	101
#define motorRecordMISS	102
#define motorRecordSPMG	103
#define motorRecordLSPG	104
#define motorRecordSTOP	105
#define motorRecordHOMF	106
#define motorRecordHOMR	107
#define motorRecordJOGF	108
#define motorRecordJOGR	109
#define motorRecordTWF	110
#define motorRecordTWR	111
#define motorRecordTWV	112
#define motorRecordVAL	113
#define motorRecordLVAL	114
#define motorRecordDVAL	115
#define motorRecordLDVL	116
#define motorRecordRVAL	117
#define motorRecordLRVL	118
#define motorRecordRLV	119
#define motorRecordLRLV	120
#define motorRecordRBV	121
#define motorRecordDRBV	122
#define motorRecordDIFF	123
#define motorRecordRDIF	124
#define motorRecordPDIF	125
#define motorRecordRRBV	126
#define motorRecordRMP	127
#define motorRecordREP	128
#define motorRecordRVEL	129
#define motorRecordDMOV	130
#define motorRecordMOVN	131
#define motorRecordMSTA	132
#define motorRecordLVIO	133
#define motorRecordTDIR	134
#define motorRecordATHM	135
#define motorRecordPP	136
#define motorRecordMIP	137
#define motorRecordMMAP	138
#define motorRecordNMAP	139
#define motorRecordDLY	140
#define motorRecordCBAK	141
#define motorRecordPCOF	142
#define motorRecordICOF	143
#define motorRecordDCOF	144
#define motorRecordCNEN	145
#define motorRecordINIT	146
#define motorRecordPREM	147
#define motorRecordPOST	148
#define motorRecordSTOO	149
#define motorRecordDINP	150
#define motorRecordRINP	151
#define motorRecordJVEL	152
#define motorRecordJAR	153
#endif /*INCmotorH*/
#ifdef GEN_SIZE_OFFSET
int motorRecordSizeOffset(dbRecordType *pdbRecordType)
{
    motorRecord *prec = 0;
  pdbRecordType->papFldDes[0]->size=sizeof(prec->name);
  pdbRecordType->papFldDes[0]->offset=(short)((char *)&prec->name - (char *)prec);
  pdbRecordType->papFldDes[1]->size=sizeof(prec->desc);
  pdbRecordType->papFldDes[1]->offset=(short)((char *)&prec->desc - (char *)prec);
  pdbRecordType->papFldDes[2]->size=sizeof(prec->asg);
  pdbRecordType->papFldDes[2]->offset=(short)((char *)&prec->asg - (char *)prec);
  pdbRecordType->papFldDes[3]->size=sizeof(prec->scan);
  pdbRecordType->papFldDes[3]->offset=(short)((char *)&prec->scan - (char *)prec);
  pdbRecordType->papFldDes[4]->size=sizeof(prec->pini);
  pdbRecordType->papFldDes[4]->offset=(short)((char *)&prec->pini - (char *)prec);
  pdbRecordType->papFldDes[5]->size=sizeof(prec->phas);
  pdbRecordType->papFldDes[5]->offset=(short)((char *)&prec->phas - (char *)prec);
  pdbRecordType->papFldDes[6]->size=sizeof(prec->evnt);
  pdbRecordType->papFldDes[6]->offset=(short)((char *)&prec->evnt - (char *)prec);
  pdbRecordType->papFldDes[7]->size=sizeof(prec->tse);
  pdbRecordType->papFldDes[7]->offset=(short)((char *)&prec->tse - (char *)prec);
  pdbRecordType->papFldDes[8]->size=sizeof(prec->tsel);
  pdbRecordType->papFldDes[8]->offset=(short)((char *)&prec->tsel - (char *)prec);
  pdbRecordType->papFldDes[9]->size=sizeof(prec->dtyp);
  pdbRecordType->papFldDes[9]->offset=(short)((char *)&prec->dtyp - (char *)prec);
  pdbRecordType->papFldDes[10]->size=sizeof(prec->disv);
  pdbRecordType->papFldDes[10]->offset=(short)((char *)&prec->disv - (char *)prec);
  pdbRecordType->papFldDes[11]->size=sizeof(prec->disa);
  pdbRecordType->papFldDes[11]->offset=(short)((char *)&prec->disa - (char *)prec);
  pdbRecordType->papFldDes[12]->size=sizeof(prec->sdis);
  pdbRecordType->papFldDes[12]->offset=(short)((char *)&prec->sdis - (char *)prec);
  pdbRecordType->papFldDes[13]->size=sizeof(prec->mlok);
  pdbRecordType->papFldDes[13]->offset=(short)((char *)&prec->mlok - (char *)prec);
  pdbRecordType->papFldDes[14]->size=sizeof(prec->mlis);
  pdbRecordType->papFldDes[14]->offset=(short)((char *)&prec->mlis - (char *)prec);
  pdbRecordType->papFldDes[15]->size=sizeof(prec->disp);
  pdbRecordType->papFldDes[15]->offset=(short)((char *)&prec->disp - (char *)prec);
  pdbRecordType->papFldDes[16]->size=sizeof(prec->proc);
  pdbRecordType->papFldDes[16]->offset=(short)((char *)&prec->proc - (char *)prec);
  pdbRecordType->papFldDes[17]->size=sizeof(prec->stat);
  pdbRecordType->papFldDes[17]->offset=(short)((char *)&prec->stat - (char *)prec);
  pdbRecordType->papFldDes[18]->size=sizeof(prec->sevr);
  pdbRecordType->papFldDes[18]->offset=(short)((char *)&prec->sevr - (char *)prec);
  pdbRecordType->papFldDes[19]->size=sizeof(prec->nsta);
  pdbRecordType->papFldDes[19]->offset=(short)((char *)&prec->nsta - (char *)prec);
  pdbRecordType->papFldDes[20]->size=sizeof(prec->nsev);
  pdbRecordType->papFldDes[20]->offset=(short)((char *)&prec->nsev - (char *)prec);
  pdbRecordType->papFldDes[21]->size=sizeof(prec->acks);
  pdbRecordType->papFldDes[21]->offset=(short)((char *)&prec->acks - (char *)prec);
  pdbRecordType->papFldDes[22]->size=sizeof(prec->ackt);
  pdbRecordType->papFldDes[22]->offset=(short)((char *)&prec->ackt - (char *)prec);
  pdbRecordType->papFldDes[23]->size=sizeof(prec->diss);
  pdbRecordType->papFldDes[23]->offset=(short)((char *)&prec->diss - (char *)prec);
  pdbRecordType->papFldDes[24]->size=sizeof(prec->lcnt);
  pdbRecordType->papFldDes[24]->offset=(short)((char *)&prec->lcnt - (char *)prec);
  pdbRecordType->papFldDes[25]->size=sizeof(prec->pact);
  pdbRecordType->papFldDes[25]->offset=(short)((char *)&prec->pact - (char *)prec);
  pdbRecordType->papFldDes[26]->size=sizeof(prec->putf);
  pdbRecordType->papFldDes[26]->offset=(short)((char *)&prec->putf - (char *)prec);
  pdbRecordType->papFldDes[27]->size=sizeof(prec->rpro);
  pdbRecordType->papFldDes[27]->offset=(short)((char *)&prec->rpro - (char *)prec);
  pdbRecordType->papFldDes[28]->size=sizeof(prec->asp);
  pdbRecordType->papFldDes[28]->offset=(short)((char *)&prec->asp - (char *)prec);
  pdbRecordType->papFldDes[29]->size=sizeof(prec->ppn);
  pdbRecordType->papFldDes[29]->offset=(short)((char *)&prec->ppn - (char *)prec);
  pdbRecordType->papFldDes[30]->size=sizeof(prec->ppnn);
  pdbRecordType->papFldDes[30]->offset=(short)((char *)&prec->ppnn - (char *)prec);
  pdbRecordType->papFldDes[31]->size=sizeof(prec->spvt);
  pdbRecordType->papFldDes[31]->offset=(short)((char *)&prec->spvt - (char *)prec);
  pdbRecordType->papFldDes[32]->size=sizeof(prec->rset);
  pdbRecordType->papFldDes[32]->offset=(short)((char *)&prec->rset - (char *)prec);
  pdbRecordType->papFldDes[33]->size=sizeof(prec->dset);
  pdbRecordType->papFldDes[33]->offset=(short)((char *)&prec->dset - (char *)prec);
  pdbRecordType->papFldDes[34]->size=sizeof(prec->dpvt);
  pdbRecordType->papFldDes[34]->offset=(short)((char *)&prec->dpvt - (char *)prec);
  pdbRecordType->papFldDes[35]->size=sizeof(prec->rdes);
  pdbRecordType->papFldDes[35]->offset=(short)((char *)&prec->rdes - (char *)prec);
  pdbRecordType->papFldDes[36]->size=sizeof(prec->lset);
  pdbRecordType->papFldDes[36]->offset=(short)((char *)&prec->lset - (char *)prec);
  pdbRecordType->papFldDes[37]->size=sizeof(prec->prio);
  pdbRecordType->papFldDes[37]->offset=(short)((char *)&prec->prio - (char *)prec);
  pdbRecordType->papFldDes[38]->size=sizeof(prec->tpro);
  pdbRecordType->papFldDes[38]->offset=(short)((char *)&prec->tpro - (char *)prec);
  pdbRecordType->papFldDes[39]->size=sizeof(prec->bkpt);
  pdbRecordType->papFldDes[39]->offset=(short)((char *)&prec->bkpt - (char *)prec);
  pdbRecordType->papFldDes[40]->size=sizeof(prec->udf);
  pdbRecordType->papFldDes[40]->offset=(short)((char *)&prec->udf - (char *)prec);
  pdbRecordType->papFldDes[41]->size=sizeof(prec->time);
  pdbRecordType->papFldDes[41]->offset=(short)((char *)&prec->time - (char *)prec);
  pdbRecordType->papFldDes[42]->size=sizeof(prec->flnk);
  pdbRecordType->papFldDes[42]->offset=(short)((char *)&prec->flnk - (char *)prec);
  pdbRecordType->papFldDes[43]->size=sizeof(prec->vers);
  pdbRecordType->papFldDes[43]->offset=(short)((char *)&prec->vers - (char *)prec);
  pdbRecordType->papFldDes[44]->size=sizeof(prec->off);
  pdbRecordType->papFldDes[44]->offset=(short)((char *)&prec->off - (char *)prec);
  pdbRecordType->papFldDes[45]->size=sizeof(prec->foff);
  pdbRecordType->papFldDes[45]->offset=(short)((char *)&prec->foff - (char *)prec);
  pdbRecordType->papFldDes[46]->size=sizeof(prec->fof);
  pdbRecordType->papFldDes[46]->offset=(short)((char *)&prec->fof - (char *)prec);
  pdbRecordType->papFldDes[47]->size=sizeof(prec->vof);
  pdbRecordType->papFldDes[47]->offset=(short)((char *)&prec->vof - (char *)prec);
  pdbRecordType->papFldDes[48]->size=sizeof(prec->dir);
  pdbRecordType->papFldDes[48]->offset=(short)((char *)&prec->dir - (char *)prec);
  pdbRecordType->papFldDes[49]->size=sizeof(prec->set);
  pdbRecordType->papFldDes[49]->offset=(short)((char *)&prec->set - (char *)prec);
  pdbRecordType->papFldDes[50]->size=sizeof(prec->sset);
  pdbRecordType->papFldDes[50]->offset=(short)((char *)&prec->sset - (char *)prec);
  pdbRecordType->papFldDes[51]->size=sizeof(prec->suse);
  pdbRecordType->papFldDes[51]->offset=(short)((char *)&prec->suse - (char *)prec);
  pdbRecordType->papFldDes[52]->size=sizeof(prec->velo);
  pdbRecordType->papFldDes[52]->offset=(short)((char *)&prec->velo - (char *)prec);
  pdbRecordType->papFldDes[53]->size=sizeof(prec->vbas);
  pdbRecordType->papFldDes[53]->offset=(short)((char *)&prec->vbas - (char *)prec);
  pdbRecordType->papFldDes[54]->size=sizeof(prec->vmax);
  pdbRecordType->papFldDes[54]->offset=(short)((char *)&prec->vmax - (char *)prec);
  pdbRecordType->papFldDes[55]->size=sizeof(prec->s);
  pdbRecordType->papFldDes[55]->offset=(short)((char *)&prec->s - (char *)prec);
  pdbRecordType->papFldDes[56]->size=sizeof(prec->sbas);
  pdbRecordType->papFldDes[56]->offset=(short)((char *)&prec->sbas - (char *)prec);
  pdbRecordType->papFldDes[57]->size=sizeof(prec->smax);
  pdbRecordType->papFldDes[57]->offset=(short)((char *)&prec->smax - (char *)prec);
  pdbRecordType->papFldDes[58]->size=sizeof(prec->accl);
  pdbRecordType->papFldDes[58]->offset=(short)((char *)&prec->accl - (char *)prec);
  pdbRecordType->papFldDes[59]->size=sizeof(prec->bdst);
  pdbRecordType->papFldDes[59]->offset=(short)((char *)&prec->bdst - (char *)prec);
  pdbRecordType->papFldDes[60]->size=sizeof(prec->bvel);
  pdbRecordType->papFldDes[60]->offset=(short)((char *)&prec->bvel - (char *)prec);
  pdbRecordType->papFldDes[61]->size=sizeof(prec->sbak);
  pdbRecordType->papFldDes[61]->offset=(short)((char *)&prec->sbak - (char *)prec);
  pdbRecordType->papFldDes[62]->size=sizeof(prec->bacc);
  pdbRecordType->papFldDes[62]->offset=(short)((char *)&prec->bacc - (char *)prec);
  pdbRecordType->papFldDes[63]->size=sizeof(prec->frac);
  pdbRecordType->papFldDes[63]->offset=(short)((char *)&prec->frac - (char *)prec);
  pdbRecordType->papFldDes[64]->size=sizeof(prec->out);
  pdbRecordType->papFldDes[64]->offset=(short)((char *)&prec->out - (char *)prec);
  pdbRecordType->papFldDes[65]->size=sizeof(prec->card);
  pdbRecordType->papFldDes[65]->offset=(short)((char *)&prec->card - (char *)prec);
  pdbRecordType->papFldDes[66]->size=sizeof(prec->rdbl);
  pdbRecordType->papFldDes[66]->offset=(short)((char *)&prec->rdbl - (char *)prec);
  pdbRecordType->papFldDes[67]->size=sizeof(prec->dol);
  pdbRecordType->papFldDes[67]->offset=(short)((char *)&prec->dol - (char *)prec);
  pdbRecordType->papFldDes[68]->size=sizeof(prec->omsl);
  pdbRecordType->papFldDes[68]->offset=(short)((char *)&prec->omsl - (char *)prec);
  pdbRecordType->papFldDes[69]->size=sizeof(prec->rlnk);
  pdbRecordType->papFldDes[69]->offset=(short)((char *)&prec->rlnk - (char *)prec);
  pdbRecordType->papFldDes[70]->size=sizeof(prec->srev);
  pdbRecordType->papFldDes[70]->offset=(short)((char *)&prec->srev - (char *)prec);
  pdbRecordType->papFldDes[71]->size=sizeof(prec->urev);
  pdbRecordType->papFldDes[71]->offset=(short)((char *)&prec->urev - (char *)prec);
  pdbRecordType->papFldDes[72]->size=sizeof(prec->mres);
  pdbRecordType->papFldDes[72]->offset=(short)((char *)&prec->mres - (char *)prec);
  pdbRecordType->papFldDes[73]->size=sizeof(prec->eres);
  pdbRecordType->papFldDes[73]->offset=(short)((char *)&prec->eres - (char *)prec);
  pdbRecordType->papFldDes[74]->size=sizeof(prec->rres);
  pdbRecordType->papFldDes[74]->offset=(short)((char *)&prec->rres - (char *)prec);
  pdbRecordType->papFldDes[75]->size=sizeof(prec->res);
  pdbRecordType->papFldDes[75]->offset=(short)((char *)&prec->res - (char *)prec);
  pdbRecordType->papFldDes[76]->size=sizeof(prec->ueip);
  pdbRecordType->papFldDes[76]->offset=(short)((char *)&prec->ueip - (char *)prec);
  pdbRecordType->papFldDes[77]->size=sizeof(prec->urip);
  pdbRecordType->papFldDes[77]->offset=(short)((char *)&prec->urip - (char *)prec);
  pdbRecordType->papFldDes[78]->size=sizeof(prec->prec);
  pdbRecordType->papFldDes[78]->offset=(short)((char *)&prec->prec - (char *)prec);
  pdbRecordType->papFldDes[79]->size=sizeof(prec->egu);
  pdbRecordType->papFldDes[79]->offset=(short)((char *)&prec->egu - (char *)prec);
  pdbRecordType->papFldDes[80]->size=sizeof(prec->hlm);
  pdbRecordType->papFldDes[80]->offset=(short)((char *)&prec->hlm - (char *)prec);
  pdbRecordType->papFldDes[81]->size=sizeof(prec->llm);
  pdbRecordType->papFldDes[81]->offset=(short)((char *)&prec->llm - (char *)prec);
  pdbRecordType->papFldDes[82]->size=sizeof(prec->dhlm);
  pdbRecordType->papFldDes[82]->offset=(short)((char *)&prec->dhlm - (char *)prec);
  pdbRecordType->papFldDes[83]->size=sizeof(prec->dllm);
  pdbRecordType->papFldDes[83]->offset=(short)((char *)&prec->dllm - (char *)prec);
  pdbRecordType->papFldDes[84]->size=sizeof(prec->hopr);
  pdbRecordType->papFldDes[84]->offset=(short)((char *)&prec->hopr - (char *)prec);
  pdbRecordType->papFldDes[85]->size=sizeof(prec->lopr);
  pdbRecordType->papFldDes[85]->offset=(short)((char *)&prec->lopr - (char *)prec);
  pdbRecordType->papFldDes[86]->size=sizeof(prec->hls);
  pdbRecordType->papFldDes[86]->offset=(short)((char *)&prec->hls - (char *)prec);
  pdbRecordType->papFldDes[87]->size=sizeof(prec->lls);
  pdbRecordType->papFldDes[87]->offset=(short)((char *)&prec->lls - (char *)prec);
  pdbRecordType->papFldDes[88]->size=sizeof(prec->rhls);
  pdbRecordType->papFldDes[88]->offset=(short)((char *)&prec->rhls - (char *)prec);
  pdbRecordType->papFldDes[89]->size=sizeof(prec->rlls);
  pdbRecordType->papFldDes[89]->offset=(short)((char *)&prec->rlls - (char *)prec);
  pdbRecordType->papFldDes[90]->size=sizeof(prec->hihi);
  pdbRecordType->papFldDes[90]->offset=(short)((char *)&prec->hihi - (char *)prec);
  pdbRecordType->papFldDes[91]->size=sizeof(prec->lolo);
  pdbRecordType->papFldDes[91]->offset=(short)((char *)&prec->lolo - (char *)prec);
  pdbRecordType->papFldDes[92]->size=sizeof(prec->high);
  pdbRecordType->papFldDes[92]->offset=(short)((char *)&prec->high - (char *)prec);
  pdbRecordType->papFldDes[93]->size=sizeof(prec->low);
  pdbRecordType->papFldDes[93]->offset=(short)((char *)&prec->low - (char *)prec);
  pdbRecordType->papFldDes[94]->size=sizeof(prec->hhsv);
  pdbRecordType->papFldDes[94]->offset=(short)((char *)&prec->hhsv - (char *)prec);
  pdbRecordType->papFldDes[95]->size=sizeof(prec->llsv);
  pdbRecordType->papFldDes[95]->offset=(short)((char *)&prec->llsv - (char *)prec);
  pdbRecordType->papFldDes[96]->size=sizeof(prec->hsv);
  pdbRecordType->papFldDes[96]->offset=(short)((char *)&prec->hsv - (char *)prec);
  pdbRecordType->papFldDes[97]->size=sizeof(prec->lsv);
  pdbRecordType->papFldDes[97]->offset=(short)((char *)&prec->lsv - (char *)prec);
  pdbRecordType->papFldDes[98]->size=sizeof(prec->hlsv);
  pdbRecordType->papFldDes[98]->offset=(short)((char *)&prec->hlsv - (char *)prec);
  pdbRecordType->papFldDes[99]->size=sizeof(prec->rdbd);
  pdbRecordType->papFldDes[99]->offset=(short)((char *)&prec->rdbd - (char *)prec);
  pdbRecordType->papFldDes[100]->size=sizeof(prec->rcnt);
  pdbRecordType->papFldDes[100]->offset=(short)((char *)&prec->rcnt - (char *)prec);
  pdbRecordType->papFldDes[101]->size=sizeof(prec->rtry);
  pdbRecordType->papFldDes[101]->offset=(short)((char *)&prec->rtry - (char *)prec);
  pdbRecordType->papFldDes[102]->size=sizeof(prec->miss);
  pdbRecordType->papFldDes[102]->offset=(short)((char *)&prec->miss - (char *)prec);
  pdbRecordType->papFldDes[103]->size=sizeof(prec->spmg);
  pdbRecordType->papFldDes[103]->offset=(short)((char *)&prec->spmg - (char *)prec);
  pdbRecordType->papFldDes[104]->size=sizeof(prec->lspg);
  pdbRecordType->papFldDes[104]->offset=(short)((char *)&prec->lspg - (char *)prec);
  pdbRecordType->papFldDes[105]->size=sizeof(prec->stop);
  pdbRecordType->papFldDes[105]->offset=(short)((char *)&prec->stop - (char *)prec);
  pdbRecordType->papFldDes[106]->size=sizeof(prec->homf);
  pdbRecordType->papFldDes[106]->offset=(short)((char *)&prec->homf - (char *)prec);
  pdbRecordType->papFldDes[107]->size=sizeof(prec->homr);
  pdbRecordType->papFldDes[107]->offset=(short)((char *)&prec->homr - (char *)prec);
  pdbRecordType->papFldDes[108]->size=sizeof(prec->jogf);
  pdbRecordType->papFldDes[108]->offset=(short)((char *)&prec->jogf - (char *)prec);
  pdbRecordType->papFldDes[109]->size=sizeof(prec->jogr);
  pdbRecordType->papFldDes[109]->offset=(short)((char *)&prec->jogr - (char *)prec);
  pdbRecordType->papFldDes[110]->size=sizeof(prec->twf);
  pdbRecordType->papFldDes[110]->offset=(short)((char *)&prec->twf - (char *)prec);
  pdbRecordType->papFldDes[111]->size=sizeof(prec->twr);
  pdbRecordType->papFldDes[111]->offset=(short)((char *)&prec->twr - (char *)prec);
  pdbRecordType->papFldDes[112]->size=sizeof(prec->twv);
  pdbRecordType->papFldDes[112]->offset=(short)((char *)&prec->twv - (char *)prec);
  pdbRecordType->papFldDes[113]->size=sizeof(prec->val);
  pdbRecordType->papFldDes[113]->offset=(short)((char *)&prec->val - (char *)prec);
  pdbRecordType->papFldDes[114]->size=sizeof(prec->lval);
  pdbRecordType->papFldDes[114]->offset=(short)((char *)&prec->lval - (char *)prec);
  pdbRecordType->papFldDes[115]->size=sizeof(prec->dval);
  pdbRecordType->papFldDes[115]->offset=(short)((char *)&prec->dval - (char *)prec);
  pdbRecordType->papFldDes[116]->size=sizeof(prec->ldvl);
  pdbRecordType->papFldDes[116]->offset=(short)((char *)&prec->ldvl - (char *)prec);
  pdbRecordType->papFldDes[117]->size=sizeof(prec->rval);
  pdbRecordType->papFldDes[117]->offset=(short)((char *)&prec->rval - (char *)prec);
  pdbRecordType->papFldDes[118]->size=sizeof(prec->lrvl);
  pdbRecordType->papFldDes[118]->offset=(short)((char *)&prec->lrvl - (char *)prec);
  pdbRecordType->papFldDes[119]->size=sizeof(prec->rlv);
  pdbRecordType->papFldDes[119]->offset=(short)((char *)&prec->rlv - (char *)prec);
  pdbRecordType->papFldDes[120]->size=sizeof(prec->lrlv);
  pdbRecordType->papFldDes[120]->offset=(short)((char *)&prec->lrlv - (char *)prec);
  pdbRecordType->papFldDes[121]->size=sizeof(prec->rbv);
  pdbRecordType->papFldDes[121]->offset=(short)((char *)&prec->rbv - (char *)prec);
  pdbRecordType->papFldDes[122]->size=sizeof(prec->drbv);
  pdbRecordType->papFldDes[122]->offset=(short)((char *)&prec->drbv - (char *)prec);
  pdbRecordType->papFldDes[123]->size=sizeof(prec->diff);
  pdbRecordType->papFldDes[123]->offset=(short)((char *)&prec->diff - (char *)prec);
  pdbRecordType->papFldDes[124]->size=sizeof(prec->rdif);
  pdbRecordType->papFldDes[124]->offset=(short)((char *)&prec->rdif - (char *)prec);
  pdbRecordType->papFldDes[125]->size=sizeof(prec->pdif);
  pdbRecordType->papFldDes[125]->offset=(short)((char *)&prec->pdif - (char *)prec);
  pdbRecordType->papFldDes[126]->size=sizeof(prec->rrbv);
  pdbRecordType->papFldDes[126]->offset=(short)((char *)&prec->rrbv - (char *)prec);
  pdbRecordType->papFldDes[127]->size=sizeof(prec->rmp);
  pdbRecordType->papFldDes[127]->offset=(short)((char *)&prec->rmp - (char *)prec);
  pdbRecordType->papFldDes[128]->size=sizeof(prec->rep);
  pdbRecordType->papFldDes[128]->offset=(short)((char *)&prec->rep - (char *)prec);
  pdbRecordType->papFldDes[129]->size=sizeof(prec->rvel);
  pdbRecordType->papFldDes[129]->offset=(short)((char *)&prec->rvel - (char *)prec);
  pdbRecordType->papFldDes[130]->size=sizeof(prec->dmov);
  pdbRecordType->papFldDes[130]->offset=(short)((char *)&prec->dmov - (char *)prec);
  pdbRecordType->papFldDes[131]->size=sizeof(prec->movn);
  pdbRecordType->papFldDes[131]->offset=(short)((char *)&prec->movn - (char *)prec);
  pdbRecordType->papFldDes[132]->size=sizeof(prec->msta);
  pdbRecordType->papFldDes[132]->offset=(short)((char *)&prec->msta - (char *)prec);
  pdbRecordType->papFldDes[133]->size=sizeof(prec->lvio);
  pdbRecordType->papFldDes[133]->offset=(short)((char *)&prec->lvio - (char *)prec);
  pdbRecordType->papFldDes[134]->size=sizeof(prec->tdir);
  pdbRecordType->papFldDes[134]->offset=(short)((char *)&prec->tdir - (char *)prec);
  pdbRecordType->papFldDes[135]->size=sizeof(prec->athm);
  pdbRecordType->papFldDes[135]->offset=(short)((char *)&prec->athm - (char *)prec);
  pdbRecordType->papFldDes[136]->size=sizeof(prec->pp);
  pdbRecordType->papFldDes[136]->offset=(short)((char *)&prec->pp - (char *)prec);
  pdbRecordType->papFldDes[137]->size=sizeof(prec->mip);
  pdbRecordType->papFldDes[137]->offset=(short)((char *)&prec->mip - (char *)prec);
  pdbRecordType->papFldDes[138]->size=sizeof(prec->mmap);
  pdbRecordType->papFldDes[138]->offset=(short)((char *)&prec->mmap - (char *)prec);
  pdbRecordType->papFldDes[139]->size=sizeof(prec->nmap);
  pdbRecordType->papFldDes[139]->offset=(short)((char *)&prec->nmap - (char *)prec);
  pdbRecordType->papFldDes[140]->size=sizeof(prec->dly);
  pdbRecordType->papFldDes[140]->offset=(short)((char *)&prec->dly - (char *)prec);
  pdbRecordType->papFldDes[141]->size=sizeof(prec->cbak);
  pdbRecordType->papFldDes[141]->offset=(short)((char *)&prec->cbak - (char *)prec);
  pdbRecordType->papFldDes[142]->size=sizeof(prec->pcof);
  pdbRecordType->papFldDes[142]->offset=(short)((char *)&prec->pcof - (char *)prec);
  pdbRecordType->papFldDes[143]->size=sizeof(prec->icof);
  pdbRecordType->papFldDes[143]->offset=(short)((char *)&prec->icof - (char *)prec);
  pdbRecordType->papFldDes[144]->size=sizeof(prec->dcof);
  pdbRecordType->papFldDes[144]->offset=(short)((char *)&prec->dcof - (char *)prec);
  pdbRecordType->papFldDes[145]->size=sizeof(prec->cnen);
  pdbRecordType->papFldDes[145]->offset=(short)((char *)&prec->cnen - (char *)prec);
  pdbRecordType->papFldDes[146]->size=sizeof(prec->init);
  pdbRecordType->papFldDes[146]->offset=(short)((char *)&prec->init - (char *)prec);
  pdbRecordType->papFldDes[147]->size=sizeof(prec->prem);
  pdbRecordType->papFldDes[147]->offset=(short)((char *)&prec->prem - (char *)prec);
  pdbRecordType->papFldDes[148]->size=sizeof(prec->post);
  pdbRecordType->papFldDes[148]->offset=(short)((char *)&prec->post - (char *)prec);
  pdbRecordType->papFldDes[149]->size=sizeof(prec->stoo);
  pdbRecordType->papFldDes[149]->offset=(short)((char *)&prec->stoo - (char *)prec);
  pdbRecordType->papFldDes[150]->size=sizeof(prec->dinp);
  pdbRecordType->papFldDes[150]->offset=(short)((char *)&prec->dinp - (char *)prec);
  pdbRecordType->papFldDes[151]->size=sizeof(prec->rinp);
  pdbRecordType->papFldDes[151]->offset=(short)((char *)&prec->rinp - (char *)prec);
  pdbRecordType->papFldDes[152]->size=sizeof(prec->jvel);
  pdbRecordType->papFldDes[152]->offset=(short)((char *)&prec->jvel - (char *)prec);
  pdbRecordType->papFldDes[153]->size=sizeof(prec->jar);
  pdbRecordType->papFldDes[153]->offset=(short)((char *)&prec->jar - (char *)prec);
    pdbRecordType->rec_size = sizeof(*prec);
    return(0);
}
#endif /*GEN_SIZE_OFFSET*/
