// MessagePvt.h

#ifndef MessagePvtH
#define MessagePvtH

class Message;
class MessagePvt;
class MessageRouterClient;
class MessageRouterServer;
class MessageServer;


// The following is for use only by friends of Message
// and any class derived from a friend
// Although there is nothing to stop other code from including this
// file and changing fields, that is not a nice thing to do

class MessagePvt
{
public:
    MessageClient	*pMessageClient;
    MessageRouterClient *pMessageRouterClient;
    MessageRouterServer *pMessageRouterServer;
    MessageServer	*pMessageServer;
    void		*routerClientPvt;
    void		*routerServerPvt;
};

class MessageRouterClient 
{
public:
    MessageRouterClient(int loc) : location(loc) {}
    virtual ~MessageRouterClient() {}
    int getLocation() {return(location);}
    virtual void bind(const char *server,MessageClient *messageClient) = 0;
    virtual int send(Message *message) = 0;
    virtual void report() = 0;
    MessagePvt *getMessagePvt(Message *p) {return(p->pvt);}
private:
    int location;
};

class MessageRouterServer
{
public:
    MessageRouterServer(int loc) : location(loc) {}
    virtual ~MessageRouterServer() {}
    int getLocation() {return(location);}
    virtual int reply(Message *) = 0;
    virtual void report() = 0;
    MessagePvt *getMessagePvt(Message *p) {return(p->pvt);}
private:
    int location;
};

class Router;
class RouterPvt;
extern "C" {
// mrr - message router report - vxWorks shell command
int mrr(); 
int routerInit();
extern Router *prouter;
}

class Router {
public:
    void routerReport();
    void serverReport(const char *name);
    int registerClientRouter(MessageRouterClient *,int location);
    MessageRouterClient *findMessageRouterClient(int location);
    int registerServerRouter(MessageRouterServer *,int location);
    MessageRouterServer *findMessageRouterServer(int location);
    int registerServer(MessageServer *);
    MessageServer *findMessageServer(const char *name);
private:
    RouterPvt *pvt;
    // only routerInit can construct and destruct
    friend int routerInit();
    Router();
    ~Router();
    // disallow copy constructor, assignment operator
    Router(const Router&);
    Router& operator=(const Router&);
};

#endif /* MessagePvtH */
