//GpibHideosLocal.cc

/********************COPYRIGHT NOTIFICATION**********************************
This software was developed under a United States Government license
described on the COPYRIGHT_UniversityOfChicago file included as part
of this distribution.
****************************************************************************/
/*
    Autor:	Tom Coleman
    Date:	1995/Dec/10

*/
/*
This is the interface called by drvGpib.
It was originally written for hideos.
This version makes direct calls to ip support.
*/

#include <stdlib.h>
#include <stddef.h>
#include <string.h>
#include <stdio.h>
 
#include <vxWorks.h>
#include <semLib.h>
#include <tickLib.h>
 
extern "C" {
#include "drvHiDEOSGpib.h"
#include "epicsPrint.h"
}
 
#include "Gpib.h"

extern "C" {
void *GpibHideosInit(int BoardId, char *TaskName);
int GpibHideosWrite(void *td, char *Buf, unsigned long BufLen, 
              unsigned long devAddr, unsigned long TimeOut);
int GpibHideosRead(void *td, char *Buf, unsigned long BufLen, 
              unsigned long *Actual, unsigned long devAddr, 
              unsigned long TimeOut, long Eos);
int GpibHideosWriteRead(void *td, char *WBuf, unsigned long WBufLen, 
              char *RBuf, unsigned long RBufLen, unsigned long *RActual, 
              unsigned long devAddr, unsigned long TimeOut, long Eos);
int GpibHideosWriteCmd(void *td, char *Buf, unsigned long BufLen, 
              unsigned long TimeOut);
};

struct pvtData {
    Gpib* pGpib;
};

void *GpibHideosInit(int junk, char *gpibName)
{
    pvtData *ppvtData = new pvtData;
    ppvtData->pGpib = Gpib::find(gpibName);
    Gpib *pGpib = ppvtData->pGpib;
    if(pGpib==0) {
      printf("GpibHideosInit[Local]: could not find Gpib %d %s\n", junk, gpibName);
      return(0);
    }
    return(ppvtData);
}

int GpibHideosWrite(void *td, char *Buf, unsigned long BufLen, 
        unsigned long devAddr, unsigned long TimeOut)
{
    double timeout = (double)(TimeOut)/60.0; //convert ticks to seconds
    pvtData *ppvtData = (pvtData *)td;
    Gpib *pGpib = ppvtData->pGpib;
    if(pGpib==0) return(0);
    int status = pGpib->write(Buf,(int)BufLen,(int)devAddr,timeout);
    return(status);
}

int GpibHideosRead(void *td, char *Buf, unsigned long BufLen, 
            unsigned long *Actual, unsigned long devAddr, 
            unsigned long TimeOut, long Eos)
{
    double timeout = (double)(TimeOut)/60.0; //convert ticks to seconds
    pvtData *ppvtData = (pvtData *)td;
    Gpib *pGpib = ppvtData->pGpib;
    if(pGpib==0) return(0);
    int actual = 0; 
    gpibStatus status = pGpib->read(Buf,(int)BufLen,&actual,
	(int)devAddr,timeout,(int)Eos);
    *Actual = (unsigned long)actual;
    if(status!=gpibSuccess) return(ERROR);
    return(0);
}

int GpibHideosWriteRead(void *td, char *WBuf, unsigned long WBufLen, 
        char *RBuf, unsigned long RBufLen, unsigned long *RActual, 
        unsigned long devAddr, unsigned long TimeOut, long Eos)
{
    double timeout = (double)(TimeOut)/60.0; //convert ticks to seconds
    pvtData *ppvtData = (pvtData *)td;
    Gpib *pGpib = ppvtData->pGpib;
    if(pGpib==0) return(0);
    int actual = pGpib->write(WBuf,(int)WBufLen,(int)devAddr,timeout);
    if(actual!=(int)WBufLen) return(ERROR);
    gpibStatus status = pGpib->read(RBuf,(int)RBufLen,&actual,
	(int)devAddr,(int)TimeOut,(int)Eos);
    *RActual = (unsigned long)actual;
    if(status!=gpibSuccess) return(ERROR);
    return(0);
}
int GpibHideosWriteCmd(void *td, char *Buf, unsigned long BufLen, 
                unsigned long TimeOut)
{
    double timeout = (double)(TimeOut)/60.0; //convert ticks to seconds
    pvtData *ppvtData = (pvtData *)td;
    Gpib *pGpib = ppvtData->pGpib;
    if(pGpib==0) return(0);
    int status = pGpib->cmd(Buf,(int)BufLen,timeout);
    return(status);
}
