//devStringMpf.cc
#include <stdlib.h>
#include <stddef.h>
#include <string.h>
#include <stdio.h>
 
#include <semLib.h>
#include <tickLib.h>
#include <taskLib.h>
 
extern "C" {
#include "dbAccess.h"
#include "dbDefs.h"
#include "link.h"
#include "epicsPrint.h"
#include "dbCommon.h"
#include "stringinRecord.h"
#include "stringoutRecord.h"
#include "recSup.h"
}

#include "Message.h"
#include "Char8ArrayMessage.h"
#include "DevMpf.h"
#include "serialServer.h"

/* For devStringWriteRead
   INP has form Cx Sx @server,INstringPV
   INstringPV is sent and response is placed in VAL
*/
class devStringWriteRead : public DevMpf
{
public:
    devStringWriteRead(dbCommon*,DBLINK*);
    long startIO(dbCommon* pr);
    long completeIO(dbCommon* pr,Message* m);
    static long dev_init(void*);
private:
    DBADDR db;
    char buffer[100];
};

class devStringRead : public DevMpf
{
public:
    devStringRead(dbCommon*,DBLINK*);
    long startIO(dbCommon* pr);
    long completeIO(dbCommon* pr,Message* m);
    static long dev_init(void*);
private:
    char buffer[100];
};

class devStringWrite : public DevMpf
{
public:
    devStringWrite(dbCommon*,DBLINK*);
    long startIO(dbCommon* pr);
    long completeIO(dbCommon* pr,Message* m);
    static long dev_init(void*);
private:
    DBADDR db;
    char buffer[100];
};

MAKE_DSET(devSiMpfWriteRead,devStringWriteRead::dev_init)
MAKE_DSET(devSiMpfRead,devStringRead::dev_init)
MAKE_DSET(devSoMpfWrite,devStringWrite::dev_init)

long devStringWriteRead::dev_init(void* v)
{
    stringinRecord* pr = (stringinRecord*)v;
    new devStringWriteRead((dbCommon*)pr,&(pr->inp));
    return(0);
}

devStringWriteRead::devStringWriteRead(dbCommon* pr,DBLINK* l) : DevMpf(pr,l,false)
{
    const char* pv = getUserParm();
    if(pv==0) {
	epicsPrintf("%s devStringWriteRead Illegal INP field\n",pr->name);
        pr->pact = TRUE;
        return;
    }
    if(dbNameToAddr(pv,&db)) {
	epicsPrintf("%s devStringWriteRead Can't locate PC %s\n",pr->name,pv);
        pr->pact = TRUE;
        return;
    }
}

long devStringWriteRead::startIO(dbCommon *pr)
{
    stringinRecord *psi = (stringinRecord *)pr;
    long status = dbGet(&db,DBR_STRING,buffer,0,0,0);
    if(status) {
	epicsPrintf("%s dbGet for input string failed\n");
	recGblSetSevr(psi,READ_ALARM,INVALID_ALARM);
	return(0);
    }
    Char8ArrayMessage *message = new Char8ArrayMessage;
    int len = ::strlen(buffer) + 1;
    message->allocValue(len);
    message->setSize(len);
    ::strcpy(message->value,buffer);
    message->cmd = cmdWriteRead;
    message->timeout = 5;
    return(sendReply(message));
}

long devStringWriteRead::completeIO(dbCommon* pr,Message* pmessage)
{
    stringinRecord* psi = (stringinRecord*)pr;
    if((pmessage->getType()) != messageTypeChar8Array) {
	if(pmessage->getType() != messageTypeInt32) 
	    epicsPrintf("%s devStringWriteRead::completeIO illegal message type %d\n",
	    psi->name,pmessage->getType());
	recGblSetSevr(psi,READ_ALARM,INVALID_ALARM);
    } else {
        Char8ArrayMessage *pChar8ArrayMessage = (Char8ArrayMessage *)pmessage;
        int len = pChar8ArrayMessage->getSize();
        psi->val[len] = 0;
        psi->val[len+1] = 0;
        ::memcpy(psi->val,pChar8ArrayMessage->value,len);
        psi->udf = 0;
        if(pChar8ArrayMessage->status != serialServerStatusOK) {
            epicsPrintf("%s devStringWriteRead::completeIO status %d\n",
                psi->name,pChar8ArrayMessage->status);
            recGblSetSevr(psi,READ_ALARM,INVALID_ALARM);
        }
    }
    delete pmessage;
    return(MPF_OK);
}

long devStringRead::dev_init(void* v)
{
    stringinRecord* pr = (stringinRecord*)v;
    new devStringRead((dbCommon*)pr,&(pr->inp));
    return(0);
}

devStringRead::devStringRead(dbCommon* pr,DBLINK* l) : DevMpf(pr,l,false)
{ }

long devStringRead::startIO(dbCommon *)
{
    Char8ArrayMessage *message = new Char8ArrayMessage;
    int len = 1;
    message->allocValue(len);
    message->setSize(len);
    message->value[0] = 0;
    message->cmd = cmdRead;
    message->timeout = 5;
    return(sendReply(message));
}

long devStringRead::completeIO(dbCommon* pr,Message* pmessage)
{
    stringinRecord* psi = (stringinRecord*)pr;
    if((pmessage->getType()) != messageTypeChar8Array) {
	if(pmessage->getType() != messageTypeInt32) 
	    epicsPrintf("%s devStringRead::completeIO illegal message type %d\n",
	    psi->name,pmessage->getType());
	recGblSetSevr(psi,READ_ALARM,INVALID_ALARM);
    } else {
        Char8ArrayMessage *pChar8ArrayMessage = (Char8ArrayMessage *)pmessage;
        int len = pChar8ArrayMessage->getSize();
        psi->val[len] = 0;
        psi->val[len+1] = 0;
        ::memcpy(psi->val,pChar8ArrayMessage->value,len);
        psi->udf = 0;
        if(pChar8ArrayMessage->status != serialServerStatusOK) {
            epicsPrintf("%s devStringRead::completeIO status %d\n",
                psi->name,pChar8ArrayMessage->status);
            recGblSetSevr(psi,READ_ALARM,INVALID_ALARM);
        }
    }
    delete pmessage;
    return(MPF_OK);
}

long devStringWrite::dev_init(void* v)
{
    stringoutRecord* pr = (stringoutRecord*)v;
    new devStringWrite((dbCommon*)pr,&(pr->out));
    return(0);
}

devStringWrite::devStringWrite(dbCommon* pr,DBLINK* l) : DevMpf(pr,l,false)
{ }

long devStringWrite::startIO(dbCommon *pr)
{
    stringoutRecord *pso = (stringoutRecord *)pr;
    Char8ArrayMessage *message = new Char8ArrayMessage;
    int len = ::strlen(pso->val) + 1;
    message->allocValue(len);
    message->setSize(len);
    ::strcpy(message->value,pso->val);
    message->cmd = cmdWrite;
    message->timeout = 5;
    return(sendReply(message));
}

long devStringWrite::completeIO(dbCommon* pr,Message* pmessage)
{
    stringoutRecord* pso = (stringoutRecord*)pr;
    if((pmessage->getType()) != messageTypeChar8Array) {
	if(pmessage->getType() != messageTypeInt32) 
	    epicsPrintf("%s devStringWrite::completeIO illegal message type %d\n",
	    pso->name,pmessage->getType());
	recGblSetSevr(pso,READ_ALARM,INVALID_ALARM);
    } else {
        Char8ArrayMessage *pChar8ArrayMessage = (Char8ArrayMessage *)pmessage;
        pso->udf = 0;
        if(pChar8ArrayMessage->status != serialServerStatusOK) {
            epicsPrintf("%s devStringWrite::completeIO status %d\n",
                pso->name,pChar8ArrayMessage->status);
            recGblSetSevr(pso,WRITE_ALARM,INVALID_ALARM);
        }
    }
    delete pmessage;
    return(MPF_OK);
}
