//serialPortSniff.cc

/********************COPYRIGHT NOTIFICATION**********************************
This software was developed under a United States Government license
described on the COPYRIGHT_UniversityOfChicago file included as part
of this distribution.
****************************************************************************/
/*
	Author: Marty Kraimer
	Date: 21OCT98
*/
#include <stdlib.h>
#include <stddef.h>
#include <string.h>
#include <stdio.h>
#include <ctype.h>
 
#include <tickLib.h>
#include <taskLib.h>
#include <rngLib.h>
#include <sysLib.h>
 

#include "SerialPort.h"
 
class SniffData {
public:
    bool isReceive;
    unsigned char byte;
};

int ringBufSize = 1000 * sizeof(SniffData);

class SniffPrivate {
public:
    RING_ID ringbuf;
};

static void peakTask(const char *portName,int nseconds);
static char taskname[] = "serialSniff";

int serialPortSniff(const char *portName,int nseconds)
{
    taskSpawn(taskname,130,VX_FP_TASK,2000,
	(FUNCPTR)peakTask,
	(int)portName,nseconds,0,0,0,0,0,0,0,0);
    return(0);
}

static void peakHandler(void* parm,bool isReceive, unsigned char byte)
{
    SniffPrivate *pvt = (SniffPrivate *)parm;
    SniffData data;
    data.isReceive = isReceive;
    data.byte = byte;
    rngBufPut(pvt->ringbuf,(char *)&data,sizeof(SniffData));
}

static void printQueue(bool *isReceiveQueue, unsigned char *byteQueue,int len)
{
    for(int i=0;i<len; i++) 
	printf("%c%2.2x ",(isReceiveQueue[i] ? 'R' : 'T'),byteQueue[i]);
    for(int i=0;i<len; i++)
	printf("%c", (isprint(byteQueue[i]) ? byteQueue[i] : '.'));
    printf("\n");
}
    
static void peakTask(const char *portName,int nseconds)
{
    SniffPrivate pvt;
    bool isReceiveQueue[10];
    unsigned char byteQueue[10];
    int nextQueue = 0;
    unsigned int totalTicks = (unsigned int)(nseconds*sysClkRateGet());

    if((pvt.ringbuf = rngCreate(sizeof(void *) * ringBufSize))==NULL){
	printf("peakTask: initWatchDog failed\n");
	return;
    }
    if(!SerialPort::setSniffHandler(portName,&pvt,peakHandler)) return;
    unsigned int start = (unsigned int )::tickGet();
    printf("\n");
    while(true) {
        unsigned int ticksNow;
	taskDelay(sysClkRateGet());
	while (rngNBytes(pvt.ringbuf)>=(int)sizeof(SniffData)){
	    SniffData data;
	    if(rngBufGet(pvt.ringbuf,(char *)&data,
	    sizeof(SniffData))!=(int)sizeof(SniffData)) {
		printf("peakTask rngBufGet returned error\n");
	    } else {
		isReceiveQueue[nextQueue] = data.isReceive;
		byteQueue[nextQueue] = data.byte;
		if(++nextQueue == 10) {
		    printQueue(isReceiveQueue,byteQueue,nextQueue);
		    nextQueue = 0;
		}
	    }
            ticksNow = (unsigned int)tickGet() - start;
            if(ticksNow>=totalTicks) break;
	}
        ticksNow = (unsigned int)tickGet() - start;
        if(ticksNow>=totalTicks) break;
    }
    if(nextQueue>0) printQueue(isReceiveQueue,byteQueue,nextQueue);
    printf("serialPortSniff done\n");
    SerialPort::freeSniffHandler(portName);
    rngDelete(pvt.ringbuf);
}
