#include <vxWorks.h>
#include <semLib.h>
#include "ellLib.h"
#include "fast_lock.h"
#include "link.h"
#include "tsDefs.h"

#ifndef INCserialTMODH
#define INCserialTMODH
typedef enum {
	serialTMOD_Write_Read,
	serialTMOD_Write,
	serialTMOD_Read
}serialTMOD;
#endif /*INCserialTMODH*/

#ifndef INCserialSBITH
#define INCserialSBITH
typedef enum {
	serialSBIT_1,
	serialSBIT_2
}serialSBIT;
#endif /*INCserialSBITH*/

#ifndef INCserialPRTYH
#define INCserialPRTYH
typedef enum {
	serialPRTY_None,
	serialPRTY_Even,
	serialPRTY_Odd
}serialPRTY;
#endif /*INCserialPRTYH*/

#ifndef INCserialOFMTH
#define INCserialOFMTH
typedef enum {
	serialOFMT_ASCII,
	serialOFMT_Binary,
	serialOFMT_Hybrid
}serialOFMT;
#endif /*INCserialOFMTH*/

#ifndef INCserialFCTLH
#define INCserialFCTLH
typedef enum {
	serialFCTL_None,
	serialFCTL_Hardware
}serialFCTL;
#endif /*INCserialFCTLH*/

#ifndef INCserialDBITH
#define INCserialDBITH
typedef enum {
	serialDBIT_5,
	serialDBIT_6,
	serialDBIT_7,
	serialDBIT_8
}serialDBIT;
#endif /*INCserialDBITH*/

#ifndef INCserialBAUDH
#define INCserialBAUDH
typedef enum {
	serialBAUD_300,
	serialBAUD_600,
	serialBAUD_1200,
	serialBAUD_2400,
	serialBAUD_4800,
	serialBAUD_9600,
	serialBAUD_19200,
	serialBAUD_38400
}serialBAUD;
#endif /*INCserialBAUDH*/
#ifndef INCserialH
#define INCserialH
typedef struct serialRecord {
	char		name[29]; /*Record Name*/
	char		desc[29]; /*Descriptor*/
	char		asg[29]; /*Access Security Group*/
	unsigned short	scan;	/*Scan Mechanism*/
	unsigned short	pini;	/*Process at iocInit*/
	short		phas;	/*Scan Phase*/
	short		evnt;	/*Event Number*/
	short		tse;	/*Time Stamp Event*/
	DBLINK		tsel;	/*Time Stamp Link*/
	unsigned short	dtyp;	/*Device Type*/
	short		disv;	/*Disable Value*/
	short		disa;	/*Disable*/
	DBLINK		sdis;	/*Scanning Disable*/
	FAST_LOCK	mlok;	/*Monitor fastlock*/
	ELLLIST		mlis;	/*Monitor List*/
	unsigned char	disp;	/*Disable putField*/
	unsigned char	proc;	/*Force Processing*/
	unsigned short	stat;	/*Alarm Status*/
	unsigned short	sevr;	/*Alarm Severity*/
	unsigned short	nsta;	/*New Alarm Status*/
	unsigned short	nsev;	/*New Alarm Severity*/
	unsigned short	acks;	/*Alarm Ack Severity*/
	unsigned short	ackt;	/*Alarm Ack Transient*/
	unsigned short	diss;	/*Disable Alarm Sevrty*/
	unsigned char	lcnt;	/*Lock Count*/
	unsigned char	pact;	/*Record active*/
	unsigned char	putf;	/*dbPutField process*/
	unsigned char	rpro;	/*Reprocess */
	void		*asp;	/*Access Security Pvt*/
	struct putNotify *ppn;	/*addr of PUTNOTIFY*/
	struct pnWaitNode *ppnn;	/*next record PUTNOTIFY*/
	struct scan_element *spvt;	/*Scan Private*/
	struct rset	*rset;	/*Address of RSET*/
	struct dset	*dset;	/*DSET address*/
	void		*dpvt;	/*Device Private*/
	struct dbRecordType *rdes;	/*Address of dbRecordType*/
	struct lockRecord *lset;	/*Lock Set*/
	unsigned short	prio;	/*Scheduling Priority*/
	unsigned char	tpro;	/*Trace Processing*/
	char bkpt;	/*Break Point*/
	unsigned char	udf;	/*Undefined*/
	TS_STAMP	time;	/*Time*/
	DBLINK		flnk;	/*Forward Process Link*/
	char		val[40]; /*Value field (unused)*/
	unsigned short	tmod;	/*Transaction mode*/
	char		aout[40]; /*Output (command) string*/
	long		odel;	/*Output delimiter*/
	char		bout;	/*Output binary data*/
	void *optr;	/*Output buffer pointer*/
	long		omax;	/*Max. size of output arr*/
	long		nowt;	/*Number of bytes to writ*/
	unsigned short	ofmt;	/*Output format*/
	char		ainp[40]; /*Input (response) string*/
	long		idel;	/*Input delimiter*/
	char		binp;	/*Input binary data*/
	void *iptr;	/*Input buffer pointer*/
	long		imax;	/*Max. size of input arra*/
	long		nrrd;	/*Number of bytes to read*/
	long		nord;	/*Number of bytes read*/
	unsigned short	ifmt;	/*Input format*/
	DBLINK		inp;	/*Input Specification*/
	long		tmot;	/*Timeout (msec)*/
	unsigned short	baud;	/*Baud rate*/
	unsigned short	prty;	/*Parity*/
	unsigned short	dbit;	/*Data bits*/
	unsigned short	sbit;	/*Stop bits*/
	unsigned short	fctl;	/*Flow control*/
	char		oinp[40]; /*Previous input string*/
	long		onrd;	/*Previous nord*/
	long		onwt;	/*Previous nowt*/
} serialRecord;
#define serialRecordNAME	0
#define serialRecordDESC	1
#define serialRecordASG	2
#define serialRecordSCAN	3
#define serialRecordPINI	4
#define serialRecordPHAS	5
#define serialRecordEVNT	6
#define serialRecordTSE	7
#define serialRecordTSEL	8
#define serialRecordDTYP	9
#define serialRecordDISV	10
#define serialRecordDISA	11
#define serialRecordSDIS	12
#define serialRecordMLOK	13
#define serialRecordMLIS	14
#define serialRecordDISP	15
#define serialRecordPROC	16
#define serialRecordSTAT	17
#define serialRecordSEVR	18
#define serialRecordNSTA	19
#define serialRecordNSEV	20
#define serialRecordACKS	21
#define serialRecordACKT	22
#define serialRecordDISS	23
#define serialRecordLCNT	24
#define serialRecordPACT	25
#define serialRecordPUTF	26
#define serialRecordRPRO	27
#define serialRecordASP	28
#define serialRecordPPN	29
#define serialRecordPPNN	30
#define serialRecordSPVT	31
#define serialRecordRSET	32
#define serialRecordDSET	33
#define serialRecordDPVT	34
#define serialRecordRDES	35
#define serialRecordLSET	36
#define serialRecordPRIO	37
#define serialRecordTPRO	38
#define serialRecordBKPT	39
#define serialRecordUDF	40
#define serialRecordTIME	41
#define serialRecordFLNK	42
#define serialRecordVAL	43
#define serialRecordTMOD	44
#define serialRecordAOUT	45
#define serialRecordODEL	46
#define serialRecordBOUT	47
#define serialRecordOPTR	48
#define serialRecordOMAX	49
#define serialRecordNOWT	50
#define serialRecordOFMT	51
#define serialRecordAINP	52
#define serialRecordIDEL	53
#define serialRecordBINP	54
#define serialRecordIPTR	55
#define serialRecordIMAX	56
#define serialRecordNRRD	57
#define serialRecordNORD	58
#define serialRecordIFMT	59
#define serialRecordINP	60
#define serialRecordTMOT	61
#define serialRecordBAUD	62
#define serialRecordPRTY	63
#define serialRecordDBIT	64
#define serialRecordSBIT	65
#define serialRecordFCTL	66
#define serialRecordOINP	67
#define serialRecordONRD	68
#define serialRecordONWT	69
#endif /*INCserialH*/
#ifdef GEN_SIZE_OFFSET
int serialRecordSizeOffset(dbRecordType *pdbRecordType)
{
    serialRecord *prec = 0;
  pdbRecordType->papFldDes[0]->size=sizeof(prec->name);
  pdbRecordType->papFldDes[0]->offset=(short)((char *)&prec->name - (char *)prec);
  pdbRecordType->papFldDes[1]->size=sizeof(prec->desc);
  pdbRecordType->papFldDes[1]->offset=(short)((char *)&prec->desc - (char *)prec);
  pdbRecordType->papFldDes[2]->size=sizeof(prec->asg);
  pdbRecordType->papFldDes[2]->offset=(short)((char *)&prec->asg - (char *)prec);
  pdbRecordType->papFldDes[3]->size=sizeof(prec->scan);
  pdbRecordType->papFldDes[3]->offset=(short)((char *)&prec->scan - (char *)prec);
  pdbRecordType->papFldDes[4]->size=sizeof(prec->pini);
  pdbRecordType->papFldDes[4]->offset=(short)((char *)&prec->pini - (char *)prec);
  pdbRecordType->papFldDes[5]->size=sizeof(prec->phas);
  pdbRecordType->papFldDes[5]->offset=(short)((char *)&prec->phas - (char *)prec);
  pdbRecordType->papFldDes[6]->size=sizeof(prec->evnt);
  pdbRecordType->papFldDes[6]->offset=(short)((char *)&prec->evnt - (char *)prec);
  pdbRecordType->papFldDes[7]->size=sizeof(prec->tse);
  pdbRecordType->papFldDes[7]->offset=(short)((char *)&prec->tse - (char *)prec);
  pdbRecordType->papFldDes[8]->size=sizeof(prec->tsel);
  pdbRecordType->papFldDes[8]->offset=(short)((char *)&prec->tsel - (char *)prec);
  pdbRecordType->papFldDes[9]->size=sizeof(prec->dtyp);
  pdbRecordType->papFldDes[9]->offset=(short)((char *)&prec->dtyp - (char *)prec);
  pdbRecordType->papFldDes[10]->size=sizeof(prec->disv);
  pdbRecordType->papFldDes[10]->offset=(short)((char *)&prec->disv - (char *)prec);
  pdbRecordType->papFldDes[11]->size=sizeof(prec->disa);
  pdbRecordType->papFldDes[11]->offset=(short)((char *)&prec->disa - (char *)prec);
  pdbRecordType->papFldDes[12]->size=sizeof(prec->sdis);
  pdbRecordType->papFldDes[12]->offset=(short)((char *)&prec->sdis - (char *)prec);
  pdbRecordType->papFldDes[13]->size=sizeof(prec->mlok);
  pdbRecordType->papFldDes[13]->offset=(short)((char *)&prec->mlok - (char *)prec);
  pdbRecordType->papFldDes[14]->size=sizeof(prec->mlis);
  pdbRecordType->papFldDes[14]->offset=(short)((char *)&prec->mlis - (char *)prec);
  pdbRecordType->papFldDes[15]->size=sizeof(prec->disp);
  pdbRecordType->papFldDes[15]->offset=(short)((char *)&prec->disp - (char *)prec);
  pdbRecordType->papFldDes[16]->size=sizeof(prec->proc);
  pdbRecordType->papFldDes[16]->offset=(short)((char *)&prec->proc - (char *)prec);
  pdbRecordType->papFldDes[17]->size=sizeof(prec->stat);
  pdbRecordType->papFldDes[17]->offset=(short)((char *)&prec->stat - (char *)prec);
  pdbRecordType->papFldDes[18]->size=sizeof(prec->sevr);
  pdbRecordType->papFldDes[18]->offset=(short)((char *)&prec->sevr - (char *)prec);
  pdbRecordType->papFldDes[19]->size=sizeof(prec->nsta);
  pdbRecordType->papFldDes[19]->offset=(short)((char *)&prec->nsta - (char *)prec);
  pdbRecordType->papFldDes[20]->size=sizeof(prec->nsev);
  pdbRecordType->papFldDes[20]->offset=(short)((char *)&prec->nsev - (char *)prec);
  pdbRecordType->papFldDes[21]->size=sizeof(prec->acks);
  pdbRecordType->papFldDes[21]->offset=(short)((char *)&prec->acks - (char *)prec);
  pdbRecordType->papFldDes[22]->size=sizeof(prec->ackt);
  pdbRecordType->papFldDes[22]->offset=(short)((char *)&prec->ackt - (char *)prec);
  pdbRecordType->papFldDes[23]->size=sizeof(prec->diss);
  pdbRecordType->papFldDes[23]->offset=(short)((char *)&prec->diss - (char *)prec);
  pdbRecordType->papFldDes[24]->size=sizeof(prec->lcnt);
  pdbRecordType->papFldDes[24]->offset=(short)((char *)&prec->lcnt - (char *)prec);
  pdbRecordType->papFldDes[25]->size=sizeof(prec->pact);
  pdbRecordType->papFldDes[25]->offset=(short)((char *)&prec->pact - (char *)prec);
  pdbRecordType->papFldDes[26]->size=sizeof(prec->putf);
  pdbRecordType->papFldDes[26]->offset=(short)((char *)&prec->putf - (char *)prec);
  pdbRecordType->papFldDes[27]->size=sizeof(prec->rpro);
  pdbRecordType->papFldDes[27]->offset=(short)((char *)&prec->rpro - (char *)prec);
  pdbRecordType->papFldDes[28]->size=sizeof(prec->asp);
  pdbRecordType->papFldDes[28]->offset=(short)((char *)&prec->asp - (char *)prec);
  pdbRecordType->papFldDes[29]->size=sizeof(prec->ppn);
  pdbRecordType->papFldDes[29]->offset=(short)((char *)&prec->ppn - (char *)prec);
  pdbRecordType->papFldDes[30]->size=sizeof(prec->ppnn);
  pdbRecordType->papFldDes[30]->offset=(short)((char *)&prec->ppnn - (char *)prec);
  pdbRecordType->papFldDes[31]->size=sizeof(prec->spvt);
  pdbRecordType->papFldDes[31]->offset=(short)((char *)&prec->spvt - (char *)prec);
  pdbRecordType->papFldDes[32]->size=sizeof(prec->rset);
  pdbRecordType->papFldDes[32]->offset=(short)((char *)&prec->rset - (char *)prec);
  pdbRecordType->papFldDes[33]->size=sizeof(prec->dset);
  pdbRecordType->papFldDes[33]->offset=(short)((char *)&prec->dset - (char *)prec);
  pdbRecordType->papFldDes[34]->size=sizeof(prec->dpvt);
  pdbRecordType->papFldDes[34]->offset=(short)((char *)&prec->dpvt - (char *)prec);
  pdbRecordType->papFldDes[35]->size=sizeof(prec->rdes);
  pdbRecordType->papFldDes[35]->offset=(short)((char *)&prec->rdes - (char *)prec);
  pdbRecordType->papFldDes[36]->size=sizeof(prec->lset);
  pdbRecordType->papFldDes[36]->offset=(short)((char *)&prec->lset - (char *)prec);
  pdbRecordType->papFldDes[37]->size=sizeof(prec->prio);
  pdbRecordType->papFldDes[37]->offset=(short)((char *)&prec->prio - (char *)prec);
  pdbRecordType->papFldDes[38]->size=sizeof(prec->tpro);
  pdbRecordType->papFldDes[38]->offset=(short)((char *)&prec->tpro - (char *)prec);
  pdbRecordType->papFldDes[39]->size=sizeof(prec->bkpt);
  pdbRecordType->papFldDes[39]->offset=(short)((char *)&prec->bkpt - (char *)prec);
  pdbRecordType->papFldDes[40]->size=sizeof(prec->udf);
  pdbRecordType->papFldDes[40]->offset=(short)((char *)&prec->udf - (char *)prec);
  pdbRecordType->papFldDes[41]->size=sizeof(prec->time);
  pdbRecordType->papFldDes[41]->offset=(short)((char *)&prec->time - (char *)prec);
  pdbRecordType->papFldDes[42]->size=sizeof(prec->flnk);
  pdbRecordType->papFldDes[42]->offset=(short)((char *)&prec->flnk - (char *)prec);
  pdbRecordType->papFldDes[43]->size=sizeof(prec->val);
  pdbRecordType->papFldDes[43]->offset=(short)((char *)&prec->val - (char *)prec);
  pdbRecordType->papFldDes[44]->size=sizeof(prec->tmod);
  pdbRecordType->papFldDes[44]->offset=(short)((char *)&prec->tmod - (char *)prec);
  pdbRecordType->papFldDes[45]->size=sizeof(prec->aout);
  pdbRecordType->papFldDes[45]->offset=(short)((char *)&prec->aout - (char *)prec);
  pdbRecordType->papFldDes[46]->size=sizeof(prec->odel);
  pdbRecordType->papFldDes[46]->offset=(short)((char *)&prec->odel - (char *)prec);
  pdbRecordType->papFldDes[47]->size=sizeof(prec->bout);
  pdbRecordType->papFldDes[47]->offset=(short)((char *)&prec->bout - (char *)prec);
  pdbRecordType->papFldDes[48]->size=sizeof(prec->optr);
  pdbRecordType->papFldDes[48]->offset=(short)((char *)&prec->optr - (char *)prec);
  pdbRecordType->papFldDes[49]->size=sizeof(prec->omax);
  pdbRecordType->papFldDes[49]->offset=(short)((char *)&prec->omax - (char *)prec);
  pdbRecordType->papFldDes[50]->size=sizeof(prec->nowt);
  pdbRecordType->papFldDes[50]->offset=(short)((char *)&prec->nowt - (char *)prec);
  pdbRecordType->papFldDes[51]->size=sizeof(prec->ofmt);
  pdbRecordType->papFldDes[51]->offset=(short)((char *)&prec->ofmt - (char *)prec);
  pdbRecordType->papFldDes[52]->size=sizeof(prec->ainp);
  pdbRecordType->papFldDes[52]->offset=(short)((char *)&prec->ainp - (char *)prec);
  pdbRecordType->papFldDes[53]->size=sizeof(prec->idel);
  pdbRecordType->papFldDes[53]->offset=(short)((char *)&prec->idel - (char *)prec);
  pdbRecordType->papFldDes[54]->size=sizeof(prec->binp);
  pdbRecordType->papFldDes[54]->offset=(short)((char *)&prec->binp - (char *)prec);
  pdbRecordType->papFldDes[55]->size=sizeof(prec->iptr);
  pdbRecordType->papFldDes[55]->offset=(short)((char *)&prec->iptr - (char *)prec);
  pdbRecordType->papFldDes[56]->size=sizeof(prec->imax);
  pdbRecordType->papFldDes[56]->offset=(short)((char *)&prec->imax - (char *)prec);
  pdbRecordType->papFldDes[57]->size=sizeof(prec->nrrd);
  pdbRecordType->papFldDes[57]->offset=(short)((char *)&prec->nrrd - (char *)prec);
  pdbRecordType->papFldDes[58]->size=sizeof(prec->nord);
  pdbRecordType->papFldDes[58]->offset=(short)((char *)&prec->nord - (char *)prec);
  pdbRecordType->papFldDes[59]->size=sizeof(prec->ifmt);
  pdbRecordType->papFldDes[59]->offset=(short)((char *)&prec->ifmt - (char *)prec);
  pdbRecordType->papFldDes[60]->size=sizeof(prec->inp);
  pdbRecordType->papFldDes[60]->offset=(short)((char *)&prec->inp - (char *)prec);
  pdbRecordType->papFldDes[61]->size=sizeof(prec->tmot);
  pdbRecordType->papFldDes[61]->offset=(short)((char *)&prec->tmot - (char *)prec);
  pdbRecordType->papFldDes[62]->size=sizeof(prec->baud);
  pdbRecordType->papFldDes[62]->offset=(short)((char *)&prec->baud - (char *)prec);
  pdbRecordType->papFldDes[63]->size=sizeof(prec->prty);
  pdbRecordType->papFldDes[63]->offset=(short)((char *)&prec->prty - (char *)prec);
  pdbRecordType->papFldDes[64]->size=sizeof(prec->dbit);
  pdbRecordType->papFldDes[64]->offset=(short)((char *)&prec->dbit - (char *)prec);
  pdbRecordType->papFldDes[65]->size=sizeof(prec->sbit);
  pdbRecordType->papFldDes[65]->offset=(short)((char *)&prec->sbit - (char *)prec);
  pdbRecordType->papFldDes[66]->size=sizeof(prec->fctl);
  pdbRecordType->papFldDes[66]->offset=(short)((char *)&prec->fctl - (char *)prec);
  pdbRecordType->papFldDes[67]->size=sizeof(prec->oinp);
  pdbRecordType->papFldDes[67]->offset=(short)((char *)&prec->oinp - (char *)prec);
  pdbRecordType->papFldDes[68]->size=sizeof(prec->onrd);
  pdbRecordType->papFldDes[68]->offset=(short)((char *)&prec->onrd - (char *)prec);
  pdbRecordType->papFldDes[69]->size=sizeof(prec->onwt);
  pdbRecordType->papFldDes[69]->offset=(short)((char *)&prec->onwt - (char *)prec);
    pdbRecordType->rec_size = sizeof(*prec);
    return(0);
}
#endif /*GEN_SIZE_OFFSET*/
