#define YYBYACC 1
#define YYMAJOR 1
#define YYMINOR 9
#define yyclearin (yychar=(-1))
#define yyerrok (yyerrflag=0)
#define YYRECOVERING (yyerrflag!=0)
#ifdef __STDC__
static int yyparse(void);
#else
static int yyparse();
#endif
#define YYPREFIX "yy"

/**************************************************************************
 *
 *     Author:	Jim Kowalkowski
 *
 *      Experimental Physics and Industrial Control System (EPICS)
 *
 *      Copyright 1991, the Regents of the University of California,
 *      and the University of Chicago Board of Governors.
 *
 *      This software was produced under  U.S. Government contracts:
 *      (W-7405-ENG-36) at the Los Alamos National Laboratory,
 *      and (W-31-109-ENG-38) at Argonne National Laboratory.
 *
 *      Initial development by:
 *              The Controls and Automation Group (AT-8)
 *              Ground Test Accelerator
 *              Accelerator Technology Division
 *              Los Alamos National Laboratory
 *
 *      Co-developed with
 *              The Controls and Computing Group
 *              Accelerator Systems Division
 *              Advanced Photon Source
 *              Argonne National Laboratory
 *
 * Modification Log:
 * -----------------
 * .01	10-29-93	jbk	initial version
 *
 ***********************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "macLib.h"
#include "dbStaticLib.h"
#include "dbmf.h"
#include "epicsVersion.h"

#define VAR_MAX_SUB_SIZE 300
static char *subst_buffer= NULL;
static int subst_used;
static int line_num;
static MAC_HANDLE *macHandle = NULL;

struct db_app_node
{
	char* name;
	struct db_app_node* next;
};
typedef struct db_app_node DB_APP_NODE;

DB_APP_NODE* DbApplList=(DB_APP_NODE*)NULL;
static DB_APP_NODE* DbCurrentListHead=(DB_APP_NODE*)NULL;
static DB_APP_NODE* DbCurrentListTail=(DB_APP_NODE*)NULL;

static int yyerror();
static void sub_pvname(char*,char*);

#ifdef vxWorks
static DBENTRY* pdbentry;
extern struct dbBase *pdbbase;
#endif

static char* strduplicate(char* x)
{
	char* c;
	c=(char*)malloc(strlen(x)+1);
	strcpy(c,x);
	return c;
}
 
typedef union
{
    int	Int;
	char Char;
	char *Str;
	double Real;
} YYSTYPE;
#define COMMA 257
#define WORD 258
#define VALUE 259
#define FIELD 260
#define O_BRACE 261
#define C_BRACE 262
#define O_PAREN 263
#define C_PAREN 264
#define DATABASE 265
#define RECORD 266
#define NOWHERE 267
#define APPL 268
#define YYERRCODE 256
static short yylhs[] = {                                        -1,
    0,    0,    0,    1,    1,    2,    4,    4,    5,    6,
    3,    3,    3,    8,    7,    7,    9,   10,   10,   11,
   11,   12,   12,   13,
};
static short yylen[] = {                                         2,
    3,    2,    1,    3,    5,    4,    0,    3,    2,    3,
    0,    2,    2,    4,    0,    2,    3,    5,    5,    0,
    3,    0,    2,    6,
};
static short yydefred[] = {                                      0,
    0,    0,    0,    0,    0,    0,    0,   12,   13,    0,
    0,    1,    0,    0,    0,    0,    4,    0,   11,    0,
   22,   17,    0,    0,    0,    0,    0,    0,    0,   14,
    5,    9,   15,    8,    6,    0,    0,    0,   21,   23,
    0,   18,   19,    0,   10,   16,    0,    0,    0,   24,
};
static short yydgoto[] = {                                       2,
    5,   12,    3,   19,   26,   34,   41,    8,    9,   14,
   22,   29,   40,
};
static short yysindex[] = {                                   -259,
 -252,    0, -263, -244, -239, -245, -240,    0,    0, -257,
 -242,    0, -234, -235, -232, -231,    0, -233,    0, -228,
    0,    0, -230, -229, -227, -225, -258, -238, -243,    0,
    0,    0,    0,    0,    0, -226, -224, -222,    0,    0,
 -250,    0,    0, -219,    0,    0, -215, -216, -220,    0,
};
static short yyrindex[] = {                                      2,
    0,    0,   31,    0,   32,    0,    0,    0,    0,    0,
 -253,    0,    0,    1,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
};
static short yygindex[] = {                                      0,
    0,    0,   14,    0,    0,    0,    0,    0,    4,    0,
    0,    0,    0,
};
#define YYTABLESIZE 270
static short yytable[] = {                                      16,
   20,   11,    6,   35,    7,    1,   17,    6,    7,    7,
    4,   45,    7,   10,    7,    6,   38,   13,   39,   36,
   37,   11,   15,   20,   18,   21,   23,   24,   28,   25,
    3,    2,   27,   30,   31,   33,   32,   42,   47,   43,
   44,   48,   49,   50,   46,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,   20,    0,    0,    0,   20,   11,   20,   11,
};
static short yycheck[] = {                                     257,
    0,    0,  266,  262,  268,  265,  264,  266,  262,  268,
  263,  262,  266,  258,  268,  266,  260,  263,  262,  258,
  259,  261,  263,  258,  267,  261,  259,  259,  257,  263,
    0,    0,   19,  264,  264,  261,  264,  264,  258,  264,
  263,  257,  259,  264,   41,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  262,   -1,   -1,   -1,  266,  266,  268,  268,
};
#define YYFINAL 2
#ifndef YYDEBUG
#define YYDEBUG 0
#endif
#define YYMAXTOKEN 268
#if YYDEBUG
static char *yyname[] = {
"end-of-file",0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,"COMMA","WORD","VALUE","FIELD",
"O_BRACE","C_BRACE","O_PAREN","C_PAREN","DATABASE","RECORD","NOWHERE","APPL",
};
static char *yyrule[] = {
"$accept : database",
"database : DATABASE d_head d_body",
"database : DATABASE d_head",
"database : db_components",
"d_head : O_PAREN WORD C_PAREN",
"d_head : O_PAREN WORD COMMA VALUE C_PAREN",
"d_body : O_BRACE nowhere_records db_components C_BRACE",
"nowhere_records :",
"nowhere_records : NOWHERE n_head n_body",
"n_head : O_PAREN C_PAREN",
"n_body : O_BRACE records C_BRACE",
"db_components :",
"db_components : db_components applic",
"db_components : db_components record",
"applic : APPL O_PAREN VALUE C_PAREN",
"records :",
"records : records record",
"record : RECORD r_head r_body",
"r_head : O_PAREN WORD COMMA WORD C_PAREN",
"r_head : O_PAREN WORD COMMA VALUE C_PAREN",
"r_body :",
"r_body : O_BRACE fields C_BRACE",
"fields :",
"fields : fields field",
"field : FIELD O_PAREN WORD COMMA VALUE C_PAREN",
};
#endif
#ifdef YYSTACKSIZE
#undef YYMAXDEPTH
#define YYMAXDEPTH YYSTACKSIZE
#else
#ifdef YYMAXDEPTH
#define YYSTACKSIZE YYMAXDEPTH
#else
#define YYSTACKSIZE 500
#define YYMAXDEPTH 500
#endif
#endif
#if YYDEBUG
static int yydebug;
#endif
static int yynerrs;
static int yyerrflag;
static int yychar;
static short *yyssp;
static YYSTYPE *yyvsp;
static YYSTYPE yyval;
static YYSTYPE yylval;
static short yyss[YYSTACKSIZE];
static YYSTYPE yyvs[YYSTACKSIZE];
#define yystacksize YYSTACKSIZE
 
#include "dbLoadRecords_lex.c"
 
static int yyerror(str)
char  *str;
{
    fprintf(stderr,"db file parse, Error line %d : %s\n",line_num, yytext);
    return(0);
}

static int is_not_inited = 1;
 
#define MAXPATHLEN 300
#define DIR_SEPARATOR '/'
int dbLoadRecords(char* pfilename, char* pattern, char* ppath)
{
	FILE* fp;
	char    **macPairs;
	char pfullpath[MAXPATHLEN] = "";
	int i;

	line_num=0;

#ifdef vxWorks
	if(pdbbase==NULL)
	{
		fprintf(stderr,"dbLoadRecords: dbLoadDatabase not called\n");
		return -1;
	}
#endif

	/* Prepend optional argument ppath to pfilename */
	if ((strlen(pfilename) + ppath?strlen(ppath):0) > (MAXPATHLEN-2)) {
		fprintf(stderr,"dbLoadRecords: path is too long\n");
		return(-1);
	}
	if (ppath && strlen(ppath)) {
		strcpy(pfullpath, ppath);
		i = strlen(pfullpath);
		if ((pfullpath[i-1] != DIR_SEPARATOR) &&
				(pfilename[0] != DIR_SEPARATOR)) {
			pfullpath[i] = DIR_SEPARATOR;
			pfullpath[i+1] = '\0';
		}
	}
	strcat(pfullpath, pfilename);

	if( pattern && *pattern )
	{
		subst_buffer = malloc(VAR_MAX_SUB_SIZE);
		subst_used = 1;
		if(macCreateHandle(&macHandle,NULL)) {
		    fprintf(stderr,"dbLoadRecords macCreateHandle error\n");
		    return -1;
		}
		macParseDefns(macHandle,pattern,&macPairs);
		if(macPairs == NULL) {
		    macDeleteHandle(macHandle);
		    macHandle = NULL;
		} else {
		    macInstallMacros(macHandle,macPairs);
		    free((void *)macPairs);
		}
	}
	else
		subst_used = 0;
	if( !(fp=fopen(pfullpath,"r")) )
	{
		fprintf(stderr,"dbLoadRecords: error opening file\n");
		return -1;
	}

	if(is_not_inited)
	{
		yyin=fp;
		is_not_inited=0;
	}
	else
	{
		yyrestart(fp);
	}

#ifdef vxWorks
	pdbentry=dbAllocEntry(pdbbase);
#endif

	yyparse();

#ifdef vxWorks
	dbFreeEntry(pdbentry);
#endif

	if(subst_used) {
	    macDeleteHandle(macHandle);
	    macHandle = NULL;
	    free((void *)subst_buffer);
	    subst_buffer = NULL;
	}

	fclose(fp);

	if(DbCurrentListHead==(DB_APP_NODE*)NULL)
	{
		/* set up a default list to put on the master application list */
		DbCurrentListHead=(DB_APP_NODE*)malloc(sizeof(DB_APP_NODE));
		DbCurrentListTail=DbCurrentListHead;
		DbCurrentListHead->name=strduplicate(pfullpath);
		DbCurrentListHead->next=(DB_APP_NODE*)NULL;
	}

	DbCurrentListTail->next=DbApplList;
	DbApplList=DbCurrentListHead;
	DbCurrentListHead=(DB_APP_NODE*)NULL;
	DbCurrentListTail=(DB_APP_NODE*)NULL;

	return 0;
}

static void sub_pvname(char* type, char* name)
{
#ifdef vxWorks
		if( dbFindRecordType(pdbentry,type) )
			fprintf(stderr,"Cannot find record type %s\n",type);
#endif

		if(subst_used)
		{
			int n;

			n = macExpandString(macHandle,name,subst_buffer,
			    VAR_MAX_SUB_SIZE-1);
			if(n<0) fprintf(stderr,"macExpandString failed\n");
#ifdef vxWorks
			dbCreateRecord(pdbentry,subst_buffer);
#else
			printf("record(%s,\"%s\") {",type,subst_buffer);
#endif
		}
		else
		{
#ifdef vxWorks
			dbCreateRecord(pdbentry,name);
#else
			printf("record(%s,\"%s\") {",type,name);
#endif
		}
}

#ifdef vxWorks
int dbAppList()
{
	DB_APP_NODE* an;

	for(an=DbApplList;an;an=an->next)
		printf("%s\n",an->name);

	return 0;
}
#endif
#define YYABORT goto yyabort
#define YYREJECT goto yyabort
#define YYACCEPT goto yyaccept
#define YYERROR goto yyerrlab
#ifdef __STDC__
static int
yyparse(void)
#else
static int
yyparse()
#endif
{
    register int yym, yyn, yystate;
#if YYDEBUG
    register char *yys;
    extern char *getenv();

    if (yys = getenv("YYDEBUG"))
    {
        yyn = *yys;
        if (yyn >= '0' && yyn <= '9')
            yydebug = yyn - '0';
    }
#endif

    yynerrs = 0;
    yyerrflag = 0;
    yychar = (-1);

    yyssp = yyss;
    yyvsp = yyvs;
    *yyssp = yystate = 0;

yyloop:
    if (yyn = yydefred[yystate]) goto yyreduce;
    if (yychar < 0)
    {
        if ((yychar = yylex()) < 0) yychar = 0;
#if YYDEBUG
        if (yydebug)
        {
            yys = 0;
            if (yychar <= YYMAXTOKEN) yys = yyname[yychar];
            if (!yys) yys = "illegal-symbol";
            printf("%sdebug: state %d, reading %d (%s)\n",
                    YYPREFIX, yystate, yychar, yys);
        }
#endif
    }
    if ((yyn = yysindex[yystate]) && (yyn += yychar) >= 0 &&
            yyn <= YYTABLESIZE && yycheck[yyn] == yychar)
    {
#if YYDEBUG
        if (yydebug)
            printf("%sdebug: state %d, shifting to state %d\n",
                    YYPREFIX, yystate, yytable[yyn]);
#endif
        if (yyssp >= yyss + yystacksize - 1)
        {
            goto yyoverflow;
        }
        *++yyssp = yystate = yytable[yyn];
        *++yyvsp = yylval;
        yychar = (-1);
        if (yyerrflag > 0)  --yyerrflag;
        goto yyloop;
    }
    if ((yyn = yyrindex[yystate]) && (yyn += yychar) >= 0 &&
            yyn <= YYTABLESIZE && yycheck[yyn] == yychar)
    {
        yyn = yytable[yyn];
        goto yyreduce;
    }
    if (yyerrflag) goto yyinrecovery;
    yyerror("syntax error");
    ++yynerrs;
yyinrecovery:
    if (yyerrflag < 3)
    {
        yyerrflag = 3;
        for (;;)
        {
            if ((yyn = yysindex[*yyssp]) && (yyn += YYERRCODE) >= 0 &&
                    yyn <= YYTABLESIZE && yycheck[yyn] == YYERRCODE)
            {
#if YYDEBUG
                if (yydebug)
                    printf("%sdebug: state %d, error recovery shifting\
 to state %d\n", YYPREFIX, *yyssp, yytable[yyn]);
#endif
                if (yyssp >= yyss + yystacksize - 1)
                {
                    goto yyoverflow;
                }
                *++yyssp = yystate = yytable[yyn];
                *++yyvsp = yylval;
                goto yyloop;
            }
            else
            {
#if YYDEBUG
                if (yydebug)
                    printf("%sdebug: error recovery discarding state %d\n",
                            YYPREFIX, *yyssp);
#endif
                if (yyssp <= yyss) goto yyabort;
                --yyssp;
                --yyvsp;
            }
        }
    }
    else
    {
        if (yychar == 0) goto yyabort;
#if YYDEBUG
        if (yydebug)
        {
            yys = 0;
            if (yychar <= YYMAXTOKEN) yys = yyname[yychar];
            if (!yys) yys = "illegal-symbol";
            printf("%sdebug: state %d, error recovery discards token %d (%s)\n",
                    YYPREFIX, yystate, yychar, yys);
        }
#endif
        yychar = (-1);
        goto yyloop;
    }
yyreduce:
#if YYDEBUG
    if (yydebug)
        printf("%sdebug: state %d, reducing by rule %d (%s)\n",
                YYPREFIX, yystate, yyn, yyrule[yyn]);
#endif
    yym = yylen[yyn];
    yyval = yyvsp[1-yym];
    switch (yyn)
    {
case 4:
{ dbmfFree(yyvsp[-1].Str); }
break;
case 5:
{ dbmfFree(yyvsp[-3].Str); dbmfFree(yyvsp[-1].Str); }
break;
case 14:
{
		DB_APP_NODE* an=(DB_APP_NODE*)malloc(sizeof(DB_APP_NODE*));

		if(subst_used)
		{
			int n;

			n = macExpandString(macHandle,yyvsp[-1].Str,subst_buffer,
			    VAR_MAX_SUB_SIZE-1);
			if(n<0) fprintf(stderr,"macExpandString failed\n");
#ifdef vxWorks
			an->name=strduplicate(subst_buffer);
			dbmfFree(yyvsp[-1].Str);
#else
			printf("\napplication(\"%s\")\n",subst_buffer);
#endif
		}
		else
		{
#ifdef vxWorks
			an->name=strduplicate(yyvsp[-1].Str);
			dbmfFree(yyvsp[-1].Str);
#else
			printf("\napplication(\"%s\")\n",yyvsp[-1].Str);
#endif
		}
		if(DbCurrentListHead==(DB_APP_NODE*)NULL) DbCurrentListTail=an;

		an->next=DbCurrentListHead;
		DbCurrentListHead=an;
	}
break;
case 17:
{
#ifndef vxWorks
		 printf("}\n");
#endif
	}
break;
case 18:
{
		sub_pvname(yyvsp[-3].Str,yyvsp[-1].Str);
		dbmfFree(yyvsp[-3].Str); dbmfFree(yyvsp[-1].Str);
	}
break;
case 19:
{
		sub_pvname(yyvsp[-3].Str,yyvsp[-1].Str);
		dbmfFree(yyvsp[-3].Str); dbmfFree(yyvsp[-1].Str);
	}
break;
case 24:
{
#ifdef vxWorks
		if( dbFindField(pdbentry,yyvsp[-3].Str) )
			fprintf(stderr,"Cannot find field %s\n",yyvsp[-3].Str);
#endif
		if(subst_used)
		{
			int n;

			n = macExpandString(macHandle,yyvsp[-1].Str,subst_buffer,
			    VAR_MAX_SUB_SIZE-1);
			if(n<0) fprintf(stderr,"macExpandString failed\n");
#ifdef vxWorks
			if( dbPutString(pdbentry, subst_buffer) )
				fprintf(stderr,"Cannot set field %s to %s\n",
					yyvsp[-3].Str,subst_buffer);
#else
			printf("\n\t\tfield(%s, \"%s\")",yyvsp[-3].Str,subst_buffer);
#endif
		}
		else
		{
#ifdef vxWorks
			if( dbPutString(pdbentry, yyvsp[-1].Str) )
				fprintf(stderr,"Cannot set field %s to %s\n",yyvsp[-3].Str,yyvsp[-1].Str);
#else
			printf("\n\t\tfield(%s, \"%s\")",yyvsp[-3].Str,yyvsp[-1].Str);
#endif
		}
		dbmfFree(yyvsp[-3].Str); dbmfFree(yyvsp[-1].Str);
	}
break;
    }
    yyssp -= yym;
    yystate = *yyssp;
    yyvsp -= yym;
    yym = yylhs[yyn];
    if (yystate == 0 && yym == 0)
    {
#if YYDEBUG
        if (yydebug)
            printf("%sdebug: after reduction, shifting from state 0 to\
 state %d\n", YYPREFIX, YYFINAL);
#endif
        yystate = YYFINAL;
        *++yyssp = YYFINAL;
        *++yyvsp = yyval;
        if (yychar < 0)
        {
            if ((yychar = yylex()) < 0) yychar = 0;
#if YYDEBUG
            if (yydebug)
            {
                yys = 0;
                if (yychar <= YYMAXTOKEN) yys = yyname[yychar];
                if (!yys) yys = "illegal-symbol";
                printf("%sdebug: state %d, reading %d (%s)\n",
                        YYPREFIX, YYFINAL, yychar, yys);
            }
#endif
        }
        if (yychar == 0) goto yyaccept;
        goto yyloop;
    }
    if ((yyn = yygindex[yym]) && (yyn += yystate) >= 0 &&
            yyn <= YYTABLESIZE && yycheck[yyn] == yystate)
        yystate = yytable[yyn];
    else
        yystate = yydgoto[yym];
#if YYDEBUG
    if (yydebug)
        printf("%sdebug: after reduction, shifting from state %d \
to state %d\n", YYPREFIX, *yyssp, yystate);
#endif
    if (yyssp >= yyss + yystacksize - 1)
    {
        goto yyoverflow;
    }
    *++yyssp = yystate;
    *++yyvsp = yyval;
    goto yyloop;
yyoverflow:
    yyerror("yacc stack overflow");
yyabort:
    return (1);
yyaccept:
    return (0);
}
