#define YYBYACC 1
#define YYMAJOR 1
#define YYMINOR 9
#define yyclearin (yychar=(-1))
#define yyerrok (yyerrflag=0)
#define YYRECOVERING (yyerrflag!=0)
#ifdef __STDC__
static int yyparse(void);
#else
static int yyparse();
#endif
#define YYPREFIX "yy"

/**************************************************************************
 *
 *     Author:	Jim Kowalkowski
 *
 *      Experimental Physics and Industrial Control System (EPICS)
 *
 *      Copyright 1991, the Regents of the University of California,
 *      and the University of Chicago Board of Governors.
 *
 *      This software was produced under  U.S. Government contracts:
 *      (W-7405-ENG-36) at the Los Alamos National Laboratory,
 *      and (W-31-109-ENG-38) at Argonne National Laboratory.
 *
 *      Initial development by:
 *              The Controls and Automation Group (AT-8)
 *              Ground Test Accelerator
 *              Accelerator Technology Division
 *              Los Alamos National Laboratory
 *
 *      Co-developed with
 *              The Controls and Computing Group
 *              Accelerator Systems Division
 *              Advanced Photon Source
 *              Argonne National Laboratory
 *
 * Modification Log:
 * -----------------
 * .01	10-29-93	jbk	initial version
 *
 ***********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <string.h>

#ifdef vxWorks
#include <symLib.h>
#include <sysSymTbl.h>
#endif

#include "macLib.h"
#include "dbmf.h"
#include "epicsVersion.h"
#ifdef _WIN32
#include "getopt.h"
#endif

#define MAXPATHLEN 300
static char db_path[MAXPATHLEN] = "";

static int line_num;
static int yyerror();
int dbLoadTemplate(char* sub_file, char* template_path);

int dbLoadRecords(char* pfilename, char* pattern, char* ppath);

#ifdef vxWorks
#define VAR_MAX_VAR_STRING 5000
#define VAR_MAX_VARS 100
#else
#define VAR_MAX_VAR_STRING 50000
#define VAR_MAX_VARS 700
#endif

static char *sub_collect = NULL;
static MAC_HANDLE *macHandle = NULL;
static char** vars = NULL;
static char* db_file_name = NULL;
static int var_count,sub_count;
static SYM_TYPE symTyp;
static char **symVal;

typedef union
{
    int	Int;
    char Char;
    char *Str;
    double Real;
} YYSTYPE;
#define WORD 257
#define QUOTE 258
#define DBFILE 259
#define PATTERN 260
#define EQUALS 261
#define O_PAREN 262
#define C_PAREN 263
#define O_BRACE 264
#define C_BRACE 265
#define YYERRCODE 256
static short yylhs[] = {                                        -1,
    0,    0,    1,    1,    3,    3,    4,    4,    2,    2,
    5,    8,    8,    9,    6,    6,   10,   10,   11,   11,
   12,   12,    7,    7,   13,   13,   14,   14,   15,   15,
};
static short yylen[] = {                                         2,
    1,    1,    2,    1,    4,    1,    2,    3,    3,    1,
    3,    2,    1,    1,    2,    1,    4,    3,    2,    1,
    1,    1,    2,    1,    4,    3,    2,    1,    3,    3,
};
static short yydefred[] = {                                      0,
    0,    0,    0,    0,    0,    0,    2,    4,    0,    0,
   24,    0,    0,    0,    0,    0,    0,   28,    3,    0,
   23,    0,    8,   14,    0,   13,    0,    0,    0,   16,
    0,   26,   27,    0,   25,   11,   12,    0,   22,   21,
    0,   20,   15,   29,   30,    5,    0,   18,   19,   17,
};
static short yydgoto[] = {                                       5,
    6,    7,    8,    9,   15,   29,   10,   25,   26,   30,
   41,   42,   11,   17,   18,
};
static short yysindex[] = {                                   -237,
 -264, -245, -234, -243,    0, -225,    0,    0, -223, -232,
    0, -243, -241, -215, -231, -217, -248,    0,    0, -236,
    0, -247,    0,    0, -246,    0, -221, -220, -231,    0,
 -218,    0,    0, -219,    0,    0,    0, -220,    0,    0,
 -252,    0,    0,    0,    0,    0, -250,    0,    0,    0,
};
static short yyrindex[] = {                                      0,
    0,    0,    0,    0,    0,   45,    0,    0,    3,    1,
    0,    0,    4,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    2,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
};
static short yygindex[] = {                                      0,
    0,   27,   42,    0,    0,    0,    0,    0,   24,   21,
   13,  -12,   43,   40,   14,
};
#define YYTABLESIZE 268
static short yytable[] = {                                      12,
   10,    9,    6,    7,   39,   40,   39,   40,   16,   16,
   24,   13,   48,   16,   50,   23,   32,   35,   36,    1,
    1,    2,    3,    3,    1,   27,    4,    4,   49,   14,
   33,    4,   28,    2,   49,   33,   39,   40,   44,   45,
   20,   24,   38,   31,    1,   46,   34,   19,   37,   43,
   47,   22,   21,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    6,    7,    0,    0,   10,    9,    7,
};
static short yycheck[] = {                                     264,
    0,    0,    0,    0,  257,  258,  257,  258,  257,  257,
  257,  257,  265,  257,  265,  257,  265,  265,  265,  257,
  257,  259,  260,  260,  257,  257,  264,  264,   41,  264,
   17,  264,  264,  259,   47,   22,  257,  258,  257,  258,
  264,  257,  264,  261,    0,  265,   20,    6,   25,   29,
   38,   12,   10,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  259,  259,   -1,   -1,  265,  265,  264,
};
#define YYFINAL 5
#ifndef YYDEBUG
#define YYDEBUG 0
#endif
#define YYMAXTOKEN 265
#if YYDEBUG
static char *yyname[] = {
"end-of-file",0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,"WORD","QUOTE","DBFILE","PATTERN",
"EQUALS","O_PAREN","C_PAREN","O_BRACE","C_BRACE",
};
static char *yyrule[] = {
"$accept : template",
"template : templs",
"template : subst",
"templs : templs templ",
"templs : templ",
"templ : templ_head O_BRACE subst C_BRACE",
"templ : templ_head",
"templ_head : DBFILE WORD",
"templ_head : DBFILE WORD WORD",
"subst : PATTERN pattern subs",
"subst : var_subs",
"pattern : O_BRACE vars C_BRACE",
"vars : vars var",
"vars : var",
"var : WORD",
"subs : subs sub",
"subs : sub",
"sub : WORD O_BRACE vals C_BRACE",
"sub : O_BRACE vals C_BRACE",
"vals : vals val",
"vals : val",
"val : QUOTE",
"val : WORD",
"var_subs : var_subs var_sub",
"var_subs : var_sub",
"var_sub : WORD O_BRACE sub_pats C_BRACE",
"var_sub : O_BRACE sub_pats C_BRACE",
"sub_pats : sub_pats sub_pat",
"sub_pats : sub_pat",
"sub_pat : WORD EQUALS WORD",
"sub_pat : WORD EQUALS QUOTE",
};
#endif
#ifdef YYSTACKSIZE
#undef YYMAXDEPTH
#define YYMAXDEPTH YYSTACKSIZE
#else
#ifdef YYMAXDEPTH
#define YYSTACKSIZE YYMAXDEPTH
#else
#define YYSTACKSIZE 500
#define YYMAXDEPTH 500
#endif
#endif
#if YYDEBUG
static int yydebug;
#endif
static int yynerrs;
static int yyerrflag;
static int yychar;
static short *yyssp;
static YYSTYPE *yyvsp;
static YYSTYPE yyval;
static YYSTYPE yylval;
static short yyss[YYSTACKSIZE];
static YYSTYPE yyvs[YYSTACKSIZE];
#define yystacksize YYSTACKSIZE
 
#include "dbLoadTemplate_lex.c"
 
static int yyerror(char* str)
{
	fprintf(stderr,"Substitution file parse error\n");
	fprintf(stderr,"line %d:%s\n",line_num,yytext);
	return(0);
}

static int is_not_inited = 1;
 
int dbLoadTemplate(char* sub_file, char* template_path)
{
	FILE *fp;
	int ind;

	line_num=0;

	if( !sub_file || !*sub_file)
	{
		fprintf(stderr,"must specify variable substitution file\n");
		return -1;
	}

	if( !(fp=fopen(sub_file,"r")) )
	{
		fprintf(stderr,"dbLoadTemplate: error opening sub file %s\n",sub_file);
		return -1;
	}

	if (template_path && strlen(template_path)) {
		strcpy(db_path, template_path);
	} else {
		db_path[0] = '\0';
	}

	vars = (char**)malloc(VAR_MAX_VARS * sizeof(char*));
	sub_collect = malloc(VAR_MAX_VAR_STRING);
	sub_collect[0]='\0';
	var_count=0;
	sub_count=0;

	if(is_not_inited)
	{
		yyin=fp;
		is_not_inited=0;
	}
	else
	{
		yyrestart(fp);
	}

	yyparse();
	for(ind=0;ind<var_count;ind++) dbmfFree(vars[ind]);
	free(vars);
	free(sub_collect);
	vars = NULL;
	fclose(fp);
	if(db_file_name){
	    dbmfFree((void *)db_file_name);
	    db_file_name = NULL;
	}
	return 0;
}

#ifndef vxWorks
/* this is template loader similar to vxWorks one for .db files */
int main(int argc, char** argv)
{
	extern char* optarg;
	extern int optind;
	char* name = (char*)NULL;
	char* path = (char*)NULL;
	int no_error = 1;
	int c;

	optind = 1;
	while(no_error && (c=getopt(argc,argv,"s:p:"))!=-1)
	{
		switch(c)
		{
		case 's':
			if(name) dbmfFree(name);
			name = dbmfMalloc(strlen(optarg));
			strcpy(name,optarg);
			break;
		case 'p':
			if (path) dbmfFree(path);
			path = dbmfMalloc(strlen(optarg));
			strcpy(path,optarg);
			break;
		default: no_error=0; break;
		}
	}

	if(!no_error || optind>=argc)
	{
		fprintf(stderr,"Usage: %s <-s name> <-p path> sub_file\n",argv[0]);
		fprintf(stderr,"\n\twhere name is the output database name,\n");
		fprintf(stderr,"\tsub_file is the variable substitution file,\n");
		fprintf(stderr,"\tand path is to be prepended to the file name\n");
		fprintf(stderr,"\tfound in that substitution file.\n");
		fprintf(stderr,"\n\tThis program uses the sub_file to produce a\n");
		fprintf(stderr,"\tdatabase of name name to standard out.\n");
		exit(1);
	}

	if(!name) {
	    name = dbmfMalloc(strlen("Composite") + 1);
	    strcpy(name,"Composite");
	}
	dbLoadTemplate(argv[optind], path);
	dbmfFree((void *)name);
	if (path) dbmfFree((void *)path);
	return(0);
}
#endif
#define YYABORT goto yyabort
#define YYREJECT goto yyabort
#define YYACCEPT goto yyaccept
#define YYERROR goto yyerrlab
#ifdef __STDC__
static int
yyparse(void)
#else
static int
yyparse()
#endif
{
    register int yym, yyn, yystate;
#if YYDEBUG
    register char *yys;
    extern char *getenv();

    if (yys = getenv("YYDEBUG"))
    {
        yyn = *yys;
        if (yyn >= '0' && yyn <= '9')
            yydebug = yyn - '0';
    }
#endif

    yynerrs = 0;
    yyerrflag = 0;
    yychar = (-1);

    yyssp = yyss;
    yyvsp = yyvs;
    *yyssp = yystate = 0;

yyloop:
    if (yyn = yydefred[yystate]) goto yyreduce;
    if (yychar < 0)
    {
        if ((yychar = yylex()) < 0) yychar = 0;
#if YYDEBUG
        if (yydebug)
        {
            yys = 0;
            if (yychar <= YYMAXTOKEN) yys = yyname[yychar];
            if (!yys) yys = "illegal-symbol";
            printf("%sdebug: state %d, reading %d (%s)\n",
                    YYPREFIX, yystate, yychar, yys);
        }
#endif
    }
    if ((yyn = yysindex[yystate]) && (yyn += yychar) >= 0 &&
            yyn <= YYTABLESIZE && yycheck[yyn] == yychar)
    {
#if YYDEBUG
        if (yydebug)
            printf("%sdebug: state %d, shifting to state %d\n",
                    YYPREFIX, yystate, yytable[yyn]);
#endif
        if (yyssp >= yyss + yystacksize - 1)
        {
            goto yyoverflow;
        }
        *++yyssp = yystate = yytable[yyn];
        *++yyvsp = yylval;
        yychar = (-1);
        if (yyerrflag > 0)  --yyerrflag;
        goto yyloop;
    }
    if ((yyn = yyrindex[yystate]) && (yyn += yychar) >= 0 &&
            yyn <= YYTABLESIZE && yycheck[yyn] == yychar)
    {
        yyn = yytable[yyn];
        goto yyreduce;
    }
    if (yyerrflag) goto yyinrecovery;
    yyerror("syntax error");
    ++yynerrs;
yyinrecovery:
    if (yyerrflag < 3)
    {
        yyerrflag = 3;
        for (;;)
        {
            if ((yyn = yysindex[*yyssp]) && (yyn += YYERRCODE) >= 0 &&
                    yyn <= YYTABLESIZE && yycheck[yyn] == YYERRCODE)
            {
#if YYDEBUG
                if (yydebug)
                    printf("%sdebug: state %d, error recovery shifting\
 to state %d\n", YYPREFIX, *yyssp, yytable[yyn]);
#endif
                if (yyssp >= yyss + yystacksize - 1)
                {
                    goto yyoverflow;
                }
                *++yyssp = yystate = yytable[yyn];
                *++yyvsp = yylval;
                goto yyloop;
            }
            else
            {
#if YYDEBUG
                if (yydebug)
                    printf("%sdebug: error recovery discarding state %d\n",
                            YYPREFIX, *yyssp);
#endif
                if (yyssp <= yyss) goto yyabort;
                --yyssp;
                --yyvsp;
            }
        }
    }
    else
    {
        if (yychar == 0) goto yyabort;
#if YYDEBUG
        if (yydebug)
        {
            yys = 0;
            if (yychar <= YYMAXTOKEN) yys = yyname[yychar];
            if (!yys) yys = "illegal-symbol";
            printf("%sdebug: state %d, error recovery discards token %d (%s)\n",
                    YYPREFIX, yystate, yychar, yys);
        }
#endif
        yychar = (-1);
        goto yyloop;
    }
yyreduce:
#if YYDEBUG
    if (yydebug)
        printf("%sdebug: state %d, reducing by rule %d (%s)\n",
                YYPREFIX, yystate, yyn, yyrule[yyn]);
#endif
    yym = yylen[yyn];
    yyval = yyvsp[1-yym];
    switch (yyn)
    {
case 6:
{
		if(db_file_name)
			dbLoadRecords(db_file_name,NULL,db_path);
		else
			fprintf(stderr,"Error: no db file name given\n");
	}
break;
case 7:
{
		var_count=0;
		if(db_file_name) dbmfFree(db_file_name);
		db_file_name = dbmfMalloc(strlen(yyvsp[0].Str)+1);
		strcpy(db_file_name,yyvsp[0].Str);
		dbmfFree(yyvsp[0].Str);
	}
break;
case 8:
{
#ifdef vxWorks
		var_count=0;
		if(db_file_name) dbmfFree(db_file_name);
		db_file_name = dbmfMalloc(strlen(yyvsp[-1].Str)+1);
		strcpy(db_file_name,yyvsp[-1].Str);
		dbmfFree(yyvsp[-1].Str);
                /* First assume that the string is a vxWorks symbol, if not then
                   assume it is a literal path */
                if (symFindByName(sysSymTbl,yyvsp[0].Str,(char **)&symVal,&symTyp)==OK)
                        strcpy(db_path, *symVal);
                else
                        strcpy(db_path,yyvsp[0].Str);
                dbmfFree(yyvsp[0].Str);
#endif
	}
break;
case 11:
{ 
#ifdef ERROR_STUFF
		int i;
		for(i=0;i<var_count;i++) fprintf(stderr,"variable=(%s)\n",vars[i]);
		fprintf(stderr,"var_count=%d\n",var_count);
#endif
	}
break;
case 14:
{
	    vars[var_count] = dbmfMalloc(strlen(yyvsp[0].Str)+1);
	    strcpy(vars[var_count],yyvsp[0].Str);
	    var_count++;
	    dbmfFree(yyvsp[0].Str);
	}
break;
case 17:
{
		sub_collect[strlen(sub_collect)-1]='\0';
#ifdef ERROR_STUFF
		fprintf(stderr,"dbLoadRecords(%s)\n",sub_collect);
#endif
		if(db_file_name)
			dbLoadRecords(db_file_name,sub_collect,db_path);
		else
			fprintf(stderr,"Error: no db file name given\n");
		dbmfFree(yyvsp[-3].Str);
		sub_collect[0]='\0';
		sub_count=0;
	}
break;
case 18:
{
		sub_collect[strlen(sub_collect)-1]='\0';
#ifdef ERROR_STUFF
		fprintf(stderr,"dbLoadRecords(%s)\n",sub_collect);
#endif
		if(db_file_name)
			dbLoadRecords(db_file_name,sub_collect,db_path);
		else
			fprintf(stderr,"Error: no db file name given\n");
		sub_collect[0]='\0';
		sub_count=0;
	}
break;
case 21:
{
		if(sub_count<=var_count)
		{
			strcat(sub_collect,vars[sub_count]);
			strcat(sub_collect,"=\"");
			strcat(sub_collect,yyvsp[0].Str);
			strcat(sub_collect,"\",");
			sub_count++;
		}
		dbmfFree(yyvsp[0].Str);
	}
break;
case 22:
{
		if(sub_count<=var_count)
		{
			strcat(sub_collect,vars[sub_count]);
			strcat(sub_collect,"=");
			strcat(sub_collect,yyvsp[0].Str);
			strcat(sub_collect,",");
			sub_count++;
		}
		dbmfFree(yyvsp[0].Str);
	}
break;
case 25:
{
		sub_collect[strlen(sub_collect)-1]='\0';
#ifdef ERROR_STUFF
		fprintf(stderr,"dbLoadRecords(%s)\n",sub_collect);
#endif
		if(db_file_name)
			dbLoadRecords(db_file_name,sub_collect,db_path);
		else
			fprintf(stderr,"Error: no db file name given\n");
		dbmfFree(yyvsp[-3].Str);
		sub_collect[0]='\0';
		sub_count=0;
	}
break;
case 26:
{
		sub_collect[strlen(sub_collect)-1]='\0';
#ifdef ERROR_STUFF
		fprintf(stderr,"dbLoadRecords(%s)\n",sub_collect);
#endif
		if(db_file_name)
			dbLoadRecords(db_file_name,sub_collect,db_path);
		else
			fprintf(stderr,"Error: no db file name given\n");
		sub_collect[0]='\0';
		sub_count=0;
	}
break;
case 29:
{
		strcat(sub_collect,yyvsp[-2].Str);
		strcat(sub_collect,"=");
		strcat(sub_collect,yyvsp[0].Str);
		strcat(sub_collect,",");
		dbmfFree(yyvsp[-2].Str); dbmfFree(yyvsp[0].Str);
		sub_count++;
	}
break;
case 30:
{
		strcat(sub_collect,yyvsp[-2].Str);
		strcat(sub_collect,"=\"");
		strcat(sub_collect,yyvsp[0].Str);
		strcat(sub_collect,"\",");
		dbmfFree(yyvsp[-2].Str); dbmfFree(yyvsp[0].Str);
		sub_count++;
	}
break;
    }
    yyssp -= yym;
    yystate = *yyssp;
    yyvsp -= yym;
    yym = yylhs[yyn];
    if (yystate == 0 && yym == 0)
    {
#if YYDEBUG
        if (yydebug)
            printf("%sdebug: after reduction, shifting from state 0 to\
 state %d\n", YYPREFIX, YYFINAL);
#endif
        yystate = YYFINAL;
        *++yyssp = YYFINAL;
        *++yyvsp = yyval;
        if (yychar < 0)
        {
            if ((yychar = yylex()) < 0) yychar = 0;
#if YYDEBUG
            if (yydebug)
            {
                yys = 0;
                if (yychar <= YYMAXTOKEN) yys = yyname[yychar];
                if (!yys) yys = "illegal-symbol";
                printf("%sdebug: state %d, reading %d (%s)\n",
                        YYPREFIX, YYFINAL, yychar, yys);
            }
#endif
        }
        if (yychar == 0) goto yyaccept;
        goto yyloop;
    }
    if ((yyn = yygindex[yym]) && (yyn += yystate) >= 0 &&
            yyn <= YYTABLESIZE && yycheck[yyn] == yystate)
        yystate = yytable[yyn];
    else
        yystate = yydgoto[yym];
#if YYDEBUG
    if (yydebug)
        printf("%sdebug: after reduction, shifting from state %d \
to state %d\n", YYPREFIX, *yyssp, yystate);
#endif
    if (yyssp >= yyss + yystacksize - 1)
    {
        goto yyoverflow;
    }
    *++yyssp = yystate;
    *++yyvsp = yyval;
    goto yyloop;
yyoverflow:
    yyerror("yacc stack overflow");
yyabort:
    return (1);
yyaccept:
    return (0);
}
