#include <vxWorks.h>
#include <semLib.h>
#include "ellLib.h"
#include "fast_lock.h"
#include "link.h"
#include "tsDefs.h"

#ifndef INCsscanPAUSH
#define INCsscanPAUSH
typedef enum {
	sscanPAUS_Go,
	sscanPAUS_Pause
}sscanPAUS;
#endif /*INCsscanPAUSH*/

#ifndef INCsscanPASMH
#define INCsscanPASMH
typedef enum {
	sscanPASM_Stay,
	sscanPASM_Start_Pos,
	sscanPASM_Prior_Pos,
	sscanPASM_Peak_Pos,
	sscanPASM_Valley_Pos,
	sscanPASM_RisingEdge_Pos,
	sscanPASM_FallingEdge_Pos
}sscanPASM;
#endif /*INCsscanPASMH*/

#ifndef INCsscanP1SMH
#define INCsscanP1SMH
typedef enum {
	sscanP1SM_Linear,
	sscanP1SM_Table,
	sscanP1SM_On_The_Fly
}sscanP1SM;
#endif /*INCsscanP1SMH*/

#ifndef INCsscanP1NVH
#define INCsscanP1NVH
typedef enum {
	sscanP1NV_PV_OK,
	sscanP1NV_No_PV,
	sscanP1NV_PV_NoRead,
	sscanP1NV_PV_xxx,
	sscanP1NV_PV_NoWrite,
	sscanP1NV_PV_yyy,
	sscanP1NV_PV_NC
}sscanP1NV;
#endif /*INCsscanP1NVH*/

#ifndef INCsscanP1ARH
#define INCsscanP1ARH
typedef enum {
	sscanP1AR_Absolute,
	sscanP1AR_Relative
}sscanP1AR;
#endif /*INCsscanP1ARH*/

#ifndef INCsscanFPTSH
#define INCsscanFPTSH
typedef enum {
	sscanFPTS_No,
	sscanFPTS_Freeze
}sscanFPTS;
#endif /*INCsscanFPTSH*/

#ifndef INCsscanFFOH
#define INCsscanFFOH
typedef enum {
	sscanFFO_Use_F_Flags,
	sscanFFO_Override
}sscanFFO;
#endif /*INCsscanFFOH*/

#ifndef INCsscanFAZEH
#define INCsscanFAZEH
typedef enum {
	sscanFAZE_IDLE,
	sscanFAZE_INIT_SCAN,
	sscanFAZE_BEFORE_SCAN,
	sscanFAZE_BEFORE_SCAN_WAIT,
	sscanFAZE_MOVE_MOTORS,
	sscanFAZE_CHECK_MOTORS,
	sscanFAZE_TRIG_DETCTRS,
	sscanFAZE_READ_DETCTRS,
	sscanFAZE_START_FLY,
	sscanFAZE_RETRACE_MOVE,
	sscanFAZE_RETRACE_WAIT,
	sscanFAZE_AFTER_SCAN_DO,
	sscanFAZE_AFTER_SCAN_WAIT,
	sscanFAZE_SCAN_DONE,
	sscanFAZE_PREVIEW,
	sscanFAZE_SCAN_PENDING
}sscanFAZE;
#endif /*INCsscanFAZEH*/

#ifndef INCsscanACQMH
#define INCsscanACQMH
typedef enum {
	sscanACQM_NORMAL,
	sscanACQM_ACC,
	sscanACQM_ADD,
	sscanACQM_ARRAY
}sscanACQM;
#endif /*INCsscanACQMH*/
#ifndef INCsscanH
#define INCsscanH
typedef struct sscanRecord {
	char		name[29]; /*Record Name*/
	char		desc[29]; /*Descriptor*/
	char		asg[29]; /*Access Security Group*/
	unsigned short	scan;	/*Scan Mechanism*/
	unsigned short	pini;	/*Process at iocInit*/
	short		phas;	/*Scan Phase*/
	short		evnt;	/*Event Number*/
	short		tse;	/*Time Stamp Event*/
	DBLINK		tsel;	/*Time Stamp Link*/
	unsigned short	dtyp;	/*Device Type*/
	short		disv;	/*Disable Value*/
	short		disa;	/*Disable*/
	DBLINK		sdis;	/*Scanning Disable*/
	FAST_LOCK	mlok;	/*Monitor fastlock*/
	ELLLIST		mlis;	/*Monitor List*/
	unsigned char	disp;	/*Disable putField*/
	unsigned char	proc;	/*Force Processing*/
	unsigned short	stat;	/*Alarm Status*/
	unsigned short	sevr;	/*Alarm Severity*/
	unsigned short	nsta;	/*New Alarm Status*/
	unsigned short	nsev;	/*New Alarm Severity*/
	unsigned short	acks;	/*Alarm Ack Severity*/
	unsigned short	ackt;	/*Alarm Ack Transient*/
	unsigned short	diss;	/*Disable Alarm Sevrty*/
	unsigned char	lcnt;	/*Lock Count*/
	unsigned char	pact;	/*Record active*/
	unsigned char	putf;	/*dbPutField process*/
	unsigned char	rpro;	/*Reprocess */
	void		*asp;	/*Access Security Pvt*/
	struct putNotify *ppn;	/*addr of PUTNOTIFY*/
	struct pnWaitNode *ppnn;	/*next record PUTNOTIFY*/
	struct scan_element *spvt;	/*Scan Private*/
	struct rset	*rset;	/*Address of RSET*/
	struct dset	*dset;	/*DSET address*/
	void		*dpvt;	/*Device Private*/
	struct dbRecordType *rdes;	/*Address of dbRecordType*/
	struct lockRecord *lset;	/*Lock Set*/
	unsigned short	prio;	/*Scheduling Priority*/
	unsigned char	tpro;	/*Trace Processing*/
	char bkpt;	/*Break Point*/
	unsigned char	udf;	/*Undefined*/
	TS_STAMP	time;	/*Time*/
	DBLINK		flnk;	/*Forward Process Link*/
	double		vers;	/*Code Version*/
	double		val;	/*Value Field*/
	char		smsg[40]; /*Record State Msg*/
	unsigned short	cmnd;	/*Command Field*/
	unsigned char	alrt;	/*Operator Alert*/
	void *           rpvt;	/*Ptr to Pvt Struct*/
	short		mpts;	/*Max # of Points*/
	short		exsc;	/*Execute Scan*/
	short		xsc;	/*Internal execScan*/
	unsigned char	pxsc;	/*Previous XScan*/
	unsigned char	busy;	/*Scan in progress*/
	unsigned char	kill;	/*Abort right now*/
	short		wait;	/*Wait for client(s)*/
	short		wcnt;	/*Wait count)*/
	short		awct;	/*Auto WCNT*/
	short		wtng;	/*Waiting for client(s)*/
	short		data;	/*Scan data ready*/
	short		refd;	/*Reference detector*/
	short		npts;	/*Number of Points*/
	unsigned short	fpts;	/*Freeze Num of Points*/
	unsigned short	ffo;	/*Freeze Flag Override*/
	short		cpt;	/*Current Point*/
	short		dpt;	/*Desired Point*/
	short		pcpt;	/*Point ofLast Posting*/
	unsigned short	pasm;	/*After Scan Mode*/
	unsigned long	tolp;	/*Time of Last Posting*/
	char		p1pv[40]; /*Positioner 1 PV Name*/
	char		p2pv[40]; /*Positioner 2 PV Name*/
	char		p3pv[40]; /*Positioner 3 PV Name*/
	char		p4pv[40]; /*Positioner 4 PV Name*/
	char		r1pv[40]; /*P1 Readback  PV Name*/
	char		r2pv[40]; /*P2 Readback  PV Name*/
	char		r3pv[40]; /*P3 Readback  PV Name*/
	char		r4pv[40]; /*P4 Readback  PV Name*/
	char		d01pv[40]; /*Detector PV Name*/
	char		d02pv[40]; /*Detector PV Name*/
	char		d03pv[40]; /*Detector PV Name*/
	char		d04pv[40]; /*Detector PV Name*/
	char		d05pv[40]; /*Detector PV Name*/
	char		d06pv[40]; /*Detector PV Name*/
	char		d07pv[40]; /*Detector PV Name*/
	char		d08pv[40]; /*Detector PV Name*/
	char		d09pv[40]; /*Detector PV Name*/
	char		d10pv[40]; /*Detector PV Name*/
	char		d11pv[40]; /*Detector PV Name*/
	char		d12pv[40]; /*Detector PV Name*/
	char		d13pv[40]; /*Detector PV Name*/
	char		d14pv[40]; /*Detector PV Name*/
	char		d15pv[40]; /*Detector PV Name*/
	char		d16pv[40]; /*Detector PV Name*/
	char		d17pv[40]; /*Detector PV Name*/
	char		d18pv[40]; /*Detector PV Name*/
	char		d19pv[40]; /*Detector PV Name*/
	char		d20pv[40]; /*Detector PV Name*/
	char		d21pv[40]; /*Detector PV Name*/
	char		d22pv[40]; /*Detector PV Name*/
	char		d23pv[40]; /*Detector PV Name*/
	char		d24pv[40]; /*Detector PV Name*/
	char		d25pv[40]; /*Detector PV Name*/
	char		d26pv[40]; /*Detector PV Name*/
	char		d27pv[40]; /*Detector PV Name*/
	char		d28pv[40]; /*Detector PV Name*/
	char		d29pv[40]; /*Detector PV Name*/
	char		d30pv[40]; /*Detector PV Name*/
	char		d31pv[40]; /*Detector PV Name*/
	char		d32pv[40]; /*Detector PV Name*/
	char		d33pv[40]; /*Detector PV Name*/
	char		d34pv[40]; /*Detector PV Name*/
	char		d35pv[40]; /*Detector PV Name*/
	char		d36pv[40]; /*Detector PV Name*/
	char		d37pv[40]; /*Detector PV Name*/
	char		d38pv[40]; /*Detector PV Name*/
	char		d39pv[40]; /*Detector PV Name*/
	char		d40pv[40]; /*Detector PV Name*/
	char		d41pv[40]; /*Detector PV Name*/
	char		d42pv[40]; /*Detector PV Name*/
	char		d43pv[40]; /*Detector PV Name*/
	char		d44pv[40]; /*Detector PV Name*/
	char		d45pv[40]; /*Detector PV Name*/
	char		d46pv[40]; /*Detector PV Name*/
	char		d47pv[40]; /*Detector PV Name*/
	char		d48pv[40]; /*Detector PV Name*/
	char		d49pv[40]; /*Detector PV Name*/
	char		d50pv[40]; /*Detector PV Name*/
	char		d51pv[40]; /*Detector PV Name*/
	char		d52pv[40]; /*Detector PV Name*/
	char		d53pv[40]; /*Detector PV Name*/
	char		d54pv[40]; /*Detector PV Name*/
	char		d55pv[40]; /*Detector PV Name*/
	char		d56pv[40]; /*Detector PV Name*/
	char		d57pv[40]; /*Detector PV Name*/
	char		d58pv[40]; /*Detector PV Name*/
	char		d59pv[40]; /*Detector PV Name*/
	char		d60pv[40]; /*Detector PV Name*/
	char		d61pv[40]; /*Detector PV Name*/
	char		d62pv[40]; /*Detector PV Name*/
	char		d63pv[40]; /*Detector PV Name*/
	char		d64pv[40]; /*Detector PV Name*/
	char		d65pv[40]; /*Detector PV Name*/
	char		d66pv[40]; /*Detector PV Name*/
	char		d67pv[40]; /*Detector PV Name*/
	char		d68pv[40]; /*Detector PV Name*/
	char		d69pv[40]; /*Detector PV Name*/
	char		d70pv[40]; /*Detector PV Name*/
	char		d1pv[40]; /*Detector PV Name*/
	char		d2pv[40]; /*Detector PV Name*/
	char		d3pv[40]; /*Detector PV Name*/
	char		d4pv[40]; /*Detector PV Name*/
	char		d5pv[40]; /*Detector PV Name*/
	char		d6pv[40]; /*Detector PV Name*/
	char		d7pv[40]; /*Detector PV Name*/
	char		d8pv[40]; /*Detector PV Name*/
	char		d9pv[40]; /*Detector PV Name*/
	char		dapv[40]; /*Detector PV Name*/
	char		dbpv[40]; /*Detector PV Name*/
	char		dcpv[40]; /*Detector PV Name*/
	char		ddpv[40]; /*Detector PV Name*/
	char		depv[40]; /*Detector PV Name*/
	char		dfpv[40]; /*Detector PV Name*/
	char		t1pv[40]; /*Trigger 1    PV Name*/
	char		t2pv[40]; /*Trigger 2    PV Name*/
	char		t3pv[40]; /*Trigger 3    PV Name*/
	char		t4pv[40]; /*Trigger 4    PV Name*/
	char		bspv[40]; /*Before Scan  PV Name*/
	char		aspv[40]; /*After Scan   PV Name*/
	unsigned short	p1nv;	/*P1  PV Status*/
	unsigned short	p2nv;	/*P2  PV Status*/
	unsigned short	p3nv;	/*P3  PV Status*/
	unsigned short	p4nv;	/*P4  PV Status*/
	unsigned short	r1nv;	/*R1  PV Status*/
	unsigned short	r2nv;	/*R2  PV Status*/
	unsigned short	r3nv;	/*R3  PV Status*/
	unsigned short	r4nv;	/*R4  PV Status*/
	unsigned short	d01nv;	/*PV Status*/
	unsigned short	d02nv;	/*PV Status*/
	unsigned short	d03nv;	/*PV Status*/
	unsigned short	d04nv;	/*PV Status*/
	unsigned short	d05nv;	/*PV Status*/
	unsigned short	d06nv;	/*PV Status*/
	unsigned short	d07nv;	/*PV Status*/
	unsigned short	d08nv;	/*PV Status*/
	unsigned short	d09nv;	/*PV Status*/
	unsigned short	d10nv;	/*PV Status*/
	unsigned short	d11nv;	/*PV Status*/
	unsigned short	d12nv;	/*PV Status*/
	unsigned short	d13nv;	/*PV Status*/
	unsigned short	d14nv;	/*PV Status*/
	unsigned short	d15nv;	/*PV Status*/
	unsigned short	d16nv;	/*PV Status*/
	unsigned short	d17nv;	/*PV Status*/
	unsigned short	d18nv;	/*PV Status*/
	unsigned short	d19nv;	/*PV Status*/
	unsigned short	d20nv;	/*PV Status*/
	unsigned short	d21nv;	/*PV Status*/
	unsigned short	d22nv;	/*PV Status*/
	unsigned short	d23nv;	/*PV Status*/
	unsigned short	d24nv;	/*PV Status*/
	unsigned short	d25nv;	/*PV Status*/
	unsigned short	d26nv;	/*PV Status*/
	unsigned short	d27nv;	/*PV Status*/
	unsigned short	d28nv;	/*PV Status*/
	unsigned short	d29nv;	/*PV Status*/
	unsigned short	d30nv;	/*PV Status*/
	unsigned short	d31nv;	/*PV Status*/
	unsigned short	d32nv;	/*PV Status*/
	unsigned short	d33nv;	/*PV Status*/
	unsigned short	d34nv;	/*PV Status*/
	unsigned short	d35nv;	/*PV Status*/
	unsigned short	d36nv;	/*PV Status*/
	unsigned short	d37nv;	/*PV Status*/
	unsigned short	d38nv;	/*PV Status*/
	unsigned short	d39nv;	/*PV Status*/
	unsigned short	d40nv;	/*PV Status*/
	unsigned short	d41nv;	/*PV Status*/
	unsigned short	d42nv;	/*PV Status*/
	unsigned short	d43nv;	/*PV Status*/
	unsigned short	d44nv;	/*PV Status*/
	unsigned short	d45nv;	/*PV Status*/
	unsigned short	d46nv;	/*PV Status*/
	unsigned short	d47nv;	/*PV Status*/
	unsigned short	d48nv;	/*PV Status*/
	unsigned short	d49nv;	/*PV Status*/
	unsigned short	d50nv;	/*PV Status*/
	unsigned short	d51nv;	/*PV Status*/
	unsigned short	d52nv;	/*PV Status*/
	unsigned short	d53nv;	/*PV Status*/
	unsigned short	d54nv;	/*PV Status*/
	unsigned short	d55nv;	/*PV Status*/
	unsigned short	d56nv;	/*PV Status*/
	unsigned short	d57nv;	/*PV Status*/
	unsigned short	d58nv;	/*PV Status*/
	unsigned short	d59nv;	/*PV Status*/
	unsigned short	d60nv;	/*PV Status*/
	unsigned short	d61nv;	/*PV Status*/
	unsigned short	d62nv;	/*PV Status*/
	unsigned short	d63nv;	/*PV Status*/
	unsigned short	d64nv;	/*PV Status*/
	unsigned short	d65nv;	/*PV Status*/
	unsigned short	d66nv;	/*PV Status*/
	unsigned short	d67nv;	/*PV Status*/
	unsigned short	d68nv;	/*PV Status*/
	unsigned short	d69nv;	/*PV Status*/
	unsigned short	d70nv;	/*PV Status*/
	unsigned short	d1nv;	/*PV Status*/
	unsigned short	d2nv;	/*PV Status*/
	unsigned short	d3nv;	/*PV Status*/
	unsigned short	d4nv;	/*PV Status*/
	unsigned short	d5nv;	/*PV Status*/
	unsigned short	d6nv;	/*PV Status*/
	unsigned short	d7nv;	/*PV Status*/
	unsigned short	d8nv;	/*PV Status*/
	unsigned short	d9nv;	/*PV Status*/
	unsigned short	danv;	/*PV Status*/
	unsigned short	dbnv;	/*PV Status*/
	unsigned short	dcnv;	/*PV Status*/
	unsigned short	ddnv;	/*PV Status*/
	unsigned short	denv;	/*PV Status*/
	unsigned short	dfnv;	/*PV Status*/
	unsigned short	t1nv;	/*T1  PV Status*/
	unsigned short	t2nv;	/*T2  PV Status*/
	unsigned short	t3nv;	/*T3  PV Status*/
	unsigned short	t4nv;	/*T4  PV Status*/
	unsigned short	bsnv;	/*BeforeScan PV Status*/
	unsigned short	asnv;	/*After Scan PV Status*/
	double		p1pp;	/*P1 Previous Position*/
	double		p1cv;	/*P1 Current Value*/
	double		p1dv;	/*P1 Desired Value*/
	double		p1lv;	/*P1 Last Value Posted*/
	double		p1sp;	/*P1 Start Position*/
	double		p1si;	/*P1 Step Increment*/
	double		p1ep;	/*P1 End Position*/
	double		p1cp;	/*P1 Center Position*/
	double		p1wd;	/*P1 Scan Width*/
	double		r1cv;	/*P1 Readback Value*/
	double		r1lv;	/*P1 Rdbk Last Val Pst*/
	double		r1dl;	/*P1 Readback Delta*/
	double		p1hr;	/*P1 High Oper Range*/
	double		p1lr;	/*P1 Low  Oper Range*/
	double *         p1pa;	/*P1 Step Array*/
	double *         p1ra;	/*P1 Readback Array*/
	unsigned short	p1fs;	/*P1 Freeze Start Pos*/
	unsigned short	p1fi;	/*P1 Freeze Step Inc*/
	unsigned short	p1fe;	/*P1 Freeze End Pos*/
	unsigned short	p1fc;	/*P1 Freeze Center Pos*/
	unsigned short	p1fw;	/*P1 Freeze Width*/
	unsigned short	p1sm;	/*P1 Step Mode*/
	unsigned short	p1ar;	/*P1 Absolute/Relative*/
	char		p1eu[16]; /*P1 Engineering Units*/
	short		p1pr;	/*P1 Display Precision*/
	double		p2pp;	/*P2 Previous Position*/
	double		p2cv;	/*P2 Current Value*/
	double		p2dv;	/*P2 Desired Value*/
	double		p2lv;	/*P2 Last Value Posted*/
	double		p2sp;	/*P2 Start Position*/
	double		p2si;	/*P2 Step Increment*/
	double		p2ep;	/*P2 End Position*/
	double		p2cp;	/*P2 Center Position*/
	double		p2wd;	/*P2 Scan Width*/
	double		r2cv;	/*P2 Readback Value*/
	double		r2lv;	/*P2 Rdbk Last Val Pst*/
	double		r2dl;	/*P2 Readback Delta*/
	double		p2hr;	/*P2 High Oper Range*/
	double		p2lr;	/*P2 Low  Oper Range*/
	double *         p2pa;	/*P2 Step Array*/
	double *         p2ra;	/*P2 Readback Array*/
	unsigned short	p2fs;	/*P2 Freeze Start Pos*/
	unsigned short	p2fi;	/*P2 Freeze Step Inc*/
	unsigned short	p2fe;	/*P2 Freeze End Pos*/
	unsigned short	p2fc;	/*P2 Freeze Center Pos*/
	unsigned short	p2fw;	/*P2 Freeze Width*/
	unsigned short	p2sm;	/*P2 Step Mode*/
	unsigned short	p2ar;	/*P2 Absolute/Relative*/
	char		p2eu[16]; /*P2 Engineering Units*/
	short		p2pr;	/*P2 Display Precision*/
	double		p3pp;	/*P3 Previous Position*/
	double		p3cv;	/*P3 Current Value*/
	double		p3dv;	/*P3 Desired Value*/
	double		p3lv;	/*P3 Last Value Posted*/
	double		p3sp;	/*P3 Start Position*/
	double		p3si;	/*P3 Step Increment*/
	double		p3ep;	/*P3 End Position*/
	double		p3cp;	/*P3 Center Position*/
	double		p3wd;	/*P3 Scan Width*/
	double		r3cv;	/*P3 Readback Value*/
	double		r3lv;	/*P3 Rdbk Last Val Pst*/
	double		r3dl;	/*P3 Readback Delta*/
	double		p3hr;	/*P3 High Oper Range*/
	double		p3lr;	/*P3 Low  Oper Range*/
	double *         p3pa;	/*P3 Step Array*/
	double *         p3ra;	/*P3 Readback Array*/
	unsigned short	p3fs;	/*P3 Freeze Start Pos*/
	unsigned short	p3fi;	/*P3 Freeze Step Inc*/
	unsigned short	p3fe;	/*P3 Freeze End Pos*/
	unsigned short	p3fc;	/*P3 Freeze Center Pos*/
	unsigned short	p3fw;	/*P3 Freeze Width*/
	unsigned short	p3sm;	/*P3 Step Mode*/
	unsigned short	p3ar;	/*P3 Absolute/Relative*/
	char		p3eu[16]; /*P3 Engineering Units*/
	short		p3pr;	/*P3 Display Precision*/
	double		p4pp;	/*P4 Previous Position*/
	double		p4cv;	/*P4 Current Value*/
	double		p4dv;	/*P4 Desired Value*/
	double		p4lv;	/*P4 Last Value Posted*/
	double		p4sp;	/*P4 Start Position*/
	double		p4si;	/*P4 Step Increment*/
	double		p4ep;	/*P4 End Position*/
	double		p4cp;	/*P4 Center Position*/
	double		p4wd;	/*P4 Scan Width*/
	double		r4cv;	/*P4 Readback Value*/
	double		r4lv;	/*P4 Rdbk Last Val Pst*/
	double		r4dl;	/*P4 Readback Delta*/
	double		p4hr;	/*P4 High Oper Range*/
	double		p4lr;	/*P4 Low  Oper Range*/
	double *         p4pa;	/*P4 Step Array*/
	double *         p4ra;	/*P4 Readback Array*/
	unsigned short	p4fs;	/*P4 Freeze Start Pos*/
	unsigned short	p4fi;	/*P4 Freeze Step Inc*/
	unsigned short	p4fe;	/*P4 Freeze End Pos*/
	unsigned short	p4fc;	/*P4 Freeze Center Pos*/
	unsigned short	p4fw;	/*P4 Freeze Width*/
	unsigned short	p4sm;	/*P4 Step Mode*/
	unsigned short	p4ar;	/*P4 Absolute/Relative*/
	char		p4eu[16]; /*P4 Engineering Units*/
	short		p4pr;	/*P4 Display Precision*/
	double		d01hr;	/*High Oper Range*/
	double		d01lr;	/*Low  Oper Range*/
	float *          d01da;	/*Data Array*/
	float		d01cv;	/*Current Value*/
	float		d01lv;	/*Last Value Posted*/
	unsigned long	d01ne;	/*# of Elements/Pt*/
	char		d01eu[16]; /*Engineering Units*/
	short		d01pr;	/*Display Precision*/
	double		d02hr;	/*High Oper Range*/
	double		d02lr;	/*Low  Oper Range*/
	float *          d02da;	/*Data Array*/
	float		d02cv;	/*Current Value*/
	float		d02lv;	/*Last Value Posted*/
	unsigned long	d02ne;	/*# of Elements/Pt*/
	char		d02eu[16]; /*Engineering Units*/
	short		d02pr;	/*Display Precision*/
	double		d03hr;	/*High Oper Range*/
	double		d03lr;	/*Low  Oper Range*/
	float *          d03da;	/*Data Array*/
	float		d03cv;	/*Current Value*/
	float		d03lv;	/*Last Value Posted*/
	unsigned long	d03ne;	/*# of Elements/Pt*/
	char		d03eu[16]; /*Engineering Units*/
	short		d03pr;	/*Display Precision*/
	double		d04hr;	/*High Oper Range*/
	double		d04lr;	/*Low  Oper Range*/
	float *          d04da;	/*Data Array*/
	float		d04cv;	/*Current Value*/
	float		d04lv;	/*Last Value Posted*/
	unsigned long	d04ne;	/*# of Elements/Pt*/
	char		d04eu[16]; /*Engineering Units*/
	short		d04pr;	/*Display Precision*/
	double		d05hr;	/*High Oper Range*/
	double		d05lr;	/*Low  Oper Range*/
	float *          d05da;	/*Data Array*/
	float		d05cv;	/*Current Value*/
	float		d05lv;	/*Last Value Posted*/
	unsigned long	d05ne;	/*# of Elements/Pt*/
	char		d05eu[16]; /*Engineering Units*/
	short		d05pr;	/*Display Precision*/
	double		d06hr;	/*High Oper Range*/
	double		d06lr;	/*Low  Oper Range*/
	float *          d06da;	/*Data Array*/
	float		d06cv;	/*Current Value*/
	float		d06lv;	/*Last Value Posted*/
	unsigned long	d06ne;	/*# of Elements/Pt*/
	char		d06eu[16]; /*Engineering Units*/
	short		d06pr;	/*Display Precision*/
	double		d07hr;	/*High Oper Range*/
	double		d07lr;	/*Low  Oper Range*/
	float *          d07da;	/*Data Array*/
	float		d07cv;	/*Current Value*/
	float		d07lv;	/*Last Value Posted*/
	unsigned long	d07ne;	/*# of Elements/Pt*/
	char		d07eu[16]; /*Engineering Units*/
	short		d07pr;	/*Display Precision*/
	double		d08hr;	/*High Oper Range*/
	double		d08lr;	/*Low  Oper Range*/
	float *          d08da;	/*Data Array*/
	float		d08cv;	/*Current Value*/
	float		d08lv;	/*Last Value Posted*/
	unsigned long	d08ne;	/*# of Elements/Pt*/
	char		d08eu[16]; /*Engineering Units*/
	short		d08pr;	/*Display Precision*/
	double		d09hr;	/*High Oper Range*/
	double		d09lr;	/*Low  Oper Range*/
	float *          d09da;	/*Data Array*/
	float		d09cv;	/*Current Value*/
	float		d09lv;	/*Last Value Posted*/
	unsigned long	d09ne;	/*# of Elements/Pt*/
	char		d09eu[16]; /*Engineering Units*/
	short		d09pr;	/*Display Precision*/
	double		d10hr;	/*High Oper Range*/
	double		d10lr;	/*Low  Oper Range*/
	float *          d10da;	/*Data Array*/
	float		d10cv;	/*Current Value*/
	float		d10lv;	/*Last Value Posted*/
	unsigned long	d10ne;	/*# of Elements/Pt*/
	char		d10eu[16]; /*Engineering Units*/
	short		d10pr;	/*Display Precision*/
	double		d11hr;	/*High Oper Range*/
	double		d11lr;	/*Low  Oper Range*/
	float *          d11da;	/*Data Array*/
	float		d11cv;	/*Current Value*/
	float		d11lv;	/*Last Value Posted*/
	unsigned long	d11ne;	/*# of Elements/Pt*/
	char		d11eu[16]; /*Engineering Units*/
	short		d11pr;	/*Display Precision*/
	double		d12hr;	/*High Oper Range*/
	double		d12lr;	/*Low  Oper Range*/
	float *          d12da;	/*Data Array*/
	float		d12cv;	/*Current Value*/
	float		d12lv;	/*Last Value Posted*/
	unsigned long	d12ne;	/*# of Elements/Pt*/
	char		d12eu[16]; /*Engineering Units*/
	short		d12pr;	/*Display Precision*/
	double		d13hr;	/*High Oper Range*/
	double		d13lr;	/*Low  Oper Range*/
	float *          d13da;	/*Data Array*/
	float		d13cv;	/*Current Value*/
	float		d13lv;	/*Last Value Posted*/
	unsigned long	d13ne;	/*# of Elements/Pt*/
	char		d13eu[16]; /*Engineering Units*/
	short		d13pr;	/*Display Precision*/
	double		d14hr;	/*High Oper Range*/
	double		d14lr;	/*Low  Oper Range*/
	float *          d14da;	/*Data Array*/
	float		d14cv;	/*Current Value*/
	float		d14lv;	/*Last Value Posted*/
	unsigned long	d14ne;	/*# of Elements/Pt*/
	char		d14eu[16]; /*Engineering Units*/
	short		d14pr;	/*Display Precision*/
	double		d15hr;	/*High Oper Range*/
	double		d15lr;	/*Low  Oper Range*/
	float *          d15da;	/*Data Array*/
	float		d15cv;	/*Current Value*/
	float		d15lv;	/*Last Value Posted*/
	unsigned long	d15ne;	/*# of Elements/Pt*/
	char		d15eu[16]; /*Engineering Units*/
	short		d15pr;	/*Display Precision*/
	double		d16hr;	/*High Oper Range*/
	double		d16lr;	/*Low  Oper Range*/
	float *          d16da;	/*Data Array*/
	float		d16cv;	/*Current Value*/
	float		d16lv;	/*Last Value Posted*/
	unsigned long	d16ne;	/*# of Elements/Pt*/
	char		d16eu[16]; /*Engineering Units*/
	short		d16pr;	/*Display Precision*/
	double		d17hr;	/*High Oper Range*/
	double		d17lr;	/*Low  Oper Range*/
	float *          d17da;	/*Data Array*/
	float		d17cv;	/*Current Value*/
	float		d17lv;	/*Last Value Posted*/
	unsigned long	d17ne;	/*# of Elements/Pt*/
	char		d17eu[16]; /*Engineering Units*/
	short		d17pr;	/*Display Precision*/
	double		d18hr;	/*High Oper Range*/
	double		d18lr;	/*Low  Oper Range*/
	float *          d18da;	/*Data Array*/
	float		d18cv;	/*Current Value*/
	float		d18lv;	/*Last Value Posted*/
	unsigned long	d18ne;	/*# of Elements/Pt*/
	char		d18eu[16]; /*Engineering Units*/
	short		d18pr;	/*Display Precision*/
	double		d19hr;	/*High Oper Range*/
	double		d19lr;	/*Low  Oper Range*/
	float *          d19da;	/*Data Array*/
	float		d19cv;	/*Current Value*/
	float		d19lv;	/*Last Value Posted*/
	unsigned long	d19ne;	/*# of Elements/Pt*/
	char		d19eu[16]; /*Engineering Units*/
	short		d19pr;	/*Display Precision*/
	double		d20hr;	/*High Oper Range*/
	double		d20lr;	/*Low  Oper Range*/
	float *          d20da;	/*Data Array*/
	float		d20cv;	/*Current Value*/
	float		d20lv;	/*Last Value Posted*/
	unsigned long	d20ne;	/*# of Elements/Pt*/
	char		d20eu[16]; /*Engineering Units*/
	short		d20pr;	/*Display Precision*/
	double		d21hr;	/*High Oper Range*/
	double		d21lr;	/*Low  Oper Range*/
	float *          d21da;	/*Data Array*/
	float		d21cv;	/*Current Value*/
	float		d21lv;	/*Last Value Posted*/
	unsigned long	d21ne;	/*# of Elements/Pt*/
	char		d21eu[16]; /*Engineering Units*/
	short		d21pr;	/*Display Precision*/
	double		d22hr;	/*High Oper Range*/
	double		d22lr;	/*Low  Oper Range*/
	float *          d22da;	/*Data Array*/
	float		d22cv;	/*Current Value*/
	float		d22lv;	/*Last Value Posted*/
	unsigned long	d22ne;	/*# of Elements/Pt*/
	char		d22eu[16]; /*Engineering Units*/
	short		d22pr;	/*Display Precision*/
	double		d23hr;	/*High Oper Range*/
	double		d23lr;	/*Low  Oper Range*/
	float *          d23da;	/*Data Array*/
	float		d23cv;	/*Current Value*/
	float		d23lv;	/*Last Value Posted*/
	unsigned long	d23ne;	/*# of Elements/Pt*/
	char		d23eu[16]; /*Engineering Units*/
	short		d23pr;	/*Display Precision*/
	double		d24hr;	/*High Oper Range*/
	double		d24lr;	/*Low  Oper Range*/
	float *          d24da;	/*Data Array*/
	float		d24cv;	/*Current Value*/
	float		d24lv;	/*Last Value Posted*/
	unsigned long	d24ne;	/*# of Elements/Pt*/
	char		d24eu[16]; /*Engineering Units*/
	short		d24pr;	/*Display Precision*/
	double		d25hr;	/*High Oper Range*/
	double		d25lr;	/*Low  Oper Range*/
	float *          d25da;	/*Data Array*/
	float		d25cv;	/*Current Value*/
	float		d25lv;	/*Last Value Posted*/
	unsigned long	d25ne;	/*# of Elements/Pt*/
	char		d25eu[16]; /*Engineering Units*/
	short		d25pr;	/*Display Precision*/
	double		d26hr;	/*High Oper Range*/
	double		d26lr;	/*Low  Oper Range*/
	float *          d26da;	/*Data Array*/
	float		d26cv;	/*Current Value*/
	float		d26lv;	/*Last Value Posted*/
	unsigned long	d26ne;	/*# of Elements/Pt*/
	char		d26eu[16]; /*Engineering Units*/
	short		d26pr;	/*Display Precision*/
	double		d27hr;	/*High Oper Range*/
	double		d27lr;	/*Low  Oper Range*/
	float *          d27da;	/*Data Array*/
	float		d27cv;	/*Current Value*/
	float		d27lv;	/*Last Value Posted*/
	unsigned long	d27ne;	/*# of Elements/Pt*/
	char		d27eu[16]; /*Engineering Units*/
	short		d27pr;	/*Display Precision*/
	double		d28hr;	/*High Oper Range*/
	double		d28lr;	/*Low  Oper Range*/
	float *          d28da;	/*Data Array*/
	float		d28cv;	/*Current Value*/
	float		d28lv;	/*Last Value Posted*/
	unsigned long	d28ne;	/*# of Elements/Pt*/
	char		d28eu[16]; /*Engineering Units*/
	short		d28pr;	/*Display Precision*/
	double		d29hr;	/*High Oper Range*/
	double		d29lr;	/*Low  Oper Range*/
	float *          d29da;	/*Data Array*/
	float		d29cv;	/*Current Value*/
	float		d29lv;	/*Last Value Posted*/
	unsigned long	d29ne;	/*# of Elements/Pt*/
	char		d29eu[16]; /*Engineering Units*/
	short		d29pr;	/*Display Precision*/
	double		d30hr;	/*High Oper Range*/
	double		d30lr;	/*Low  Oper Range*/
	float *          d30da;	/*Data Array*/
	float		d30cv;	/*Current Value*/
	float		d30lv;	/*Last Value Posted*/
	unsigned long	d30ne;	/*# of Elements/Pt*/
	char		d30eu[16]; /*Engineering Units*/
	short		d30pr;	/*Display Precision*/
	double		d31hr;	/*High Oper Range*/
	double		d31lr;	/*Low  Oper Range*/
	float *          d31da;	/*Data Array*/
	float		d31cv;	/*Current Value*/
	float		d31lv;	/*Last Value Posted*/
	unsigned long	d31ne;	/*# of Elements/Pt*/
	char		d31eu[16]; /*Engineering Units*/
	short		d31pr;	/*Display Precision*/
	double		d32hr;	/*High Oper Range*/
	double		d32lr;	/*Low  Oper Range*/
	float *          d32da;	/*Data Array*/
	float		d32cv;	/*Current Value*/
	float		d32lv;	/*Last Value Posted*/
	unsigned long	d32ne;	/*# of Elements/Pt*/
	char		d32eu[16]; /*Engineering Units*/
	short		d32pr;	/*Display Precision*/
	double		d33hr;	/*High Oper Range*/
	double		d33lr;	/*Low  Oper Range*/
	float *          d33da;	/*Data Array*/
	float		d33cv;	/*Current Value*/
	float		d33lv;	/*Last Value Posted*/
	unsigned long	d33ne;	/*# of Elements/Pt*/
	char		d33eu[16]; /*Engineering Units*/
	short		d33pr;	/*Display Precision*/
	double		d34hr;	/*High Oper Range*/
	double		d34lr;	/*Low  Oper Range*/
	float *          d34da;	/*Data Array*/
	float		d34cv;	/*Current Value*/
	float		d34lv;	/*Last Value Posted*/
	unsigned long	d34ne;	/*# of Elements/Pt*/
	char		d34eu[16]; /*Engineering Units*/
	short		d34pr;	/*Display Precision*/
	double		d35hr;	/*High Oper Range*/
	double		d35lr;	/*Low  Oper Range*/
	float *          d35da;	/*Data Array*/
	float		d35cv;	/*Current Value*/
	float		d35lv;	/*Last Value Posted*/
	unsigned long	d35ne;	/*# of Elements/Pt*/
	char		d35eu[16]; /*Engineering Units*/
	short		d35pr;	/*Display Precision*/
	double		d36hr;	/*High Oper Range*/
	double		d36lr;	/*Low  Oper Range*/
	float *          d36da;	/*Data Array*/
	float		d36cv;	/*Current Value*/
	float		d36lv;	/*Last Value Posted*/
	unsigned long	d36ne;	/*# of Elements/Pt*/
	char		d36eu[16]; /*Engineering Units*/
	short		d36pr;	/*Display Precision*/
	double		d37hr;	/*High Oper Range*/
	double		d37lr;	/*Low  Oper Range*/
	float *          d37da;	/*Data Array*/
	float		d37cv;	/*Current Value*/
	float		d37lv;	/*Last Value Posted*/
	unsigned long	d37ne;	/*# of Elements/Pt*/
	char		d37eu[16]; /*Engineering Units*/
	short		d37pr;	/*Display Precision*/
	double		d38hr;	/*High Oper Range*/
	double		d38lr;	/*Low  Oper Range*/
	float *          d38da;	/*Data Array*/
	float		d38cv;	/*Current Value*/
	float		d38lv;	/*Last Value Posted*/
	unsigned long	d38ne;	/*# of Elements/Pt*/
	char		d38eu[16]; /*Engineering Units*/
	short		d38pr;	/*Display Precision*/
	double		d39hr;	/*High Oper Range*/
	double		d39lr;	/*Low  Oper Range*/
	float *          d39da;	/*Data Array*/
	float		d39cv;	/*Current Value*/
	float		d39lv;	/*Last Value Posted*/
	unsigned long	d39ne;	/*# of Elements/Pt*/
	char		d39eu[16]; /*Engineering Units*/
	short		d39pr;	/*Display Precision*/
	double		d40hr;	/*High Oper Range*/
	double		d40lr;	/*Low  Oper Range*/
	float *          d40da;	/*Data Array*/
	float		d40cv;	/*Current Value*/
	float		d40lv;	/*Last Value Posted*/
	unsigned long	d40ne;	/*# of Elements/Pt*/
	char		d40eu[16]; /*Engineering Units*/
	short		d40pr;	/*Display Precision*/
	double		d41hr;	/*High Oper Range*/
	double		d41lr;	/*Low  Oper Range*/
	float *          d41da;	/*Data Array*/
	float		d41cv;	/*Current Value*/
	float		d41lv;	/*Last Value Posted*/
	unsigned long	d41ne;	/*# of Elements/Pt*/
	char		d41eu[16]; /*Engineering Units*/
	short		d41pr;	/*Display Precision*/
	double		d42hr;	/*High Oper Range*/
	double		d42lr;	/*Low  Oper Range*/
	float *          d42da;	/*Data Array*/
	float		d42cv;	/*Current Value*/
	float		d42lv;	/*Last Value Posted*/
	unsigned long	d42ne;	/*# of Elements/Pt*/
	char		d42eu[16]; /*Engineering Units*/
	short		d42pr;	/*Display Precision*/
	double		d43hr;	/*High Oper Range*/
	double		d43lr;	/*Low  Oper Range*/
	float *          d43da;	/*Data Array*/
	float		d43cv;	/*Current Value*/
	float		d43lv;	/*Last Value Posted*/
	unsigned long	d43ne;	/*# of Elements/Pt*/
	char		d43eu[16]; /*Engineering Units*/
	short		d43pr;	/*Display Precision*/
	double		d44hr;	/*High Oper Range*/
	double		d44lr;	/*Low  Oper Range*/
	float *          d44da;	/*Data Array*/
	float		d44cv;	/*Current Value*/
	float		d44lv;	/*Last Value Posted*/
	unsigned long	d44ne;	/*# of Elements/Pt*/
	char		d44eu[16]; /*Engineering Units*/
	short		d44pr;	/*Display Precision*/
	double		d45hr;	/*High Oper Range*/
	double		d45lr;	/*Low  Oper Range*/
	float *          d45da;	/*Data Array*/
	float		d45cv;	/*Current Value*/
	float		d45lv;	/*Last Value Posted*/
	unsigned long	d45ne;	/*# of Elements/Pt*/
	char		d45eu[16]; /*Engineering Units*/
	short		d45pr;	/*Display Precision*/
	double		d46hr;	/*High Oper Range*/
	double		d46lr;	/*Low  Oper Range*/
	float *          d46da;	/*Data Array*/
	float		d46cv;	/*Current Value*/
	float		d46lv;	/*Last Value Posted*/
	unsigned long	d46ne;	/*# of Elements/Pt*/
	char		d46eu[16]; /*Engineering Units*/
	short		d46pr;	/*Display Precision*/
	double		d47hr;	/*High Oper Range*/
	double		d47lr;	/*Low  Oper Range*/
	float *          d47da;	/*Data Array*/
	float		d47cv;	/*Current Value*/
	float		d47lv;	/*Last Value Posted*/
	unsigned long	d47ne;	/*# of Elements/Pt*/
	char		d47eu[16]; /*Engineering Units*/
	short		d47pr;	/*Display Precision*/
	double		d48hr;	/*High Oper Range*/
	double		d48lr;	/*Low  Oper Range*/
	float *          d48da;	/*Data Array*/
	float		d48cv;	/*Current Value*/
	float		d48lv;	/*Last Value Posted*/
	unsigned long	d48ne;	/*# of Elements/Pt*/
	char		d48eu[16]; /*Engineering Units*/
	short		d48pr;	/*Display Precision*/
	double		d49hr;	/*High Oper Range*/
	double		d49lr;	/*Low  Oper Range*/
	float *          d49da;	/*Data Array*/
	float		d49cv;	/*Current Value*/
	float		d49lv;	/*Last Value Posted*/
	unsigned long	d49ne;	/*# of Elements/Pt*/
	char		d49eu[16]; /*Engineering Units*/
	short		d49pr;	/*Display Precision*/
	double		d50hr;	/*High Oper Range*/
	double		d50lr;	/*Low  Oper Range*/
	float *          d50da;	/*Data Array*/
	float		d50cv;	/*Current Value*/
	float		d50lv;	/*Last Value Posted*/
	unsigned long	d50ne;	/*# of Elements/Pt*/
	char		d50eu[16]; /*Engineering Units*/
	short		d50pr;	/*Display Precision*/
	double		d51hr;	/*High Oper Range*/
	double		d51lr;	/*Low  Oper Range*/
	float *          d51da;	/*Data Array*/
	float		d51cv;	/*Current Value*/
	float		d51lv;	/*Last Value Posted*/
	unsigned long	d51ne;	/*# of Elements/Pt*/
	char		d51eu[16]; /*Engineering Units*/
	short		d51pr;	/*Display Precision*/
	double		d52hr;	/*High Oper Range*/
	double		d52lr;	/*Low  Oper Range*/
	float *          d52da;	/*Data Array*/
	float		d52cv;	/*Current Value*/
	float		d52lv;	/*Last Value Posted*/
	unsigned long	d52ne;	/*# of Elements/Pt*/
	char		d52eu[16]; /*Engineering Units*/
	short		d52pr;	/*Display Precision*/
	double		d53hr;	/*High Oper Range*/
	double		d53lr;	/*Low  Oper Range*/
	float *          d53da;	/*Data Array*/
	float		d53cv;	/*Current Value*/
	float		d53lv;	/*Last Value Posted*/
	unsigned long	d53ne;	/*# of Elements/Pt*/
	char		d53eu[16]; /*Engineering Units*/
	short		d53pr;	/*Display Precision*/
	double		d54hr;	/*High Oper Range*/
	double		d54lr;	/*Low  Oper Range*/
	float *          d54da;	/*Data Array*/
	float		d54cv;	/*Current Value*/
	float		d54lv;	/*Last Value Posted*/
	unsigned long	d54ne;	/*# of Elements/Pt*/
	char		d54eu[16]; /*Engineering Units*/
	short		d54pr;	/*Display Precision*/
	double		d55hr;	/*High Oper Range*/
	double		d55lr;	/*Low  Oper Range*/
	float *          d55da;	/*Data Array*/
	float		d55cv;	/*Current Value*/
	float		d55lv;	/*Last Value Posted*/
	unsigned long	d55ne;	/*# of Elements/Pt*/
	char		d55eu[16]; /*Engineering Units*/
	short		d55pr;	/*Display Precision*/
	double		d56hr;	/*High Oper Range*/
	double		d56lr;	/*Low  Oper Range*/
	float *          d56da;	/*Data Array*/
	float		d56cv;	/*Current Value*/
	float		d56lv;	/*Last Value Posted*/
	unsigned long	d56ne;	/*# of Elements/Pt*/
	char		d56eu[16]; /*Engineering Units*/
	short		d56pr;	/*Display Precision*/
	double		d57hr;	/*High Oper Range*/
	double		d57lr;	/*Low  Oper Range*/
	float *          d57da;	/*Data Array*/
	float		d57cv;	/*Current Value*/
	float		d57lv;	/*Last Value Posted*/
	unsigned long	d57ne;	/*# of Elements/Pt*/
	char		d57eu[16]; /*Engineering Units*/
	short		d57pr;	/*Display Precision*/
	double		d58hr;	/*High Oper Range*/
	double		d58lr;	/*Low  Oper Range*/
	float *          d58da;	/*Data Array*/
	float		d58cv;	/*Current Value*/
	float		d58lv;	/*Last Value Posted*/
	unsigned long	d58ne;	/*# of Elements/Pt*/
	char		d58eu[16]; /*Engineering Units*/
	short		d58pr;	/*Display Precision*/
	double		d59hr;	/*High Oper Range*/
	double		d59lr;	/*Low  Oper Range*/
	float *          d59da;	/*Data Array*/
	float		d59cv;	/*Current Value*/
	float		d59lv;	/*Last Value Posted*/
	unsigned long	d59ne;	/*# of Elements/Pt*/
	char		d59eu[16]; /*Engineering Units*/
	short		d59pr;	/*Display Precision*/
	double		d60hr;	/*High Oper Range*/
	double		d60lr;	/*Low  Oper Range*/
	float *          d60da;	/*Data Array*/
	float		d60cv;	/*Current Value*/
	float		d60lv;	/*Last Value Posted*/
	unsigned long	d60ne;	/*# of Elements/Pt*/
	char		d60eu[16]; /*Engineering Units*/
	short		d60pr;	/*Display Precision*/
	double		d61hr;	/*High Oper Range*/
	double		d61lr;	/*Low  Oper Range*/
	float *          d61da;	/*Data Array*/
	float		d61cv;	/*Current Value*/
	float		d61lv;	/*Last Value Posted*/
	unsigned long	d61ne;	/*# of Elements/Pt*/
	char		d61eu[16]; /*Engineering Units*/
	short		d61pr;	/*Display Precision*/
	double		d62hr;	/*High Oper Range*/
	double		d62lr;	/*Low  Oper Range*/
	float *          d62da;	/*Data Array*/
	float		d62cv;	/*Current Value*/
	float		d62lv;	/*Last Value Posted*/
	unsigned long	d62ne;	/*# of Elements/Pt*/
	char		d62eu[16]; /*Engineering Units*/
	short		d62pr;	/*Display Precision*/
	double		d63hr;	/*High Oper Range*/
	double		d63lr;	/*Low  Oper Range*/
	float *          d63da;	/*Data Array*/
	float		d63cv;	/*Current Value*/
	float		d63lv;	/*Last Value Posted*/
	unsigned long	d63ne;	/*# of Elements/Pt*/
	char		d63eu[16]; /*Engineering Units*/
	short		d63pr;	/*Display Precision*/
	double		d64hr;	/*High Oper Range*/
	double		d64lr;	/*Low  Oper Range*/
	float *          d64da;	/*Data Array*/
	float		d64cv;	/*Current Value*/
	float		d64lv;	/*Last Value Posted*/
	unsigned long	d64ne;	/*# of Elements/Pt*/
	char		d64eu[16]; /*Engineering Units*/
	short		d64pr;	/*Display Precision*/
	double		d65hr;	/*High Oper Range*/
	double		d65lr;	/*Low  Oper Range*/
	float *          d65da;	/*Data Array*/
	float		d65cv;	/*Current Value*/
	float		d65lv;	/*Last Value Posted*/
	unsigned long	d65ne;	/*# of Elements/Pt*/
	char		d65eu[16]; /*Engineering Units*/
	short		d65pr;	/*Display Precision*/
	double		d66hr;	/*High Oper Range*/
	double		d66lr;	/*Low  Oper Range*/
	float *          d66da;	/*Data Array*/
	float		d66cv;	/*Current Value*/
	float		d66lv;	/*Last Value Posted*/
	unsigned long	d66ne;	/*# of Elements/Pt*/
	char		d66eu[16]; /*Engineering Units*/
	short		d66pr;	/*Display Precision*/
	double		d67hr;	/*High Oper Range*/
	double		d67lr;	/*Low  Oper Range*/
	float *          d67da;	/*Data Array*/
	float		d67cv;	/*Current Value*/
	float		d67lv;	/*Last Value Posted*/
	unsigned long	d67ne;	/*# of Elements/Pt*/
	char		d67eu[16]; /*Engineering Units*/
	short		d67pr;	/*Display Precision*/
	double		d68hr;	/*High Oper Range*/
	double		d68lr;	/*Low  Oper Range*/
	float *          d68da;	/*Data Array*/
	float		d68cv;	/*Current Value*/
	float		d68lv;	/*Last Value Posted*/
	unsigned long	d68ne;	/*# of Elements/Pt*/
	char		d68eu[16]; /*Engineering Units*/
	short		d68pr;	/*Display Precision*/
	double		d69hr;	/*High Oper Range*/
	double		d69lr;	/*Low  Oper Range*/
	float *          d69da;	/*Data Array*/
	float		d69cv;	/*Current Value*/
	float		d69lv;	/*Last Value Posted*/
	unsigned long	d69ne;	/*# of Elements/Pt*/
	char		d69eu[16]; /*Engineering Units*/
	short		d69pr;	/*Display Precision*/
	double		d70hr;	/*High Oper Range*/
	double		d70lr;	/*Low  Oper Range*/
	float *          d70da;	/*Data Array*/
	float		d70cv;	/*Current Value*/
	float		d70lv;	/*Last Value Posted*/
	unsigned long	d70ne;	/*# of Elements/Pt*/
	char		d70eu[16]; /*Engineering Units*/
	short		d70pr;	/*Display Precision*/
	double		d1hr;	/*High Oper Range*/
	double		d1lr;	/*Low  Oper Range*/
	float *          d1da;	/*Data Array*/
	float		d1cv;	/*Current Value*/
	float		d1lv;	/*Last Value Posted*/
	unsigned long	d1ne;	/*# of Elements/Pt*/
	char		d1eu[16]; /*Engineering Units*/
	short		d1pr;	/*Display Precision*/
	double		d2hr;	/*High Oper Range*/
	double		d2lr;	/*Low  Oper Range*/
	float *          d2da;	/*Data Array*/
	float		d2cv;	/*Current Value*/
	float		d2lv;	/*Last Value Posted*/
	unsigned long	d2ne;	/*# of Elements/Pt*/
	char		d2eu[16]; /*Engineering Units*/
	short		d2pr;	/*Display Precision*/
	double		d3hr;	/*High Oper Range*/
	double		d3lr;	/*Low  Oper Range*/
	float *          d3da;	/*Data Array*/
	float		d3cv;	/*Current Value*/
	float		d3lv;	/*Last Value Posted*/
	unsigned long	d3ne;	/*# of Elements/Pt*/
	char		d3eu[16]; /*Engineering Units*/
	short		d3pr;	/*Display Precision*/
	double		d4hr;	/*High Oper Range*/
	double		d4lr;	/*Low  Oper Range*/
	float *          d4da;	/*Data Array*/
	float		d4cv;	/*Current Value*/
	float		d4lv;	/*Last Value Posted*/
	unsigned long	d4ne;	/*# of Elements/Pt*/
	char		d4eu[16]; /*Engineering Units*/
	short		d4pr;	/*Display Precision*/
	double		d5hr;	/*High Oper Range*/
	double		d5lr;	/*Low  Oper Range*/
	float *          d5da;	/*Data Array*/
	float		d5cv;	/*Current Value*/
	float		d5lv;	/*Last Value Posted*/
	unsigned long	d5ne;	/*# of Elements/Pt*/
	char		d5eu[16]; /*Engineering Units*/
	short		d5pr;	/*Display Precision*/
	double		d6hr;	/*High Oper Range*/
	double		d6lr;	/*Low  Oper Range*/
	float *          d6da;	/*Data Array*/
	float		d6cv;	/*Current Value*/
	float		d6lv;	/*Last Value Posted*/
	unsigned long	d6ne;	/*# of Elements/Pt*/
	char		d6eu[16]; /*Engineering Units*/
	short		d6pr;	/*Display Precision*/
	double		d7hr;	/*High Oper Range*/
	double		d7lr;	/*Low  Oper Range*/
	float *          d7da;	/*Data Array*/
	float		d7cv;	/*Current Value*/
	float		d7lv;	/*Last Value Posted*/
	unsigned long	d7ne;	/*# of Elements/Pt*/
	char		d7eu[16]; /*Engineering Units*/
	short		d7pr;	/*Display Precision*/
	double		d8hr;	/*High Oper Range*/
	double		d8lr;	/*Low  Oper Range*/
	float *          d8da;	/*Data Array*/
	float		d8cv;	/*Current Value*/
	float		d8lv;	/*Last Value Posted*/
	unsigned long	d8ne;	/*# of Elements/Pt*/
	char		d8eu[16]; /*Engineering Units*/
	short		d8pr;	/*Display Precision*/
	double		d9hr;	/*High Oper Range*/
	double		d9lr;	/*Low  Oper Range*/
	float *          d9da;	/*Data Array*/
	float		d9cv;	/*Current Value*/
	float		d9lv;	/*Last Value Posted*/
	unsigned long	d9ne;	/*# of Elements/Pt*/
	char		d9eu[16]; /*Engineering Units*/
	short		d9pr;	/*Display Precision*/
	double		dahr;	/*High Oper Range*/
	double		dalr;	/*Low  Oper Range*/
	float *          dada;	/*Data Array*/
	float		dacv;	/*Current Value*/
	float		dalv;	/*LastValue Posted*/
	unsigned long	dane;	/*# of Elements/Pt*/
	char		daeu[16]; /*EngineeringUnits*/
	short		dapr;	/*DisplayPrecision*/
	double		dbhr;	/*High Oper Range*/
	double		dblr;	/*Low  Oper Range*/
	float *          dbda;	/*Data Array*/
	float		dbcv;	/*Current Value*/
	float		dblv;	/*LastValue Posted*/
	unsigned long	dbne;	/*# of Elements/Pt*/
	char		dbeu[16]; /*EngineeringUnits*/
	short		dbpr;	/*DisplayPrecision*/
	double		dchr;	/*High Oper Range*/
	double		dclr;	/*Low  Oper Range*/
	float *          dcda;	/*Data Array*/
	float		dccv;	/*Current Value*/
	float		dclv;	/*LastValue Posted*/
	unsigned long	dcne;	/*# of Elements/Pt*/
	char		dceu[16]; /*EngineeringUnits*/
	short		dcpr;	/*DisplayPrecision*/
	double		ddhr;	/*High Oper Range*/
	double		ddlr;	/*Low  Oper Range*/
	float *          ddda;	/*Data Array*/
	float		ddcv;	/*Current Value*/
	float		ddlv;	/*LastValue Posted*/
	unsigned long	ddne;	/*# of Elements/Pt*/
	char		ddeu[16]; /*EngineeringUnits*/
	short		ddpr;	/*DisplayPrecision*/
	double		dehr;	/*High Oper Range*/
	double		delr;	/*Low  Oper Range*/
	float *          deda;	/*Data Array*/
	float		decv;	/*Current Value*/
	float		delv;	/*LastValue Posted*/
	unsigned long	dene;	/*# of Elements/Pt*/
	char		deeu[16]; /*EngineeringUnits*/
	short		depr;	/*DisplayPrecision*/
	double		dfhr;	/*High Oper Range*/
	double		dflr;	/*Low  Oper Range*/
	float *          dfda;	/*Data Array*/
	float		dfcv;	/*Current Value*/
	float		dflv;	/*LastValue Posted*/
	unsigned long	dfne;	/*# of Elements/Pt*/
	char		dfeu[16]; /*EngineeringUnits*/
	short		dfpr;	/*DisplayPrecision*/
	float		t1cd;	/*T1 Cmnd*/
	float		t2cd;	/*T2 Cmnd*/
	float		t3cd;	/*T3 Cmnd*/
	float		t4cd;	/*T4 Cmnd*/
	float		bscd;	/*Before Scan Cmnd*/
	float		ascd;	/*After Scan Cmnd*/
	unsigned short	paus;	/*Go/Pause control*/
	unsigned short	lpau;	/*Last value of Go/Pause*/
	float		pdly;	/*Positioner-settling delay*/
	float		ddly;	/*Detector-settling delay*/
	float		rdly;	/*Pause resume delay*/
	unsigned short	faze;	/*Scan phase*/
	unsigned short	acqm;	/*Acquisition mode*/
} sscanRecord;
#define sscanRecordNAME	0
#define sscanRecordDESC	1
#define sscanRecordASG	2
#define sscanRecordSCAN	3
#define sscanRecordPINI	4
#define sscanRecordPHAS	5
#define sscanRecordEVNT	6
#define sscanRecordTSE	7
#define sscanRecordTSEL	8
#define sscanRecordDTYP	9
#define sscanRecordDISV	10
#define sscanRecordDISA	11
#define sscanRecordSDIS	12
#define sscanRecordMLOK	13
#define sscanRecordMLIS	14
#define sscanRecordDISP	15
#define sscanRecordPROC	16
#define sscanRecordSTAT	17
#define sscanRecordSEVR	18
#define sscanRecordNSTA	19
#define sscanRecordNSEV	20
#define sscanRecordACKS	21
#define sscanRecordACKT	22
#define sscanRecordDISS	23
#define sscanRecordLCNT	24
#define sscanRecordPACT	25
#define sscanRecordPUTF	26
#define sscanRecordRPRO	27
#define sscanRecordASP	28
#define sscanRecordPPN	29
#define sscanRecordPPNN	30
#define sscanRecordSPVT	31
#define sscanRecordRSET	32
#define sscanRecordDSET	33
#define sscanRecordDPVT	34
#define sscanRecordRDES	35
#define sscanRecordLSET	36
#define sscanRecordPRIO	37
#define sscanRecordTPRO	38
#define sscanRecordBKPT	39
#define sscanRecordUDF	40
#define sscanRecordTIME	41
#define sscanRecordFLNK	42
#define sscanRecordVERS	43
#define sscanRecordVAL	44
#define sscanRecordSMSG	45
#define sscanRecordCMND	46
#define sscanRecordALRT	47
#define sscanRecordRPVT	48
#define sscanRecordMPTS	49
#define sscanRecordEXSC	50
#define sscanRecordXSC	51
#define sscanRecordPXSC	52
#define sscanRecordBUSY	53
#define sscanRecordKILL	54
#define sscanRecordWAIT	55
#define sscanRecordWCNT	56
#define sscanRecordAWCT	57
#define sscanRecordWTNG	58
#define sscanRecordDATA	59
#define sscanRecordREFD	60
#define sscanRecordNPTS	61
#define sscanRecordFPTS	62
#define sscanRecordFFO	63
#define sscanRecordCPT	64
#define sscanRecordDPT	65
#define sscanRecordPCPT	66
#define sscanRecordPASM	67
#define sscanRecordTOLP	68
#define sscanRecordP1PV	69
#define sscanRecordP2PV	70
#define sscanRecordP3PV	71
#define sscanRecordP4PV	72
#define sscanRecordR1PV	73
#define sscanRecordR2PV	74
#define sscanRecordR3PV	75
#define sscanRecordR4PV	76
#define sscanRecordD01PV	77
#define sscanRecordD02PV	78
#define sscanRecordD03PV	79
#define sscanRecordD04PV	80
#define sscanRecordD05PV	81
#define sscanRecordD06PV	82
#define sscanRecordD07PV	83
#define sscanRecordD08PV	84
#define sscanRecordD09PV	85
#define sscanRecordD10PV	86
#define sscanRecordD11PV	87
#define sscanRecordD12PV	88
#define sscanRecordD13PV	89
#define sscanRecordD14PV	90
#define sscanRecordD15PV	91
#define sscanRecordD16PV	92
#define sscanRecordD17PV	93
#define sscanRecordD18PV	94
#define sscanRecordD19PV	95
#define sscanRecordD20PV	96
#define sscanRecordD21PV	97
#define sscanRecordD22PV	98
#define sscanRecordD23PV	99
#define sscanRecordD24PV	100
#define sscanRecordD25PV	101
#define sscanRecordD26PV	102
#define sscanRecordD27PV	103
#define sscanRecordD28PV	104
#define sscanRecordD29PV	105
#define sscanRecordD30PV	106
#define sscanRecordD31PV	107
#define sscanRecordD32PV	108
#define sscanRecordD33PV	109
#define sscanRecordD34PV	110
#define sscanRecordD35PV	111
#define sscanRecordD36PV	112
#define sscanRecordD37PV	113
#define sscanRecordD38PV	114
#define sscanRecordD39PV	115
#define sscanRecordD40PV	116
#define sscanRecordD41PV	117
#define sscanRecordD42PV	118
#define sscanRecordD43PV	119
#define sscanRecordD44PV	120
#define sscanRecordD45PV	121
#define sscanRecordD46PV	122
#define sscanRecordD47PV	123
#define sscanRecordD48PV	124
#define sscanRecordD49PV	125
#define sscanRecordD50PV	126
#define sscanRecordD51PV	127
#define sscanRecordD52PV	128
#define sscanRecordD53PV	129
#define sscanRecordD54PV	130
#define sscanRecordD55PV	131
#define sscanRecordD56PV	132
#define sscanRecordD57PV	133
#define sscanRecordD58PV	134
#define sscanRecordD59PV	135
#define sscanRecordD60PV	136
#define sscanRecordD61PV	137
#define sscanRecordD62PV	138
#define sscanRecordD63PV	139
#define sscanRecordD64PV	140
#define sscanRecordD65PV	141
#define sscanRecordD66PV	142
#define sscanRecordD67PV	143
#define sscanRecordD68PV	144
#define sscanRecordD69PV	145
#define sscanRecordD70PV	146
#define sscanRecordD1PV	147
#define sscanRecordD2PV	148
#define sscanRecordD3PV	149
#define sscanRecordD4PV	150
#define sscanRecordD5PV	151
#define sscanRecordD6PV	152
#define sscanRecordD7PV	153
#define sscanRecordD8PV	154
#define sscanRecordD9PV	155
#define sscanRecordDAPV	156
#define sscanRecordDBPV	157
#define sscanRecordDCPV	158
#define sscanRecordDDPV	159
#define sscanRecordDEPV	160
#define sscanRecordDFPV	161
#define sscanRecordT1PV	162
#define sscanRecordT2PV	163
#define sscanRecordT3PV	164
#define sscanRecordT4PV	165
#define sscanRecordBSPV	166
#define sscanRecordASPV	167
#define sscanRecordP1NV	168
#define sscanRecordP2NV	169
#define sscanRecordP3NV	170
#define sscanRecordP4NV	171
#define sscanRecordR1NV	172
#define sscanRecordR2NV	173
#define sscanRecordR3NV	174
#define sscanRecordR4NV	175
#define sscanRecordD01NV	176
#define sscanRecordD02NV	177
#define sscanRecordD03NV	178
#define sscanRecordD04NV	179
#define sscanRecordD05NV	180
#define sscanRecordD06NV	181
#define sscanRecordD07NV	182
#define sscanRecordD08NV	183
#define sscanRecordD09NV	184
#define sscanRecordD10NV	185
#define sscanRecordD11NV	186
#define sscanRecordD12NV	187
#define sscanRecordD13NV	188
#define sscanRecordD14NV	189
#define sscanRecordD15NV	190
#define sscanRecordD16NV	191
#define sscanRecordD17NV	192
#define sscanRecordD18NV	193
#define sscanRecordD19NV	194
#define sscanRecordD20NV	195
#define sscanRecordD21NV	196
#define sscanRecordD22NV	197
#define sscanRecordD23NV	198
#define sscanRecordD24NV	199
#define sscanRecordD25NV	200
#define sscanRecordD26NV	201
#define sscanRecordD27NV	202
#define sscanRecordD28NV	203
#define sscanRecordD29NV	204
#define sscanRecordD30NV	205
#define sscanRecordD31NV	206
#define sscanRecordD32NV	207
#define sscanRecordD33NV	208
#define sscanRecordD34NV	209
#define sscanRecordD35NV	210
#define sscanRecordD36NV	211
#define sscanRecordD37NV	212
#define sscanRecordD38NV	213
#define sscanRecordD39NV	214
#define sscanRecordD40NV	215
#define sscanRecordD41NV	216
#define sscanRecordD42NV	217
#define sscanRecordD43NV	218
#define sscanRecordD44NV	219
#define sscanRecordD45NV	220
#define sscanRecordD46NV	221
#define sscanRecordD47NV	222
#define sscanRecordD48NV	223
#define sscanRecordD49NV	224
#define sscanRecordD50NV	225
#define sscanRecordD51NV	226
#define sscanRecordD52NV	227
#define sscanRecordD53NV	228
#define sscanRecordD54NV	229
#define sscanRecordD55NV	230
#define sscanRecordD56NV	231
#define sscanRecordD57NV	232
#define sscanRecordD58NV	233
#define sscanRecordD59NV	234
#define sscanRecordD60NV	235
#define sscanRecordD61NV	236
#define sscanRecordD62NV	237
#define sscanRecordD63NV	238
#define sscanRecordD64NV	239
#define sscanRecordD65NV	240
#define sscanRecordD66NV	241
#define sscanRecordD67NV	242
#define sscanRecordD68NV	243
#define sscanRecordD69NV	244
#define sscanRecordD70NV	245
#define sscanRecordD1NV	246
#define sscanRecordD2NV	247
#define sscanRecordD3NV	248
#define sscanRecordD4NV	249
#define sscanRecordD5NV	250
#define sscanRecordD6NV	251
#define sscanRecordD7NV	252
#define sscanRecordD8NV	253
#define sscanRecordD9NV	254
#define sscanRecordDANV	255
#define sscanRecordDBNV	256
#define sscanRecordDCNV	257
#define sscanRecordDDNV	258
#define sscanRecordDENV	259
#define sscanRecordDFNV	260
#define sscanRecordT1NV	261
#define sscanRecordT2NV	262
#define sscanRecordT3NV	263
#define sscanRecordT4NV	264
#define sscanRecordBSNV	265
#define sscanRecordASNV	266
#define sscanRecordP1PP	267
#define sscanRecordP1CV	268
#define sscanRecordP1DV	269
#define sscanRecordP1LV	270
#define sscanRecordP1SP	271
#define sscanRecordP1SI	272
#define sscanRecordP1EP	273
#define sscanRecordP1CP	274
#define sscanRecordP1WD	275
#define sscanRecordR1CV	276
#define sscanRecordR1LV	277
#define sscanRecordR1DL	278
#define sscanRecordP1HR	279
#define sscanRecordP1LR	280
#define sscanRecordP1PA	281
#define sscanRecordP1RA	282
#define sscanRecordP1FS	283
#define sscanRecordP1FI	284
#define sscanRecordP1FE	285
#define sscanRecordP1FC	286
#define sscanRecordP1FW	287
#define sscanRecordP1SM	288
#define sscanRecordP1AR	289
#define sscanRecordP1EU	290
#define sscanRecordP1PR	291
#define sscanRecordP2PP	292
#define sscanRecordP2CV	293
#define sscanRecordP2DV	294
#define sscanRecordP2LV	295
#define sscanRecordP2SP	296
#define sscanRecordP2SI	297
#define sscanRecordP2EP	298
#define sscanRecordP2CP	299
#define sscanRecordP2WD	300
#define sscanRecordR2CV	301
#define sscanRecordR2LV	302
#define sscanRecordR2DL	303
#define sscanRecordP2HR	304
#define sscanRecordP2LR	305
#define sscanRecordP2PA	306
#define sscanRecordP2RA	307
#define sscanRecordP2FS	308
#define sscanRecordP2FI	309
#define sscanRecordP2FE	310
#define sscanRecordP2FC	311
#define sscanRecordP2FW	312
#define sscanRecordP2SM	313
#define sscanRecordP2AR	314
#define sscanRecordP2EU	315
#define sscanRecordP2PR	316
#define sscanRecordP3PP	317
#define sscanRecordP3CV	318
#define sscanRecordP3DV	319
#define sscanRecordP3LV	320
#define sscanRecordP3SP	321
#define sscanRecordP3SI	322
#define sscanRecordP3EP	323
#define sscanRecordP3CP	324
#define sscanRecordP3WD	325
#define sscanRecordR3CV	326
#define sscanRecordR3LV	327
#define sscanRecordR3DL	328
#define sscanRecordP3HR	329
#define sscanRecordP3LR	330
#define sscanRecordP3PA	331
#define sscanRecordP3RA	332
#define sscanRecordP3FS	333
#define sscanRecordP3FI	334
#define sscanRecordP3FE	335
#define sscanRecordP3FC	336
#define sscanRecordP3FW	337
#define sscanRecordP3SM	338
#define sscanRecordP3AR	339
#define sscanRecordP3EU	340
#define sscanRecordP3PR	341
#define sscanRecordP4PP	342
#define sscanRecordP4CV	343
#define sscanRecordP4DV	344
#define sscanRecordP4LV	345
#define sscanRecordP4SP	346
#define sscanRecordP4SI	347
#define sscanRecordP4EP	348
#define sscanRecordP4CP	349
#define sscanRecordP4WD	350
#define sscanRecordR4CV	351
#define sscanRecordR4LV	352
#define sscanRecordR4DL	353
#define sscanRecordP4HR	354
#define sscanRecordP4LR	355
#define sscanRecordP4PA	356
#define sscanRecordP4RA	357
#define sscanRecordP4FS	358
#define sscanRecordP4FI	359
#define sscanRecordP4FE	360
#define sscanRecordP4FC	361
#define sscanRecordP4FW	362
#define sscanRecordP4SM	363
#define sscanRecordP4AR	364
#define sscanRecordP4EU	365
#define sscanRecordP4PR	366
#define sscanRecordD01HR	367
#define sscanRecordD01LR	368
#define sscanRecordD01DA	369
#define sscanRecordD01CV	370
#define sscanRecordD01LV	371
#define sscanRecordD01NE	372
#define sscanRecordD01EU	373
#define sscanRecordD01PR	374
#define sscanRecordD02HR	375
#define sscanRecordD02LR	376
#define sscanRecordD02DA	377
#define sscanRecordD02CV	378
#define sscanRecordD02LV	379
#define sscanRecordD02NE	380
#define sscanRecordD02EU	381
#define sscanRecordD02PR	382
#define sscanRecordD03HR	383
#define sscanRecordD03LR	384
#define sscanRecordD03DA	385
#define sscanRecordD03CV	386
#define sscanRecordD03LV	387
#define sscanRecordD03NE	388
#define sscanRecordD03EU	389
#define sscanRecordD03PR	390
#define sscanRecordD04HR	391
#define sscanRecordD04LR	392
#define sscanRecordD04DA	393
#define sscanRecordD04CV	394
#define sscanRecordD04LV	395
#define sscanRecordD04NE	396
#define sscanRecordD04EU	397
#define sscanRecordD04PR	398
#define sscanRecordD05HR	399
#define sscanRecordD05LR	400
#define sscanRecordD05DA	401
#define sscanRecordD05CV	402
#define sscanRecordD05LV	403
#define sscanRecordD05NE	404
#define sscanRecordD05EU	405
#define sscanRecordD05PR	406
#define sscanRecordD06HR	407
#define sscanRecordD06LR	408
#define sscanRecordD06DA	409
#define sscanRecordD06CV	410
#define sscanRecordD06LV	411
#define sscanRecordD06NE	412
#define sscanRecordD06EU	413
#define sscanRecordD06PR	414
#define sscanRecordD07HR	415
#define sscanRecordD07LR	416
#define sscanRecordD07DA	417
#define sscanRecordD07CV	418
#define sscanRecordD07LV	419
#define sscanRecordD07NE	420
#define sscanRecordD07EU	421
#define sscanRecordD07PR	422
#define sscanRecordD08HR	423
#define sscanRecordD08LR	424
#define sscanRecordD08DA	425
#define sscanRecordD08CV	426
#define sscanRecordD08LV	427
#define sscanRecordD08NE	428
#define sscanRecordD08EU	429
#define sscanRecordD08PR	430
#define sscanRecordD09HR	431
#define sscanRecordD09LR	432
#define sscanRecordD09DA	433
#define sscanRecordD09CV	434
#define sscanRecordD09LV	435
#define sscanRecordD09NE	436
#define sscanRecordD09EU	437
#define sscanRecordD09PR	438
#define sscanRecordD10HR	439
#define sscanRecordD10LR	440
#define sscanRecordD10DA	441
#define sscanRecordD10CV	442
#define sscanRecordD10LV	443
#define sscanRecordD10NE	444
#define sscanRecordD10EU	445
#define sscanRecordD10PR	446
#define sscanRecordD11HR	447
#define sscanRecordD11LR	448
#define sscanRecordD11DA	449
#define sscanRecordD11CV	450
#define sscanRecordD11LV	451
#define sscanRecordD11NE	452
#define sscanRecordD11EU	453
#define sscanRecordD11PR	454
#define sscanRecordD12HR	455
#define sscanRecordD12LR	456
#define sscanRecordD12DA	457
#define sscanRecordD12CV	458
#define sscanRecordD12LV	459
#define sscanRecordD12NE	460
#define sscanRecordD12EU	461
#define sscanRecordD12PR	462
#define sscanRecordD13HR	463
#define sscanRecordD13LR	464
#define sscanRecordD13DA	465
#define sscanRecordD13CV	466
#define sscanRecordD13LV	467
#define sscanRecordD13NE	468
#define sscanRecordD13EU	469
#define sscanRecordD13PR	470
#define sscanRecordD14HR	471
#define sscanRecordD14LR	472
#define sscanRecordD14DA	473
#define sscanRecordD14CV	474
#define sscanRecordD14LV	475
#define sscanRecordD14NE	476
#define sscanRecordD14EU	477
#define sscanRecordD14PR	478
#define sscanRecordD15HR	479
#define sscanRecordD15LR	480
#define sscanRecordD15DA	481
#define sscanRecordD15CV	482
#define sscanRecordD15LV	483
#define sscanRecordD15NE	484
#define sscanRecordD15EU	485
#define sscanRecordD15PR	486
#define sscanRecordD16HR	487
#define sscanRecordD16LR	488
#define sscanRecordD16DA	489
#define sscanRecordD16CV	490
#define sscanRecordD16LV	491
#define sscanRecordD16NE	492
#define sscanRecordD16EU	493
#define sscanRecordD16PR	494
#define sscanRecordD17HR	495
#define sscanRecordD17LR	496
#define sscanRecordD17DA	497
#define sscanRecordD17CV	498
#define sscanRecordD17LV	499
#define sscanRecordD17NE	500
#define sscanRecordD17EU	501
#define sscanRecordD17PR	502
#define sscanRecordD18HR	503
#define sscanRecordD18LR	504
#define sscanRecordD18DA	505
#define sscanRecordD18CV	506
#define sscanRecordD18LV	507
#define sscanRecordD18NE	508
#define sscanRecordD18EU	509
#define sscanRecordD18PR	510
#define sscanRecordD19HR	511
#define sscanRecordD19LR	512
#define sscanRecordD19DA	513
#define sscanRecordD19CV	514
#define sscanRecordD19LV	515
#define sscanRecordD19NE	516
#define sscanRecordD19EU	517
#define sscanRecordD19PR	518
#define sscanRecordD20HR	519
#define sscanRecordD20LR	520
#define sscanRecordD20DA	521
#define sscanRecordD20CV	522
#define sscanRecordD20LV	523
#define sscanRecordD20NE	524
#define sscanRecordD20EU	525
#define sscanRecordD20PR	526
#define sscanRecordD21HR	527
#define sscanRecordD21LR	528
#define sscanRecordD21DA	529
#define sscanRecordD21CV	530
#define sscanRecordD21LV	531
#define sscanRecordD21NE	532
#define sscanRecordD21EU	533
#define sscanRecordD21PR	534
#define sscanRecordD22HR	535
#define sscanRecordD22LR	536
#define sscanRecordD22DA	537
#define sscanRecordD22CV	538
#define sscanRecordD22LV	539
#define sscanRecordD22NE	540
#define sscanRecordD22EU	541
#define sscanRecordD22PR	542
#define sscanRecordD23HR	543
#define sscanRecordD23LR	544
#define sscanRecordD23DA	545
#define sscanRecordD23CV	546
#define sscanRecordD23LV	547
#define sscanRecordD23NE	548
#define sscanRecordD23EU	549
#define sscanRecordD23PR	550
#define sscanRecordD24HR	551
#define sscanRecordD24LR	552
#define sscanRecordD24DA	553
#define sscanRecordD24CV	554
#define sscanRecordD24LV	555
#define sscanRecordD24NE	556
#define sscanRecordD24EU	557
#define sscanRecordD24PR	558
#define sscanRecordD25HR	559
#define sscanRecordD25LR	560
#define sscanRecordD25DA	561
#define sscanRecordD25CV	562
#define sscanRecordD25LV	563
#define sscanRecordD25NE	564
#define sscanRecordD25EU	565
#define sscanRecordD25PR	566
#define sscanRecordD26HR	567
#define sscanRecordD26LR	568
#define sscanRecordD26DA	569
#define sscanRecordD26CV	570
#define sscanRecordD26LV	571
#define sscanRecordD26NE	572
#define sscanRecordD26EU	573
#define sscanRecordD26PR	574
#define sscanRecordD27HR	575
#define sscanRecordD27LR	576
#define sscanRecordD27DA	577
#define sscanRecordD27CV	578
#define sscanRecordD27LV	579
#define sscanRecordD27NE	580
#define sscanRecordD27EU	581
#define sscanRecordD27PR	582
#define sscanRecordD28HR	583
#define sscanRecordD28LR	584
#define sscanRecordD28DA	585
#define sscanRecordD28CV	586
#define sscanRecordD28LV	587
#define sscanRecordD28NE	588
#define sscanRecordD28EU	589
#define sscanRecordD28PR	590
#define sscanRecordD29HR	591
#define sscanRecordD29LR	592
#define sscanRecordD29DA	593
#define sscanRecordD29CV	594
#define sscanRecordD29LV	595
#define sscanRecordD29NE	596
#define sscanRecordD29EU	597
#define sscanRecordD29PR	598
#define sscanRecordD30HR	599
#define sscanRecordD30LR	600
#define sscanRecordD30DA	601
#define sscanRecordD30CV	602
#define sscanRecordD30LV	603
#define sscanRecordD30NE	604
#define sscanRecordD30EU	605
#define sscanRecordD30PR	606
#define sscanRecordD31HR	607
#define sscanRecordD31LR	608
#define sscanRecordD31DA	609
#define sscanRecordD31CV	610
#define sscanRecordD31LV	611
#define sscanRecordD31NE	612
#define sscanRecordD31EU	613
#define sscanRecordD31PR	614
#define sscanRecordD32HR	615
#define sscanRecordD32LR	616
#define sscanRecordD32DA	617
#define sscanRecordD32CV	618
#define sscanRecordD32LV	619
#define sscanRecordD32NE	620
#define sscanRecordD32EU	621
#define sscanRecordD32PR	622
#define sscanRecordD33HR	623
#define sscanRecordD33LR	624
#define sscanRecordD33DA	625
#define sscanRecordD33CV	626
#define sscanRecordD33LV	627
#define sscanRecordD33NE	628
#define sscanRecordD33EU	629
#define sscanRecordD33PR	630
#define sscanRecordD34HR	631
#define sscanRecordD34LR	632
#define sscanRecordD34DA	633
#define sscanRecordD34CV	634
#define sscanRecordD34LV	635
#define sscanRecordD34NE	636
#define sscanRecordD34EU	637
#define sscanRecordD34PR	638
#define sscanRecordD35HR	639
#define sscanRecordD35LR	640
#define sscanRecordD35DA	641
#define sscanRecordD35CV	642
#define sscanRecordD35LV	643
#define sscanRecordD35NE	644
#define sscanRecordD35EU	645
#define sscanRecordD35PR	646
#define sscanRecordD36HR	647
#define sscanRecordD36LR	648
#define sscanRecordD36DA	649
#define sscanRecordD36CV	650
#define sscanRecordD36LV	651
#define sscanRecordD36NE	652
#define sscanRecordD36EU	653
#define sscanRecordD36PR	654
#define sscanRecordD37HR	655
#define sscanRecordD37LR	656
#define sscanRecordD37DA	657
#define sscanRecordD37CV	658
#define sscanRecordD37LV	659
#define sscanRecordD37NE	660
#define sscanRecordD37EU	661
#define sscanRecordD37PR	662
#define sscanRecordD38HR	663
#define sscanRecordD38LR	664
#define sscanRecordD38DA	665
#define sscanRecordD38CV	666
#define sscanRecordD38LV	667
#define sscanRecordD38NE	668
#define sscanRecordD38EU	669
#define sscanRecordD38PR	670
#define sscanRecordD39HR	671
#define sscanRecordD39LR	672
#define sscanRecordD39DA	673
#define sscanRecordD39CV	674
#define sscanRecordD39LV	675
#define sscanRecordD39NE	676
#define sscanRecordD39EU	677
#define sscanRecordD39PR	678
#define sscanRecordD40HR	679
#define sscanRecordD40LR	680
#define sscanRecordD40DA	681
#define sscanRecordD40CV	682
#define sscanRecordD40LV	683
#define sscanRecordD40NE	684
#define sscanRecordD40EU	685
#define sscanRecordD40PR	686
#define sscanRecordD41HR	687
#define sscanRecordD41LR	688
#define sscanRecordD41DA	689
#define sscanRecordD41CV	690
#define sscanRecordD41LV	691
#define sscanRecordD41NE	692
#define sscanRecordD41EU	693
#define sscanRecordD41PR	694
#define sscanRecordD42HR	695
#define sscanRecordD42LR	696
#define sscanRecordD42DA	697
#define sscanRecordD42CV	698
#define sscanRecordD42LV	699
#define sscanRecordD42NE	700
#define sscanRecordD42EU	701
#define sscanRecordD42PR	702
#define sscanRecordD43HR	703
#define sscanRecordD43LR	704
#define sscanRecordD43DA	705
#define sscanRecordD43CV	706
#define sscanRecordD43LV	707
#define sscanRecordD43NE	708
#define sscanRecordD43EU	709
#define sscanRecordD43PR	710
#define sscanRecordD44HR	711
#define sscanRecordD44LR	712
#define sscanRecordD44DA	713
#define sscanRecordD44CV	714
#define sscanRecordD44LV	715
#define sscanRecordD44NE	716
#define sscanRecordD44EU	717
#define sscanRecordD44PR	718
#define sscanRecordD45HR	719
#define sscanRecordD45LR	720
#define sscanRecordD45DA	721
#define sscanRecordD45CV	722
#define sscanRecordD45LV	723
#define sscanRecordD45NE	724
#define sscanRecordD45EU	725
#define sscanRecordD45PR	726
#define sscanRecordD46HR	727
#define sscanRecordD46LR	728
#define sscanRecordD46DA	729
#define sscanRecordD46CV	730
#define sscanRecordD46LV	731
#define sscanRecordD46NE	732
#define sscanRecordD46EU	733
#define sscanRecordD46PR	734
#define sscanRecordD47HR	735
#define sscanRecordD47LR	736
#define sscanRecordD47DA	737
#define sscanRecordD47CV	738
#define sscanRecordD47LV	739
#define sscanRecordD47NE	740
#define sscanRecordD47EU	741
#define sscanRecordD47PR	742
#define sscanRecordD48HR	743
#define sscanRecordD48LR	744
#define sscanRecordD48DA	745
#define sscanRecordD48CV	746
#define sscanRecordD48LV	747
#define sscanRecordD48NE	748
#define sscanRecordD48EU	749
#define sscanRecordD48PR	750
#define sscanRecordD49HR	751
#define sscanRecordD49LR	752
#define sscanRecordD49DA	753
#define sscanRecordD49CV	754
#define sscanRecordD49LV	755
#define sscanRecordD49NE	756
#define sscanRecordD49EU	757
#define sscanRecordD49PR	758
#define sscanRecordD50HR	759
#define sscanRecordD50LR	760
#define sscanRecordD50DA	761
#define sscanRecordD50CV	762
#define sscanRecordD50LV	763
#define sscanRecordD50NE	764
#define sscanRecordD50EU	765
#define sscanRecordD50PR	766
#define sscanRecordD51HR	767
#define sscanRecordD51LR	768
#define sscanRecordD51DA	769
#define sscanRecordD51CV	770
#define sscanRecordD51LV	771
#define sscanRecordD51NE	772
#define sscanRecordD51EU	773
#define sscanRecordD51PR	774
#define sscanRecordD52HR	775
#define sscanRecordD52LR	776
#define sscanRecordD52DA	777
#define sscanRecordD52CV	778
#define sscanRecordD52LV	779
#define sscanRecordD52NE	780
#define sscanRecordD52EU	781
#define sscanRecordD52PR	782
#define sscanRecordD53HR	783
#define sscanRecordD53LR	784
#define sscanRecordD53DA	785
#define sscanRecordD53CV	786
#define sscanRecordD53LV	787
#define sscanRecordD53NE	788
#define sscanRecordD53EU	789
#define sscanRecordD53PR	790
#define sscanRecordD54HR	791
#define sscanRecordD54LR	792
#define sscanRecordD54DA	793
#define sscanRecordD54CV	794
#define sscanRecordD54LV	795
#define sscanRecordD54NE	796
#define sscanRecordD54EU	797
#define sscanRecordD54PR	798
#define sscanRecordD55HR	799
#define sscanRecordD55LR	800
#define sscanRecordD55DA	801
#define sscanRecordD55CV	802
#define sscanRecordD55LV	803
#define sscanRecordD55NE	804
#define sscanRecordD55EU	805
#define sscanRecordD55PR	806
#define sscanRecordD56HR	807
#define sscanRecordD56LR	808
#define sscanRecordD56DA	809
#define sscanRecordD56CV	810
#define sscanRecordD56LV	811
#define sscanRecordD56NE	812
#define sscanRecordD56EU	813
#define sscanRecordD56PR	814
#define sscanRecordD57HR	815
#define sscanRecordD57LR	816
#define sscanRecordD57DA	817
#define sscanRecordD57CV	818
#define sscanRecordD57LV	819
#define sscanRecordD57NE	820
#define sscanRecordD57EU	821
#define sscanRecordD57PR	822
#define sscanRecordD58HR	823
#define sscanRecordD58LR	824
#define sscanRecordD58DA	825
#define sscanRecordD58CV	826
#define sscanRecordD58LV	827
#define sscanRecordD58NE	828
#define sscanRecordD58EU	829
#define sscanRecordD58PR	830
#define sscanRecordD59HR	831
#define sscanRecordD59LR	832
#define sscanRecordD59DA	833
#define sscanRecordD59CV	834
#define sscanRecordD59LV	835
#define sscanRecordD59NE	836
#define sscanRecordD59EU	837
#define sscanRecordD59PR	838
#define sscanRecordD60HR	839
#define sscanRecordD60LR	840
#define sscanRecordD60DA	841
#define sscanRecordD60CV	842
#define sscanRecordD60LV	843
#define sscanRecordD60NE	844
#define sscanRecordD60EU	845
#define sscanRecordD60PR	846
#define sscanRecordD61HR	847
#define sscanRecordD61LR	848
#define sscanRecordD61DA	849
#define sscanRecordD61CV	850
#define sscanRecordD61LV	851
#define sscanRecordD61NE	852
#define sscanRecordD61EU	853
#define sscanRecordD61PR	854
#define sscanRecordD62HR	855
#define sscanRecordD62LR	856
#define sscanRecordD62DA	857
#define sscanRecordD62CV	858
#define sscanRecordD62LV	859
#define sscanRecordD62NE	860
#define sscanRecordD62EU	861
#define sscanRecordD62PR	862
#define sscanRecordD63HR	863
#define sscanRecordD63LR	864
#define sscanRecordD63DA	865
#define sscanRecordD63CV	866
#define sscanRecordD63LV	867
#define sscanRecordD63NE	868
#define sscanRecordD63EU	869
#define sscanRecordD63PR	870
#define sscanRecordD64HR	871
#define sscanRecordD64LR	872
#define sscanRecordD64DA	873
#define sscanRecordD64CV	874
#define sscanRecordD64LV	875
#define sscanRecordD64NE	876
#define sscanRecordD64EU	877
#define sscanRecordD64PR	878
#define sscanRecordD65HR	879
#define sscanRecordD65LR	880
#define sscanRecordD65DA	881
#define sscanRecordD65CV	882
#define sscanRecordD65LV	883
#define sscanRecordD65NE	884
#define sscanRecordD65EU	885
#define sscanRecordD65PR	886
#define sscanRecordD66HR	887
#define sscanRecordD66LR	888
#define sscanRecordD66DA	889
#define sscanRecordD66CV	890
#define sscanRecordD66LV	891
#define sscanRecordD66NE	892
#define sscanRecordD66EU	893
#define sscanRecordD66PR	894
#define sscanRecordD67HR	895
#define sscanRecordD67LR	896
#define sscanRecordD67DA	897
#define sscanRecordD67CV	898
#define sscanRecordD67LV	899
#define sscanRecordD67NE	900
#define sscanRecordD67EU	901
#define sscanRecordD67PR	902
#define sscanRecordD68HR	903
#define sscanRecordD68LR	904
#define sscanRecordD68DA	905
#define sscanRecordD68CV	906
#define sscanRecordD68LV	907
#define sscanRecordD68NE	908
#define sscanRecordD68EU	909
#define sscanRecordD68PR	910
#define sscanRecordD69HR	911
#define sscanRecordD69LR	912
#define sscanRecordD69DA	913
#define sscanRecordD69CV	914
#define sscanRecordD69LV	915
#define sscanRecordD69NE	916
#define sscanRecordD69EU	917
#define sscanRecordD69PR	918
#define sscanRecordD70HR	919
#define sscanRecordD70LR	920
#define sscanRecordD70DA	921
#define sscanRecordD70CV	922
#define sscanRecordD70LV	923
#define sscanRecordD70NE	924
#define sscanRecordD70EU	925
#define sscanRecordD70PR	926
#define sscanRecordD1HR	927
#define sscanRecordD1LR	928
#define sscanRecordD1DA	929
#define sscanRecordD1CV	930
#define sscanRecordD1LV	931
#define sscanRecordD1NE	932
#define sscanRecordD1EU	933
#define sscanRecordD1PR	934
#define sscanRecordD2HR	935
#define sscanRecordD2LR	936
#define sscanRecordD2DA	937
#define sscanRecordD2CV	938
#define sscanRecordD2LV	939
#define sscanRecordD2NE	940
#define sscanRecordD2EU	941
#define sscanRecordD2PR	942
#define sscanRecordD3HR	943
#define sscanRecordD3LR	944
#define sscanRecordD3DA	945
#define sscanRecordD3CV	946
#define sscanRecordD3LV	947
#define sscanRecordD3NE	948
#define sscanRecordD3EU	949
#define sscanRecordD3PR	950
#define sscanRecordD4HR	951
#define sscanRecordD4LR	952
#define sscanRecordD4DA	953
#define sscanRecordD4CV	954
#define sscanRecordD4LV	955
#define sscanRecordD4NE	956
#define sscanRecordD4EU	957
#define sscanRecordD4PR	958
#define sscanRecordD5HR	959
#define sscanRecordD5LR	960
#define sscanRecordD5DA	961
#define sscanRecordD5CV	962
#define sscanRecordD5LV	963
#define sscanRecordD5NE	964
#define sscanRecordD5EU	965
#define sscanRecordD5PR	966
#define sscanRecordD6HR	967
#define sscanRecordD6LR	968
#define sscanRecordD6DA	969
#define sscanRecordD6CV	970
#define sscanRecordD6LV	971
#define sscanRecordD6NE	972
#define sscanRecordD6EU	973
#define sscanRecordD6PR	974
#define sscanRecordD7HR	975
#define sscanRecordD7LR	976
#define sscanRecordD7DA	977
#define sscanRecordD7CV	978
#define sscanRecordD7LV	979
#define sscanRecordD7NE	980
#define sscanRecordD7EU	981
#define sscanRecordD7PR	982
#define sscanRecordD8HR	983
#define sscanRecordD8LR	984
#define sscanRecordD8DA	985
#define sscanRecordD8CV	986
#define sscanRecordD8LV	987
#define sscanRecordD8NE	988
#define sscanRecordD8EU	989
#define sscanRecordD8PR	990
#define sscanRecordD9HR	991
#define sscanRecordD9LR	992
#define sscanRecordD9DA	993
#define sscanRecordD9CV	994
#define sscanRecordD9LV	995
#define sscanRecordD9NE	996
#define sscanRecordD9EU	997
#define sscanRecordD9PR	998
#define sscanRecordDAHR	999
#define sscanRecordDALR	1000
#define sscanRecordDADA	1001
#define sscanRecordDACV	1002
#define sscanRecordDALV	1003
#define sscanRecordDANE	1004
#define sscanRecordDAEU	1005
#define sscanRecordDAPR	1006
#define sscanRecordDBHR	1007
#define sscanRecordDBLR	1008
#define sscanRecordDBDA	1009
#define sscanRecordDBCV	1010
#define sscanRecordDBLV	1011
#define sscanRecordDBNE	1012
#define sscanRecordDBEU	1013
#define sscanRecordDBPR	1014
#define sscanRecordDCHR	1015
#define sscanRecordDCLR	1016
#define sscanRecordDCDA	1017
#define sscanRecordDCCV	1018
#define sscanRecordDCLV	1019
#define sscanRecordDCNE	1020
#define sscanRecordDCEU	1021
#define sscanRecordDCPR	1022
#define sscanRecordDDHR	1023
#define sscanRecordDDLR	1024
#define sscanRecordDDDA	1025
#define sscanRecordDDCV	1026
#define sscanRecordDDLV	1027
#define sscanRecordDDNE	1028
#define sscanRecordDDEU	1029
#define sscanRecordDDPR	1030
#define sscanRecordDEHR	1031
#define sscanRecordDELR	1032
#define sscanRecordDEDA	1033
#define sscanRecordDECV	1034
#define sscanRecordDELV	1035
#define sscanRecordDENE	1036
#define sscanRecordDEEU	1037
#define sscanRecordDEPR	1038
#define sscanRecordDFHR	1039
#define sscanRecordDFLR	1040
#define sscanRecordDFDA	1041
#define sscanRecordDFCV	1042
#define sscanRecordDFLV	1043
#define sscanRecordDFNE	1044
#define sscanRecordDFEU	1045
#define sscanRecordDFPR	1046
#define sscanRecordT1CD	1047
#define sscanRecordT2CD	1048
#define sscanRecordT3CD	1049
#define sscanRecordT4CD	1050
#define sscanRecordBSCD	1051
#define sscanRecordASCD	1052
#define sscanRecordPAUS	1053
#define sscanRecordLPAU	1054
#define sscanRecordPDLY	1055
#define sscanRecordDDLY	1056
#define sscanRecordRDLY	1057
#define sscanRecordFAZE	1058
#define sscanRecordACQM	1059
#endif /*INCsscanH*/
#ifdef GEN_SIZE_OFFSET
int sscanRecordSizeOffset(dbRecordType *pdbRecordType)
{
    sscanRecord *prec = 0;
  pdbRecordType->papFldDes[0]->size=sizeof(prec->name);
  pdbRecordType->papFldDes[0]->offset=(short)((char *)&prec->name - (char *)prec);
  pdbRecordType->papFldDes[1]->size=sizeof(prec->desc);
  pdbRecordType->papFldDes[1]->offset=(short)((char *)&prec->desc - (char *)prec);
  pdbRecordType->papFldDes[2]->size=sizeof(prec->asg);
  pdbRecordType->papFldDes[2]->offset=(short)((char *)&prec->asg - (char *)prec);
  pdbRecordType->papFldDes[3]->size=sizeof(prec->scan);
  pdbRecordType->papFldDes[3]->offset=(short)((char *)&prec->scan - (char *)prec);
  pdbRecordType->papFldDes[4]->size=sizeof(prec->pini);
  pdbRecordType->papFldDes[4]->offset=(short)((char *)&prec->pini - (char *)prec);
  pdbRecordType->papFldDes[5]->size=sizeof(prec->phas);
  pdbRecordType->papFldDes[5]->offset=(short)((char *)&prec->phas - (char *)prec);
  pdbRecordType->papFldDes[6]->size=sizeof(prec->evnt);
  pdbRecordType->papFldDes[6]->offset=(short)((char *)&prec->evnt - (char *)prec);
  pdbRecordType->papFldDes[7]->size=sizeof(prec->tse);
  pdbRecordType->papFldDes[7]->offset=(short)((char *)&prec->tse - (char *)prec);
  pdbRecordType->papFldDes[8]->size=sizeof(prec->tsel);
  pdbRecordType->papFldDes[8]->offset=(short)((char *)&prec->tsel - (char *)prec);
  pdbRecordType->papFldDes[9]->size=sizeof(prec->dtyp);
  pdbRecordType->papFldDes[9]->offset=(short)((char *)&prec->dtyp - (char *)prec);
  pdbRecordType->papFldDes[10]->size=sizeof(prec->disv);
  pdbRecordType->papFldDes[10]->offset=(short)((char *)&prec->disv - (char *)prec);
  pdbRecordType->papFldDes[11]->size=sizeof(prec->disa);
  pdbRecordType->papFldDes[11]->offset=(short)((char *)&prec->disa - (char *)prec);
  pdbRecordType->papFldDes[12]->size=sizeof(prec->sdis);
  pdbRecordType->papFldDes[12]->offset=(short)((char *)&prec->sdis - (char *)prec);
  pdbRecordType->papFldDes[13]->size=sizeof(prec->mlok);
  pdbRecordType->papFldDes[13]->offset=(short)((char *)&prec->mlok - (char *)prec);
  pdbRecordType->papFldDes[14]->size=sizeof(prec->mlis);
  pdbRecordType->papFldDes[14]->offset=(short)((char *)&prec->mlis - (char *)prec);
  pdbRecordType->papFldDes[15]->size=sizeof(prec->disp);
  pdbRecordType->papFldDes[15]->offset=(short)((char *)&prec->disp - (char *)prec);
  pdbRecordType->papFldDes[16]->size=sizeof(prec->proc);
  pdbRecordType->papFldDes[16]->offset=(short)((char *)&prec->proc - (char *)prec);
  pdbRecordType->papFldDes[17]->size=sizeof(prec->stat);
  pdbRecordType->papFldDes[17]->offset=(short)((char *)&prec->stat - (char *)prec);
  pdbRecordType->papFldDes[18]->size=sizeof(prec->sevr);
  pdbRecordType->papFldDes[18]->offset=(short)((char *)&prec->sevr - (char *)prec);
  pdbRecordType->papFldDes[19]->size=sizeof(prec->nsta);
  pdbRecordType->papFldDes[19]->offset=(short)((char *)&prec->nsta - (char *)prec);
  pdbRecordType->papFldDes[20]->size=sizeof(prec->nsev);
  pdbRecordType->papFldDes[20]->offset=(short)((char *)&prec->nsev - (char *)prec);
  pdbRecordType->papFldDes[21]->size=sizeof(prec->acks);
  pdbRecordType->papFldDes[21]->offset=(short)((char *)&prec->acks - (char *)prec);
  pdbRecordType->papFldDes[22]->size=sizeof(prec->ackt);
  pdbRecordType->papFldDes[22]->offset=(short)((char *)&prec->ackt - (char *)prec);
  pdbRecordType->papFldDes[23]->size=sizeof(prec->diss);
  pdbRecordType->papFldDes[23]->offset=(short)((char *)&prec->diss - (char *)prec);
  pdbRecordType->papFldDes[24]->size=sizeof(prec->lcnt);
  pdbRecordType->papFldDes[24]->offset=(short)((char *)&prec->lcnt - (char *)prec);
  pdbRecordType->papFldDes[25]->size=sizeof(prec->pact);
  pdbRecordType->papFldDes[25]->offset=(short)((char *)&prec->pact - (char *)prec);
  pdbRecordType->papFldDes[26]->size=sizeof(prec->putf);
  pdbRecordType->papFldDes[26]->offset=(short)((char *)&prec->putf - (char *)prec);
  pdbRecordType->papFldDes[27]->size=sizeof(prec->rpro);
  pdbRecordType->papFldDes[27]->offset=(short)((char *)&prec->rpro - (char *)prec);
  pdbRecordType->papFldDes[28]->size=sizeof(prec->asp);
  pdbRecordType->papFldDes[28]->offset=(short)((char *)&prec->asp - (char *)prec);
  pdbRecordType->papFldDes[29]->size=sizeof(prec->ppn);
  pdbRecordType->papFldDes[29]->offset=(short)((char *)&prec->ppn - (char *)prec);
  pdbRecordType->papFldDes[30]->size=sizeof(prec->ppnn);
  pdbRecordType->papFldDes[30]->offset=(short)((char *)&prec->ppnn - (char *)prec);
  pdbRecordType->papFldDes[31]->size=sizeof(prec->spvt);
  pdbRecordType->papFldDes[31]->offset=(short)((char *)&prec->spvt - (char *)prec);
  pdbRecordType->papFldDes[32]->size=sizeof(prec->rset);
  pdbRecordType->papFldDes[32]->offset=(short)((char *)&prec->rset - (char *)prec);
  pdbRecordType->papFldDes[33]->size=sizeof(prec->dset);
  pdbRecordType->papFldDes[33]->offset=(short)((char *)&prec->dset - (char *)prec);
  pdbRecordType->papFldDes[34]->size=sizeof(prec->dpvt);
  pdbRecordType->papFldDes[34]->offset=(short)((char *)&prec->dpvt - (char *)prec);
  pdbRecordType->papFldDes[35]->size=sizeof(prec->rdes);
  pdbRecordType->papFldDes[35]->offset=(short)((char *)&prec->rdes - (char *)prec);
  pdbRecordType->papFldDes[36]->size=sizeof(prec->lset);
  pdbRecordType->papFldDes[36]->offset=(short)((char *)&prec->lset - (char *)prec);
  pdbRecordType->papFldDes[37]->size=sizeof(prec->prio);
  pdbRecordType->papFldDes[37]->offset=(short)((char *)&prec->prio - (char *)prec);
  pdbRecordType->papFldDes[38]->size=sizeof(prec->tpro);
  pdbRecordType->papFldDes[38]->offset=(short)((char *)&prec->tpro - (char *)prec);
  pdbRecordType->papFldDes[39]->size=sizeof(prec->bkpt);
  pdbRecordType->papFldDes[39]->offset=(short)((char *)&prec->bkpt - (char *)prec);
  pdbRecordType->papFldDes[40]->size=sizeof(prec->udf);
  pdbRecordType->papFldDes[40]->offset=(short)((char *)&prec->udf - (char *)prec);
  pdbRecordType->papFldDes[41]->size=sizeof(prec->time);
  pdbRecordType->papFldDes[41]->offset=(short)((char *)&prec->time - (char *)prec);
  pdbRecordType->papFldDes[42]->size=sizeof(prec->flnk);
  pdbRecordType->papFldDes[42]->offset=(short)((char *)&prec->flnk - (char *)prec);
  pdbRecordType->papFldDes[43]->size=sizeof(prec->vers);
  pdbRecordType->papFldDes[43]->offset=(short)((char *)&prec->vers - (char *)prec);
  pdbRecordType->papFldDes[44]->size=sizeof(prec->val);
  pdbRecordType->papFldDes[44]->offset=(short)((char *)&prec->val - (char *)prec);
  pdbRecordType->papFldDes[45]->size=sizeof(prec->smsg);
  pdbRecordType->papFldDes[45]->offset=(short)((char *)&prec->smsg - (char *)prec);
  pdbRecordType->papFldDes[46]->size=sizeof(prec->cmnd);
  pdbRecordType->papFldDes[46]->offset=(short)((char *)&prec->cmnd - (char *)prec);
  pdbRecordType->papFldDes[47]->size=sizeof(prec->alrt);
  pdbRecordType->papFldDes[47]->offset=(short)((char *)&prec->alrt - (char *)prec);
  pdbRecordType->papFldDes[48]->size=sizeof(prec->rpvt);
  pdbRecordType->papFldDes[48]->offset=(short)((char *)&prec->rpvt - (char *)prec);
  pdbRecordType->papFldDes[49]->size=sizeof(prec->mpts);
  pdbRecordType->papFldDes[49]->offset=(short)((char *)&prec->mpts - (char *)prec);
  pdbRecordType->papFldDes[50]->size=sizeof(prec->exsc);
  pdbRecordType->papFldDes[50]->offset=(short)((char *)&prec->exsc - (char *)prec);
  pdbRecordType->papFldDes[51]->size=sizeof(prec->xsc);
  pdbRecordType->papFldDes[51]->offset=(short)((char *)&prec->xsc - (char *)prec);
  pdbRecordType->papFldDes[52]->size=sizeof(prec->pxsc);
  pdbRecordType->papFldDes[52]->offset=(short)((char *)&prec->pxsc - (char *)prec);
  pdbRecordType->papFldDes[53]->size=sizeof(prec->busy);
  pdbRecordType->papFldDes[53]->offset=(short)((char *)&prec->busy - (char *)prec);
  pdbRecordType->papFldDes[54]->size=sizeof(prec->kill);
  pdbRecordType->papFldDes[54]->offset=(short)((char *)&prec->kill - (char *)prec);
  pdbRecordType->papFldDes[55]->size=sizeof(prec->wait);
  pdbRecordType->papFldDes[55]->offset=(short)((char *)&prec->wait - (char *)prec);
  pdbRecordType->papFldDes[56]->size=sizeof(prec->wcnt);
  pdbRecordType->papFldDes[56]->offset=(short)((char *)&prec->wcnt - (char *)prec);
  pdbRecordType->papFldDes[57]->size=sizeof(prec->awct);
  pdbRecordType->papFldDes[57]->offset=(short)((char *)&prec->awct - (char *)prec);
  pdbRecordType->papFldDes[58]->size=sizeof(prec->wtng);
  pdbRecordType->papFldDes[58]->offset=(short)((char *)&prec->wtng - (char *)prec);
  pdbRecordType->papFldDes[59]->size=sizeof(prec->data);
  pdbRecordType->papFldDes[59]->offset=(short)((char *)&prec->data - (char *)prec);
  pdbRecordType->papFldDes[60]->size=sizeof(prec->refd);
  pdbRecordType->papFldDes[60]->offset=(short)((char *)&prec->refd - (char *)prec);
  pdbRecordType->papFldDes[61]->size=sizeof(prec->npts);
  pdbRecordType->papFldDes[61]->offset=(short)((char *)&prec->npts - (char *)prec);
  pdbRecordType->papFldDes[62]->size=sizeof(prec->fpts);
  pdbRecordType->papFldDes[62]->offset=(short)((char *)&prec->fpts - (char *)prec);
  pdbRecordType->papFldDes[63]->size=sizeof(prec->ffo);
  pdbRecordType->papFldDes[63]->offset=(short)((char *)&prec->ffo - (char *)prec);
  pdbRecordType->papFldDes[64]->size=sizeof(prec->cpt);
  pdbRecordType->papFldDes[64]->offset=(short)((char *)&prec->cpt - (char *)prec);
  pdbRecordType->papFldDes[65]->size=sizeof(prec->dpt);
  pdbRecordType->papFldDes[65]->offset=(short)((char *)&prec->dpt - (char *)prec);
  pdbRecordType->papFldDes[66]->size=sizeof(prec->pcpt);
  pdbRecordType->papFldDes[66]->offset=(short)((char *)&prec->pcpt - (char *)prec);
  pdbRecordType->papFldDes[67]->size=sizeof(prec->pasm);
  pdbRecordType->papFldDes[67]->offset=(short)((char *)&prec->pasm - (char *)prec);
  pdbRecordType->papFldDes[68]->size=sizeof(prec->tolp);
  pdbRecordType->papFldDes[68]->offset=(short)((char *)&prec->tolp - (char *)prec);
  pdbRecordType->papFldDes[69]->size=sizeof(prec->p1pv);
  pdbRecordType->papFldDes[69]->offset=(short)((char *)&prec->p1pv - (char *)prec);
  pdbRecordType->papFldDes[70]->size=sizeof(prec->p2pv);
  pdbRecordType->papFldDes[70]->offset=(short)((char *)&prec->p2pv - (char *)prec);
  pdbRecordType->papFldDes[71]->size=sizeof(prec->p3pv);
  pdbRecordType->papFldDes[71]->offset=(short)((char *)&prec->p3pv - (char *)prec);
  pdbRecordType->papFldDes[72]->size=sizeof(prec->p4pv);
  pdbRecordType->papFldDes[72]->offset=(short)((char *)&prec->p4pv - (char *)prec);
  pdbRecordType->papFldDes[73]->size=sizeof(prec->r1pv);
  pdbRecordType->papFldDes[73]->offset=(short)((char *)&prec->r1pv - (char *)prec);
  pdbRecordType->papFldDes[74]->size=sizeof(prec->r2pv);
  pdbRecordType->papFldDes[74]->offset=(short)((char *)&prec->r2pv - (char *)prec);
  pdbRecordType->papFldDes[75]->size=sizeof(prec->r3pv);
  pdbRecordType->papFldDes[75]->offset=(short)((char *)&prec->r3pv - (char *)prec);
  pdbRecordType->papFldDes[76]->size=sizeof(prec->r4pv);
  pdbRecordType->papFldDes[76]->offset=(short)((char *)&prec->r4pv - (char *)prec);
  pdbRecordType->papFldDes[77]->size=sizeof(prec->d01pv);
  pdbRecordType->papFldDes[77]->offset=(short)((char *)&prec->d01pv - (char *)prec);
  pdbRecordType->papFldDes[78]->size=sizeof(prec->d02pv);
  pdbRecordType->papFldDes[78]->offset=(short)((char *)&prec->d02pv - (char *)prec);
  pdbRecordType->papFldDes[79]->size=sizeof(prec->d03pv);
  pdbRecordType->papFldDes[79]->offset=(short)((char *)&prec->d03pv - (char *)prec);
  pdbRecordType->papFldDes[80]->size=sizeof(prec->d04pv);
  pdbRecordType->papFldDes[80]->offset=(short)((char *)&prec->d04pv - (char *)prec);
  pdbRecordType->papFldDes[81]->size=sizeof(prec->d05pv);
  pdbRecordType->papFldDes[81]->offset=(short)((char *)&prec->d05pv - (char *)prec);
  pdbRecordType->papFldDes[82]->size=sizeof(prec->d06pv);
  pdbRecordType->papFldDes[82]->offset=(short)((char *)&prec->d06pv - (char *)prec);
  pdbRecordType->papFldDes[83]->size=sizeof(prec->d07pv);
  pdbRecordType->papFldDes[83]->offset=(short)((char *)&prec->d07pv - (char *)prec);
  pdbRecordType->papFldDes[84]->size=sizeof(prec->d08pv);
  pdbRecordType->papFldDes[84]->offset=(short)((char *)&prec->d08pv - (char *)prec);
  pdbRecordType->papFldDes[85]->size=sizeof(prec->d09pv);
  pdbRecordType->papFldDes[85]->offset=(short)((char *)&prec->d09pv - (char *)prec);
  pdbRecordType->papFldDes[86]->size=sizeof(prec->d10pv);
  pdbRecordType->papFldDes[86]->offset=(short)((char *)&prec->d10pv - (char *)prec);
  pdbRecordType->papFldDes[87]->size=sizeof(prec->d11pv);
  pdbRecordType->papFldDes[87]->offset=(short)((char *)&prec->d11pv - (char *)prec);
  pdbRecordType->papFldDes[88]->size=sizeof(prec->d12pv);
  pdbRecordType->papFldDes[88]->offset=(short)((char *)&prec->d12pv - (char *)prec);
  pdbRecordType->papFldDes[89]->size=sizeof(prec->d13pv);
  pdbRecordType->papFldDes[89]->offset=(short)((char *)&prec->d13pv - (char *)prec);
  pdbRecordType->papFldDes[90]->size=sizeof(prec->d14pv);
  pdbRecordType->papFldDes[90]->offset=(short)((char *)&prec->d14pv - (char *)prec);
  pdbRecordType->papFldDes[91]->size=sizeof(prec->d15pv);
  pdbRecordType->papFldDes[91]->offset=(short)((char *)&prec->d15pv - (char *)prec);
  pdbRecordType->papFldDes[92]->size=sizeof(prec->d16pv);
  pdbRecordType->papFldDes[92]->offset=(short)((char *)&prec->d16pv - (char *)prec);
  pdbRecordType->papFldDes[93]->size=sizeof(prec->d17pv);
  pdbRecordType->papFldDes[93]->offset=(short)((char *)&prec->d17pv - (char *)prec);
  pdbRecordType->papFldDes[94]->size=sizeof(prec->d18pv);
  pdbRecordType->papFldDes[94]->offset=(short)((char *)&prec->d18pv - (char *)prec);
  pdbRecordType->papFldDes[95]->size=sizeof(prec->d19pv);
  pdbRecordType->papFldDes[95]->offset=(short)((char *)&prec->d19pv - (char *)prec);
  pdbRecordType->papFldDes[96]->size=sizeof(prec->d20pv);
  pdbRecordType->papFldDes[96]->offset=(short)((char *)&prec->d20pv - (char *)prec);
  pdbRecordType->papFldDes[97]->size=sizeof(prec->d21pv);
  pdbRecordType->papFldDes[97]->offset=(short)((char *)&prec->d21pv - (char *)prec);
  pdbRecordType->papFldDes[98]->size=sizeof(prec->d22pv);
  pdbRecordType->papFldDes[98]->offset=(short)((char *)&prec->d22pv - (char *)prec);
  pdbRecordType->papFldDes[99]->size=sizeof(prec->d23pv);
  pdbRecordType->papFldDes[99]->offset=(short)((char *)&prec->d23pv - (char *)prec);
  pdbRecordType->papFldDes[100]->size=sizeof(prec->d24pv);
  pdbRecordType->papFldDes[100]->offset=(short)((char *)&prec->d24pv - (char *)prec);
  pdbRecordType->papFldDes[101]->size=sizeof(prec->d25pv);
  pdbRecordType->papFldDes[101]->offset=(short)((char *)&prec->d25pv - (char *)prec);
  pdbRecordType->papFldDes[102]->size=sizeof(prec->d26pv);
  pdbRecordType->papFldDes[102]->offset=(short)((char *)&prec->d26pv - (char *)prec);
  pdbRecordType->papFldDes[103]->size=sizeof(prec->d27pv);
  pdbRecordType->papFldDes[103]->offset=(short)((char *)&prec->d27pv - (char *)prec);
  pdbRecordType->papFldDes[104]->size=sizeof(prec->d28pv);
  pdbRecordType->papFldDes[104]->offset=(short)((char *)&prec->d28pv - (char *)prec);
  pdbRecordType->papFldDes[105]->size=sizeof(prec->d29pv);
  pdbRecordType->papFldDes[105]->offset=(short)((char *)&prec->d29pv - (char *)prec);
  pdbRecordType->papFldDes[106]->size=sizeof(prec->d30pv);
  pdbRecordType->papFldDes[106]->offset=(short)((char *)&prec->d30pv - (char *)prec);
  pdbRecordType->papFldDes[107]->size=sizeof(prec->d31pv);
  pdbRecordType->papFldDes[107]->offset=(short)((char *)&prec->d31pv - (char *)prec);
  pdbRecordType->papFldDes[108]->size=sizeof(prec->d32pv);
  pdbRecordType->papFldDes[108]->offset=(short)((char *)&prec->d32pv - (char *)prec);
  pdbRecordType->papFldDes[109]->size=sizeof(prec->d33pv);
  pdbRecordType->papFldDes[109]->offset=(short)((char *)&prec->d33pv - (char *)prec);
  pdbRecordType->papFldDes[110]->size=sizeof(prec->d34pv);
  pdbRecordType->papFldDes[110]->offset=(short)((char *)&prec->d34pv - (char *)prec);
  pdbRecordType->papFldDes[111]->size=sizeof(prec->d35pv);
  pdbRecordType->papFldDes[111]->offset=(short)((char *)&prec->d35pv - (char *)prec);
  pdbRecordType->papFldDes[112]->size=sizeof(prec->d36pv);
  pdbRecordType->papFldDes[112]->offset=(short)((char *)&prec->d36pv - (char *)prec);
  pdbRecordType->papFldDes[113]->size=sizeof(prec->d37pv);
  pdbRecordType->papFldDes[113]->offset=(short)((char *)&prec->d37pv - (char *)prec);
  pdbRecordType->papFldDes[114]->size=sizeof(prec->d38pv);
  pdbRecordType->papFldDes[114]->offset=(short)((char *)&prec->d38pv - (char *)prec);
  pdbRecordType->papFldDes[115]->size=sizeof(prec->d39pv);
  pdbRecordType->papFldDes[115]->offset=(short)((char *)&prec->d39pv - (char *)prec);
  pdbRecordType->papFldDes[116]->size=sizeof(prec->d40pv);
  pdbRecordType->papFldDes[116]->offset=(short)((char *)&prec->d40pv - (char *)prec);
  pdbRecordType->papFldDes[117]->size=sizeof(prec->d41pv);
  pdbRecordType->papFldDes[117]->offset=(short)((char *)&prec->d41pv - (char *)prec);
  pdbRecordType->papFldDes[118]->size=sizeof(prec->d42pv);
  pdbRecordType->papFldDes[118]->offset=(short)((char *)&prec->d42pv - (char *)prec);
  pdbRecordType->papFldDes[119]->size=sizeof(prec->d43pv);
  pdbRecordType->papFldDes[119]->offset=(short)((char *)&prec->d43pv - (char *)prec);
  pdbRecordType->papFldDes[120]->size=sizeof(prec->d44pv);
  pdbRecordType->papFldDes[120]->offset=(short)((char *)&prec->d44pv - (char *)prec);
  pdbRecordType->papFldDes[121]->size=sizeof(prec->d45pv);
  pdbRecordType->papFldDes[121]->offset=(short)((char *)&prec->d45pv - (char *)prec);
  pdbRecordType->papFldDes[122]->size=sizeof(prec->d46pv);
  pdbRecordType->papFldDes[122]->offset=(short)((char *)&prec->d46pv - (char *)prec);
  pdbRecordType->papFldDes[123]->size=sizeof(prec->d47pv);
  pdbRecordType->papFldDes[123]->offset=(short)((char *)&prec->d47pv - (char *)prec);
  pdbRecordType->papFldDes[124]->size=sizeof(prec->d48pv);
  pdbRecordType->papFldDes[124]->offset=(short)((char *)&prec->d48pv - (char *)prec);
  pdbRecordType->papFldDes[125]->size=sizeof(prec->d49pv);
  pdbRecordType->papFldDes[125]->offset=(short)((char *)&prec->d49pv - (char *)prec);
  pdbRecordType->papFldDes[126]->size=sizeof(prec->d50pv);
  pdbRecordType->papFldDes[126]->offset=(short)((char *)&prec->d50pv - (char *)prec);
  pdbRecordType->papFldDes[127]->size=sizeof(prec->d51pv);
  pdbRecordType->papFldDes[127]->offset=(short)((char *)&prec->d51pv - (char *)prec);
  pdbRecordType->papFldDes[128]->size=sizeof(prec->d52pv);
  pdbRecordType->papFldDes[128]->offset=(short)((char *)&prec->d52pv - (char *)prec);
  pdbRecordType->papFldDes[129]->size=sizeof(prec->d53pv);
  pdbRecordType->papFldDes[129]->offset=(short)((char *)&prec->d53pv - (char *)prec);
  pdbRecordType->papFldDes[130]->size=sizeof(prec->d54pv);
  pdbRecordType->papFldDes[130]->offset=(short)((char *)&prec->d54pv - (char *)prec);
  pdbRecordType->papFldDes[131]->size=sizeof(prec->d55pv);
  pdbRecordType->papFldDes[131]->offset=(short)((char *)&prec->d55pv - (char *)prec);
  pdbRecordType->papFldDes[132]->size=sizeof(prec->d56pv);
  pdbRecordType->papFldDes[132]->offset=(short)((char *)&prec->d56pv - (char *)prec);
  pdbRecordType->papFldDes[133]->size=sizeof(prec->d57pv);
  pdbRecordType->papFldDes[133]->offset=(short)((char *)&prec->d57pv - (char *)prec);
  pdbRecordType->papFldDes[134]->size=sizeof(prec->d58pv);
  pdbRecordType->papFldDes[134]->offset=(short)((char *)&prec->d58pv - (char *)prec);
  pdbRecordType->papFldDes[135]->size=sizeof(prec->d59pv);
  pdbRecordType->papFldDes[135]->offset=(short)((char *)&prec->d59pv - (char *)prec);
  pdbRecordType->papFldDes[136]->size=sizeof(prec->d60pv);
  pdbRecordType->papFldDes[136]->offset=(short)((char *)&prec->d60pv - (char *)prec);
  pdbRecordType->papFldDes[137]->size=sizeof(prec->d61pv);
  pdbRecordType->papFldDes[137]->offset=(short)((char *)&prec->d61pv - (char *)prec);
  pdbRecordType->papFldDes[138]->size=sizeof(prec->d62pv);
  pdbRecordType->papFldDes[138]->offset=(short)((char *)&prec->d62pv - (char *)prec);
  pdbRecordType->papFldDes[139]->size=sizeof(prec->d63pv);
  pdbRecordType->papFldDes[139]->offset=(short)((char *)&prec->d63pv - (char *)prec);
  pdbRecordType->papFldDes[140]->size=sizeof(prec->d64pv);
  pdbRecordType->papFldDes[140]->offset=(short)((char *)&prec->d64pv - (char *)prec);
  pdbRecordType->papFldDes[141]->size=sizeof(prec->d65pv);
  pdbRecordType->papFldDes[141]->offset=(short)((char *)&prec->d65pv - (char *)prec);
  pdbRecordType->papFldDes[142]->size=sizeof(prec->d66pv);
  pdbRecordType->papFldDes[142]->offset=(short)((char *)&prec->d66pv - (char *)prec);
  pdbRecordType->papFldDes[143]->size=sizeof(prec->d67pv);
  pdbRecordType->papFldDes[143]->offset=(short)((char *)&prec->d67pv - (char *)prec);
  pdbRecordType->papFldDes[144]->size=sizeof(prec->d68pv);
  pdbRecordType->papFldDes[144]->offset=(short)((char *)&prec->d68pv - (char *)prec);
  pdbRecordType->papFldDes[145]->size=sizeof(prec->d69pv);
  pdbRecordType->papFldDes[145]->offset=(short)((char *)&prec->d69pv - (char *)prec);
  pdbRecordType->papFldDes[146]->size=sizeof(prec->d70pv);
  pdbRecordType->papFldDes[146]->offset=(short)((char *)&prec->d70pv - (char *)prec);
  pdbRecordType->papFldDes[147]->size=sizeof(prec->d1pv);
  pdbRecordType->papFldDes[147]->offset=(short)((char *)&prec->d1pv - (char *)prec);
  pdbRecordType->papFldDes[148]->size=sizeof(prec->d2pv);
  pdbRecordType->papFldDes[148]->offset=(short)((char *)&prec->d2pv - (char *)prec);
  pdbRecordType->papFldDes[149]->size=sizeof(prec->d3pv);
  pdbRecordType->papFldDes[149]->offset=(short)((char *)&prec->d3pv - (char *)prec);
  pdbRecordType->papFldDes[150]->size=sizeof(prec->d4pv);
  pdbRecordType->papFldDes[150]->offset=(short)((char *)&prec->d4pv - (char *)prec);
  pdbRecordType->papFldDes[151]->size=sizeof(prec->d5pv);
  pdbRecordType->papFldDes[151]->offset=(short)((char *)&prec->d5pv - (char *)prec);
  pdbRecordType->papFldDes[152]->size=sizeof(prec->d6pv);
  pdbRecordType->papFldDes[152]->offset=(short)((char *)&prec->d6pv - (char *)prec);
  pdbRecordType->papFldDes[153]->size=sizeof(prec->d7pv);
  pdbRecordType->papFldDes[153]->offset=(short)((char *)&prec->d7pv - (char *)prec);
  pdbRecordType->papFldDes[154]->size=sizeof(prec->d8pv);
  pdbRecordType->papFldDes[154]->offset=(short)((char *)&prec->d8pv - (char *)prec);
  pdbRecordType->papFldDes[155]->size=sizeof(prec->d9pv);
  pdbRecordType->papFldDes[155]->offset=(short)((char *)&prec->d9pv - (char *)prec);
  pdbRecordType->papFldDes[156]->size=sizeof(prec->dapv);
  pdbRecordType->papFldDes[156]->offset=(short)((char *)&prec->dapv - (char *)prec);
  pdbRecordType->papFldDes[157]->size=sizeof(prec->dbpv);
  pdbRecordType->papFldDes[157]->offset=(short)((char *)&prec->dbpv - (char *)prec);
  pdbRecordType->papFldDes[158]->size=sizeof(prec->dcpv);
  pdbRecordType->papFldDes[158]->offset=(short)((char *)&prec->dcpv - (char *)prec);
  pdbRecordType->papFldDes[159]->size=sizeof(prec->ddpv);
  pdbRecordType->papFldDes[159]->offset=(short)((char *)&prec->ddpv - (char *)prec);
  pdbRecordType->papFldDes[160]->size=sizeof(prec->depv);
  pdbRecordType->papFldDes[160]->offset=(short)((char *)&prec->depv - (char *)prec);
  pdbRecordType->papFldDes[161]->size=sizeof(prec->dfpv);
  pdbRecordType->papFldDes[161]->offset=(short)((char *)&prec->dfpv - (char *)prec);
  pdbRecordType->papFldDes[162]->size=sizeof(prec->t1pv);
  pdbRecordType->papFldDes[162]->offset=(short)((char *)&prec->t1pv - (char *)prec);
  pdbRecordType->papFldDes[163]->size=sizeof(prec->t2pv);
  pdbRecordType->papFldDes[163]->offset=(short)((char *)&prec->t2pv - (char *)prec);
  pdbRecordType->papFldDes[164]->size=sizeof(prec->t3pv);
  pdbRecordType->papFldDes[164]->offset=(short)((char *)&prec->t3pv - (char *)prec);
  pdbRecordType->papFldDes[165]->size=sizeof(prec->t4pv);
  pdbRecordType->papFldDes[165]->offset=(short)((char *)&prec->t4pv - (char *)prec);
  pdbRecordType->papFldDes[166]->size=sizeof(prec->bspv);
  pdbRecordType->papFldDes[166]->offset=(short)((char *)&prec->bspv - (char *)prec);
  pdbRecordType->papFldDes[167]->size=sizeof(prec->aspv);
  pdbRecordType->papFldDes[167]->offset=(short)((char *)&prec->aspv - (char *)prec);
  pdbRecordType->papFldDes[168]->size=sizeof(prec->p1nv);
  pdbRecordType->papFldDes[168]->offset=(short)((char *)&prec->p1nv - (char *)prec);
  pdbRecordType->papFldDes[169]->size=sizeof(prec->p2nv);
  pdbRecordType->papFldDes[169]->offset=(short)((char *)&prec->p2nv - (char *)prec);
  pdbRecordType->papFldDes[170]->size=sizeof(prec->p3nv);
  pdbRecordType->papFldDes[170]->offset=(short)((char *)&prec->p3nv - (char *)prec);
  pdbRecordType->papFldDes[171]->size=sizeof(prec->p4nv);
  pdbRecordType->papFldDes[171]->offset=(short)((char *)&prec->p4nv - (char *)prec);
  pdbRecordType->papFldDes[172]->size=sizeof(prec->r1nv);
  pdbRecordType->papFldDes[172]->offset=(short)((char *)&prec->r1nv - (char *)prec);
  pdbRecordType->papFldDes[173]->size=sizeof(prec->r2nv);
  pdbRecordType->papFldDes[173]->offset=(short)((char *)&prec->r2nv - (char *)prec);
  pdbRecordType->papFldDes[174]->size=sizeof(prec->r3nv);
  pdbRecordType->papFldDes[174]->offset=(short)((char *)&prec->r3nv - (char *)prec);
  pdbRecordType->papFldDes[175]->size=sizeof(prec->r4nv);
  pdbRecordType->papFldDes[175]->offset=(short)((char *)&prec->r4nv - (char *)prec);
  pdbRecordType->papFldDes[176]->size=sizeof(prec->d01nv);
  pdbRecordType->papFldDes[176]->offset=(short)((char *)&prec->d01nv - (char *)prec);
  pdbRecordType->papFldDes[177]->size=sizeof(prec->d02nv);
  pdbRecordType->papFldDes[177]->offset=(short)((char *)&prec->d02nv - (char *)prec);
  pdbRecordType->papFldDes[178]->size=sizeof(prec->d03nv);
  pdbRecordType->papFldDes[178]->offset=(short)((char *)&prec->d03nv - (char *)prec);
  pdbRecordType->papFldDes[179]->size=sizeof(prec->d04nv);
  pdbRecordType->papFldDes[179]->offset=(short)((char *)&prec->d04nv - (char *)prec);
  pdbRecordType->papFldDes[180]->size=sizeof(prec->d05nv);
  pdbRecordType->papFldDes[180]->offset=(short)((char *)&prec->d05nv - (char *)prec);
  pdbRecordType->papFldDes[181]->size=sizeof(prec->d06nv);
  pdbRecordType->papFldDes[181]->offset=(short)((char *)&prec->d06nv - (char *)prec);
  pdbRecordType->papFldDes[182]->size=sizeof(prec->d07nv);
  pdbRecordType->papFldDes[182]->offset=(short)((char *)&prec->d07nv - (char *)prec);
  pdbRecordType->papFldDes[183]->size=sizeof(prec->d08nv);
  pdbRecordType->papFldDes[183]->offset=(short)((char *)&prec->d08nv - (char *)prec);
  pdbRecordType->papFldDes[184]->size=sizeof(prec->d09nv);
  pdbRecordType->papFldDes[184]->offset=(short)((char *)&prec->d09nv - (char *)prec);
  pdbRecordType->papFldDes[185]->size=sizeof(prec->d10nv);
  pdbRecordType->papFldDes[185]->offset=(short)((char *)&prec->d10nv - (char *)prec);
  pdbRecordType->papFldDes[186]->size=sizeof(prec->d11nv);
  pdbRecordType->papFldDes[186]->offset=(short)((char *)&prec->d11nv - (char *)prec);
  pdbRecordType->papFldDes[187]->size=sizeof(prec->d12nv);
  pdbRecordType->papFldDes[187]->offset=(short)((char *)&prec->d12nv - (char *)prec);
  pdbRecordType->papFldDes[188]->size=sizeof(prec->d13nv);
  pdbRecordType->papFldDes[188]->offset=(short)((char *)&prec->d13nv - (char *)prec);
  pdbRecordType->papFldDes[189]->size=sizeof(prec->d14nv);
  pdbRecordType->papFldDes[189]->offset=(short)((char *)&prec->d14nv - (char *)prec);
  pdbRecordType->papFldDes[190]->size=sizeof(prec->d15nv);
  pdbRecordType->papFldDes[190]->offset=(short)((char *)&prec->d15nv - (char *)prec);
  pdbRecordType->papFldDes[191]->size=sizeof(prec->d16nv);
  pdbRecordType->papFldDes[191]->offset=(short)((char *)&prec->d16nv - (char *)prec);
  pdbRecordType->papFldDes[192]->size=sizeof(prec->d17nv);
  pdbRecordType->papFldDes[192]->offset=(short)((char *)&prec->d17nv - (char *)prec);
  pdbRecordType->papFldDes[193]->size=sizeof(prec->d18nv);
  pdbRecordType->papFldDes[193]->offset=(short)((char *)&prec->d18nv - (char *)prec);
  pdbRecordType->papFldDes[194]->size=sizeof(prec->d19nv);
  pdbRecordType->papFldDes[194]->offset=(short)((char *)&prec->d19nv - (char *)prec);
  pdbRecordType->papFldDes[195]->size=sizeof(prec->d20nv);
  pdbRecordType->papFldDes[195]->offset=(short)((char *)&prec->d20nv - (char *)prec);
  pdbRecordType->papFldDes[196]->size=sizeof(prec->d21nv);
  pdbRecordType->papFldDes[196]->offset=(short)((char *)&prec->d21nv - (char *)prec);
  pdbRecordType->papFldDes[197]->size=sizeof(prec->d22nv);
  pdbRecordType->papFldDes[197]->offset=(short)((char *)&prec->d22nv - (char *)prec);
  pdbRecordType->papFldDes[198]->size=sizeof(prec->d23nv);
  pdbRecordType->papFldDes[198]->offset=(short)((char *)&prec->d23nv - (char *)prec);
  pdbRecordType->papFldDes[199]->size=sizeof(prec->d24nv);
  pdbRecordType->papFldDes[199]->offset=(short)((char *)&prec->d24nv - (char *)prec);
  pdbRecordType->papFldDes[200]->size=sizeof(prec->d25nv);
  pdbRecordType->papFldDes[200]->offset=(short)((char *)&prec->d25nv - (char *)prec);
  pdbRecordType->papFldDes[201]->size=sizeof(prec->d26nv);
  pdbRecordType->papFldDes[201]->offset=(short)((char *)&prec->d26nv - (char *)prec);
  pdbRecordType->papFldDes[202]->size=sizeof(prec->d27nv);
  pdbRecordType->papFldDes[202]->offset=(short)((char *)&prec->d27nv - (char *)prec);
  pdbRecordType->papFldDes[203]->size=sizeof(prec->d28nv);
  pdbRecordType->papFldDes[203]->offset=(short)((char *)&prec->d28nv - (char *)prec);
  pdbRecordType->papFldDes[204]->size=sizeof(prec->d29nv);
  pdbRecordType->papFldDes[204]->offset=(short)((char *)&prec->d29nv - (char *)prec);
  pdbRecordType->papFldDes[205]->size=sizeof(prec->d30nv);
  pdbRecordType->papFldDes[205]->offset=(short)((char *)&prec->d30nv - (char *)prec);
  pdbRecordType->papFldDes[206]->size=sizeof(prec->d31nv);
  pdbRecordType->papFldDes[206]->offset=(short)((char *)&prec->d31nv - (char *)prec);
  pdbRecordType->papFldDes[207]->size=sizeof(prec->d32nv);
  pdbRecordType->papFldDes[207]->offset=(short)((char *)&prec->d32nv - (char *)prec);
  pdbRecordType->papFldDes[208]->size=sizeof(prec->d33nv);
  pdbRecordType->papFldDes[208]->offset=(short)((char *)&prec->d33nv - (char *)prec);
  pdbRecordType->papFldDes[209]->size=sizeof(prec->d34nv);
  pdbRecordType->papFldDes[209]->offset=(short)((char *)&prec->d34nv - (char *)prec);
  pdbRecordType->papFldDes[210]->size=sizeof(prec->d35nv);
  pdbRecordType->papFldDes[210]->offset=(short)((char *)&prec->d35nv - (char *)prec);
  pdbRecordType->papFldDes[211]->size=sizeof(prec->d36nv);
  pdbRecordType->papFldDes[211]->offset=(short)((char *)&prec->d36nv - (char *)prec);
  pdbRecordType->papFldDes[212]->size=sizeof(prec->d37nv);
  pdbRecordType->papFldDes[212]->offset=(short)((char *)&prec->d37nv - (char *)prec);
  pdbRecordType->papFldDes[213]->size=sizeof(prec->d38nv);
  pdbRecordType->papFldDes[213]->offset=(short)((char *)&prec->d38nv - (char *)prec);
  pdbRecordType->papFldDes[214]->size=sizeof(prec->d39nv);
  pdbRecordType->papFldDes[214]->offset=(short)((char *)&prec->d39nv - (char *)prec);
  pdbRecordType->papFldDes[215]->size=sizeof(prec->d40nv);
  pdbRecordType->papFldDes[215]->offset=(short)((char *)&prec->d40nv - (char *)prec);
  pdbRecordType->papFldDes[216]->size=sizeof(prec->d41nv);
  pdbRecordType->papFldDes[216]->offset=(short)((char *)&prec->d41nv - (char *)prec);
  pdbRecordType->papFldDes[217]->size=sizeof(prec->d42nv);
  pdbRecordType->papFldDes[217]->offset=(short)((char *)&prec->d42nv - (char *)prec);
  pdbRecordType->papFldDes[218]->size=sizeof(prec->d43nv);
  pdbRecordType->papFldDes[218]->offset=(short)((char *)&prec->d43nv - (char *)prec);
  pdbRecordType->papFldDes[219]->size=sizeof(prec->d44nv);
  pdbRecordType->papFldDes[219]->offset=(short)((char *)&prec->d44nv - (char *)prec);
  pdbRecordType->papFldDes[220]->size=sizeof(prec->d45nv);
  pdbRecordType->papFldDes[220]->offset=(short)((char *)&prec->d45nv - (char *)prec);
  pdbRecordType->papFldDes[221]->size=sizeof(prec->d46nv);
  pdbRecordType->papFldDes[221]->offset=(short)((char *)&prec->d46nv - (char *)prec);
  pdbRecordType->papFldDes[222]->size=sizeof(prec->d47nv);
  pdbRecordType->papFldDes[222]->offset=(short)((char *)&prec->d47nv - (char *)prec);
  pdbRecordType->papFldDes[223]->size=sizeof(prec->d48nv);
  pdbRecordType->papFldDes[223]->offset=(short)((char *)&prec->d48nv - (char *)prec);
  pdbRecordType->papFldDes[224]->size=sizeof(prec->d49nv);
  pdbRecordType->papFldDes[224]->offset=(short)((char *)&prec->d49nv - (char *)prec);
  pdbRecordType->papFldDes[225]->size=sizeof(prec->d50nv);
  pdbRecordType->papFldDes[225]->offset=(short)((char *)&prec->d50nv - (char *)prec);
  pdbRecordType->papFldDes[226]->size=sizeof(prec->d51nv);
  pdbRecordType->papFldDes[226]->offset=(short)((char *)&prec->d51nv - (char *)prec);
  pdbRecordType->papFldDes[227]->size=sizeof(prec->d52nv);
  pdbRecordType->papFldDes[227]->offset=(short)((char *)&prec->d52nv - (char *)prec);
  pdbRecordType->papFldDes[228]->size=sizeof(prec->d53nv);
  pdbRecordType->papFldDes[228]->offset=(short)((char *)&prec->d53nv - (char *)prec);
  pdbRecordType->papFldDes[229]->size=sizeof(prec->d54nv);
  pdbRecordType->papFldDes[229]->offset=(short)((char *)&prec->d54nv - (char *)prec);
  pdbRecordType->papFldDes[230]->size=sizeof(prec->d55nv);
  pdbRecordType->papFldDes[230]->offset=(short)((char *)&prec->d55nv - (char *)prec);
  pdbRecordType->papFldDes[231]->size=sizeof(prec->d56nv);
  pdbRecordType->papFldDes[231]->offset=(short)((char *)&prec->d56nv - (char *)prec);
  pdbRecordType->papFldDes[232]->size=sizeof(prec->d57nv);
  pdbRecordType->papFldDes[232]->offset=(short)((char *)&prec->d57nv - (char *)prec);
  pdbRecordType->papFldDes[233]->size=sizeof(prec->d58nv);
  pdbRecordType->papFldDes[233]->offset=(short)((char *)&prec->d58nv - (char *)prec);
  pdbRecordType->papFldDes[234]->size=sizeof(prec->d59nv);
  pdbRecordType->papFldDes[234]->offset=(short)((char *)&prec->d59nv - (char *)prec);
  pdbRecordType->papFldDes[235]->size=sizeof(prec->d60nv);
  pdbRecordType->papFldDes[235]->offset=(short)((char *)&prec->d60nv - (char *)prec);
  pdbRecordType->papFldDes[236]->size=sizeof(prec->d61nv);
  pdbRecordType->papFldDes[236]->offset=(short)((char *)&prec->d61nv - (char *)prec);
  pdbRecordType->papFldDes[237]->size=sizeof(prec->d62nv);
  pdbRecordType->papFldDes[237]->offset=(short)((char *)&prec->d62nv - (char *)prec);
  pdbRecordType->papFldDes[238]->size=sizeof(prec->d63nv);
  pdbRecordType->papFldDes[238]->offset=(short)((char *)&prec->d63nv - (char *)prec);
  pdbRecordType->papFldDes[239]->size=sizeof(prec->d64nv);
  pdbRecordType->papFldDes[239]->offset=(short)((char *)&prec->d64nv - (char *)prec);
  pdbRecordType->papFldDes[240]->size=sizeof(prec->d65nv);
  pdbRecordType->papFldDes[240]->offset=(short)((char *)&prec->d65nv - (char *)prec);
  pdbRecordType->papFldDes[241]->size=sizeof(prec->d66nv);
  pdbRecordType->papFldDes[241]->offset=(short)((char *)&prec->d66nv - (char *)prec);
  pdbRecordType->papFldDes[242]->size=sizeof(prec->d67nv);
  pdbRecordType->papFldDes[242]->offset=(short)((char *)&prec->d67nv - (char *)prec);
  pdbRecordType->papFldDes[243]->size=sizeof(prec->d68nv);
  pdbRecordType->papFldDes[243]->offset=(short)((char *)&prec->d68nv - (char *)prec);
  pdbRecordType->papFldDes[244]->size=sizeof(prec->d69nv);
  pdbRecordType->papFldDes[244]->offset=(short)((char *)&prec->d69nv - (char *)prec);
  pdbRecordType->papFldDes[245]->size=sizeof(prec->d70nv);
  pdbRecordType->papFldDes[245]->offset=(short)((char *)&prec->d70nv - (char *)prec);
  pdbRecordType->papFldDes[246]->size=sizeof(prec->d1nv);
  pdbRecordType->papFldDes[246]->offset=(short)((char *)&prec->d1nv - (char *)prec);
  pdbRecordType->papFldDes[247]->size=sizeof(prec->d2nv);
  pdbRecordType->papFldDes[247]->offset=(short)((char *)&prec->d2nv - (char *)prec);
  pdbRecordType->papFldDes[248]->size=sizeof(prec->d3nv);
  pdbRecordType->papFldDes[248]->offset=(short)((char *)&prec->d3nv - (char *)prec);
  pdbRecordType->papFldDes[249]->size=sizeof(prec->d4nv);
  pdbRecordType->papFldDes[249]->offset=(short)((char *)&prec->d4nv - (char *)prec);
  pdbRecordType->papFldDes[250]->size=sizeof(prec->d5nv);
  pdbRecordType->papFldDes[250]->offset=(short)((char *)&prec->d5nv - (char *)prec);
  pdbRecordType->papFldDes[251]->size=sizeof(prec->d6nv);
  pdbRecordType->papFldDes[251]->offset=(short)((char *)&prec->d6nv - (char *)prec);
  pdbRecordType->papFldDes[252]->size=sizeof(prec->d7nv);
  pdbRecordType->papFldDes[252]->offset=(short)((char *)&prec->d7nv - (char *)prec);
  pdbRecordType->papFldDes[253]->size=sizeof(prec->d8nv);
  pdbRecordType->papFldDes[253]->offset=(short)((char *)&prec->d8nv - (char *)prec);
  pdbRecordType->papFldDes[254]->size=sizeof(prec->d9nv);
  pdbRecordType->papFldDes[254]->offset=(short)((char *)&prec->d9nv - (char *)prec);
  pdbRecordType->papFldDes[255]->size=sizeof(prec->danv);
  pdbRecordType->papFldDes[255]->offset=(short)((char *)&prec->danv - (char *)prec);
  pdbRecordType->papFldDes[256]->size=sizeof(prec->dbnv);
  pdbRecordType->papFldDes[256]->offset=(short)((char *)&prec->dbnv - (char *)prec);
  pdbRecordType->papFldDes[257]->size=sizeof(prec->dcnv);
  pdbRecordType->papFldDes[257]->offset=(short)((char *)&prec->dcnv - (char *)prec);
  pdbRecordType->papFldDes[258]->size=sizeof(prec->ddnv);
  pdbRecordType->papFldDes[258]->offset=(short)((char *)&prec->ddnv - (char *)prec);
  pdbRecordType->papFldDes[259]->size=sizeof(prec->denv);
  pdbRecordType->papFldDes[259]->offset=(short)((char *)&prec->denv - (char *)prec);
  pdbRecordType->papFldDes[260]->size=sizeof(prec->dfnv);
  pdbRecordType->papFldDes[260]->offset=(short)((char *)&prec->dfnv - (char *)prec);
  pdbRecordType->papFldDes[261]->size=sizeof(prec->t1nv);
  pdbRecordType->papFldDes[261]->offset=(short)((char *)&prec->t1nv - (char *)prec);
  pdbRecordType->papFldDes[262]->size=sizeof(prec->t2nv);
  pdbRecordType->papFldDes[262]->offset=(short)((char *)&prec->t2nv - (char *)prec);
  pdbRecordType->papFldDes[263]->size=sizeof(prec->t3nv);
  pdbRecordType->papFldDes[263]->offset=(short)((char *)&prec->t3nv - (char *)prec);
  pdbRecordType->papFldDes[264]->size=sizeof(prec->t4nv);
  pdbRecordType->papFldDes[264]->offset=(short)((char *)&prec->t4nv - (char *)prec);
  pdbRecordType->papFldDes[265]->size=sizeof(prec->bsnv);
  pdbRecordType->papFldDes[265]->offset=(short)((char *)&prec->bsnv - (char *)prec);
  pdbRecordType->papFldDes[266]->size=sizeof(prec->asnv);
  pdbRecordType->papFldDes[266]->offset=(short)((char *)&prec->asnv - (char *)prec);
  pdbRecordType->papFldDes[267]->size=sizeof(prec->p1pp);
  pdbRecordType->papFldDes[267]->offset=(short)((char *)&prec->p1pp - (char *)prec);
  pdbRecordType->papFldDes[268]->size=sizeof(prec->p1cv);
  pdbRecordType->papFldDes[268]->offset=(short)((char *)&prec->p1cv - (char *)prec);
  pdbRecordType->papFldDes[269]->size=sizeof(prec->p1dv);
  pdbRecordType->papFldDes[269]->offset=(short)((char *)&prec->p1dv - (char *)prec);
  pdbRecordType->papFldDes[270]->size=sizeof(prec->p1lv);
  pdbRecordType->papFldDes[270]->offset=(short)((char *)&prec->p1lv - (char *)prec);
  pdbRecordType->papFldDes[271]->size=sizeof(prec->p1sp);
  pdbRecordType->papFldDes[271]->offset=(short)((char *)&prec->p1sp - (char *)prec);
  pdbRecordType->papFldDes[272]->size=sizeof(prec->p1si);
  pdbRecordType->papFldDes[272]->offset=(short)((char *)&prec->p1si - (char *)prec);
  pdbRecordType->papFldDes[273]->size=sizeof(prec->p1ep);
  pdbRecordType->papFldDes[273]->offset=(short)((char *)&prec->p1ep - (char *)prec);
  pdbRecordType->papFldDes[274]->size=sizeof(prec->p1cp);
  pdbRecordType->papFldDes[274]->offset=(short)((char *)&prec->p1cp - (char *)prec);
  pdbRecordType->papFldDes[275]->size=sizeof(prec->p1wd);
  pdbRecordType->papFldDes[275]->offset=(short)((char *)&prec->p1wd - (char *)prec);
  pdbRecordType->papFldDes[276]->size=sizeof(prec->r1cv);
  pdbRecordType->papFldDes[276]->offset=(short)((char *)&prec->r1cv - (char *)prec);
  pdbRecordType->papFldDes[277]->size=sizeof(prec->r1lv);
  pdbRecordType->papFldDes[277]->offset=(short)((char *)&prec->r1lv - (char *)prec);
  pdbRecordType->papFldDes[278]->size=sizeof(prec->r1dl);
  pdbRecordType->papFldDes[278]->offset=(short)((char *)&prec->r1dl - (char *)prec);
  pdbRecordType->papFldDes[279]->size=sizeof(prec->p1hr);
  pdbRecordType->papFldDes[279]->offset=(short)((char *)&prec->p1hr - (char *)prec);
  pdbRecordType->papFldDes[280]->size=sizeof(prec->p1lr);
  pdbRecordType->papFldDes[280]->offset=(short)((char *)&prec->p1lr - (char *)prec);
  pdbRecordType->papFldDes[281]->size=sizeof(prec->p1pa);
  pdbRecordType->papFldDes[281]->offset=(short)((char *)&prec->p1pa - (char *)prec);
  pdbRecordType->papFldDes[282]->size=sizeof(prec->p1ra);
  pdbRecordType->papFldDes[282]->offset=(short)((char *)&prec->p1ra - (char *)prec);
  pdbRecordType->papFldDes[283]->size=sizeof(prec->p1fs);
  pdbRecordType->papFldDes[283]->offset=(short)((char *)&prec->p1fs - (char *)prec);
  pdbRecordType->papFldDes[284]->size=sizeof(prec->p1fi);
  pdbRecordType->papFldDes[284]->offset=(short)((char *)&prec->p1fi - (char *)prec);
  pdbRecordType->papFldDes[285]->size=sizeof(prec->p1fe);
  pdbRecordType->papFldDes[285]->offset=(short)((char *)&prec->p1fe - (char *)prec);
  pdbRecordType->papFldDes[286]->size=sizeof(prec->p1fc);
  pdbRecordType->papFldDes[286]->offset=(short)((char *)&prec->p1fc - (char *)prec);
  pdbRecordType->papFldDes[287]->size=sizeof(prec->p1fw);
  pdbRecordType->papFldDes[287]->offset=(short)((char *)&prec->p1fw - (char *)prec);
  pdbRecordType->papFldDes[288]->size=sizeof(prec->p1sm);
  pdbRecordType->papFldDes[288]->offset=(short)((char *)&prec->p1sm - (char *)prec);
  pdbRecordType->papFldDes[289]->size=sizeof(prec->p1ar);
  pdbRecordType->papFldDes[289]->offset=(short)((char *)&prec->p1ar - (char *)prec);
  pdbRecordType->papFldDes[290]->size=sizeof(prec->p1eu);
  pdbRecordType->papFldDes[290]->offset=(short)((char *)&prec->p1eu - (char *)prec);
  pdbRecordType->papFldDes[291]->size=sizeof(prec->p1pr);
  pdbRecordType->papFldDes[291]->offset=(short)((char *)&prec->p1pr - (char *)prec);
  pdbRecordType->papFldDes[292]->size=sizeof(prec->p2pp);
  pdbRecordType->papFldDes[292]->offset=(short)((char *)&prec->p2pp - (char *)prec);
  pdbRecordType->papFldDes[293]->size=sizeof(prec->p2cv);
  pdbRecordType->papFldDes[293]->offset=(short)((char *)&prec->p2cv - (char *)prec);
  pdbRecordType->papFldDes[294]->size=sizeof(prec->p2dv);
  pdbRecordType->papFldDes[294]->offset=(short)((char *)&prec->p2dv - (char *)prec);
  pdbRecordType->papFldDes[295]->size=sizeof(prec->p2lv);
  pdbRecordType->papFldDes[295]->offset=(short)((char *)&prec->p2lv - (char *)prec);
  pdbRecordType->papFldDes[296]->size=sizeof(prec->p2sp);
  pdbRecordType->papFldDes[296]->offset=(short)((char *)&prec->p2sp - (char *)prec);
  pdbRecordType->papFldDes[297]->size=sizeof(prec->p2si);
  pdbRecordType->papFldDes[297]->offset=(short)((char *)&prec->p2si - (char *)prec);
  pdbRecordType->papFldDes[298]->size=sizeof(prec->p2ep);
  pdbRecordType->papFldDes[298]->offset=(short)((char *)&prec->p2ep - (char *)prec);
  pdbRecordType->papFldDes[299]->size=sizeof(prec->p2cp);
  pdbRecordType->papFldDes[299]->offset=(short)((char *)&prec->p2cp - (char *)prec);
  pdbRecordType->papFldDes[300]->size=sizeof(prec->p2wd);
  pdbRecordType->papFldDes[300]->offset=(short)((char *)&prec->p2wd - (char *)prec);
  pdbRecordType->papFldDes[301]->size=sizeof(prec->r2cv);
  pdbRecordType->papFldDes[301]->offset=(short)((char *)&prec->r2cv - (char *)prec);
  pdbRecordType->papFldDes[302]->size=sizeof(prec->r2lv);
  pdbRecordType->papFldDes[302]->offset=(short)((char *)&prec->r2lv - (char *)prec);
  pdbRecordType->papFldDes[303]->size=sizeof(prec->r2dl);
  pdbRecordType->papFldDes[303]->offset=(short)((char *)&prec->r2dl - (char *)prec);
  pdbRecordType->papFldDes[304]->size=sizeof(prec->p2hr);
  pdbRecordType->papFldDes[304]->offset=(short)((char *)&prec->p2hr - (char *)prec);
  pdbRecordType->papFldDes[305]->size=sizeof(prec->p2lr);
  pdbRecordType->papFldDes[305]->offset=(short)((char *)&prec->p2lr - (char *)prec);
  pdbRecordType->papFldDes[306]->size=sizeof(prec->p2pa);
  pdbRecordType->papFldDes[306]->offset=(short)((char *)&prec->p2pa - (char *)prec);
  pdbRecordType->papFldDes[307]->size=sizeof(prec->p2ra);
  pdbRecordType->papFldDes[307]->offset=(short)((char *)&prec->p2ra - (char *)prec);
  pdbRecordType->papFldDes[308]->size=sizeof(prec->p2fs);
  pdbRecordType->papFldDes[308]->offset=(short)((char *)&prec->p2fs - (char *)prec);
  pdbRecordType->papFldDes[309]->size=sizeof(prec->p2fi);
  pdbRecordType->papFldDes[309]->offset=(short)((char *)&prec->p2fi - (char *)prec);
  pdbRecordType->papFldDes[310]->size=sizeof(prec->p2fe);
  pdbRecordType->papFldDes[310]->offset=(short)((char *)&prec->p2fe - (char *)prec);
  pdbRecordType->papFldDes[311]->size=sizeof(prec->p2fc);
  pdbRecordType->papFldDes[311]->offset=(short)((char *)&prec->p2fc - (char *)prec);
  pdbRecordType->papFldDes[312]->size=sizeof(prec->p2fw);
  pdbRecordType->papFldDes[312]->offset=(short)((char *)&prec->p2fw - (char *)prec);
  pdbRecordType->papFldDes[313]->size=sizeof(prec->p2sm);
  pdbRecordType->papFldDes[313]->offset=(short)((char *)&prec->p2sm - (char *)prec);
  pdbRecordType->papFldDes[314]->size=sizeof(prec->p2ar);
  pdbRecordType->papFldDes[314]->offset=(short)((char *)&prec->p2ar - (char *)prec);
  pdbRecordType->papFldDes[315]->size=sizeof(prec->p2eu);
  pdbRecordType->papFldDes[315]->offset=(short)((char *)&prec->p2eu - (char *)prec);
  pdbRecordType->papFldDes[316]->size=sizeof(prec->p2pr);
  pdbRecordType->papFldDes[316]->offset=(short)((char *)&prec->p2pr - (char *)prec);
  pdbRecordType->papFldDes[317]->size=sizeof(prec->p3pp);
  pdbRecordType->papFldDes[317]->offset=(short)((char *)&prec->p3pp - (char *)prec);
  pdbRecordType->papFldDes[318]->size=sizeof(prec->p3cv);
  pdbRecordType->papFldDes[318]->offset=(short)((char *)&prec->p3cv - (char *)prec);
  pdbRecordType->papFldDes[319]->size=sizeof(prec->p3dv);
  pdbRecordType->papFldDes[319]->offset=(short)((char *)&prec->p3dv - (char *)prec);
  pdbRecordType->papFldDes[320]->size=sizeof(prec->p3lv);
  pdbRecordType->papFldDes[320]->offset=(short)((char *)&prec->p3lv - (char *)prec);
  pdbRecordType->papFldDes[321]->size=sizeof(prec->p3sp);
  pdbRecordType->papFldDes[321]->offset=(short)((char *)&prec->p3sp - (char *)prec);
  pdbRecordType->papFldDes[322]->size=sizeof(prec->p3si);
  pdbRecordType->papFldDes[322]->offset=(short)((char *)&prec->p3si - (char *)prec);
  pdbRecordType->papFldDes[323]->size=sizeof(prec->p3ep);
  pdbRecordType->papFldDes[323]->offset=(short)((char *)&prec->p3ep - (char *)prec);
  pdbRecordType->papFldDes[324]->size=sizeof(prec->p3cp);
  pdbRecordType->papFldDes[324]->offset=(short)((char *)&prec->p3cp - (char *)prec);
  pdbRecordType->papFldDes[325]->size=sizeof(prec->p3wd);
  pdbRecordType->papFldDes[325]->offset=(short)((char *)&prec->p3wd - (char *)prec);
  pdbRecordType->papFldDes[326]->size=sizeof(prec->r3cv);
  pdbRecordType->papFldDes[326]->offset=(short)((char *)&prec->r3cv - (char *)prec);
  pdbRecordType->papFldDes[327]->size=sizeof(prec->r3lv);
  pdbRecordType->papFldDes[327]->offset=(short)((char *)&prec->r3lv - (char *)prec);
  pdbRecordType->papFldDes[328]->size=sizeof(prec->r3dl);
  pdbRecordType->papFldDes[328]->offset=(short)((char *)&prec->r3dl - (char *)prec);
  pdbRecordType->papFldDes[329]->size=sizeof(prec->p3hr);
  pdbRecordType->papFldDes[329]->offset=(short)((char *)&prec->p3hr - (char *)prec);
  pdbRecordType->papFldDes[330]->size=sizeof(prec->p3lr);
  pdbRecordType->papFldDes[330]->offset=(short)((char *)&prec->p3lr - (char *)prec);
  pdbRecordType->papFldDes[331]->size=sizeof(prec->p3pa);
  pdbRecordType->papFldDes[331]->offset=(short)((char *)&prec->p3pa - (char *)prec);
  pdbRecordType->papFldDes[332]->size=sizeof(prec->p3ra);
  pdbRecordType->papFldDes[332]->offset=(short)((char *)&prec->p3ra - (char *)prec);
  pdbRecordType->papFldDes[333]->size=sizeof(prec->p3fs);
  pdbRecordType->papFldDes[333]->offset=(short)((char *)&prec->p3fs - (char *)prec);
  pdbRecordType->papFldDes[334]->size=sizeof(prec->p3fi);
  pdbRecordType->papFldDes[334]->offset=(short)((char *)&prec->p3fi - (char *)prec);
  pdbRecordType->papFldDes[335]->size=sizeof(prec->p3fe);
  pdbRecordType->papFldDes[335]->offset=(short)((char *)&prec->p3fe - (char *)prec);
  pdbRecordType->papFldDes[336]->size=sizeof(prec->p3fc);
  pdbRecordType->papFldDes[336]->offset=(short)((char *)&prec->p3fc - (char *)prec);
  pdbRecordType->papFldDes[337]->size=sizeof(prec->p3fw);
  pdbRecordType->papFldDes[337]->offset=(short)((char *)&prec->p3fw - (char *)prec);
  pdbRecordType->papFldDes[338]->size=sizeof(prec->p3sm);
  pdbRecordType->papFldDes[338]->offset=(short)((char *)&prec->p3sm - (char *)prec);
  pdbRecordType->papFldDes[339]->size=sizeof(prec->p3ar);
  pdbRecordType->papFldDes[339]->offset=(short)((char *)&prec->p3ar - (char *)prec);
  pdbRecordType->papFldDes[340]->size=sizeof(prec->p3eu);
  pdbRecordType->papFldDes[340]->offset=(short)((char *)&prec->p3eu - (char *)prec);
  pdbRecordType->papFldDes[341]->size=sizeof(prec->p3pr);
  pdbRecordType->papFldDes[341]->offset=(short)((char *)&prec->p3pr - (char *)prec);
  pdbRecordType->papFldDes[342]->size=sizeof(prec->p4pp);
  pdbRecordType->papFldDes[342]->offset=(short)((char *)&prec->p4pp - (char *)prec);
  pdbRecordType->papFldDes[343]->size=sizeof(prec->p4cv);
  pdbRecordType->papFldDes[343]->offset=(short)((char *)&prec->p4cv - (char *)prec);
  pdbRecordType->papFldDes[344]->size=sizeof(prec->p4dv);
  pdbRecordType->papFldDes[344]->offset=(short)((char *)&prec->p4dv - (char *)prec);
  pdbRecordType->papFldDes[345]->size=sizeof(prec->p4lv);
  pdbRecordType->papFldDes[345]->offset=(short)((char *)&prec->p4lv - (char *)prec);
  pdbRecordType->papFldDes[346]->size=sizeof(prec->p4sp);
  pdbRecordType->papFldDes[346]->offset=(short)((char *)&prec->p4sp - (char *)prec);
  pdbRecordType->papFldDes[347]->size=sizeof(prec->p4si);
  pdbRecordType->papFldDes[347]->offset=(short)((char *)&prec->p4si - (char *)prec);
  pdbRecordType->papFldDes[348]->size=sizeof(prec->p4ep);
  pdbRecordType->papFldDes[348]->offset=(short)((char *)&prec->p4ep - (char *)prec);
  pdbRecordType->papFldDes[349]->size=sizeof(prec->p4cp);
  pdbRecordType->papFldDes[349]->offset=(short)((char *)&prec->p4cp - (char *)prec);
  pdbRecordType->papFldDes[350]->size=sizeof(prec->p4wd);
  pdbRecordType->papFldDes[350]->offset=(short)((char *)&prec->p4wd - (char *)prec);
  pdbRecordType->papFldDes[351]->size=sizeof(prec->r4cv);
  pdbRecordType->papFldDes[351]->offset=(short)((char *)&prec->r4cv - (char *)prec);
  pdbRecordType->papFldDes[352]->size=sizeof(prec->r4lv);
  pdbRecordType->papFldDes[352]->offset=(short)((char *)&prec->r4lv - (char *)prec);
  pdbRecordType->papFldDes[353]->size=sizeof(prec->r4dl);
  pdbRecordType->papFldDes[353]->offset=(short)((char *)&prec->r4dl - (char *)prec);
  pdbRecordType->papFldDes[354]->size=sizeof(prec->p4hr);
  pdbRecordType->papFldDes[354]->offset=(short)((char *)&prec->p4hr - (char *)prec);
  pdbRecordType->papFldDes[355]->size=sizeof(prec->p4lr);
  pdbRecordType->papFldDes[355]->offset=(short)((char *)&prec->p4lr - (char *)prec);
  pdbRecordType->papFldDes[356]->size=sizeof(prec->p4pa);
  pdbRecordType->papFldDes[356]->offset=(short)((char *)&prec->p4pa - (char *)prec);
  pdbRecordType->papFldDes[357]->size=sizeof(prec->p4ra);
  pdbRecordType->papFldDes[357]->offset=(short)((char *)&prec->p4ra - (char *)prec);
  pdbRecordType->papFldDes[358]->size=sizeof(prec->p4fs);
  pdbRecordType->papFldDes[358]->offset=(short)((char *)&prec->p4fs - (char *)prec);
  pdbRecordType->papFldDes[359]->size=sizeof(prec->p4fi);
  pdbRecordType->papFldDes[359]->offset=(short)((char *)&prec->p4fi - (char *)prec);
  pdbRecordType->papFldDes[360]->size=sizeof(prec->p4fe);
  pdbRecordType->papFldDes[360]->offset=(short)((char *)&prec->p4fe - (char *)prec);
  pdbRecordType->papFldDes[361]->size=sizeof(prec->p4fc);
  pdbRecordType->papFldDes[361]->offset=(short)((char *)&prec->p4fc - (char *)prec);
  pdbRecordType->papFldDes[362]->size=sizeof(prec->p4fw);
  pdbRecordType->papFldDes[362]->offset=(short)((char *)&prec->p4fw - (char *)prec);
  pdbRecordType->papFldDes[363]->size=sizeof(prec->p4sm);
  pdbRecordType->papFldDes[363]->offset=(short)((char *)&prec->p4sm - (char *)prec);
  pdbRecordType->papFldDes[364]->size=sizeof(prec->p4ar);
  pdbRecordType->papFldDes[364]->offset=(short)((char *)&prec->p4ar - (char *)prec);
  pdbRecordType->papFldDes[365]->size=sizeof(prec->p4eu);
  pdbRecordType->papFldDes[365]->offset=(short)((char *)&prec->p4eu - (char *)prec);
  pdbRecordType->papFldDes[366]->size=sizeof(prec->p4pr);
  pdbRecordType->papFldDes[366]->offset=(short)((char *)&prec->p4pr - (char *)prec);
  pdbRecordType->papFldDes[367]->size=sizeof(prec->d01hr);
  pdbRecordType->papFldDes[367]->offset=(short)((char *)&prec->d01hr - (char *)prec);
  pdbRecordType->papFldDes[368]->size=sizeof(prec->d01lr);
  pdbRecordType->papFldDes[368]->offset=(short)((char *)&prec->d01lr - (char *)prec);
  pdbRecordType->papFldDes[369]->size=sizeof(prec->d01da);
  pdbRecordType->papFldDes[369]->offset=(short)((char *)&prec->d01da - (char *)prec);
  pdbRecordType->papFldDes[370]->size=sizeof(prec->d01cv);
  pdbRecordType->papFldDes[370]->offset=(short)((char *)&prec->d01cv - (char *)prec);
  pdbRecordType->papFldDes[371]->size=sizeof(prec->d01lv);
  pdbRecordType->papFldDes[371]->offset=(short)((char *)&prec->d01lv - (char *)prec);
  pdbRecordType->papFldDes[372]->size=sizeof(prec->d01ne);
  pdbRecordType->papFldDes[372]->offset=(short)((char *)&prec->d01ne - (char *)prec);
  pdbRecordType->papFldDes[373]->size=sizeof(prec->d01eu);
  pdbRecordType->papFldDes[373]->offset=(short)((char *)&prec->d01eu - (char *)prec);
  pdbRecordType->papFldDes[374]->size=sizeof(prec->d01pr);
  pdbRecordType->papFldDes[374]->offset=(short)((char *)&prec->d01pr - (char *)prec);
  pdbRecordType->papFldDes[375]->size=sizeof(prec->d02hr);
  pdbRecordType->papFldDes[375]->offset=(short)((char *)&prec->d02hr - (char *)prec);
  pdbRecordType->papFldDes[376]->size=sizeof(prec->d02lr);
  pdbRecordType->papFldDes[376]->offset=(short)((char *)&prec->d02lr - (char *)prec);
  pdbRecordType->papFldDes[377]->size=sizeof(prec->d02da);
  pdbRecordType->papFldDes[377]->offset=(short)((char *)&prec->d02da - (char *)prec);
  pdbRecordType->papFldDes[378]->size=sizeof(prec->d02cv);
  pdbRecordType->papFldDes[378]->offset=(short)((char *)&prec->d02cv - (char *)prec);
  pdbRecordType->papFldDes[379]->size=sizeof(prec->d02lv);
  pdbRecordType->papFldDes[379]->offset=(short)((char *)&prec->d02lv - (char *)prec);
  pdbRecordType->papFldDes[380]->size=sizeof(prec->d02ne);
  pdbRecordType->papFldDes[380]->offset=(short)((char *)&prec->d02ne - (char *)prec);
  pdbRecordType->papFldDes[381]->size=sizeof(prec->d02eu);
  pdbRecordType->papFldDes[381]->offset=(short)((char *)&prec->d02eu - (char *)prec);
  pdbRecordType->papFldDes[382]->size=sizeof(prec->d02pr);
  pdbRecordType->papFldDes[382]->offset=(short)((char *)&prec->d02pr - (char *)prec);
  pdbRecordType->papFldDes[383]->size=sizeof(prec->d03hr);
  pdbRecordType->papFldDes[383]->offset=(short)((char *)&prec->d03hr - (char *)prec);
  pdbRecordType->papFldDes[384]->size=sizeof(prec->d03lr);
  pdbRecordType->papFldDes[384]->offset=(short)((char *)&prec->d03lr - (char *)prec);
  pdbRecordType->papFldDes[385]->size=sizeof(prec->d03da);
  pdbRecordType->papFldDes[385]->offset=(short)((char *)&prec->d03da - (char *)prec);
  pdbRecordType->papFldDes[386]->size=sizeof(prec->d03cv);
  pdbRecordType->papFldDes[386]->offset=(short)((char *)&prec->d03cv - (char *)prec);
  pdbRecordType->papFldDes[387]->size=sizeof(prec->d03lv);
  pdbRecordType->papFldDes[387]->offset=(short)((char *)&prec->d03lv - (char *)prec);
  pdbRecordType->papFldDes[388]->size=sizeof(prec->d03ne);
  pdbRecordType->papFldDes[388]->offset=(short)((char *)&prec->d03ne - (char *)prec);
  pdbRecordType->papFldDes[389]->size=sizeof(prec->d03eu);
  pdbRecordType->papFldDes[389]->offset=(short)((char *)&prec->d03eu - (char *)prec);
  pdbRecordType->papFldDes[390]->size=sizeof(prec->d03pr);
  pdbRecordType->papFldDes[390]->offset=(short)((char *)&prec->d03pr - (char *)prec);
  pdbRecordType->papFldDes[391]->size=sizeof(prec->d04hr);
  pdbRecordType->papFldDes[391]->offset=(short)((char *)&prec->d04hr - (char *)prec);
  pdbRecordType->papFldDes[392]->size=sizeof(prec->d04lr);
  pdbRecordType->papFldDes[392]->offset=(short)((char *)&prec->d04lr - (char *)prec);
  pdbRecordType->papFldDes[393]->size=sizeof(prec->d04da);
  pdbRecordType->papFldDes[393]->offset=(short)((char *)&prec->d04da - (char *)prec);
  pdbRecordType->papFldDes[394]->size=sizeof(prec->d04cv);
  pdbRecordType->papFldDes[394]->offset=(short)((char *)&prec->d04cv - (char *)prec);
  pdbRecordType->papFldDes[395]->size=sizeof(prec->d04lv);
  pdbRecordType->papFldDes[395]->offset=(short)((char *)&prec->d04lv - (char *)prec);
  pdbRecordType->papFldDes[396]->size=sizeof(prec->d04ne);
  pdbRecordType->papFldDes[396]->offset=(short)((char *)&prec->d04ne - (char *)prec);
  pdbRecordType->papFldDes[397]->size=sizeof(prec->d04eu);
  pdbRecordType->papFldDes[397]->offset=(short)((char *)&prec->d04eu - (char *)prec);
  pdbRecordType->papFldDes[398]->size=sizeof(prec->d04pr);
  pdbRecordType->papFldDes[398]->offset=(short)((char *)&prec->d04pr - (char *)prec);
  pdbRecordType->papFldDes[399]->size=sizeof(prec->d05hr);
  pdbRecordType->papFldDes[399]->offset=(short)((char *)&prec->d05hr - (char *)prec);
  pdbRecordType->papFldDes[400]->size=sizeof(prec->d05lr);
  pdbRecordType->papFldDes[400]->offset=(short)((char *)&prec->d05lr - (char *)prec);
  pdbRecordType->papFldDes[401]->size=sizeof(prec->d05da);
  pdbRecordType->papFldDes[401]->offset=(short)((char *)&prec->d05da - (char *)prec);
  pdbRecordType->papFldDes[402]->size=sizeof(prec->d05cv);
  pdbRecordType->papFldDes[402]->offset=(short)((char *)&prec->d05cv - (char *)prec);
  pdbRecordType->papFldDes[403]->size=sizeof(prec->d05lv);
  pdbRecordType->papFldDes[403]->offset=(short)((char *)&prec->d05lv - (char *)prec);
  pdbRecordType->papFldDes[404]->size=sizeof(prec->d05ne);
  pdbRecordType->papFldDes[404]->offset=(short)((char *)&prec->d05ne - (char *)prec);
  pdbRecordType->papFldDes[405]->size=sizeof(prec->d05eu);
  pdbRecordType->papFldDes[405]->offset=(short)((char *)&prec->d05eu - (char *)prec);
  pdbRecordType->papFldDes[406]->size=sizeof(prec->d05pr);
  pdbRecordType->papFldDes[406]->offset=(short)((char *)&prec->d05pr - (char *)prec);
  pdbRecordType->papFldDes[407]->size=sizeof(prec->d06hr);
  pdbRecordType->papFldDes[407]->offset=(short)((char *)&prec->d06hr - (char *)prec);
  pdbRecordType->papFldDes[408]->size=sizeof(prec->d06lr);
  pdbRecordType->papFldDes[408]->offset=(short)((char *)&prec->d06lr - (char *)prec);
  pdbRecordType->papFldDes[409]->size=sizeof(prec->d06da);
  pdbRecordType->papFldDes[409]->offset=(short)((char *)&prec->d06da - (char *)prec);
  pdbRecordType->papFldDes[410]->size=sizeof(prec->d06cv);
  pdbRecordType->papFldDes[410]->offset=(short)((char *)&prec->d06cv - (char *)prec);
  pdbRecordType->papFldDes[411]->size=sizeof(prec->d06lv);
  pdbRecordType->papFldDes[411]->offset=(short)((char *)&prec->d06lv - (char *)prec);
  pdbRecordType->papFldDes[412]->size=sizeof(prec->d06ne);
  pdbRecordType->papFldDes[412]->offset=(short)((char *)&prec->d06ne - (char *)prec);
  pdbRecordType->papFldDes[413]->size=sizeof(prec->d06eu);
  pdbRecordType->papFldDes[413]->offset=(short)((char *)&prec->d06eu - (char *)prec);
  pdbRecordType->papFldDes[414]->size=sizeof(prec->d06pr);
  pdbRecordType->papFldDes[414]->offset=(short)((char *)&prec->d06pr - (char *)prec);
  pdbRecordType->papFldDes[415]->size=sizeof(prec->d07hr);
  pdbRecordType->papFldDes[415]->offset=(short)((char *)&prec->d07hr - (char *)prec);
  pdbRecordType->papFldDes[416]->size=sizeof(prec->d07lr);
  pdbRecordType->papFldDes[416]->offset=(short)((char *)&prec->d07lr - (char *)prec);
  pdbRecordType->papFldDes[417]->size=sizeof(prec->d07da);
  pdbRecordType->papFldDes[417]->offset=(short)((char *)&prec->d07da - (char *)prec);
  pdbRecordType->papFldDes[418]->size=sizeof(prec->d07cv);
  pdbRecordType->papFldDes[418]->offset=(short)((char *)&prec->d07cv - (char *)prec);
  pdbRecordType->papFldDes[419]->size=sizeof(prec->d07lv);
  pdbRecordType->papFldDes[419]->offset=(short)((char *)&prec->d07lv - (char *)prec);
  pdbRecordType->papFldDes[420]->size=sizeof(prec->d07ne);
  pdbRecordType->papFldDes[420]->offset=(short)((char *)&prec->d07ne - (char *)prec);
  pdbRecordType->papFldDes[421]->size=sizeof(prec->d07eu);
  pdbRecordType->papFldDes[421]->offset=(short)((char *)&prec->d07eu - (char *)prec);
  pdbRecordType->papFldDes[422]->size=sizeof(prec->d07pr);
  pdbRecordType->papFldDes[422]->offset=(short)((char *)&prec->d07pr - (char *)prec);
  pdbRecordType->papFldDes[423]->size=sizeof(prec->d08hr);
  pdbRecordType->papFldDes[423]->offset=(short)((char *)&prec->d08hr - (char *)prec);
  pdbRecordType->papFldDes[424]->size=sizeof(prec->d08lr);
  pdbRecordType->papFldDes[424]->offset=(short)((char *)&prec->d08lr - (char *)prec);
  pdbRecordType->papFldDes[425]->size=sizeof(prec->d08da);
  pdbRecordType->papFldDes[425]->offset=(short)((char *)&prec->d08da - (char *)prec);
  pdbRecordType->papFldDes[426]->size=sizeof(prec->d08cv);
  pdbRecordType->papFldDes[426]->offset=(short)((char *)&prec->d08cv - (char *)prec);
  pdbRecordType->papFldDes[427]->size=sizeof(prec->d08lv);
  pdbRecordType->papFldDes[427]->offset=(short)((char *)&prec->d08lv - (char *)prec);
  pdbRecordType->papFldDes[428]->size=sizeof(prec->d08ne);
  pdbRecordType->papFldDes[428]->offset=(short)((char *)&prec->d08ne - (char *)prec);
  pdbRecordType->papFldDes[429]->size=sizeof(prec->d08eu);
  pdbRecordType->papFldDes[429]->offset=(short)((char *)&prec->d08eu - (char *)prec);
  pdbRecordType->papFldDes[430]->size=sizeof(prec->d08pr);
  pdbRecordType->papFldDes[430]->offset=(short)((char *)&prec->d08pr - (char *)prec);
  pdbRecordType->papFldDes[431]->size=sizeof(prec->d09hr);
  pdbRecordType->papFldDes[431]->offset=(short)((char *)&prec->d09hr - (char *)prec);
  pdbRecordType->papFldDes[432]->size=sizeof(prec->d09lr);
  pdbRecordType->papFldDes[432]->offset=(short)((char *)&prec->d09lr - (char *)prec);
  pdbRecordType->papFldDes[433]->size=sizeof(prec->d09da);
  pdbRecordType->papFldDes[433]->offset=(short)((char *)&prec->d09da - (char *)prec);
  pdbRecordType->papFldDes[434]->size=sizeof(prec->d09cv);
  pdbRecordType->papFldDes[434]->offset=(short)((char *)&prec->d09cv - (char *)prec);
  pdbRecordType->papFldDes[435]->size=sizeof(prec->d09lv);
  pdbRecordType->papFldDes[435]->offset=(short)((char *)&prec->d09lv - (char *)prec);
  pdbRecordType->papFldDes[436]->size=sizeof(prec->d09ne);
  pdbRecordType->papFldDes[436]->offset=(short)((char *)&prec->d09ne - (char *)prec);
  pdbRecordType->papFldDes[437]->size=sizeof(prec->d09eu);
  pdbRecordType->papFldDes[437]->offset=(short)((char *)&prec->d09eu - (char *)prec);
  pdbRecordType->papFldDes[438]->size=sizeof(prec->d09pr);
  pdbRecordType->papFldDes[438]->offset=(short)((char *)&prec->d09pr - (char *)prec);
  pdbRecordType->papFldDes[439]->size=sizeof(prec->d10hr);
  pdbRecordType->papFldDes[439]->offset=(short)((char *)&prec->d10hr - (char *)prec);
  pdbRecordType->papFldDes[440]->size=sizeof(prec->d10lr);
  pdbRecordType->papFldDes[440]->offset=(short)((char *)&prec->d10lr - (char *)prec);
  pdbRecordType->papFldDes[441]->size=sizeof(prec->d10da);
  pdbRecordType->papFldDes[441]->offset=(short)((char *)&prec->d10da - (char *)prec);
  pdbRecordType->papFldDes[442]->size=sizeof(prec->d10cv);
  pdbRecordType->papFldDes[442]->offset=(short)((char *)&prec->d10cv - (char *)prec);
  pdbRecordType->papFldDes[443]->size=sizeof(prec->d10lv);
  pdbRecordType->papFldDes[443]->offset=(short)((char *)&prec->d10lv - (char *)prec);
  pdbRecordType->papFldDes[444]->size=sizeof(prec->d10ne);
  pdbRecordType->papFldDes[444]->offset=(short)((char *)&prec->d10ne - (char *)prec);
  pdbRecordType->papFldDes[445]->size=sizeof(prec->d10eu);
  pdbRecordType->papFldDes[445]->offset=(short)((char *)&prec->d10eu - (char *)prec);
  pdbRecordType->papFldDes[446]->size=sizeof(prec->d10pr);
  pdbRecordType->papFldDes[446]->offset=(short)((char *)&prec->d10pr - (char *)prec);
  pdbRecordType->papFldDes[447]->size=sizeof(prec->d11hr);
  pdbRecordType->papFldDes[447]->offset=(short)((char *)&prec->d11hr - (char *)prec);
  pdbRecordType->papFldDes[448]->size=sizeof(prec->d11lr);
  pdbRecordType->papFldDes[448]->offset=(short)((char *)&prec->d11lr - (char *)prec);
  pdbRecordType->papFldDes[449]->size=sizeof(prec->d11da);
  pdbRecordType->papFldDes[449]->offset=(short)((char *)&prec->d11da - (char *)prec);
  pdbRecordType->papFldDes[450]->size=sizeof(prec->d11cv);
  pdbRecordType->papFldDes[450]->offset=(short)((char *)&prec->d11cv - (char *)prec);
  pdbRecordType->papFldDes[451]->size=sizeof(prec->d11lv);
  pdbRecordType->papFldDes[451]->offset=(short)((char *)&prec->d11lv - (char *)prec);
  pdbRecordType->papFldDes[452]->size=sizeof(prec->d11ne);
  pdbRecordType->papFldDes[452]->offset=(short)((char *)&prec->d11ne - (char *)prec);
  pdbRecordType->papFldDes[453]->size=sizeof(prec->d11eu);
  pdbRecordType->papFldDes[453]->offset=(short)((char *)&prec->d11eu - (char *)prec);
  pdbRecordType->papFldDes[454]->size=sizeof(prec->d11pr);
  pdbRecordType->papFldDes[454]->offset=(short)((char *)&prec->d11pr - (char *)prec);
  pdbRecordType->papFldDes[455]->size=sizeof(prec->d12hr);
  pdbRecordType->papFldDes[455]->offset=(short)((char *)&prec->d12hr - (char *)prec);
  pdbRecordType->papFldDes[456]->size=sizeof(prec->d12lr);
  pdbRecordType->papFldDes[456]->offset=(short)((char *)&prec->d12lr - (char *)prec);
  pdbRecordType->papFldDes[457]->size=sizeof(prec->d12da);
  pdbRecordType->papFldDes[457]->offset=(short)((char *)&prec->d12da - (char *)prec);
  pdbRecordType->papFldDes[458]->size=sizeof(prec->d12cv);
  pdbRecordType->papFldDes[458]->offset=(short)((char *)&prec->d12cv - (char *)prec);
  pdbRecordType->papFldDes[459]->size=sizeof(prec->d12lv);
  pdbRecordType->papFldDes[459]->offset=(short)((char *)&prec->d12lv - (char *)prec);
  pdbRecordType->papFldDes[460]->size=sizeof(prec->d12ne);
  pdbRecordType->papFldDes[460]->offset=(short)((char *)&prec->d12ne - (char *)prec);
  pdbRecordType->papFldDes[461]->size=sizeof(prec->d12eu);
  pdbRecordType->papFldDes[461]->offset=(short)((char *)&prec->d12eu - (char *)prec);
  pdbRecordType->papFldDes[462]->size=sizeof(prec->d12pr);
  pdbRecordType->papFldDes[462]->offset=(short)((char *)&prec->d12pr - (char *)prec);
  pdbRecordType->papFldDes[463]->size=sizeof(prec->d13hr);
  pdbRecordType->papFldDes[463]->offset=(short)((char *)&prec->d13hr - (char *)prec);
  pdbRecordType->papFldDes[464]->size=sizeof(prec->d13lr);
  pdbRecordType->papFldDes[464]->offset=(short)((char *)&prec->d13lr - (char *)prec);
  pdbRecordType->papFldDes[465]->size=sizeof(prec->d13da);
  pdbRecordType->papFldDes[465]->offset=(short)((char *)&prec->d13da - (char *)prec);
  pdbRecordType->papFldDes[466]->size=sizeof(prec->d13cv);
  pdbRecordType->papFldDes[466]->offset=(short)((char *)&prec->d13cv - (char *)prec);
  pdbRecordType->papFldDes[467]->size=sizeof(prec->d13lv);
  pdbRecordType->papFldDes[467]->offset=(short)((char *)&prec->d13lv - (char *)prec);
  pdbRecordType->papFldDes[468]->size=sizeof(prec->d13ne);
  pdbRecordType->papFldDes[468]->offset=(short)((char *)&prec->d13ne - (char *)prec);
  pdbRecordType->papFldDes[469]->size=sizeof(prec->d13eu);
  pdbRecordType->papFldDes[469]->offset=(short)((char *)&prec->d13eu - (char *)prec);
  pdbRecordType->papFldDes[470]->size=sizeof(prec->d13pr);
  pdbRecordType->papFldDes[470]->offset=(short)((char *)&prec->d13pr - (char *)prec);
  pdbRecordType->papFldDes[471]->size=sizeof(prec->d14hr);
  pdbRecordType->papFldDes[471]->offset=(short)((char *)&prec->d14hr - (char *)prec);
  pdbRecordType->papFldDes[472]->size=sizeof(prec->d14lr);
  pdbRecordType->papFldDes[472]->offset=(short)((char *)&prec->d14lr - (char *)prec);
  pdbRecordType->papFldDes[473]->size=sizeof(prec->d14da);
  pdbRecordType->papFldDes[473]->offset=(short)((char *)&prec->d14da - (char *)prec);
  pdbRecordType->papFldDes[474]->size=sizeof(prec->d14cv);
  pdbRecordType->papFldDes[474]->offset=(short)((char *)&prec->d14cv - (char *)prec);
  pdbRecordType->papFldDes[475]->size=sizeof(prec->d14lv);
  pdbRecordType->papFldDes[475]->offset=(short)((char *)&prec->d14lv - (char *)prec);
  pdbRecordType->papFldDes[476]->size=sizeof(prec->d14ne);
  pdbRecordType->papFldDes[476]->offset=(short)((char *)&prec->d14ne - (char *)prec);
  pdbRecordType->papFldDes[477]->size=sizeof(prec->d14eu);
  pdbRecordType->papFldDes[477]->offset=(short)((char *)&prec->d14eu - (char *)prec);
  pdbRecordType->papFldDes[478]->size=sizeof(prec->d14pr);
  pdbRecordType->papFldDes[478]->offset=(short)((char *)&prec->d14pr - (char *)prec);
  pdbRecordType->papFldDes[479]->size=sizeof(prec->d15hr);
  pdbRecordType->papFldDes[479]->offset=(short)((char *)&prec->d15hr - (char *)prec);
  pdbRecordType->papFldDes[480]->size=sizeof(prec->d15lr);
  pdbRecordType->papFldDes[480]->offset=(short)((char *)&prec->d15lr - (char *)prec);
  pdbRecordType->papFldDes[481]->size=sizeof(prec->d15da);
  pdbRecordType->papFldDes[481]->offset=(short)((char *)&prec->d15da - (char *)prec);
  pdbRecordType->papFldDes[482]->size=sizeof(prec->d15cv);
  pdbRecordType->papFldDes[482]->offset=(short)((char *)&prec->d15cv - (char *)prec);
  pdbRecordType->papFldDes[483]->size=sizeof(prec->d15lv);
  pdbRecordType->papFldDes[483]->offset=(short)((char *)&prec->d15lv - (char *)prec);
  pdbRecordType->papFldDes[484]->size=sizeof(prec->d15ne);
  pdbRecordType->papFldDes[484]->offset=(short)((char *)&prec->d15ne - (char *)prec);
  pdbRecordType->papFldDes[485]->size=sizeof(prec->d15eu);
  pdbRecordType->papFldDes[485]->offset=(short)((char *)&prec->d15eu - (char *)prec);
  pdbRecordType->papFldDes[486]->size=sizeof(prec->d15pr);
  pdbRecordType->papFldDes[486]->offset=(short)((char *)&prec->d15pr - (char *)prec);
  pdbRecordType->papFldDes[487]->size=sizeof(prec->d16hr);
  pdbRecordType->papFldDes[487]->offset=(short)((char *)&prec->d16hr - (char *)prec);
  pdbRecordType->papFldDes[488]->size=sizeof(prec->d16lr);
  pdbRecordType->papFldDes[488]->offset=(short)((char *)&prec->d16lr - (char *)prec);
  pdbRecordType->papFldDes[489]->size=sizeof(prec->d16da);
  pdbRecordType->papFldDes[489]->offset=(short)((char *)&prec->d16da - (char *)prec);
  pdbRecordType->papFldDes[490]->size=sizeof(prec->d16cv);
  pdbRecordType->papFldDes[490]->offset=(short)((char *)&prec->d16cv - (char *)prec);
  pdbRecordType->papFldDes[491]->size=sizeof(prec->d16lv);
  pdbRecordType->papFldDes[491]->offset=(short)((char *)&prec->d16lv - (char *)prec);
  pdbRecordType->papFldDes[492]->size=sizeof(prec->d16ne);
  pdbRecordType->papFldDes[492]->offset=(short)((char *)&prec->d16ne - (char *)prec);
  pdbRecordType->papFldDes[493]->size=sizeof(prec->d16eu);
  pdbRecordType->papFldDes[493]->offset=(short)((char *)&prec->d16eu - (char *)prec);
  pdbRecordType->papFldDes[494]->size=sizeof(prec->d16pr);
  pdbRecordType->papFldDes[494]->offset=(short)((char *)&prec->d16pr - (char *)prec);
  pdbRecordType->papFldDes[495]->size=sizeof(prec->d17hr);
  pdbRecordType->papFldDes[495]->offset=(short)((char *)&prec->d17hr - (char *)prec);
  pdbRecordType->papFldDes[496]->size=sizeof(prec->d17lr);
  pdbRecordType->papFldDes[496]->offset=(short)((char *)&prec->d17lr - (char *)prec);
  pdbRecordType->papFldDes[497]->size=sizeof(prec->d17da);
  pdbRecordType->papFldDes[497]->offset=(short)((char *)&prec->d17da - (char *)prec);
  pdbRecordType->papFldDes[498]->size=sizeof(prec->d17cv);
  pdbRecordType->papFldDes[498]->offset=(short)((char *)&prec->d17cv - (char *)prec);
  pdbRecordType->papFldDes[499]->size=sizeof(prec->d17lv);
  pdbRecordType->papFldDes[499]->offset=(short)((char *)&prec->d17lv - (char *)prec);
  pdbRecordType->papFldDes[500]->size=sizeof(prec->d17ne);
  pdbRecordType->papFldDes[500]->offset=(short)((char *)&prec->d17ne - (char *)prec);
  pdbRecordType->papFldDes[501]->size=sizeof(prec->d17eu);
  pdbRecordType->papFldDes[501]->offset=(short)((char *)&prec->d17eu - (char *)prec);
  pdbRecordType->papFldDes[502]->size=sizeof(prec->d17pr);
  pdbRecordType->papFldDes[502]->offset=(short)((char *)&prec->d17pr - (char *)prec);
  pdbRecordType->papFldDes[503]->size=sizeof(prec->d18hr);
  pdbRecordType->papFldDes[503]->offset=(short)((char *)&prec->d18hr - (char *)prec);
  pdbRecordType->papFldDes[504]->size=sizeof(prec->d18lr);
  pdbRecordType->papFldDes[504]->offset=(short)((char *)&prec->d18lr - (char *)prec);
  pdbRecordType->papFldDes[505]->size=sizeof(prec->d18da);
  pdbRecordType->papFldDes[505]->offset=(short)((char *)&prec->d18da - (char *)prec);
  pdbRecordType->papFldDes[506]->size=sizeof(prec->d18cv);
  pdbRecordType->papFldDes[506]->offset=(short)((char *)&prec->d18cv - (char *)prec);
  pdbRecordType->papFldDes[507]->size=sizeof(prec->d18lv);
  pdbRecordType->papFldDes[507]->offset=(short)((char *)&prec->d18lv - (char *)prec);
  pdbRecordType->papFldDes[508]->size=sizeof(prec->d18ne);
  pdbRecordType->papFldDes[508]->offset=(short)((char *)&prec->d18ne - (char *)prec);
  pdbRecordType->papFldDes[509]->size=sizeof(prec->d18eu);
  pdbRecordType->papFldDes[509]->offset=(short)((char *)&prec->d18eu - (char *)prec);
  pdbRecordType->papFldDes[510]->size=sizeof(prec->d18pr);
  pdbRecordType->papFldDes[510]->offset=(short)((char *)&prec->d18pr - (char *)prec);
  pdbRecordType->papFldDes[511]->size=sizeof(prec->d19hr);
  pdbRecordType->papFldDes[511]->offset=(short)((char *)&prec->d19hr - (char *)prec);
  pdbRecordType->papFldDes[512]->size=sizeof(prec->d19lr);
  pdbRecordType->papFldDes[512]->offset=(short)((char *)&prec->d19lr - (char *)prec);
  pdbRecordType->papFldDes[513]->size=sizeof(prec->d19da);
  pdbRecordType->papFldDes[513]->offset=(short)((char *)&prec->d19da - (char *)prec);
  pdbRecordType->papFldDes[514]->size=sizeof(prec->d19cv);
  pdbRecordType->papFldDes[514]->offset=(short)((char *)&prec->d19cv - (char *)prec);
  pdbRecordType->papFldDes[515]->size=sizeof(prec->d19lv);
  pdbRecordType->papFldDes[515]->offset=(short)((char *)&prec->d19lv - (char *)prec);
  pdbRecordType->papFldDes[516]->size=sizeof(prec->d19ne);
  pdbRecordType->papFldDes[516]->offset=(short)((char *)&prec->d19ne - (char *)prec);
  pdbRecordType->papFldDes[517]->size=sizeof(prec->d19eu);
  pdbRecordType->papFldDes[517]->offset=(short)((char *)&prec->d19eu - (char *)prec);
  pdbRecordType->papFldDes[518]->size=sizeof(prec->d19pr);
  pdbRecordType->papFldDes[518]->offset=(short)((char *)&prec->d19pr - (char *)prec);
  pdbRecordType->papFldDes[519]->size=sizeof(prec->d20hr);
  pdbRecordType->papFldDes[519]->offset=(short)((char *)&prec->d20hr - (char *)prec);
  pdbRecordType->papFldDes[520]->size=sizeof(prec->d20lr);
  pdbRecordType->papFldDes[520]->offset=(short)((char *)&prec->d20lr - (char *)prec);
  pdbRecordType->papFldDes[521]->size=sizeof(prec->d20da);
  pdbRecordType->papFldDes[521]->offset=(short)((char *)&prec->d20da - (char *)prec);
  pdbRecordType->papFldDes[522]->size=sizeof(prec->d20cv);
  pdbRecordType->papFldDes[522]->offset=(short)((char *)&prec->d20cv - (char *)prec);
  pdbRecordType->papFldDes[523]->size=sizeof(prec->d20lv);
  pdbRecordType->papFldDes[523]->offset=(short)((char *)&prec->d20lv - (char *)prec);
  pdbRecordType->papFldDes[524]->size=sizeof(prec->d20ne);
  pdbRecordType->papFldDes[524]->offset=(short)((char *)&prec->d20ne - (char *)prec);
  pdbRecordType->papFldDes[525]->size=sizeof(prec->d20eu);
  pdbRecordType->papFldDes[525]->offset=(short)((char *)&prec->d20eu - (char *)prec);
  pdbRecordType->papFldDes[526]->size=sizeof(prec->d20pr);
  pdbRecordType->papFldDes[526]->offset=(short)((char *)&prec->d20pr - (char *)prec);
  pdbRecordType->papFldDes[527]->size=sizeof(prec->d21hr);
  pdbRecordType->papFldDes[527]->offset=(short)((char *)&prec->d21hr - (char *)prec);
  pdbRecordType->papFldDes[528]->size=sizeof(prec->d21lr);
  pdbRecordType->papFldDes[528]->offset=(short)((char *)&prec->d21lr - (char *)prec);
  pdbRecordType->papFldDes[529]->size=sizeof(prec->d21da);
  pdbRecordType->papFldDes[529]->offset=(short)((char *)&prec->d21da - (char *)prec);
  pdbRecordType->papFldDes[530]->size=sizeof(prec->d21cv);
  pdbRecordType->papFldDes[530]->offset=(short)((char *)&prec->d21cv - (char *)prec);
  pdbRecordType->papFldDes[531]->size=sizeof(prec->d21lv);
  pdbRecordType->papFldDes[531]->offset=(short)((char *)&prec->d21lv - (char *)prec);
  pdbRecordType->papFldDes[532]->size=sizeof(prec->d21ne);
  pdbRecordType->papFldDes[532]->offset=(short)((char *)&prec->d21ne - (char *)prec);
  pdbRecordType->papFldDes[533]->size=sizeof(prec->d21eu);
  pdbRecordType->papFldDes[533]->offset=(short)((char *)&prec->d21eu - (char *)prec);
  pdbRecordType->papFldDes[534]->size=sizeof(prec->d21pr);
  pdbRecordType->papFldDes[534]->offset=(short)((char *)&prec->d21pr - (char *)prec);
  pdbRecordType->papFldDes[535]->size=sizeof(prec->d22hr);
  pdbRecordType->papFldDes[535]->offset=(short)((char *)&prec->d22hr - (char *)prec);
  pdbRecordType->papFldDes[536]->size=sizeof(prec->d22lr);
  pdbRecordType->papFldDes[536]->offset=(short)((char *)&prec->d22lr - (char *)prec);
  pdbRecordType->papFldDes[537]->size=sizeof(prec->d22da);
  pdbRecordType->papFldDes[537]->offset=(short)((char *)&prec->d22da - (char *)prec);
  pdbRecordType->papFldDes[538]->size=sizeof(prec->d22cv);
  pdbRecordType->papFldDes[538]->offset=(short)((char *)&prec->d22cv - (char *)prec);
  pdbRecordType->papFldDes[539]->size=sizeof(prec->d22lv);
  pdbRecordType->papFldDes[539]->offset=(short)((char *)&prec->d22lv - (char *)prec);
  pdbRecordType->papFldDes[540]->size=sizeof(prec->d22ne);
  pdbRecordType->papFldDes[540]->offset=(short)((char *)&prec->d22ne - (char *)prec);
  pdbRecordType->papFldDes[541]->size=sizeof(prec->d22eu);
  pdbRecordType->papFldDes[541]->offset=(short)((char *)&prec->d22eu - (char *)prec);
  pdbRecordType->papFldDes[542]->size=sizeof(prec->d22pr);
  pdbRecordType->papFldDes[542]->offset=(short)((char *)&prec->d22pr - (char *)prec);
  pdbRecordType->papFldDes[543]->size=sizeof(prec->d23hr);
  pdbRecordType->papFldDes[543]->offset=(short)((char *)&prec->d23hr - (char *)prec);
  pdbRecordType->papFldDes[544]->size=sizeof(prec->d23lr);
  pdbRecordType->papFldDes[544]->offset=(short)((char *)&prec->d23lr - (char *)prec);
  pdbRecordType->papFldDes[545]->size=sizeof(prec->d23da);
  pdbRecordType->papFldDes[545]->offset=(short)((char *)&prec->d23da - (char *)prec);
  pdbRecordType->papFldDes[546]->size=sizeof(prec->d23cv);
  pdbRecordType->papFldDes[546]->offset=(short)((char *)&prec->d23cv - (char *)prec);
  pdbRecordType->papFldDes[547]->size=sizeof(prec->d23lv);
  pdbRecordType->papFldDes[547]->offset=(short)((char *)&prec->d23lv - (char *)prec);
  pdbRecordType->papFldDes[548]->size=sizeof(prec->d23ne);
  pdbRecordType->papFldDes[548]->offset=(short)((char *)&prec->d23ne - (char *)prec);
  pdbRecordType->papFldDes[549]->size=sizeof(prec->d23eu);
  pdbRecordType->papFldDes[549]->offset=(short)((char *)&prec->d23eu - (char *)prec);
  pdbRecordType->papFldDes[550]->size=sizeof(prec->d23pr);
  pdbRecordType->papFldDes[550]->offset=(short)((char *)&prec->d23pr - (char *)prec);
  pdbRecordType->papFldDes[551]->size=sizeof(prec->d24hr);
  pdbRecordType->papFldDes[551]->offset=(short)((char *)&prec->d24hr - (char *)prec);
  pdbRecordType->papFldDes[552]->size=sizeof(prec->d24lr);
  pdbRecordType->papFldDes[552]->offset=(short)((char *)&prec->d24lr - (char *)prec);
  pdbRecordType->papFldDes[553]->size=sizeof(prec->d24da);
  pdbRecordType->papFldDes[553]->offset=(short)((char *)&prec->d24da - (char *)prec);
  pdbRecordType->papFldDes[554]->size=sizeof(prec->d24cv);
  pdbRecordType->papFldDes[554]->offset=(short)((char *)&prec->d24cv - (char *)prec);
  pdbRecordType->papFldDes[555]->size=sizeof(prec->d24lv);
  pdbRecordType->papFldDes[555]->offset=(short)((char *)&prec->d24lv - (char *)prec);
  pdbRecordType->papFldDes[556]->size=sizeof(prec->d24ne);
  pdbRecordType->papFldDes[556]->offset=(short)((char *)&prec->d24ne - (char *)prec);
  pdbRecordType->papFldDes[557]->size=sizeof(prec->d24eu);
  pdbRecordType->papFldDes[557]->offset=(short)((char *)&prec->d24eu - (char *)prec);
  pdbRecordType->papFldDes[558]->size=sizeof(prec->d24pr);
  pdbRecordType->papFldDes[558]->offset=(short)((char *)&prec->d24pr - (char *)prec);
  pdbRecordType->papFldDes[559]->size=sizeof(prec->d25hr);
  pdbRecordType->papFldDes[559]->offset=(short)((char *)&prec->d25hr - (char *)prec);
  pdbRecordType->papFldDes[560]->size=sizeof(prec->d25lr);
  pdbRecordType->papFldDes[560]->offset=(short)((char *)&prec->d25lr - (char *)prec);
  pdbRecordType->papFldDes[561]->size=sizeof(prec->d25da);
  pdbRecordType->papFldDes[561]->offset=(short)((char *)&prec->d25da - (char *)prec);
  pdbRecordType->papFldDes[562]->size=sizeof(prec->d25cv);
  pdbRecordType->papFldDes[562]->offset=(short)((char *)&prec->d25cv - (char *)prec);
  pdbRecordType->papFldDes[563]->size=sizeof(prec->d25lv);
  pdbRecordType->papFldDes[563]->offset=(short)((char *)&prec->d25lv - (char *)prec);
  pdbRecordType->papFldDes[564]->size=sizeof(prec->d25ne);
  pdbRecordType->papFldDes[564]->offset=(short)((char *)&prec->d25ne - (char *)prec);
  pdbRecordType->papFldDes[565]->size=sizeof(prec->d25eu);
  pdbRecordType->papFldDes[565]->offset=(short)((char *)&prec->d25eu - (char *)prec);
  pdbRecordType->papFldDes[566]->size=sizeof(prec->d25pr);
  pdbRecordType->papFldDes[566]->offset=(short)((char *)&prec->d25pr - (char *)prec);
  pdbRecordType->papFldDes[567]->size=sizeof(prec->d26hr);
  pdbRecordType->papFldDes[567]->offset=(short)((char *)&prec->d26hr - (char *)prec);
  pdbRecordType->papFldDes[568]->size=sizeof(prec->d26lr);
  pdbRecordType->papFldDes[568]->offset=(short)((char *)&prec->d26lr - (char *)prec);
  pdbRecordType->papFldDes[569]->size=sizeof(prec->d26da);
  pdbRecordType->papFldDes[569]->offset=(short)((char *)&prec->d26da - (char *)prec);
  pdbRecordType->papFldDes[570]->size=sizeof(prec->d26cv);
  pdbRecordType->papFldDes[570]->offset=(short)((char *)&prec->d26cv - (char *)prec);
  pdbRecordType->papFldDes[571]->size=sizeof(prec->d26lv);
  pdbRecordType->papFldDes[571]->offset=(short)((char *)&prec->d26lv - (char *)prec);
  pdbRecordType->papFldDes[572]->size=sizeof(prec->d26ne);
  pdbRecordType->papFldDes[572]->offset=(short)((char *)&prec->d26ne - (char *)prec);
  pdbRecordType->papFldDes[573]->size=sizeof(prec->d26eu);
  pdbRecordType->papFldDes[573]->offset=(short)((char *)&prec->d26eu - (char *)prec);
  pdbRecordType->papFldDes[574]->size=sizeof(prec->d26pr);
  pdbRecordType->papFldDes[574]->offset=(short)((char *)&prec->d26pr - (char *)prec);
  pdbRecordType->papFldDes[575]->size=sizeof(prec->d27hr);
  pdbRecordType->papFldDes[575]->offset=(short)((char *)&prec->d27hr - (char *)prec);
  pdbRecordType->papFldDes[576]->size=sizeof(prec->d27lr);
  pdbRecordType->papFldDes[576]->offset=(short)((char *)&prec->d27lr - (char *)prec);
  pdbRecordType->papFldDes[577]->size=sizeof(prec->d27da);
  pdbRecordType->papFldDes[577]->offset=(short)((char *)&prec->d27da - (char *)prec);
  pdbRecordType->papFldDes[578]->size=sizeof(prec->d27cv);
  pdbRecordType->papFldDes[578]->offset=(short)((char *)&prec->d27cv - (char *)prec);
  pdbRecordType->papFldDes[579]->size=sizeof(prec->d27lv);
  pdbRecordType->papFldDes[579]->offset=(short)((char *)&prec->d27lv - (char *)prec);
  pdbRecordType->papFldDes[580]->size=sizeof(prec->d27ne);
  pdbRecordType->papFldDes[580]->offset=(short)((char *)&prec->d27ne - (char *)prec);
  pdbRecordType->papFldDes[581]->size=sizeof(prec->d27eu);
  pdbRecordType->papFldDes[581]->offset=(short)((char *)&prec->d27eu - (char *)prec);
  pdbRecordType->papFldDes[582]->size=sizeof(prec->d27pr);
  pdbRecordType->papFldDes[582]->offset=(short)((char *)&prec->d27pr - (char *)prec);
  pdbRecordType->papFldDes[583]->size=sizeof(prec->d28hr);
  pdbRecordType->papFldDes[583]->offset=(short)((char *)&prec->d28hr - (char *)prec);
  pdbRecordType->papFldDes[584]->size=sizeof(prec->d28lr);
  pdbRecordType->papFldDes[584]->offset=(short)((char *)&prec->d28lr - (char *)prec);
  pdbRecordType->papFldDes[585]->size=sizeof(prec->d28da);
  pdbRecordType->papFldDes[585]->offset=(short)((char *)&prec->d28da - (char *)prec);
  pdbRecordType->papFldDes[586]->size=sizeof(prec->d28cv);
  pdbRecordType->papFldDes[586]->offset=(short)((char *)&prec->d28cv - (char *)prec);
  pdbRecordType->papFldDes[587]->size=sizeof(prec->d28lv);
  pdbRecordType->papFldDes[587]->offset=(short)((char *)&prec->d28lv - (char *)prec);
  pdbRecordType->papFldDes[588]->size=sizeof(prec->d28ne);
  pdbRecordType->papFldDes[588]->offset=(short)((char *)&prec->d28ne - (char *)prec);
  pdbRecordType->papFldDes[589]->size=sizeof(prec->d28eu);
  pdbRecordType->papFldDes[589]->offset=(short)((char *)&prec->d28eu - (char *)prec);
  pdbRecordType->papFldDes[590]->size=sizeof(prec->d28pr);
  pdbRecordType->papFldDes[590]->offset=(short)((char *)&prec->d28pr - (char *)prec);
  pdbRecordType->papFldDes[591]->size=sizeof(prec->d29hr);
  pdbRecordType->papFldDes[591]->offset=(short)((char *)&prec->d29hr - (char *)prec);
  pdbRecordType->papFldDes[592]->size=sizeof(prec->d29lr);
  pdbRecordType->papFldDes[592]->offset=(short)((char *)&prec->d29lr - (char *)prec);
  pdbRecordType->papFldDes[593]->size=sizeof(prec->d29da);
  pdbRecordType->papFldDes[593]->offset=(short)((char *)&prec->d29da - (char *)prec);
  pdbRecordType->papFldDes[594]->size=sizeof(prec->d29cv);
  pdbRecordType->papFldDes[594]->offset=(short)((char *)&prec->d29cv - (char *)prec);
  pdbRecordType->papFldDes[595]->size=sizeof(prec->d29lv);
  pdbRecordType->papFldDes[595]->offset=(short)((char *)&prec->d29lv - (char *)prec);
  pdbRecordType->papFldDes[596]->size=sizeof(prec->d29ne);
  pdbRecordType->papFldDes[596]->offset=(short)((char *)&prec->d29ne - (char *)prec);
  pdbRecordType->papFldDes[597]->size=sizeof(prec->d29eu);
  pdbRecordType->papFldDes[597]->offset=(short)((char *)&prec->d29eu - (char *)prec);
  pdbRecordType->papFldDes[598]->size=sizeof(prec->d29pr);
  pdbRecordType->papFldDes[598]->offset=(short)((char *)&prec->d29pr - (char *)prec);
  pdbRecordType->papFldDes[599]->size=sizeof(prec->d30hr);
  pdbRecordType->papFldDes[599]->offset=(short)((char *)&prec->d30hr - (char *)prec);
  pdbRecordType->papFldDes[600]->size=sizeof(prec->d30lr);
  pdbRecordType->papFldDes[600]->offset=(short)((char *)&prec->d30lr - (char *)prec);
  pdbRecordType->papFldDes[601]->size=sizeof(prec->d30da);
  pdbRecordType->papFldDes[601]->offset=(short)((char *)&prec->d30da - (char *)prec);
  pdbRecordType->papFldDes[602]->size=sizeof(prec->d30cv);
  pdbRecordType->papFldDes[602]->offset=(short)((char *)&prec->d30cv - (char *)prec);
  pdbRecordType->papFldDes[603]->size=sizeof(prec->d30lv);
  pdbRecordType->papFldDes[603]->offset=(short)((char *)&prec->d30lv - (char *)prec);
  pdbRecordType->papFldDes[604]->size=sizeof(prec->d30ne);
  pdbRecordType->papFldDes[604]->offset=(short)((char *)&prec->d30ne - (char *)prec);
  pdbRecordType->papFldDes[605]->size=sizeof(prec->d30eu);
  pdbRecordType->papFldDes[605]->offset=(short)((char *)&prec->d30eu - (char *)prec);
  pdbRecordType->papFldDes[606]->size=sizeof(prec->d30pr);
  pdbRecordType->papFldDes[606]->offset=(short)((char *)&prec->d30pr - (char *)prec);
  pdbRecordType->papFldDes[607]->size=sizeof(prec->d31hr);
  pdbRecordType->papFldDes[607]->offset=(short)((char *)&prec->d31hr - (char *)prec);
  pdbRecordType->papFldDes[608]->size=sizeof(prec->d31lr);
  pdbRecordType->papFldDes[608]->offset=(short)((char *)&prec->d31lr - (char *)prec);
  pdbRecordType->papFldDes[609]->size=sizeof(prec->d31da);
  pdbRecordType->papFldDes[609]->offset=(short)((char *)&prec->d31da - (char *)prec);
  pdbRecordType->papFldDes[610]->size=sizeof(prec->d31cv);
  pdbRecordType->papFldDes[610]->offset=(short)((char *)&prec->d31cv - (char *)prec);
  pdbRecordType->papFldDes[611]->size=sizeof(prec->d31lv);
  pdbRecordType->papFldDes[611]->offset=(short)((char *)&prec->d31lv - (char *)prec);
  pdbRecordType->papFldDes[612]->size=sizeof(prec->d31ne);
  pdbRecordType->papFldDes[612]->offset=(short)((char *)&prec->d31ne - (char *)prec);
  pdbRecordType->papFldDes[613]->size=sizeof(prec->d31eu);
  pdbRecordType->papFldDes[613]->offset=(short)((char *)&prec->d31eu - (char *)prec);
  pdbRecordType->papFldDes[614]->size=sizeof(prec->d31pr);
  pdbRecordType->papFldDes[614]->offset=(short)((char *)&prec->d31pr - (char *)prec);
  pdbRecordType->papFldDes[615]->size=sizeof(prec->d32hr);
  pdbRecordType->papFldDes[615]->offset=(short)((char *)&prec->d32hr - (char *)prec);
  pdbRecordType->papFldDes[616]->size=sizeof(prec->d32lr);
  pdbRecordType->papFldDes[616]->offset=(short)((char *)&prec->d32lr - (char *)prec);
  pdbRecordType->papFldDes[617]->size=sizeof(prec->d32da);
  pdbRecordType->papFldDes[617]->offset=(short)((char *)&prec->d32da - (char *)prec);
  pdbRecordType->papFldDes[618]->size=sizeof(prec->d32cv);
  pdbRecordType->papFldDes[618]->offset=(short)((char *)&prec->d32cv - (char *)prec);
  pdbRecordType->papFldDes[619]->size=sizeof(prec->d32lv);
  pdbRecordType->papFldDes[619]->offset=(short)((char *)&prec->d32lv - (char *)prec);
  pdbRecordType->papFldDes[620]->size=sizeof(prec->d32ne);
  pdbRecordType->papFldDes[620]->offset=(short)((char *)&prec->d32ne - (char *)prec);
  pdbRecordType->papFldDes[621]->size=sizeof(prec->d32eu);
  pdbRecordType->papFldDes[621]->offset=(short)((char *)&prec->d32eu - (char *)prec);
  pdbRecordType->papFldDes[622]->size=sizeof(prec->d32pr);
  pdbRecordType->papFldDes[622]->offset=(short)((char *)&prec->d32pr - (char *)prec);
  pdbRecordType->papFldDes[623]->size=sizeof(prec->d33hr);
  pdbRecordType->papFldDes[623]->offset=(short)((char *)&prec->d33hr - (char *)prec);
  pdbRecordType->papFldDes[624]->size=sizeof(prec->d33lr);
  pdbRecordType->papFldDes[624]->offset=(short)((char *)&prec->d33lr - (char *)prec);
  pdbRecordType->papFldDes[625]->size=sizeof(prec->d33da);
  pdbRecordType->papFldDes[625]->offset=(short)((char *)&prec->d33da - (char *)prec);
  pdbRecordType->papFldDes[626]->size=sizeof(prec->d33cv);
  pdbRecordType->papFldDes[626]->offset=(short)((char *)&prec->d33cv - (char *)prec);
  pdbRecordType->papFldDes[627]->size=sizeof(prec->d33lv);
  pdbRecordType->papFldDes[627]->offset=(short)((char *)&prec->d33lv - (char *)prec);
  pdbRecordType->papFldDes[628]->size=sizeof(prec->d33ne);
  pdbRecordType->papFldDes[628]->offset=(short)((char *)&prec->d33ne - (char *)prec);
  pdbRecordType->papFldDes[629]->size=sizeof(prec->d33eu);
  pdbRecordType->papFldDes[629]->offset=(short)((char *)&prec->d33eu - (char *)prec);
  pdbRecordType->papFldDes[630]->size=sizeof(prec->d33pr);
  pdbRecordType->papFldDes[630]->offset=(short)((char *)&prec->d33pr - (char *)prec);
  pdbRecordType->papFldDes[631]->size=sizeof(prec->d34hr);
  pdbRecordType->papFldDes[631]->offset=(short)((char *)&prec->d34hr - (char *)prec);
  pdbRecordType->papFldDes[632]->size=sizeof(prec->d34lr);
  pdbRecordType->papFldDes[632]->offset=(short)((char *)&prec->d34lr - (char *)prec);
  pdbRecordType->papFldDes[633]->size=sizeof(prec->d34da);
  pdbRecordType->papFldDes[633]->offset=(short)((char *)&prec->d34da - (char *)prec);
  pdbRecordType->papFldDes[634]->size=sizeof(prec->d34cv);
  pdbRecordType->papFldDes[634]->offset=(short)((char *)&prec->d34cv - (char *)prec);
  pdbRecordType->papFldDes[635]->size=sizeof(prec->d34lv);
  pdbRecordType->papFldDes[635]->offset=(short)((char *)&prec->d34lv - (char *)prec);
  pdbRecordType->papFldDes[636]->size=sizeof(prec->d34ne);
  pdbRecordType->papFldDes[636]->offset=(short)((char *)&prec->d34ne - (char *)prec);
  pdbRecordType->papFldDes[637]->size=sizeof(prec->d34eu);
  pdbRecordType->papFldDes[637]->offset=(short)((char *)&prec->d34eu - (char *)prec);
  pdbRecordType->papFldDes[638]->size=sizeof(prec->d34pr);
  pdbRecordType->papFldDes[638]->offset=(short)((char *)&prec->d34pr - (char *)prec);
  pdbRecordType->papFldDes[639]->size=sizeof(prec->d35hr);
  pdbRecordType->papFldDes[639]->offset=(short)((char *)&prec->d35hr - (char *)prec);
  pdbRecordType->papFldDes[640]->size=sizeof(prec->d35lr);
  pdbRecordType->papFldDes[640]->offset=(short)((char *)&prec->d35lr - (char *)prec);
  pdbRecordType->papFldDes[641]->size=sizeof(prec->d35da);
  pdbRecordType->papFldDes[641]->offset=(short)((char *)&prec->d35da - (char *)prec);
  pdbRecordType->papFldDes[642]->size=sizeof(prec->d35cv);
  pdbRecordType->papFldDes[642]->offset=(short)((char *)&prec->d35cv - (char *)prec);
  pdbRecordType->papFldDes[643]->size=sizeof(prec->d35lv);
  pdbRecordType->papFldDes[643]->offset=(short)((char *)&prec->d35lv - (char *)prec);
  pdbRecordType->papFldDes[644]->size=sizeof(prec->d35ne);
  pdbRecordType->papFldDes[644]->offset=(short)((char *)&prec->d35ne - (char *)prec);
  pdbRecordType->papFldDes[645]->size=sizeof(prec->d35eu);
  pdbRecordType->papFldDes[645]->offset=(short)((char *)&prec->d35eu - (char *)prec);
  pdbRecordType->papFldDes[646]->size=sizeof(prec->d35pr);
  pdbRecordType->papFldDes[646]->offset=(short)((char *)&prec->d35pr - (char *)prec);
  pdbRecordType->papFldDes[647]->size=sizeof(prec->d36hr);
  pdbRecordType->papFldDes[647]->offset=(short)((char *)&prec->d36hr - (char *)prec);
  pdbRecordType->papFldDes[648]->size=sizeof(prec->d36lr);
  pdbRecordType->papFldDes[648]->offset=(short)((char *)&prec->d36lr - (char *)prec);
  pdbRecordType->papFldDes[649]->size=sizeof(prec->d36da);
  pdbRecordType->papFldDes[649]->offset=(short)((char *)&prec->d36da - (char *)prec);
  pdbRecordType->papFldDes[650]->size=sizeof(prec->d36cv);
  pdbRecordType->papFldDes[650]->offset=(short)((char *)&prec->d36cv - (char *)prec);
  pdbRecordType->papFldDes[651]->size=sizeof(prec->d36lv);
  pdbRecordType->papFldDes[651]->offset=(short)((char *)&prec->d36lv - (char *)prec);
  pdbRecordType->papFldDes[652]->size=sizeof(prec->d36ne);
  pdbRecordType->papFldDes[652]->offset=(short)((char *)&prec->d36ne - (char *)prec);
  pdbRecordType->papFldDes[653]->size=sizeof(prec->d36eu);
  pdbRecordType->papFldDes[653]->offset=(short)((char *)&prec->d36eu - (char *)prec);
  pdbRecordType->papFldDes[654]->size=sizeof(prec->d36pr);
  pdbRecordType->papFldDes[654]->offset=(short)((char *)&prec->d36pr - (char *)prec);
  pdbRecordType->papFldDes[655]->size=sizeof(prec->d37hr);
  pdbRecordType->papFldDes[655]->offset=(short)((char *)&prec->d37hr - (char *)prec);
  pdbRecordType->papFldDes[656]->size=sizeof(prec->d37lr);
  pdbRecordType->papFldDes[656]->offset=(short)((char *)&prec->d37lr - (char *)prec);
  pdbRecordType->papFldDes[657]->size=sizeof(prec->d37da);
  pdbRecordType->papFldDes[657]->offset=(short)((char *)&prec->d37da - (char *)prec);
  pdbRecordType->papFldDes[658]->size=sizeof(prec->d37cv);
  pdbRecordType->papFldDes[658]->offset=(short)((char *)&prec->d37cv - (char *)prec);
  pdbRecordType->papFldDes[659]->size=sizeof(prec->d37lv);
  pdbRecordType->papFldDes[659]->offset=(short)((char *)&prec->d37lv - (char *)prec);
  pdbRecordType->papFldDes[660]->size=sizeof(prec->d37ne);
  pdbRecordType->papFldDes[660]->offset=(short)((char *)&prec->d37ne - (char *)prec);
  pdbRecordType->papFldDes[661]->size=sizeof(prec->d37eu);
  pdbRecordType->papFldDes[661]->offset=(short)((char *)&prec->d37eu - (char *)prec);
  pdbRecordType->papFldDes[662]->size=sizeof(prec->d37pr);
  pdbRecordType->papFldDes[662]->offset=(short)((char *)&prec->d37pr - (char *)prec);
  pdbRecordType->papFldDes[663]->size=sizeof(prec->d38hr);
  pdbRecordType->papFldDes[663]->offset=(short)((char *)&prec->d38hr - (char *)prec);
  pdbRecordType->papFldDes[664]->size=sizeof(prec->d38lr);
  pdbRecordType->papFldDes[664]->offset=(short)((char *)&prec->d38lr - (char *)prec);
  pdbRecordType->papFldDes[665]->size=sizeof(prec->d38da);
  pdbRecordType->papFldDes[665]->offset=(short)((char *)&prec->d38da - (char *)prec);
  pdbRecordType->papFldDes[666]->size=sizeof(prec->d38cv);
  pdbRecordType->papFldDes[666]->offset=(short)((char *)&prec->d38cv - (char *)prec);
  pdbRecordType->papFldDes[667]->size=sizeof(prec->d38lv);
  pdbRecordType->papFldDes[667]->offset=(short)((char *)&prec->d38lv - (char *)prec);
  pdbRecordType->papFldDes[668]->size=sizeof(prec->d38ne);
  pdbRecordType->papFldDes[668]->offset=(short)((char *)&prec->d38ne - (char *)prec);
  pdbRecordType->papFldDes[669]->size=sizeof(prec->d38eu);
  pdbRecordType->papFldDes[669]->offset=(short)((char *)&prec->d38eu - (char *)prec);
  pdbRecordType->papFldDes[670]->size=sizeof(prec->d38pr);
  pdbRecordType->papFldDes[670]->offset=(short)((char *)&prec->d38pr - (char *)prec);
  pdbRecordType->papFldDes[671]->size=sizeof(prec->d39hr);
  pdbRecordType->papFldDes[671]->offset=(short)((char *)&prec->d39hr - (char *)prec);
  pdbRecordType->papFldDes[672]->size=sizeof(prec->d39lr);
  pdbRecordType->papFldDes[672]->offset=(short)((char *)&prec->d39lr - (char *)prec);
  pdbRecordType->papFldDes[673]->size=sizeof(prec->d39da);
  pdbRecordType->papFldDes[673]->offset=(short)((char *)&prec->d39da - (char *)prec);
  pdbRecordType->papFldDes[674]->size=sizeof(prec->d39cv);
  pdbRecordType->papFldDes[674]->offset=(short)((char *)&prec->d39cv - (char *)prec);
  pdbRecordType->papFldDes[675]->size=sizeof(prec->d39lv);
  pdbRecordType->papFldDes[675]->offset=(short)((char *)&prec->d39lv - (char *)prec);
  pdbRecordType->papFldDes[676]->size=sizeof(prec->d39ne);
  pdbRecordType->papFldDes[676]->offset=(short)((char *)&prec->d39ne - (char *)prec);
  pdbRecordType->papFldDes[677]->size=sizeof(prec->d39eu);
  pdbRecordType->papFldDes[677]->offset=(short)((char *)&prec->d39eu - (char *)prec);
  pdbRecordType->papFldDes[678]->size=sizeof(prec->d39pr);
  pdbRecordType->papFldDes[678]->offset=(short)((char *)&prec->d39pr - (char *)prec);
  pdbRecordType->papFldDes[679]->size=sizeof(prec->d40hr);
  pdbRecordType->papFldDes[679]->offset=(short)((char *)&prec->d40hr - (char *)prec);
  pdbRecordType->papFldDes[680]->size=sizeof(prec->d40lr);
  pdbRecordType->papFldDes[680]->offset=(short)((char *)&prec->d40lr - (char *)prec);
  pdbRecordType->papFldDes[681]->size=sizeof(prec->d40da);
  pdbRecordType->papFldDes[681]->offset=(short)((char *)&prec->d40da - (char *)prec);
  pdbRecordType->papFldDes[682]->size=sizeof(prec->d40cv);
  pdbRecordType->papFldDes[682]->offset=(short)((char *)&prec->d40cv - (char *)prec);
  pdbRecordType->papFldDes[683]->size=sizeof(prec->d40lv);
  pdbRecordType->papFldDes[683]->offset=(short)((char *)&prec->d40lv - (char *)prec);
  pdbRecordType->papFldDes[684]->size=sizeof(prec->d40ne);
  pdbRecordType->papFldDes[684]->offset=(short)((char *)&prec->d40ne - (char *)prec);
  pdbRecordType->papFldDes[685]->size=sizeof(prec->d40eu);
  pdbRecordType->papFldDes[685]->offset=(short)((char *)&prec->d40eu - (char *)prec);
  pdbRecordType->papFldDes[686]->size=sizeof(prec->d40pr);
  pdbRecordType->papFldDes[686]->offset=(short)((char *)&prec->d40pr - (char *)prec);
  pdbRecordType->papFldDes[687]->size=sizeof(prec->d41hr);
  pdbRecordType->papFldDes[687]->offset=(short)((char *)&prec->d41hr - (char *)prec);
  pdbRecordType->papFldDes[688]->size=sizeof(prec->d41lr);
  pdbRecordType->papFldDes[688]->offset=(short)((char *)&prec->d41lr - (char *)prec);
  pdbRecordType->papFldDes[689]->size=sizeof(prec->d41da);
  pdbRecordType->papFldDes[689]->offset=(short)((char *)&prec->d41da - (char *)prec);
  pdbRecordType->papFldDes[690]->size=sizeof(prec->d41cv);
  pdbRecordType->papFldDes[690]->offset=(short)((char *)&prec->d41cv - (char *)prec);
  pdbRecordType->papFldDes[691]->size=sizeof(prec->d41lv);
  pdbRecordType->papFldDes[691]->offset=(short)((char *)&prec->d41lv - (char *)prec);
  pdbRecordType->papFldDes[692]->size=sizeof(prec->d41ne);
  pdbRecordType->papFldDes[692]->offset=(short)((char *)&prec->d41ne - (char *)prec);
  pdbRecordType->papFldDes[693]->size=sizeof(prec->d41eu);
  pdbRecordType->papFldDes[693]->offset=(short)((char *)&prec->d41eu - (char *)prec);
  pdbRecordType->papFldDes[694]->size=sizeof(prec->d41pr);
  pdbRecordType->papFldDes[694]->offset=(short)((char *)&prec->d41pr - (char *)prec);
  pdbRecordType->papFldDes[695]->size=sizeof(prec->d42hr);
  pdbRecordType->papFldDes[695]->offset=(short)((char *)&prec->d42hr - (char *)prec);
  pdbRecordType->papFldDes[696]->size=sizeof(prec->d42lr);
  pdbRecordType->papFldDes[696]->offset=(short)((char *)&prec->d42lr - (char *)prec);
  pdbRecordType->papFldDes[697]->size=sizeof(prec->d42da);
  pdbRecordType->papFldDes[697]->offset=(short)((char *)&prec->d42da - (char *)prec);
  pdbRecordType->papFldDes[698]->size=sizeof(prec->d42cv);
  pdbRecordType->papFldDes[698]->offset=(short)((char *)&prec->d42cv - (char *)prec);
  pdbRecordType->papFldDes[699]->size=sizeof(prec->d42lv);
  pdbRecordType->papFldDes[699]->offset=(short)((char *)&prec->d42lv - (char *)prec);
  pdbRecordType->papFldDes[700]->size=sizeof(prec->d42ne);
  pdbRecordType->papFldDes[700]->offset=(short)((char *)&prec->d42ne - (char *)prec);
  pdbRecordType->papFldDes[701]->size=sizeof(prec->d42eu);
  pdbRecordType->papFldDes[701]->offset=(short)((char *)&prec->d42eu - (char *)prec);
  pdbRecordType->papFldDes[702]->size=sizeof(prec->d42pr);
  pdbRecordType->papFldDes[702]->offset=(short)((char *)&prec->d42pr - (char *)prec);
  pdbRecordType->papFldDes[703]->size=sizeof(prec->d43hr);
  pdbRecordType->papFldDes[703]->offset=(short)((char *)&prec->d43hr - (char *)prec);
  pdbRecordType->papFldDes[704]->size=sizeof(prec->d43lr);
  pdbRecordType->papFldDes[704]->offset=(short)((char *)&prec->d43lr - (char *)prec);
  pdbRecordType->papFldDes[705]->size=sizeof(prec->d43da);
  pdbRecordType->papFldDes[705]->offset=(short)((char *)&prec->d43da - (char *)prec);
  pdbRecordType->papFldDes[706]->size=sizeof(prec->d43cv);
  pdbRecordType->papFldDes[706]->offset=(short)((char *)&prec->d43cv - (char *)prec);
  pdbRecordType->papFldDes[707]->size=sizeof(prec->d43lv);
  pdbRecordType->papFldDes[707]->offset=(short)((char *)&prec->d43lv - (char *)prec);
  pdbRecordType->papFldDes[708]->size=sizeof(prec->d43ne);
  pdbRecordType->papFldDes[708]->offset=(short)((char *)&prec->d43ne - (char *)prec);
  pdbRecordType->papFldDes[709]->size=sizeof(prec->d43eu);
  pdbRecordType->papFldDes[709]->offset=(short)((char *)&prec->d43eu - (char *)prec);
  pdbRecordType->papFldDes[710]->size=sizeof(prec->d43pr);
  pdbRecordType->papFldDes[710]->offset=(short)((char *)&prec->d43pr - (char *)prec);
  pdbRecordType->papFldDes[711]->size=sizeof(prec->d44hr);
  pdbRecordType->papFldDes[711]->offset=(short)((char *)&prec->d44hr - (char *)prec);
  pdbRecordType->papFldDes[712]->size=sizeof(prec->d44lr);
  pdbRecordType->papFldDes[712]->offset=(short)((char *)&prec->d44lr - (char *)prec);
  pdbRecordType->papFldDes[713]->size=sizeof(prec->d44da);
  pdbRecordType->papFldDes[713]->offset=(short)((char *)&prec->d44da - (char *)prec);
  pdbRecordType->papFldDes[714]->size=sizeof(prec->d44cv);
  pdbRecordType->papFldDes[714]->offset=(short)((char *)&prec->d44cv - (char *)prec);
  pdbRecordType->papFldDes[715]->size=sizeof(prec->d44lv);
  pdbRecordType->papFldDes[715]->offset=(short)((char *)&prec->d44lv - (char *)prec);
  pdbRecordType->papFldDes[716]->size=sizeof(prec->d44ne);
  pdbRecordType->papFldDes[716]->offset=(short)((char *)&prec->d44ne - (char *)prec);
  pdbRecordType->papFldDes[717]->size=sizeof(prec->d44eu);
  pdbRecordType->papFldDes[717]->offset=(short)((char *)&prec->d44eu - (char *)prec);
  pdbRecordType->papFldDes[718]->size=sizeof(prec->d44pr);
  pdbRecordType->papFldDes[718]->offset=(short)((char *)&prec->d44pr - (char *)prec);
  pdbRecordType->papFldDes[719]->size=sizeof(prec->d45hr);
  pdbRecordType->papFldDes[719]->offset=(short)((char *)&prec->d45hr - (char *)prec);
  pdbRecordType->papFldDes[720]->size=sizeof(prec->d45lr);
  pdbRecordType->papFldDes[720]->offset=(short)((char *)&prec->d45lr - (char *)prec);
  pdbRecordType->papFldDes[721]->size=sizeof(prec->d45da);
  pdbRecordType->papFldDes[721]->offset=(short)((char *)&prec->d45da - (char *)prec);
  pdbRecordType->papFldDes[722]->size=sizeof(prec->d45cv);
  pdbRecordType->papFldDes[722]->offset=(short)((char *)&prec->d45cv - (char *)prec);
  pdbRecordType->papFldDes[723]->size=sizeof(prec->d45lv);
  pdbRecordType->papFldDes[723]->offset=(short)((char *)&prec->d45lv - (char *)prec);
  pdbRecordType->papFldDes[724]->size=sizeof(prec->d45ne);
  pdbRecordType->papFldDes[724]->offset=(short)((char *)&prec->d45ne - (char *)prec);
  pdbRecordType->papFldDes[725]->size=sizeof(prec->d45eu);
  pdbRecordType->papFldDes[725]->offset=(short)((char *)&prec->d45eu - (char *)prec);
  pdbRecordType->papFldDes[726]->size=sizeof(prec->d45pr);
  pdbRecordType->papFldDes[726]->offset=(short)((char *)&prec->d45pr - (char *)prec);
  pdbRecordType->papFldDes[727]->size=sizeof(prec->d46hr);
  pdbRecordType->papFldDes[727]->offset=(short)((char *)&prec->d46hr - (char *)prec);
  pdbRecordType->papFldDes[728]->size=sizeof(prec->d46lr);
  pdbRecordType->papFldDes[728]->offset=(short)((char *)&prec->d46lr - (char *)prec);
  pdbRecordType->papFldDes[729]->size=sizeof(prec->d46da);
  pdbRecordType->papFldDes[729]->offset=(short)((char *)&prec->d46da - (char *)prec);
  pdbRecordType->papFldDes[730]->size=sizeof(prec->d46cv);
  pdbRecordType->papFldDes[730]->offset=(short)((char *)&prec->d46cv - (char *)prec);
  pdbRecordType->papFldDes[731]->size=sizeof(prec->d46lv);
  pdbRecordType->papFldDes[731]->offset=(short)((char *)&prec->d46lv - (char *)prec);
  pdbRecordType->papFldDes[732]->size=sizeof(prec->d46ne);
  pdbRecordType->papFldDes[732]->offset=(short)((char *)&prec->d46ne - (char *)prec);
  pdbRecordType->papFldDes[733]->size=sizeof(prec->d46eu);
  pdbRecordType->papFldDes[733]->offset=(short)((char *)&prec->d46eu - (char *)prec);
  pdbRecordType->papFldDes[734]->size=sizeof(prec->d46pr);
  pdbRecordType->papFldDes[734]->offset=(short)((char *)&prec->d46pr - (char *)prec);
  pdbRecordType->papFldDes[735]->size=sizeof(prec->d47hr);
  pdbRecordType->papFldDes[735]->offset=(short)((char *)&prec->d47hr - (char *)prec);
  pdbRecordType->papFldDes[736]->size=sizeof(prec->d47lr);
  pdbRecordType->papFldDes[736]->offset=(short)((char *)&prec->d47lr - (char *)prec);
  pdbRecordType->papFldDes[737]->size=sizeof(prec->d47da);
  pdbRecordType->papFldDes[737]->offset=(short)((char *)&prec->d47da - (char *)prec);
  pdbRecordType->papFldDes[738]->size=sizeof(prec->d47cv);
  pdbRecordType->papFldDes[738]->offset=(short)((char *)&prec->d47cv - (char *)prec);
  pdbRecordType->papFldDes[739]->size=sizeof(prec->d47lv);
  pdbRecordType->papFldDes[739]->offset=(short)((char *)&prec->d47lv - (char *)prec);
  pdbRecordType->papFldDes[740]->size=sizeof(prec->d47ne);
  pdbRecordType->papFldDes[740]->offset=(short)((char *)&prec->d47ne - (char *)prec);
  pdbRecordType->papFldDes[741]->size=sizeof(prec->d47eu);
  pdbRecordType->papFldDes[741]->offset=(short)((char *)&prec->d47eu - (char *)prec);
  pdbRecordType->papFldDes[742]->size=sizeof(prec->d47pr);
  pdbRecordType->papFldDes[742]->offset=(short)((char *)&prec->d47pr - (char *)prec);
  pdbRecordType->papFldDes[743]->size=sizeof(prec->d48hr);
  pdbRecordType->papFldDes[743]->offset=(short)((char *)&prec->d48hr - (char *)prec);
  pdbRecordType->papFldDes[744]->size=sizeof(prec->d48lr);
  pdbRecordType->papFldDes[744]->offset=(short)((char *)&prec->d48lr - (char *)prec);
  pdbRecordType->papFldDes[745]->size=sizeof(prec->d48da);
  pdbRecordType->papFldDes[745]->offset=(short)((char *)&prec->d48da - (char *)prec);
  pdbRecordType->papFldDes[746]->size=sizeof(prec->d48cv);
  pdbRecordType->papFldDes[746]->offset=(short)((char *)&prec->d48cv - (char *)prec);
  pdbRecordType->papFldDes[747]->size=sizeof(prec->d48lv);
  pdbRecordType->papFldDes[747]->offset=(short)((char *)&prec->d48lv - (char *)prec);
  pdbRecordType->papFldDes[748]->size=sizeof(prec->d48ne);
  pdbRecordType->papFldDes[748]->offset=(short)((char *)&prec->d48ne - (char *)prec);
  pdbRecordType->papFldDes[749]->size=sizeof(prec->d48eu);
  pdbRecordType->papFldDes[749]->offset=(short)((char *)&prec->d48eu - (char *)prec);
  pdbRecordType->papFldDes[750]->size=sizeof(prec->d48pr);
  pdbRecordType->papFldDes[750]->offset=(short)((char *)&prec->d48pr - (char *)prec);
  pdbRecordType->papFldDes[751]->size=sizeof(prec->d49hr);
  pdbRecordType->papFldDes[751]->offset=(short)((char *)&prec->d49hr - (char *)prec);
  pdbRecordType->papFldDes[752]->size=sizeof(prec->d49lr);
  pdbRecordType->papFldDes[752]->offset=(short)((char *)&prec->d49lr - (char *)prec);
  pdbRecordType->papFldDes[753]->size=sizeof(prec->d49da);
  pdbRecordType->papFldDes[753]->offset=(short)((char *)&prec->d49da - (char *)prec);
  pdbRecordType->papFldDes[754]->size=sizeof(prec->d49cv);
  pdbRecordType->papFldDes[754]->offset=(short)((char *)&prec->d49cv - (char *)prec);
  pdbRecordType->papFldDes[755]->size=sizeof(prec->d49lv);
  pdbRecordType->papFldDes[755]->offset=(short)((char *)&prec->d49lv - (char *)prec);
  pdbRecordType->papFldDes[756]->size=sizeof(prec->d49ne);
  pdbRecordType->papFldDes[756]->offset=(short)((char *)&prec->d49ne - (char *)prec);
  pdbRecordType->papFldDes[757]->size=sizeof(prec->d49eu);
  pdbRecordType->papFldDes[757]->offset=(short)((char *)&prec->d49eu - (char *)prec);
  pdbRecordType->papFldDes[758]->size=sizeof(prec->d49pr);
  pdbRecordType->papFldDes[758]->offset=(short)((char *)&prec->d49pr - (char *)prec);
  pdbRecordType->papFldDes[759]->size=sizeof(prec->d50hr);
  pdbRecordType->papFldDes[759]->offset=(short)((char *)&prec->d50hr - (char *)prec);
  pdbRecordType->papFldDes[760]->size=sizeof(prec->d50lr);
  pdbRecordType->papFldDes[760]->offset=(short)((char *)&prec->d50lr - (char *)prec);
  pdbRecordType->papFldDes[761]->size=sizeof(prec->d50da);
  pdbRecordType->papFldDes[761]->offset=(short)((char *)&prec->d50da - (char *)prec);
  pdbRecordType->papFldDes[762]->size=sizeof(prec->d50cv);
  pdbRecordType->papFldDes[762]->offset=(short)((char *)&prec->d50cv - (char *)prec);
  pdbRecordType->papFldDes[763]->size=sizeof(prec->d50lv);
  pdbRecordType->papFldDes[763]->offset=(short)((char *)&prec->d50lv - (char *)prec);
  pdbRecordType->papFldDes[764]->size=sizeof(prec->d50ne);
  pdbRecordType->papFldDes[764]->offset=(short)((char *)&prec->d50ne - (char *)prec);
  pdbRecordType->papFldDes[765]->size=sizeof(prec->d50eu);
  pdbRecordType->papFldDes[765]->offset=(short)((char *)&prec->d50eu - (char *)prec);
  pdbRecordType->papFldDes[766]->size=sizeof(prec->d50pr);
  pdbRecordType->papFldDes[766]->offset=(short)((char *)&prec->d50pr - (char *)prec);
  pdbRecordType->papFldDes[767]->size=sizeof(prec->d51hr);
  pdbRecordType->papFldDes[767]->offset=(short)((char *)&prec->d51hr - (char *)prec);
  pdbRecordType->papFldDes[768]->size=sizeof(prec->d51lr);
  pdbRecordType->papFldDes[768]->offset=(short)((char *)&prec->d51lr - (char *)prec);
  pdbRecordType->papFldDes[769]->size=sizeof(prec->d51da);
  pdbRecordType->papFldDes[769]->offset=(short)((char *)&prec->d51da - (char *)prec);
  pdbRecordType->papFldDes[770]->size=sizeof(prec->d51cv);
  pdbRecordType->papFldDes[770]->offset=(short)((char *)&prec->d51cv - (char *)prec);
  pdbRecordType->papFldDes[771]->size=sizeof(prec->d51lv);
  pdbRecordType->papFldDes[771]->offset=(short)((char *)&prec->d51lv - (char *)prec);
  pdbRecordType->papFldDes[772]->size=sizeof(prec->d51ne);
  pdbRecordType->papFldDes[772]->offset=(short)((char *)&prec->d51ne - (char *)prec);
  pdbRecordType->papFldDes[773]->size=sizeof(prec->d51eu);
  pdbRecordType->papFldDes[773]->offset=(short)((char *)&prec->d51eu - (char *)prec);
  pdbRecordType->papFldDes[774]->size=sizeof(prec->d51pr);
  pdbRecordType->papFldDes[774]->offset=(short)((char *)&prec->d51pr - (char *)prec);
  pdbRecordType->papFldDes[775]->size=sizeof(prec->d52hr);
  pdbRecordType->papFldDes[775]->offset=(short)((char *)&prec->d52hr - (char *)prec);
  pdbRecordType->papFldDes[776]->size=sizeof(prec->d52lr);
  pdbRecordType->papFldDes[776]->offset=(short)((char *)&prec->d52lr - (char *)prec);
  pdbRecordType->papFldDes[777]->size=sizeof(prec->d52da);
  pdbRecordType->papFldDes[777]->offset=(short)((char *)&prec->d52da - (char *)prec);
  pdbRecordType->papFldDes[778]->size=sizeof(prec->d52cv);
  pdbRecordType->papFldDes[778]->offset=(short)((char *)&prec->d52cv - (char *)prec);
  pdbRecordType->papFldDes[779]->size=sizeof(prec->d52lv);
  pdbRecordType->papFldDes[779]->offset=(short)((char *)&prec->d52lv - (char *)prec);
  pdbRecordType->papFldDes[780]->size=sizeof(prec->d52ne);
  pdbRecordType->papFldDes[780]->offset=(short)((char *)&prec->d52ne - (char *)prec);
  pdbRecordType->papFldDes[781]->size=sizeof(prec->d52eu);
  pdbRecordType->papFldDes[781]->offset=(short)((char *)&prec->d52eu - (char *)prec);
  pdbRecordType->papFldDes[782]->size=sizeof(prec->d52pr);
  pdbRecordType->papFldDes[782]->offset=(short)((char *)&prec->d52pr - (char *)prec);
  pdbRecordType->papFldDes[783]->size=sizeof(prec->d53hr);
  pdbRecordType->papFldDes[783]->offset=(short)((char *)&prec->d53hr - (char *)prec);
  pdbRecordType->papFldDes[784]->size=sizeof(prec->d53lr);
  pdbRecordType->papFldDes[784]->offset=(short)((char *)&prec->d53lr - (char *)prec);
  pdbRecordType->papFldDes[785]->size=sizeof(prec->d53da);
  pdbRecordType->papFldDes[785]->offset=(short)((char *)&prec->d53da - (char *)prec);
  pdbRecordType->papFldDes[786]->size=sizeof(prec->d53cv);
  pdbRecordType->papFldDes[786]->offset=(short)((char *)&prec->d53cv - (char *)prec);
  pdbRecordType->papFldDes[787]->size=sizeof(prec->d53lv);
  pdbRecordType->papFldDes[787]->offset=(short)((char *)&prec->d53lv - (char *)prec);
  pdbRecordType->papFldDes[788]->size=sizeof(prec->d53ne);
  pdbRecordType->papFldDes[788]->offset=(short)((char *)&prec->d53ne - (char *)prec);
  pdbRecordType->papFldDes[789]->size=sizeof(prec->d53eu);
  pdbRecordType->papFldDes[789]->offset=(short)((char *)&prec->d53eu - (char *)prec);
  pdbRecordType->papFldDes[790]->size=sizeof(prec->d53pr);
  pdbRecordType->papFldDes[790]->offset=(short)((char *)&prec->d53pr - (char *)prec);
  pdbRecordType->papFldDes[791]->size=sizeof(prec->d54hr);
  pdbRecordType->papFldDes[791]->offset=(short)((char *)&prec->d54hr - (char *)prec);
  pdbRecordType->papFldDes[792]->size=sizeof(prec->d54lr);
  pdbRecordType->papFldDes[792]->offset=(short)((char *)&prec->d54lr - (char *)prec);
  pdbRecordType->papFldDes[793]->size=sizeof(prec->d54da);
  pdbRecordType->papFldDes[793]->offset=(short)((char *)&prec->d54da - (char *)prec);
  pdbRecordType->papFldDes[794]->size=sizeof(prec->d54cv);
  pdbRecordType->papFldDes[794]->offset=(short)((char *)&prec->d54cv - (char *)prec);
  pdbRecordType->papFldDes[795]->size=sizeof(prec->d54lv);
  pdbRecordType->papFldDes[795]->offset=(short)((char *)&prec->d54lv - (char *)prec);
  pdbRecordType->papFldDes[796]->size=sizeof(prec->d54ne);
  pdbRecordType->papFldDes[796]->offset=(short)((char *)&prec->d54ne - (char *)prec);
  pdbRecordType->papFldDes[797]->size=sizeof(prec->d54eu);
  pdbRecordType->papFldDes[797]->offset=(short)((char *)&prec->d54eu - (char *)prec);
  pdbRecordType->papFldDes[798]->size=sizeof(prec->d54pr);
  pdbRecordType->papFldDes[798]->offset=(short)((char *)&prec->d54pr - (char *)prec);
  pdbRecordType->papFldDes[799]->size=sizeof(prec->d55hr);
  pdbRecordType->papFldDes[799]->offset=(short)((char *)&prec->d55hr - (char *)prec);
  pdbRecordType->papFldDes[800]->size=sizeof(prec->d55lr);
  pdbRecordType->papFldDes[800]->offset=(short)((char *)&prec->d55lr - (char *)prec);
  pdbRecordType->papFldDes[801]->size=sizeof(prec->d55da);
  pdbRecordType->papFldDes[801]->offset=(short)((char *)&prec->d55da - (char *)prec);
  pdbRecordType->papFldDes[802]->size=sizeof(prec->d55cv);
  pdbRecordType->papFldDes[802]->offset=(short)((char *)&prec->d55cv - (char *)prec);
  pdbRecordType->papFldDes[803]->size=sizeof(prec->d55lv);
  pdbRecordType->papFldDes[803]->offset=(short)((char *)&prec->d55lv - (char *)prec);
  pdbRecordType->papFldDes[804]->size=sizeof(prec->d55ne);
  pdbRecordType->papFldDes[804]->offset=(short)((char *)&prec->d55ne - (char *)prec);
  pdbRecordType->papFldDes[805]->size=sizeof(prec->d55eu);
  pdbRecordType->papFldDes[805]->offset=(short)((char *)&prec->d55eu - (char *)prec);
  pdbRecordType->papFldDes[806]->size=sizeof(prec->d55pr);
  pdbRecordType->papFldDes[806]->offset=(short)((char *)&prec->d55pr - (char *)prec);
  pdbRecordType->papFldDes[807]->size=sizeof(prec->d56hr);
  pdbRecordType->papFldDes[807]->offset=(short)((char *)&prec->d56hr - (char *)prec);
  pdbRecordType->papFldDes[808]->size=sizeof(prec->d56lr);
  pdbRecordType->papFldDes[808]->offset=(short)((char *)&prec->d56lr - (char *)prec);
  pdbRecordType->papFldDes[809]->size=sizeof(prec->d56da);
  pdbRecordType->papFldDes[809]->offset=(short)((char *)&prec->d56da - (char *)prec);
  pdbRecordType->papFldDes[810]->size=sizeof(prec->d56cv);
  pdbRecordType->papFldDes[810]->offset=(short)((char *)&prec->d56cv - (char *)prec);
  pdbRecordType->papFldDes[811]->size=sizeof(prec->d56lv);
  pdbRecordType->papFldDes[811]->offset=(short)((char *)&prec->d56lv - (char *)prec);
  pdbRecordType->papFldDes[812]->size=sizeof(prec->d56ne);
  pdbRecordType->papFldDes[812]->offset=(short)((char *)&prec->d56ne - (char *)prec);
  pdbRecordType->papFldDes[813]->size=sizeof(prec->d56eu);
  pdbRecordType->papFldDes[813]->offset=(short)((char *)&prec->d56eu - (char *)prec);
  pdbRecordType->papFldDes[814]->size=sizeof(prec->d56pr);
  pdbRecordType->papFldDes[814]->offset=(short)((char *)&prec->d56pr - (char *)prec);
  pdbRecordType->papFldDes[815]->size=sizeof(prec->d57hr);
  pdbRecordType->papFldDes[815]->offset=(short)((char *)&prec->d57hr - (char *)prec);
  pdbRecordType->papFldDes[816]->size=sizeof(prec->d57lr);
  pdbRecordType->papFldDes[816]->offset=(short)((char *)&prec->d57lr - (char *)prec);
  pdbRecordType->papFldDes[817]->size=sizeof(prec->d57da);
  pdbRecordType->papFldDes[817]->offset=(short)((char *)&prec->d57da - (char *)prec);
  pdbRecordType->papFldDes[818]->size=sizeof(prec->d57cv);
  pdbRecordType->papFldDes[818]->offset=(short)((char *)&prec->d57cv - (char *)prec);
  pdbRecordType->papFldDes[819]->size=sizeof(prec->d57lv);
  pdbRecordType->papFldDes[819]->offset=(short)((char *)&prec->d57lv - (char *)prec);
  pdbRecordType->papFldDes[820]->size=sizeof(prec->d57ne);
  pdbRecordType->papFldDes[820]->offset=(short)((char *)&prec->d57ne - (char *)prec);
  pdbRecordType->papFldDes[821]->size=sizeof(prec->d57eu);
  pdbRecordType->papFldDes[821]->offset=(short)((char *)&prec->d57eu - (char *)prec);
  pdbRecordType->papFldDes[822]->size=sizeof(prec->d57pr);
  pdbRecordType->papFldDes[822]->offset=(short)((char *)&prec->d57pr - (char *)prec);
  pdbRecordType->papFldDes[823]->size=sizeof(prec->d58hr);
  pdbRecordType->papFldDes[823]->offset=(short)((char *)&prec->d58hr - (char *)prec);
  pdbRecordType->papFldDes[824]->size=sizeof(prec->d58lr);
  pdbRecordType->papFldDes[824]->offset=(short)((char *)&prec->d58lr - (char *)prec);
  pdbRecordType->papFldDes[825]->size=sizeof(prec->d58da);
  pdbRecordType->papFldDes[825]->offset=(short)((char *)&prec->d58da - (char *)prec);
  pdbRecordType->papFldDes[826]->size=sizeof(prec->d58cv);
  pdbRecordType->papFldDes[826]->offset=(short)((char *)&prec->d58cv - (char *)prec);
  pdbRecordType->papFldDes[827]->size=sizeof(prec->d58lv);
  pdbRecordType->papFldDes[827]->offset=(short)((char *)&prec->d58lv - (char *)prec);
  pdbRecordType->papFldDes[828]->size=sizeof(prec->d58ne);
  pdbRecordType->papFldDes[828]->offset=(short)((char *)&prec->d58ne - (char *)prec);
  pdbRecordType->papFldDes[829]->size=sizeof(prec->d58eu);
  pdbRecordType->papFldDes[829]->offset=(short)((char *)&prec->d58eu - (char *)prec);
  pdbRecordType->papFldDes[830]->size=sizeof(prec->d58pr);
  pdbRecordType->papFldDes[830]->offset=(short)((char *)&prec->d58pr - (char *)prec);
  pdbRecordType->papFldDes[831]->size=sizeof(prec->d59hr);
  pdbRecordType->papFldDes[831]->offset=(short)((char *)&prec->d59hr - (char *)prec);
  pdbRecordType->papFldDes[832]->size=sizeof(prec->d59lr);
  pdbRecordType->papFldDes[832]->offset=(short)((char *)&prec->d59lr - (char *)prec);
  pdbRecordType->papFldDes[833]->size=sizeof(prec->d59da);
  pdbRecordType->papFldDes[833]->offset=(short)((char *)&prec->d59da - (char *)prec);
  pdbRecordType->papFldDes[834]->size=sizeof(prec->d59cv);
  pdbRecordType->papFldDes[834]->offset=(short)((char *)&prec->d59cv - (char *)prec);
  pdbRecordType->papFldDes[835]->size=sizeof(prec->d59lv);
  pdbRecordType->papFldDes[835]->offset=(short)((char *)&prec->d59lv - (char *)prec);
  pdbRecordType->papFldDes[836]->size=sizeof(prec->d59ne);
  pdbRecordType->papFldDes[836]->offset=(short)((char *)&prec->d59ne - (char *)prec);
  pdbRecordType->papFldDes[837]->size=sizeof(prec->d59eu);
  pdbRecordType->papFldDes[837]->offset=(short)((char *)&prec->d59eu - (char *)prec);
  pdbRecordType->papFldDes[838]->size=sizeof(prec->d59pr);
  pdbRecordType->papFldDes[838]->offset=(short)((char *)&prec->d59pr - (char *)prec);
  pdbRecordType->papFldDes[839]->size=sizeof(prec->d60hr);
  pdbRecordType->papFldDes[839]->offset=(short)((char *)&prec->d60hr - (char *)prec);
  pdbRecordType->papFldDes[840]->size=sizeof(prec->d60lr);
  pdbRecordType->papFldDes[840]->offset=(short)((char *)&prec->d60lr - (char *)prec);
  pdbRecordType->papFldDes[841]->size=sizeof(prec->d60da);
  pdbRecordType->papFldDes[841]->offset=(short)((char *)&prec->d60da - (char *)prec);
  pdbRecordType->papFldDes[842]->size=sizeof(prec->d60cv);
  pdbRecordType->papFldDes[842]->offset=(short)((char *)&prec->d60cv - (char *)prec);
  pdbRecordType->papFldDes[843]->size=sizeof(prec->d60lv);
  pdbRecordType->papFldDes[843]->offset=(short)((char *)&prec->d60lv - (char *)prec);
  pdbRecordType->papFldDes[844]->size=sizeof(prec->d60ne);
  pdbRecordType->papFldDes[844]->offset=(short)((char *)&prec->d60ne - (char *)prec);
  pdbRecordType->papFldDes[845]->size=sizeof(prec->d60eu);
  pdbRecordType->papFldDes[845]->offset=(short)((char *)&prec->d60eu - (char *)prec);
  pdbRecordType->papFldDes[846]->size=sizeof(prec->d60pr);
  pdbRecordType->papFldDes[846]->offset=(short)((char *)&prec->d60pr - (char *)prec);
  pdbRecordType->papFldDes[847]->size=sizeof(prec->d61hr);
  pdbRecordType->papFldDes[847]->offset=(short)((char *)&prec->d61hr - (char *)prec);
  pdbRecordType->papFldDes[848]->size=sizeof(prec->d61lr);
  pdbRecordType->papFldDes[848]->offset=(short)((char *)&prec->d61lr - (char *)prec);
  pdbRecordType->papFldDes[849]->size=sizeof(prec->d61da);
  pdbRecordType->papFldDes[849]->offset=(short)((char *)&prec->d61da - (char *)prec);
  pdbRecordType->papFldDes[850]->size=sizeof(prec->d61cv);
  pdbRecordType->papFldDes[850]->offset=(short)((char *)&prec->d61cv - (char *)prec);
  pdbRecordType->papFldDes[851]->size=sizeof(prec->d61lv);
  pdbRecordType->papFldDes[851]->offset=(short)((char *)&prec->d61lv - (char *)prec);
  pdbRecordType->papFldDes[852]->size=sizeof(prec->d61ne);
  pdbRecordType->papFldDes[852]->offset=(short)((char *)&prec->d61ne - (char *)prec);
  pdbRecordType->papFldDes[853]->size=sizeof(prec->d61eu);
  pdbRecordType->papFldDes[853]->offset=(short)((char *)&prec->d61eu - (char *)prec);
  pdbRecordType->papFldDes[854]->size=sizeof(prec->d61pr);
  pdbRecordType->papFldDes[854]->offset=(short)((char *)&prec->d61pr - (char *)prec);
  pdbRecordType->papFldDes[855]->size=sizeof(prec->d62hr);
  pdbRecordType->papFldDes[855]->offset=(short)((char *)&prec->d62hr - (char *)prec);
  pdbRecordType->papFldDes[856]->size=sizeof(prec->d62lr);
  pdbRecordType->papFldDes[856]->offset=(short)((char *)&prec->d62lr - (char *)prec);
  pdbRecordType->papFldDes[857]->size=sizeof(prec->d62da);
  pdbRecordType->papFldDes[857]->offset=(short)((char *)&prec->d62da - (char *)prec);
  pdbRecordType->papFldDes[858]->size=sizeof(prec->d62cv);
  pdbRecordType->papFldDes[858]->offset=(short)((char *)&prec->d62cv - (char *)prec);
  pdbRecordType->papFldDes[859]->size=sizeof(prec->d62lv);
  pdbRecordType->papFldDes[859]->offset=(short)((char *)&prec->d62lv - (char *)prec);
  pdbRecordType->papFldDes[860]->size=sizeof(prec->d62ne);
  pdbRecordType->papFldDes[860]->offset=(short)((char *)&prec->d62ne - (char *)prec);
  pdbRecordType->papFldDes[861]->size=sizeof(prec->d62eu);
  pdbRecordType->papFldDes[861]->offset=(short)((char *)&prec->d62eu - (char *)prec);
  pdbRecordType->papFldDes[862]->size=sizeof(prec->d62pr);
  pdbRecordType->papFldDes[862]->offset=(short)((char *)&prec->d62pr - (char *)prec);
  pdbRecordType->papFldDes[863]->size=sizeof(prec->d63hr);
  pdbRecordType->papFldDes[863]->offset=(short)((char *)&prec->d63hr - (char *)prec);
  pdbRecordType->papFldDes[864]->size=sizeof(prec->d63lr);
  pdbRecordType->papFldDes[864]->offset=(short)((char *)&prec->d63lr - (char *)prec);
  pdbRecordType->papFldDes[865]->size=sizeof(prec->d63da);
  pdbRecordType->papFldDes[865]->offset=(short)((char *)&prec->d63da - (char *)prec);
  pdbRecordType->papFldDes[866]->size=sizeof(prec->d63cv);
  pdbRecordType->papFldDes[866]->offset=(short)((char *)&prec->d63cv - (char *)prec);
  pdbRecordType->papFldDes[867]->size=sizeof(prec->d63lv);
  pdbRecordType->papFldDes[867]->offset=(short)((char *)&prec->d63lv - (char *)prec);
  pdbRecordType->papFldDes[868]->size=sizeof(prec->d63ne);
  pdbRecordType->papFldDes[868]->offset=(short)((char *)&prec->d63ne - (char *)prec);
  pdbRecordType->papFldDes[869]->size=sizeof(prec->d63eu);
  pdbRecordType->papFldDes[869]->offset=(short)((char *)&prec->d63eu - (char *)prec);
  pdbRecordType->papFldDes[870]->size=sizeof(prec->d63pr);
  pdbRecordType->papFldDes[870]->offset=(short)((char *)&prec->d63pr - (char *)prec);
  pdbRecordType->papFldDes[871]->size=sizeof(prec->d64hr);
  pdbRecordType->papFldDes[871]->offset=(short)((char *)&prec->d64hr - (char *)prec);
  pdbRecordType->papFldDes[872]->size=sizeof(prec->d64lr);
  pdbRecordType->papFldDes[872]->offset=(short)((char *)&prec->d64lr - (char *)prec);
  pdbRecordType->papFldDes[873]->size=sizeof(prec->d64da);
  pdbRecordType->papFldDes[873]->offset=(short)((char *)&prec->d64da - (char *)prec);
  pdbRecordType->papFldDes[874]->size=sizeof(prec->d64cv);
  pdbRecordType->papFldDes[874]->offset=(short)((char *)&prec->d64cv - (char *)prec);
  pdbRecordType->papFldDes[875]->size=sizeof(prec->d64lv);
  pdbRecordType->papFldDes[875]->offset=(short)((char *)&prec->d64lv - (char *)prec);
  pdbRecordType->papFldDes[876]->size=sizeof(prec->d64ne);
  pdbRecordType->papFldDes[876]->offset=(short)((char *)&prec->d64ne - (char *)prec);
  pdbRecordType->papFldDes[877]->size=sizeof(prec->d64eu);
  pdbRecordType->papFldDes[877]->offset=(short)((char *)&prec->d64eu - (char *)prec);
  pdbRecordType->papFldDes[878]->size=sizeof(prec->d64pr);
  pdbRecordType->papFldDes[878]->offset=(short)((char *)&prec->d64pr - (char *)prec);
  pdbRecordType->papFldDes[879]->size=sizeof(prec->d65hr);
  pdbRecordType->papFldDes[879]->offset=(short)((char *)&prec->d65hr - (char *)prec);
  pdbRecordType->papFldDes[880]->size=sizeof(prec->d65lr);
  pdbRecordType->papFldDes[880]->offset=(short)((char *)&prec->d65lr - (char *)prec);
  pdbRecordType->papFldDes[881]->size=sizeof(prec->d65da);
  pdbRecordType->papFldDes[881]->offset=(short)((char *)&prec->d65da - (char *)prec);
  pdbRecordType->papFldDes[882]->size=sizeof(prec->d65cv);
  pdbRecordType->papFldDes[882]->offset=(short)((char *)&prec->d65cv - (char *)prec);
  pdbRecordType->papFldDes[883]->size=sizeof(prec->d65lv);
  pdbRecordType->papFldDes[883]->offset=(short)((char *)&prec->d65lv - (char *)prec);
  pdbRecordType->papFldDes[884]->size=sizeof(prec->d65ne);
  pdbRecordType->papFldDes[884]->offset=(short)((char *)&prec->d65ne - (char *)prec);
  pdbRecordType->papFldDes[885]->size=sizeof(prec->d65eu);
  pdbRecordType->papFldDes[885]->offset=(short)((char *)&prec->d65eu - (char *)prec);
  pdbRecordType->papFldDes[886]->size=sizeof(prec->d65pr);
  pdbRecordType->papFldDes[886]->offset=(short)((char *)&prec->d65pr - (char *)prec);
  pdbRecordType->papFldDes[887]->size=sizeof(prec->d66hr);
  pdbRecordType->papFldDes[887]->offset=(short)((char *)&prec->d66hr - (char *)prec);
  pdbRecordType->papFldDes[888]->size=sizeof(prec->d66lr);
  pdbRecordType->papFldDes[888]->offset=(short)((char *)&prec->d66lr - (char *)prec);
  pdbRecordType->papFldDes[889]->size=sizeof(prec->d66da);
  pdbRecordType->papFldDes[889]->offset=(short)((char *)&prec->d66da - (char *)prec);
  pdbRecordType->papFldDes[890]->size=sizeof(prec->d66cv);
  pdbRecordType->papFldDes[890]->offset=(short)((char *)&prec->d66cv - (char *)prec);
  pdbRecordType->papFldDes[891]->size=sizeof(prec->d66lv);
  pdbRecordType->papFldDes[891]->offset=(short)((char *)&prec->d66lv - (char *)prec);
  pdbRecordType->papFldDes[892]->size=sizeof(prec->d66ne);
  pdbRecordType->papFldDes[892]->offset=(short)((char *)&prec->d66ne - (char *)prec);
  pdbRecordType->papFldDes[893]->size=sizeof(prec->d66eu);
  pdbRecordType->papFldDes[893]->offset=(short)((char *)&prec->d66eu - (char *)prec);
  pdbRecordType->papFldDes[894]->size=sizeof(prec->d66pr);
  pdbRecordType->papFldDes[894]->offset=(short)((char *)&prec->d66pr - (char *)prec);
  pdbRecordType->papFldDes[895]->size=sizeof(prec->d67hr);
  pdbRecordType->papFldDes[895]->offset=(short)((char *)&prec->d67hr - (char *)prec);
  pdbRecordType->papFldDes[896]->size=sizeof(prec->d67lr);
  pdbRecordType->papFldDes[896]->offset=(short)((char *)&prec->d67lr - (char *)prec);
  pdbRecordType->papFldDes[897]->size=sizeof(prec->d67da);
  pdbRecordType->papFldDes[897]->offset=(short)((char *)&prec->d67da - (char *)prec);
  pdbRecordType->papFldDes[898]->size=sizeof(prec->d67cv);
  pdbRecordType->papFldDes[898]->offset=(short)((char *)&prec->d67cv - (char *)prec);
  pdbRecordType->papFldDes[899]->size=sizeof(prec->d67lv);
  pdbRecordType->papFldDes[899]->offset=(short)((char *)&prec->d67lv - (char *)prec);
  pdbRecordType->papFldDes[900]->size=sizeof(prec->d67ne);
  pdbRecordType->papFldDes[900]->offset=(short)((char *)&prec->d67ne - (char *)prec);
  pdbRecordType->papFldDes[901]->size=sizeof(prec->d67eu);
  pdbRecordType->papFldDes[901]->offset=(short)((char *)&prec->d67eu - (char *)prec);
  pdbRecordType->papFldDes[902]->size=sizeof(prec->d67pr);
  pdbRecordType->papFldDes[902]->offset=(short)((char *)&prec->d67pr - (char *)prec);
  pdbRecordType->papFldDes[903]->size=sizeof(prec->d68hr);
  pdbRecordType->papFldDes[903]->offset=(short)((char *)&prec->d68hr - (char *)prec);
  pdbRecordType->papFldDes[904]->size=sizeof(prec->d68lr);
  pdbRecordType->papFldDes[904]->offset=(short)((char *)&prec->d68lr - (char *)prec);
  pdbRecordType->papFldDes[905]->size=sizeof(prec->d68da);
  pdbRecordType->papFldDes[905]->offset=(short)((char *)&prec->d68da - (char *)prec);
  pdbRecordType->papFldDes[906]->size=sizeof(prec->d68cv);
  pdbRecordType->papFldDes[906]->offset=(short)((char *)&prec->d68cv - (char *)prec);
  pdbRecordType->papFldDes[907]->size=sizeof(prec->d68lv);
  pdbRecordType->papFldDes[907]->offset=(short)((char *)&prec->d68lv - (char *)prec);
  pdbRecordType->papFldDes[908]->size=sizeof(prec->d68ne);
  pdbRecordType->papFldDes[908]->offset=(short)((char *)&prec->d68ne - (char *)prec);
  pdbRecordType->papFldDes[909]->size=sizeof(prec->d68eu);
  pdbRecordType->papFldDes[909]->offset=(short)((char *)&prec->d68eu - (char *)prec);
  pdbRecordType->papFldDes[910]->size=sizeof(prec->d68pr);
  pdbRecordType->papFldDes[910]->offset=(short)((char *)&prec->d68pr - (char *)prec);
  pdbRecordType->papFldDes[911]->size=sizeof(prec->d69hr);
  pdbRecordType->papFldDes[911]->offset=(short)((char *)&prec->d69hr - (char *)prec);
  pdbRecordType->papFldDes[912]->size=sizeof(prec->d69lr);
  pdbRecordType->papFldDes[912]->offset=(short)((char *)&prec->d69lr - (char *)prec);
  pdbRecordType->papFldDes[913]->size=sizeof(prec->d69da);
  pdbRecordType->papFldDes[913]->offset=(short)((char *)&prec->d69da - (char *)prec);
  pdbRecordType->papFldDes[914]->size=sizeof(prec->d69cv);
  pdbRecordType->papFldDes[914]->offset=(short)((char *)&prec->d69cv - (char *)prec);
  pdbRecordType->papFldDes[915]->size=sizeof(prec->d69lv);
  pdbRecordType->papFldDes[915]->offset=(short)((char *)&prec->d69lv - (char *)prec);
  pdbRecordType->papFldDes[916]->size=sizeof(prec->d69ne);
  pdbRecordType->papFldDes[916]->offset=(short)((char *)&prec->d69ne - (char *)prec);
  pdbRecordType->papFldDes[917]->size=sizeof(prec->d69eu);
  pdbRecordType->papFldDes[917]->offset=(short)((char *)&prec->d69eu - (char *)prec);
  pdbRecordType->papFldDes[918]->size=sizeof(prec->d69pr);
  pdbRecordType->papFldDes[918]->offset=(short)((char *)&prec->d69pr - (char *)prec);
  pdbRecordType->papFldDes[919]->size=sizeof(prec->d70hr);
  pdbRecordType->papFldDes[919]->offset=(short)((char *)&prec->d70hr - (char *)prec);
  pdbRecordType->papFldDes[920]->size=sizeof(prec->d70lr);
  pdbRecordType->papFldDes[920]->offset=(short)((char *)&prec->d70lr - (char *)prec);
  pdbRecordType->papFldDes[921]->size=sizeof(prec->d70da);
  pdbRecordType->papFldDes[921]->offset=(short)((char *)&prec->d70da - (char *)prec);
  pdbRecordType->papFldDes[922]->size=sizeof(prec->d70cv);
  pdbRecordType->papFldDes[922]->offset=(short)((char *)&prec->d70cv - (char *)prec);
  pdbRecordType->papFldDes[923]->size=sizeof(prec->d70lv);
  pdbRecordType->papFldDes[923]->offset=(short)((char *)&prec->d70lv - (char *)prec);
  pdbRecordType->papFldDes[924]->size=sizeof(prec->d70ne);
  pdbRecordType->papFldDes[924]->offset=(short)((char *)&prec->d70ne - (char *)prec);
  pdbRecordType->papFldDes[925]->size=sizeof(prec->d70eu);
  pdbRecordType->papFldDes[925]->offset=(short)((char *)&prec->d70eu - (char *)prec);
  pdbRecordType->papFldDes[926]->size=sizeof(prec->d70pr);
  pdbRecordType->papFldDes[926]->offset=(short)((char *)&prec->d70pr - (char *)prec);
  pdbRecordType->papFldDes[927]->size=sizeof(prec->d1hr);
  pdbRecordType->papFldDes[927]->offset=(short)((char *)&prec->d1hr - (char *)prec);
  pdbRecordType->papFldDes[928]->size=sizeof(prec->d1lr);
  pdbRecordType->papFldDes[928]->offset=(short)((char *)&prec->d1lr - (char *)prec);
  pdbRecordType->papFldDes[929]->size=sizeof(prec->d1da);
  pdbRecordType->papFldDes[929]->offset=(short)((char *)&prec->d1da - (char *)prec);
  pdbRecordType->papFldDes[930]->size=sizeof(prec->d1cv);
  pdbRecordType->papFldDes[930]->offset=(short)((char *)&prec->d1cv - (char *)prec);
  pdbRecordType->papFldDes[931]->size=sizeof(prec->d1lv);
  pdbRecordType->papFldDes[931]->offset=(short)((char *)&prec->d1lv - (char *)prec);
  pdbRecordType->papFldDes[932]->size=sizeof(prec->d1ne);
  pdbRecordType->papFldDes[932]->offset=(short)((char *)&prec->d1ne - (char *)prec);
  pdbRecordType->papFldDes[933]->size=sizeof(prec->d1eu);
  pdbRecordType->papFldDes[933]->offset=(short)((char *)&prec->d1eu - (char *)prec);
  pdbRecordType->papFldDes[934]->size=sizeof(prec->d1pr);
  pdbRecordType->papFldDes[934]->offset=(short)((char *)&prec->d1pr - (char *)prec);
  pdbRecordType->papFldDes[935]->size=sizeof(prec->d2hr);
  pdbRecordType->papFldDes[935]->offset=(short)((char *)&prec->d2hr - (char *)prec);
  pdbRecordType->papFldDes[936]->size=sizeof(prec->d2lr);
  pdbRecordType->papFldDes[936]->offset=(short)((char *)&prec->d2lr - (char *)prec);
  pdbRecordType->papFldDes[937]->size=sizeof(prec->d2da);
  pdbRecordType->papFldDes[937]->offset=(short)((char *)&prec->d2da - (char *)prec);
  pdbRecordType->papFldDes[938]->size=sizeof(prec->d2cv);
  pdbRecordType->papFldDes[938]->offset=(short)((char *)&prec->d2cv - (char *)prec);
  pdbRecordType->papFldDes[939]->size=sizeof(prec->d2lv);
  pdbRecordType->papFldDes[939]->offset=(short)((char *)&prec->d2lv - (char *)prec);
  pdbRecordType->papFldDes[940]->size=sizeof(prec->d2ne);
  pdbRecordType->papFldDes[940]->offset=(short)((char *)&prec->d2ne - (char *)prec);
  pdbRecordType->papFldDes[941]->size=sizeof(prec->d2eu);
  pdbRecordType->papFldDes[941]->offset=(short)((char *)&prec->d2eu - (char *)prec);
  pdbRecordType->papFldDes[942]->size=sizeof(prec->d2pr);
  pdbRecordType->papFldDes[942]->offset=(short)((char *)&prec->d2pr - (char *)prec);
  pdbRecordType->papFldDes[943]->size=sizeof(prec->d3hr);
  pdbRecordType->papFldDes[943]->offset=(short)((char *)&prec->d3hr - (char *)prec);
  pdbRecordType->papFldDes[944]->size=sizeof(prec->d3lr);
  pdbRecordType->papFldDes[944]->offset=(short)((char *)&prec->d3lr - (char *)prec);
  pdbRecordType->papFldDes[945]->size=sizeof(prec->d3da);
  pdbRecordType->papFldDes[945]->offset=(short)((char *)&prec->d3da - (char *)prec);
  pdbRecordType->papFldDes[946]->size=sizeof(prec->d3cv);
  pdbRecordType->papFldDes[946]->offset=(short)((char *)&prec->d3cv - (char *)prec);
  pdbRecordType->papFldDes[947]->size=sizeof(prec->d3lv);
  pdbRecordType->papFldDes[947]->offset=(short)((char *)&prec->d3lv - (char *)prec);
  pdbRecordType->papFldDes[948]->size=sizeof(prec->d3ne);
  pdbRecordType->papFldDes[948]->offset=(short)((char *)&prec->d3ne - (char *)prec);
  pdbRecordType->papFldDes[949]->size=sizeof(prec->d3eu);
  pdbRecordType->papFldDes[949]->offset=(short)((char *)&prec->d3eu - (char *)prec);
  pdbRecordType->papFldDes[950]->size=sizeof(prec->d3pr);
  pdbRecordType->papFldDes[950]->offset=(short)((char *)&prec->d3pr - (char *)prec);
  pdbRecordType->papFldDes[951]->size=sizeof(prec->d4hr);
  pdbRecordType->papFldDes[951]->offset=(short)((char *)&prec->d4hr - (char *)prec);
  pdbRecordType->papFldDes[952]->size=sizeof(prec->d4lr);
  pdbRecordType->papFldDes[952]->offset=(short)((char *)&prec->d4lr - (char *)prec);
  pdbRecordType->papFldDes[953]->size=sizeof(prec->d4da);
  pdbRecordType->papFldDes[953]->offset=(short)((char *)&prec->d4da - (char *)prec);
  pdbRecordType->papFldDes[954]->size=sizeof(prec->d4cv);
  pdbRecordType->papFldDes[954]->offset=(short)((char *)&prec->d4cv - (char *)prec);
  pdbRecordType->papFldDes[955]->size=sizeof(prec->d4lv);
  pdbRecordType->papFldDes[955]->offset=(short)((char *)&prec->d4lv - (char *)prec);
  pdbRecordType->papFldDes[956]->size=sizeof(prec->d4ne);
  pdbRecordType->papFldDes[956]->offset=(short)((char *)&prec->d4ne - (char *)prec);
  pdbRecordType->papFldDes[957]->size=sizeof(prec->d4eu);
  pdbRecordType->papFldDes[957]->offset=(short)((char *)&prec->d4eu - (char *)prec);
  pdbRecordType->papFldDes[958]->size=sizeof(prec->d4pr);
  pdbRecordType->papFldDes[958]->offset=(short)((char *)&prec->d4pr - (char *)prec);
  pdbRecordType->papFldDes[959]->size=sizeof(prec->d5hr);
  pdbRecordType->papFldDes[959]->offset=(short)((char *)&prec->d5hr - (char *)prec);
  pdbRecordType->papFldDes[960]->size=sizeof(prec->d5lr);
  pdbRecordType->papFldDes[960]->offset=(short)((char *)&prec->d5lr - (char *)prec);
  pdbRecordType->papFldDes[961]->size=sizeof(prec->d5da);
  pdbRecordType->papFldDes[961]->offset=(short)((char *)&prec->d5da - (char *)prec);
  pdbRecordType->papFldDes[962]->size=sizeof(prec->d5cv);
  pdbRecordType->papFldDes[962]->offset=(short)((char *)&prec->d5cv - (char *)prec);
  pdbRecordType->papFldDes[963]->size=sizeof(prec->d5lv);
  pdbRecordType->papFldDes[963]->offset=(short)((char *)&prec->d5lv - (char *)prec);
  pdbRecordType->papFldDes[964]->size=sizeof(prec->d5ne);
  pdbRecordType->papFldDes[964]->offset=(short)((char *)&prec->d5ne - (char *)prec);
  pdbRecordType->papFldDes[965]->size=sizeof(prec->d5eu);
  pdbRecordType->papFldDes[965]->offset=(short)((char *)&prec->d5eu - (char *)prec);
  pdbRecordType->papFldDes[966]->size=sizeof(prec->d5pr);
  pdbRecordType->papFldDes[966]->offset=(short)((char *)&prec->d5pr - (char *)prec);
  pdbRecordType->papFldDes[967]->size=sizeof(prec->d6hr);
  pdbRecordType->papFldDes[967]->offset=(short)((char *)&prec->d6hr - (char *)prec);
  pdbRecordType->papFldDes[968]->size=sizeof(prec->d6lr);
  pdbRecordType->papFldDes[968]->offset=(short)((char *)&prec->d6lr - (char *)prec);
  pdbRecordType->papFldDes[969]->size=sizeof(prec->d6da);
  pdbRecordType->papFldDes[969]->offset=(short)((char *)&prec->d6da - (char *)prec);
  pdbRecordType->papFldDes[970]->size=sizeof(prec->d6cv);
  pdbRecordType->papFldDes[970]->offset=(short)((char *)&prec->d6cv - (char *)prec);
  pdbRecordType->papFldDes[971]->size=sizeof(prec->d6lv);
  pdbRecordType->papFldDes[971]->offset=(short)((char *)&prec->d6lv - (char *)prec);
  pdbRecordType->papFldDes[972]->size=sizeof(prec->d6ne);
  pdbRecordType->papFldDes[972]->offset=(short)((char *)&prec->d6ne - (char *)prec);
  pdbRecordType->papFldDes[973]->size=sizeof(prec->d6eu);
  pdbRecordType->papFldDes[973]->offset=(short)((char *)&prec->d6eu - (char *)prec);
  pdbRecordType->papFldDes[974]->size=sizeof(prec->d6pr);
  pdbRecordType->papFldDes[974]->offset=(short)((char *)&prec->d6pr - (char *)prec);
  pdbRecordType->papFldDes[975]->size=sizeof(prec->d7hr);
  pdbRecordType->papFldDes[975]->offset=(short)((char *)&prec->d7hr - (char *)prec);
  pdbRecordType->papFldDes[976]->size=sizeof(prec->d7lr);
  pdbRecordType->papFldDes[976]->offset=(short)((char *)&prec->d7lr - (char *)prec);
  pdbRecordType->papFldDes[977]->size=sizeof(prec->d7da);
  pdbRecordType->papFldDes[977]->offset=(short)((char *)&prec->d7da - (char *)prec);
  pdbRecordType->papFldDes[978]->size=sizeof(prec->d7cv);
  pdbRecordType->papFldDes[978]->offset=(short)((char *)&prec->d7cv - (char *)prec);
  pdbRecordType->papFldDes[979]->size=sizeof(prec->d7lv);
  pdbRecordType->papFldDes[979]->offset=(short)((char *)&prec->d7lv - (char *)prec);
  pdbRecordType->papFldDes[980]->size=sizeof(prec->d7ne);
  pdbRecordType->papFldDes[980]->offset=(short)((char *)&prec->d7ne - (char *)prec);
  pdbRecordType->papFldDes[981]->size=sizeof(prec->d7eu);
  pdbRecordType->papFldDes[981]->offset=(short)((char *)&prec->d7eu - (char *)prec);
  pdbRecordType->papFldDes[982]->size=sizeof(prec->d7pr);
  pdbRecordType->papFldDes[982]->offset=(short)((char *)&prec->d7pr - (char *)prec);
  pdbRecordType->papFldDes[983]->size=sizeof(prec->d8hr);
  pdbRecordType->papFldDes[983]->offset=(short)((char *)&prec->d8hr - (char *)prec);
  pdbRecordType->papFldDes[984]->size=sizeof(prec->d8lr);
  pdbRecordType->papFldDes[984]->offset=(short)((char *)&prec->d8lr - (char *)prec);
  pdbRecordType->papFldDes[985]->size=sizeof(prec->d8da);
  pdbRecordType->papFldDes[985]->offset=(short)((char *)&prec->d8da - (char *)prec);
  pdbRecordType->papFldDes[986]->size=sizeof(prec->d8cv);
  pdbRecordType->papFldDes[986]->offset=(short)((char *)&prec->d8cv - (char *)prec);
  pdbRecordType->papFldDes[987]->size=sizeof(prec->d8lv);
  pdbRecordType->papFldDes[987]->offset=(short)((char *)&prec->d8lv - (char *)prec);
  pdbRecordType->papFldDes[988]->size=sizeof(prec->d8ne);
  pdbRecordType->papFldDes[988]->offset=(short)((char *)&prec->d8ne - (char *)prec);
  pdbRecordType->papFldDes[989]->size=sizeof(prec->d8eu);
  pdbRecordType->papFldDes[989]->offset=(short)((char *)&prec->d8eu - (char *)prec);
  pdbRecordType->papFldDes[990]->size=sizeof(prec->d8pr);
  pdbRecordType->papFldDes[990]->offset=(short)((char *)&prec->d8pr - (char *)prec);
  pdbRecordType->papFldDes[991]->size=sizeof(prec->d9hr);
  pdbRecordType->papFldDes[991]->offset=(short)((char *)&prec->d9hr - (char *)prec);
  pdbRecordType->papFldDes[992]->size=sizeof(prec->d9lr);
  pdbRecordType->papFldDes[992]->offset=(short)((char *)&prec->d9lr - (char *)prec);
  pdbRecordType->papFldDes[993]->size=sizeof(prec->d9da);
  pdbRecordType->papFldDes[993]->offset=(short)((char *)&prec->d9da - (char *)prec);
  pdbRecordType->papFldDes[994]->size=sizeof(prec->d9cv);
  pdbRecordType->papFldDes[994]->offset=(short)((char *)&prec->d9cv - (char *)prec);
  pdbRecordType->papFldDes[995]->size=sizeof(prec->d9lv);
  pdbRecordType->papFldDes[995]->offset=(short)((char *)&prec->d9lv - (char *)prec);
  pdbRecordType->papFldDes[996]->size=sizeof(prec->d9ne);
  pdbRecordType->papFldDes[996]->offset=(short)((char *)&prec->d9ne - (char *)prec);
  pdbRecordType->papFldDes[997]->size=sizeof(prec->d9eu);
  pdbRecordType->papFldDes[997]->offset=(short)((char *)&prec->d9eu - (char *)prec);
  pdbRecordType->papFldDes[998]->size=sizeof(prec->d9pr);
  pdbRecordType->papFldDes[998]->offset=(short)((char *)&prec->d9pr - (char *)prec);
  pdbRecordType->papFldDes[999]->size=sizeof(prec->dahr);
  pdbRecordType->papFldDes[999]->offset=(short)((char *)&prec->dahr - (char *)prec);
  pdbRecordType->papFldDes[1000]->size=sizeof(prec->dalr);
  pdbRecordType->papFldDes[1000]->offset=(short)((char *)&prec->dalr - (char *)prec);
  pdbRecordType->papFldDes[1001]->size=sizeof(prec->dada);
  pdbRecordType->papFldDes[1001]->offset=(short)((char *)&prec->dada - (char *)prec);
  pdbRecordType->papFldDes[1002]->size=sizeof(prec->dacv);
  pdbRecordType->papFldDes[1002]->offset=(short)((char *)&prec->dacv - (char *)prec);
  pdbRecordType->papFldDes[1003]->size=sizeof(prec->dalv);
  pdbRecordType->papFldDes[1003]->offset=(short)((char *)&prec->dalv - (char *)prec);
  pdbRecordType->papFldDes[1004]->size=sizeof(prec->dane);
  pdbRecordType->papFldDes[1004]->offset=(short)((char *)&prec->dane - (char *)prec);
  pdbRecordType->papFldDes[1005]->size=sizeof(prec->daeu);
  pdbRecordType->papFldDes[1005]->offset=(short)((char *)&prec->daeu - (char *)prec);
  pdbRecordType->papFldDes[1006]->size=sizeof(prec->dapr);
  pdbRecordType->papFldDes[1006]->offset=(short)((char *)&prec->dapr - (char *)prec);
  pdbRecordType->papFldDes[1007]->size=sizeof(prec->dbhr);
  pdbRecordType->papFldDes[1007]->offset=(short)((char *)&prec->dbhr - (char *)prec);
  pdbRecordType->papFldDes[1008]->size=sizeof(prec->dblr);
  pdbRecordType->papFldDes[1008]->offset=(short)((char *)&prec->dblr - (char *)prec);
  pdbRecordType->papFldDes[1009]->size=sizeof(prec->dbda);
  pdbRecordType->papFldDes[1009]->offset=(short)((char *)&prec->dbda - (char *)prec);
  pdbRecordType->papFldDes[1010]->size=sizeof(prec->dbcv);
  pdbRecordType->papFldDes[1010]->offset=(short)((char *)&prec->dbcv - (char *)prec);
  pdbRecordType->papFldDes[1011]->size=sizeof(prec->dblv);
  pdbRecordType->papFldDes[1011]->offset=(short)((char *)&prec->dblv - (char *)prec);
  pdbRecordType->papFldDes[1012]->size=sizeof(prec->dbne);
  pdbRecordType->papFldDes[1012]->offset=(short)((char *)&prec->dbne - (char *)prec);
  pdbRecordType->papFldDes[1013]->size=sizeof(prec->dbeu);
  pdbRecordType->papFldDes[1013]->offset=(short)((char *)&prec->dbeu - (char *)prec);
  pdbRecordType->papFldDes[1014]->size=sizeof(prec->dbpr);
  pdbRecordType->papFldDes[1014]->offset=(short)((char *)&prec->dbpr - (char *)prec);
  pdbRecordType->papFldDes[1015]->size=sizeof(prec->dchr);
  pdbRecordType->papFldDes[1015]->offset=(short)((char *)&prec->dchr - (char *)prec);
  pdbRecordType->papFldDes[1016]->size=sizeof(prec->dclr);
  pdbRecordType->papFldDes[1016]->offset=(short)((char *)&prec->dclr - (char *)prec);
  pdbRecordType->papFldDes[1017]->size=sizeof(prec->dcda);
  pdbRecordType->papFldDes[1017]->offset=(short)((char *)&prec->dcda - (char *)prec);
  pdbRecordType->papFldDes[1018]->size=sizeof(prec->dccv);
  pdbRecordType->papFldDes[1018]->offset=(short)((char *)&prec->dccv - (char *)prec);
  pdbRecordType->papFldDes[1019]->size=sizeof(prec->dclv);
  pdbRecordType->papFldDes[1019]->offset=(short)((char *)&prec->dclv - (char *)prec);
  pdbRecordType->papFldDes[1020]->size=sizeof(prec->dcne);
  pdbRecordType->papFldDes[1020]->offset=(short)((char *)&prec->dcne - (char *)prec);
  pdbRecordType->papFldDes[1021]->size=sizeof(prec->dceu);
  pdbRecordType->papFldDes[1021]->offset=(short)((char *)&prec->dceu - (char *)prec);
  pdbRecordType->papFldDes[1022]->size=sizeof(prec->dcpr);
  pdbRecordType->papFldDes[1022]->offset=(short)((char *)&prec->dcpr - (char *)prec);
  pdbRecordType->papFldDes[1023]->size=sizeof(prec->ddhr);
  pdbRecordType->papFldDes[1023]->offset=(short)((char *)&prec->ddhr - (char *)prec);
  pdbRecordType->papFldDes[1024]->size=sizeof(prec->ddlr);
  pdbRecordType->papFldDes[1024]->offset=(short)((char *)&prec->ddlr - (char *)prec);
  pdbRecordType->papFldDes[1025]->size=sizeof(prec->ddda);
  pdbRecordType->papFldDes[1025]->offset=(short)((char *)&prec->ddda - (char *)prec);
  pdbRecordType->papFldDes[1026]->size=sizeof(prec->ddcv);
  pdbRecordType->papFldDes[1026]->offset=(short)((char *)&prec->ddcv - (char *)prec);
  pdbRecordType->papFldDes[1027]->size=sizeof(prec->ddlv);
  pdbRecordType->papFldDes[1027]->offset=(short)((char *)&prec->ddlv - (char *)prec);
  pdbRecordType->papFldDes[1028]->size=sizeof(prec->ddne);
  pdbRecordType->papFldDes[1028]->offset=(short)((char *)&prec->ddne - (char *)prec);
  pdbRecordType->papFldDes[1029]->size=sizeof(prec->ddeu);
  pdbRecordType->papFldDes[1029]->offset=(short)((char *)&prec->ddeu - (char *)prec);
  pdbRecordType->papFldDes[1030]->size=sizeof(prec->ddpr);
  pdbRecordType->papFldDes[1030]->offset=(short)((char *)&prec->ddpr - (char *)prec);
  pdbRecordType->papFldDes[1031]->size=sizeof(prec->dehr);
  pdbRecordType->papFldDes[1031]->offset=(short)((char *)&prec->dehr - (char *)prec);
  pdbRecordType->papFldDes[1032]->size=sizeof(prec->delr);
  pdbRecordType->papFldDes[1032]->offset=(short)((char *)&prec->delr - (char *)prec);
  pdbRecordType->papFldDes[1033]->size=sizeof(prec->deda);
  pdbRecordType->papFldDes[1033]->offset=(short)((char *)&prec->deda - (char *)prec);
  pdbRecordType->papFldDes[1034]->size=sizeof(prec->decv);
  pdbRecordType->papFldDes[1034]->offset=(short)((char *)&prec->decv - (char *)prec);
  pdbRecordType->papFldDes[1035]->size=sizeof(prec->delv);
  pdbRecordType->papFldDes[1035]->offset=(short)((char *)&prec->delv - (char *)prec);
  pdbRecordType->papFldDes[1036]->size=sizeof(prec->dene);
  pdbRecordType->papFldDes[1036]->offset=(short)((char *)&prec->dene - (char *)prec);
  pdbRecordType->papFldDes[1037]->size=sizeof(prec->deeu);
  pdbRecordType->papFldDes[1037]->offset=(short)((char *)&prec->deeu - (char *)prec);
  pdbRecordType->papFldDes[1038]->size=sizeof(prec->depr);
  pdbRecordType->papFldDes[1038]->offset=(short)((char *)&prec->depr - (char *)prec);
  pdbRecordType->papFldDes[1039]->size=sizeof(prec->dfhr);
  pdbRecordType->papFldDes[1039]->offset=(short)((char *)&prec->dfhr - (char *)prec);
  pdbRecordType->papFldDes[1040]->size=sizeof(prec->dflr);
  pdbRecordType->papFldDes[1040]->offset=(short)((char *)&prec->dflr - (char *)prec);
  pdbRecordType->papFldDes[1041]->size=sizeof(prec->dfda);
  pdbRecordType->papFldDes[1041]->offset=(short)((char *)&prec->dfda - (char *)prec);
  pdbRecordType->papFldDes[1042]->size=sizeof(prec->dfcv);
  pdbRecordType->papFldDes[1042]->offset=(short)((char *)&prec->dfcv - (char *)prec);
  pdbRecordType->papFldDes[1043]->size=sizeof(prec->dflv);
  pdbRecordType->papFldDes[1043]->offset=(short)((char *)&prec->dflv - (char *)prec);
  pdbRecordType->papFldDes[1044]->size=sizeof(prec->dfne);
  pdbRecordType->papFldDes[1044]->offset=(short)((char *)&prec->dfne - (char *)prec);
  pdbRecordType->papFldDes[1045]->size=sizeof(prec->dfeu);
  pdbRecordType->papFldDes[1045]->offset=(short)((char *)&prec->dfeu - (char *)prec);
  pdbRecordType->papFldDes[1046]->size=sizeof(prec->dfpr);
  pdbRecordType->papFldDes[1046]->offset=(short)((char *)&prec->dfpr - (char *)prec);
  pdbRecordType->papFldDes[1047]->size=sizeof(prec->t1cd);
  pdbRecordType->papFldDes[1047]->offset=(short)((char *)&prec->t1cd - (char *)prec);
  pdbRecordType->papFldDes[1048]->size=sizeof(prec->t2cd);
  pdbRecordType->papFldDes[1048]->offset=(short)((char *)&prec->t2cd - (char *)prec);
  pdbRecordType->papFldDes[1049]->size=sizeof(prec->t3cd);
  pdbRecordType->papFldDes[1049]->offset=(short)((char *)&prec->t3cd - (char *)prec);
  pdbRecordType->papFldDes[1050]->size=sizeof(prec->t4cd);
  pdbRecordType->papFldDes[1050]->offset=(short)((char *)&prec->t4cd - (char *)prec);
  pdbRecordType->papFldDes[1051]->size=sizeof(prec->bscd);
  pdbRecordType->papFldDes[1051]->offset=(short)((char *)&prec->bscd - (char *)prec);
  pdbRecordType->papFldDes[1052]->size=sizeof(prec->ascd);
  pdbRecordType->papFldDes[1052]->offset=(short)((char *)&prec->ascd - (char *)prec);
  pdbRecordType->papFldDes[1053]->size=sizeof(prec->paus);
  pdbRecordType->papFldDes[1053]->offset=(short)((char *)&prec->paus - (char *)prec);
  pdbRecordType->papFldDes[1054]->size=sizeof(prec->lpau);
  pdbRecordType->papFldDes[1054]->offset=(short)((char *)&prec->lpau - (char *)prec);
  pdbRecordType->papFldDes[1055]->size=sizeof(prec->pdly);
  pdbRecordType->papFldDes[1055]->offset=(short)((char *)&prec->pdly - (char *)prec);
  pdbRecordType->papFldDes[1056]->size=sizeof(prec->ddly);
  pdbRecordType->papFldDes[1056]->offset=(short)((char *)&prec->ddly - (char *)prec);
  pdbRecordType->papFldDes[1057]->size=sizeof(prec->rdly);
  pdbRecordType->papFldDes[1057]->offset=(short)((char *)&prec->rdly - (char *)prec);
  pdbRecordType->papFldDes[1058]->size=sizeof(prec->faze);
  pdbRecordType->papFldDes[1058]->offset=(short)((char *)&prec->faze - (char *)prec);
  pdbRecordType->papFldDes[1059]->size=sizeof(prec->acqm);
  pdbRecordType->papFldDes[1059]->offset=(short)((char *)&prec->acqm - (char *)prec);
    pdbRecordType->rec_size = sizeof(*prec);
    return(0);
}
#endif /*GEN_SIZE_OFFSET*/
